/****************************************************************************
** Meta object code from reading C++ file 'qquickimagebase_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/quick/items/qquickimagebase_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickimagebase_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickImageBase_t {
    QByteArrayData data[44];
    char stringdata0[588];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickImageBase_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickImageBase_t qt_meta_stringdata_QQuickImageBase = {
    {
QT_MOC_LITERAL(0, 0, 15), // "QQuickImageBase"
QT_MOC_LITERAL(1, 16, 11), // "QML.Element"
QT_MOC_LITERAL(2, 28, 9), // "ImageBase"
QT_MOC_LITERAL(3, 38, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(4, 62, 2), // "14"
QT_MOC_LITERAL(5, 65, 13), // "QML.Creatable"
QT_MOC_LITERAL(6, 79, 5), // "false"
QT_MOC_LITERAL(7, 85, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(8, 107, 36), // "ImageBase is an abstract base..."
QT_MOC_LITERAL(9, 144, 13), // "sourceChanged"
QT_MOC_LITERAL(10, 158, 0), // ""
QT_MOC_LITERAL(11, 159, 17), // "sourceSizeChanged"
QT_MOC_LITERAL(12, 177, 13), // "statusChanged"
QT_MOC_LITERAL(13, 191, 23), // "QQuickImageBase::Status"
QT_MOC_LITERAL(14, 215, 15), // "progressChanged"
QT_MOC_LITERAL(15, 231, 8), // "progress"
QT_MOC_LITERAL(16, 240, 19), // "asynchronousChanged"
QT_MOC_LITERAL(17, 260, 12), // "cacheChanged"
QT_MOC_LITERAL(18, 273, 13), // "mirrorChanged"
QT_MOC_LITERAL(19, 287, 19), // "currentFrameChanged"
QT_MOC_LITERAL(20, 307, 17), // "frameCountChanged"
QT_MOC_LITERAL(21, 325, 21), // "sourceClipRectChanged"
QT_MOC_LITERAL(22, 347, 17), // "colorSpaceChanged"
QT_MOC_LITERAL(23, 365, 15), // "requestFinished"
QT_MOC_LITERAL(24, 381, 15), // "requestProgress"
QT_MOC_LITERAL(25, 397, 6), // "status"
QT_MOC_LITERAL(26, 404, 6), // "Status"
QT_MOC_LITERAL(27, 411, 6), // "source"
QT_MOC_LITERAL(28, 418, 12), // "asynchronous"
QT_MOC_LITERAL(29, 431, 5), // "cache"
QT_MOC_LITERAL(30, 437, 10), // "sourceSize"
QT_MOC_LITERAL(31, 448, 6), // "mirror"
QT_MOC_LITERAL(32, 455, 12), // "currentFrame"
QT_MOC_LITERAL(33, 468, 10), // "frameCount"
QT_MOC_LITERAL(34, 479, 10), // "colorSpace"
QT_MOC_LITERAL(35, 490, 17), // "LoadPixmapOptions"
QT_MOC_LITERAL(36, 508, 16), // "LoadPixmapOption"
QT_MOC_LITERAL(37, 525, 8), // "NoOption"
QT_MOC_LITERAL(38, 534, 9), // "HandleDPR"
QT_MOC_LITERAL(39, 544, 18), // "UseProviderOptions"
QT_MOC_LITERAL(40, 563, 4), // "Null"
QT_MOC_LITERAL(41, 568, 5), // "Ready"
QT_MOC_LITERAL(42, 574, 7), // "Loading"
QT_MOC_LITERAL(43, 582, 5) // "Error"

    },
    "QQuickImageBase\0QML.Element\0ImageBase\0"
    "QML.AddedInMinorVersion\0""14\0QML.Creatable\0"
    "false\0QML.UncreatableReason\0"
    "ImageBase is an abstract base class.\0"
    "sourceChanged\0\0sourceSizeChanged\0"
    "statusChanged\0QQuickImageBase::Status\0"
    "progressChanged\0progress\0asynchronousChanged\0"
    "cacheChanged\0mirrorChanged\0"
    "currentFrameChanged\0frameCountChanged\0"
    "sourceClipRectChanged\0colorSpaceChanged\0"
    "requestFinished\0requestProgress\0status\0"
    "Status\0source\0asynchronous\0cache\0"
    "sourceSize\0mirror\0currentFrame\0"
    "frameCount\0colorSpace\0LoadPixmapOptions\0"
    "LoadPixmapOption\0NoOption\0HandleDPR\0"
    "UseProviderOptions\0Null\0Ready\0Loading\0"
    "Error"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickImageBase[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
      13,   22, // methods
      10,  123, // properties
       2,  173, // enums/sets
       0,    0, // constructors
       0,       // flags
      11,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,

 // signals: name, argc, parameters, tag, flags
       9,    1,  100,   10, 0x06 /* Public */,
      11,    0,  103,   10, 0x06 /* Public */,
      12,    1,  104,   10, 0x06 /* Public */,
      14,    1,  107,   10, 0x06 /* Public */,
      16,    0,  110,   10, 0x06 /* Public */,
      17,    0,  111,   10, 0x06 /* Public */,
      18,    0,  112,   10, 0x06 /* Public */,
      19,    0,  113,   10, 0x86 /* Public | MethodRevisioned */,
      20,    0,  114,   10, 0x86 /* Public | MethodRevisioned */,
      21,    0,  115,   10, 0x86 /* Public | MethodRevisioned */,
      22,    0,  116,   10, 0x86 /* Public | MethodRevisioned */,

 // slots: name, argc, parameters, tag, flags
      23,    0,  117,   10, 0x08 /* Private */,
      24,    2,  118,   10, 0x08 /* Private */,

 // signals: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
      14,
      14,
      15,
      15,

 // slots: revision
       0,
       0,

 // signals: parameters
    QMetaType::Void, QMetaType::QUrl,   10,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 13,   10,
    QMetaType::Void, QMetaType::QReal,   15,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::LongLong, QMetaType::LongLong,   10,   10,

 // properties: name, type, flags
      25, 0x80000000 | 26, 0x00495009,
      27, QMetaType::QUrl, 0x00495103,
      15, QMetaType::QReal, 0x00495001,
      28, QMetaType::Bool, 0x00495103,
      29, QMetaType::Bool, 0x00495103,
      30, QMetaType::QSize, 0x00495107,
      31, QMetaType::Bool, 0x00495103,
      32, QMetaType::Int, 0x00c95103,
      33, QMetaType::Int, 0x00c95001,
      34, QMetaType::QColorSpace, 0x00c95103,

 // properties: notify_signal_id
       2,
       0,
       3,
       4,
       5,
       1,
       6,
       7,
       8,
      10,

 // properties: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
      14,
      14,
      15,

 // enums: name, alias, flags, count, data
      35,   36, 0x1,    3,  183,
      26,   26, 0x0,    4,  189,

 // enum data: key, value
      37, uint(QQuickImageBase::NoOption),
      38, uint(QQuickImageBase::HandleDPR),
      39, uint(QQuickImageBase::UseProviderOptions),
      40, uint(QQuickImageBase::Null),
      41, uint(QQuickImageBase::Ready),
      42, uint(QQuickImageBase::Loading),
      43, uint(QQuickImageBase::Error),

       0        // eod
};

void QQuickImageBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickImageBase *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->sourceChanged((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 1: _t->sourceSizeChanged(); break;
        case 2: _t->statusChanged((*reinterpret_cast< QQuickImageBase::Status(*)>(_a[1]))); break;
        case 3: _t->progressChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 4: _t->asynchronousChanged(); break;
        case 5: _t->cacheChanged(); break;
        case 6: _t->mirrorChanged(); break;
        case 7: _t->currentFrameChanged(); break;
        case 8: _t->frameCountChanged(); break;
        case 9: _t->sourceClipRectChanged(); break;
        case 10: _t->colorSpaceChanged(); break;
        case 11: _t->requestFinished(); break;
        case 12: _t->requestProgress((*reinterpret_cast< qint64(*)>(_a[1])),(*reinterpret_cast< qint64(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickImageBase::*)(const QUrl & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageBase::sourceChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickImageBase::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageBase::sourceSizeChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickImageBase::*)(QQuickImageBase::Status );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageBase::statusChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickImageBase::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageBase::progressChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickImageBase::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageBase::asynchronousChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickImageBase::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageBase::cacheChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickImageBase::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageBase::mirrorChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickImageBase::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageBase::currentFrameChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickImageBase::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageBase::frameCountChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickImageBase::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageBase::sourceClipRectChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QQuickImageBase::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageBase::colorSpaceChanged)) {
                *result = 10;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickImageBase *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< Status*>(_v) = _t->status(); break;
        case 1: *reinterpret_cast< QUrl*>(_v) = _t->source(); break;
        case 2: *reinterpret_cast< qreal*>(_v) = _t->progress(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->asynchronous(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->cache(); break;
        case 5: *reinterpret_cast< QSize*>(_v) = _t->sourceSize(); break;
        case 6: *reinterpret_cast< bool*>(_v) = _t->mirror(); break;
        case 7: *reinterpret_cast< int*>(_v) = _t->currentFrame(); break;
        case 8: *reinterpret_cast< int*>(_v) = _t->frameCount(); break;
        case 9: *reinterpret_cast< QColorSpace*>(_v) = _t->colorSpace(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickImageBase *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setSource(*reinterpret_cast< QUrl*>(_v)); break;
        case 3: _t->setAsynchronous(*reinterpret_cast< bool*>(_v)); break;
        case 4: _t->setCache(*reinterpret_cast< bool*>(_v)); break;
        case 5: _t->setSourceSize(*reinterpret_cast< QSize*>(_v)); break;
        case 6: _t->setMirror(*reinterpret_cast< bool*>(_v)); break;
        case 7: _t->setCurrentFrame(*reinterpret_cast< int*>(_v)); break;
        case 9: _t->setColorSpace(*reinterpret_cast< QColorSpace*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        QQuickImageBase *_t = static_cast<QQuickImageBase *>(_o);
        (void)_t;
        switch (_id) {
        case 5: _t->resetSourceSize(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickImageBase::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickImplicitSizeItem::staticMetaObject>(),
    qt_meta_stringdata_QQuickImageBase.data,
    qt_meta_data_QQuickImageBase,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickImageBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickImageBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickImageBase.stringdata0))
        return static_cast<void*>(this);
    return QQuickImplicitSizeItem::qt_metacast(_clname);
}

int QQuickImageBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickImplicitSizeItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 10;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickImageBase::sourceChanged(const QUrl & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QQuickImageBase::sourceSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickImageBase::statusChanged(QQuickImageBase::Status _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QQuickImageBase::progressChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QQuickImageBase::asynchronousChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickImageBase::cacheChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickImageBase::mirrorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQuickImageBase::currentFrameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QQuickImageBase::frameCountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QQuickImageBase::sourceClipRectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QQuickImageBase::colorSpaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
