/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#include <qquickboundaryrule_p.h>

void qml_register_types_Qt_labs_animation()
{
    qmlRegisterTypesAndRevisions<QQuickBoundaryRule>("Qt.labs.animation", 1);
    qmlRegisterModule("Qt.labs.animation", 1, 0);
}

static const QQmlModuleRegistration registration("Qt.labs.animation", 1, qml_register_types_Qt_labs_animation);
