/****************************************************************************
** Meta object code from reading C++ file 'qquicktargetdirection_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/particles/qquicktargetdirection_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquicktargetdirection_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickTargetDirection_t {
    QByteArrayData data[27];
    char stringdata0[419];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickTargetDirection_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickTargetDirection_t qt_meta_stringdata_QQuickTargetDirection = {
    {
QT_MOC_LITERAL(0, 0, 21), // "QQuickTargetDirection"
QT_MOC_LITERAL(1, 22, 11), // "QML.Element"
QT_MOC_LITERAL(2, 34, 15), // "TargetDirection"
QT_MOC_LITERAL(3, 50, 14), // "targetXChanged"
QT_MOC_LITERAL(4, 65, 0), // ""
QT_MOC_LITERAL(5, 66, 3), // "arg"
QT_MOC_LITERAL(6, 70, 14), // "targetYChanged"
QT_MOC_LITERAL(7, 85, 22), // "targetVariationChanged"
QT_MOC_LITERAL(8, 108, 16), // "magnitudeChanged"
QT_MOC_LITERAL(9, 125, 28), // "proprotionalMagnitudeChanged"
QT_MOC_LITERAL(10, 154, 25), // "magnitudeVariationChanged"
QT_MOC_LITERAL(11, 180, 17), // "targetItemChanged"
QT_MOC_LITERAL(12, 198, 11), // "QQuickItem*"
QT_MOC_LITERAL(13, 210, 10), // "setTargetX"
QT_MOC_LITERAL(14, 221, 10), // "setTargetY"
QT_MOC_LITERAL(15, 232, 18), // "setTargetVariation"
QT_MOC_LITERAL(16, 251, 12), // "setMagnitude"
QT_MOC_LITERAL(17, 264, 24), // "setProportionalMagnitude"
QT_MOC_LITERAL(18, 289, 21), // "setMagnitudeVariation"
QT_MOC_LITERAL(19, 311, 13), // "setTargetItem"
QT_MOC_LITERAL(20, 325, 7), // "targetX"
QT_MOC_LITERAL(21, 333, 7), // "targetY"
QT_MOC_LITERAL(22, 341, 10), // "targetItem"
QT_MOC_LITERAL(23, 352, 15), // "targetVariation"
QT_MOC_LITERAL(24, 368, 21), // "proportionalMagnitude"
QT_MOC_LITERAL(25, 390, 9), // "magnitude"
QT_MOC_LITERAL(26, 400, 18) // "magnitudeVariation"

    },
    "QQuickTargetDirection\0QML.Element\0"
    "TargetDirection\0targetXChanged\0\0arg\0"
    "targetYChanged\0targetVariationChanged\0"
    "magnitudeChanged\0proprotionalMagnitudeChanged\0"
    "magnitudeVariationChanged\0targetItemChanged\0"
    "QQuickItem*\0setTargetX\0setTargetY\0"
    "setTargetVariation\0setMagnitude\0"
    "setProportionalMagnitude\0setMagnitudeVariation\0"
    "setTargetItem\0targetX\0targetY\0targetItem\0"
    "targetVariation\0proportionalMagnitude\0"
    "magnitude\0magnitudeVariation"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickTargetDirection[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      14,   16, // methods
       7,  128, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    1,   86,    4, 0x06 /* Public */,
       6,    1,   89,    4, 0x06 /* Public */,
       7,    1,   92,    4, 0x06 /* Public */,
       8,    1,   95,    4, 0x06 /* Public */,
       9,    1,   98,    4, 0x06 /* Public */,
      10,    1,  101,    4, 0x06 /* Public */,
      11,    1,  104,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      13,    1,  107,    4, 0x0a /* Public */,
      14,    1,  110,    4, 0x0a /* Public */,
      15,    1,  113,    4, 0x0a /* Public */,
      16,    1,  116,    4, 0x0a /* Public */,
      17,    1,  119,    4, 0x0a /* Public */,
      18,    1,  122,    4, 0x0a /* Public */,
      19,    1,  125,    4, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QReal,    5,
    QMetaType::Void, QMetaType::QReal,    5,
    QMetaType::Void, QMetaType::QReal,    5,
    QMetaType::Void, QMetaType::QReal,    5,
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void, QMetaType::QReal,    5,
    QMetaType::Void, 0x80000000 | 12,    5,

 // slots: parameters
    QMetaType::Void, QMetaType::QReal,    5,
    QMetaType::Void, QMetaType::QReal,    5,
    QMetaType::Void, QMetaType::QReal,    5,
    QMetaType::Void, QMetaType::QReal,    5,
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void, QMetaType::QReal,    5,
    QMetaType::Void, 0x80000000 | 12,    5,

 // properties: name, type, flags
      20, QMetaType::QReal, 0x00495103,
      21, QMetaType::QReal, 0x00495103,
      22, 0x80000000 | 12, 0x0049510b,
      23, QMetaType::QReal, 0x00495103,
      24, QMetaType::Bool, 0x00495103,
      25, QMetaType::QReal, 0x00495103,
      26, QMetaType::QReal, 0x00495103,

 // properties: notify_signal_id
       0,
       1,
       6,
       2,
       4,
       3,
       5,

       0        // eod
};

void QQuickTargetDirection::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickTargetDirection *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->targetXChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 1: _t->targetYChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 2: _t->targetVariationChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 3: _t->magnitudeChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 4: _t->proprotionalMagnitudeChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->magnitudeVariationChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 6: _t->targetItemChanged((*reinterpret_cast< QQuickItem*(*)>(_a[1]))); break;
        case 7: _t->setTargetX((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 8: _t->setTargetY((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 9: _t->setTargetVariation((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 10: _t->setMagnitude((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 11: _t->setProportionalMagnitude((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 12: _t->setMagnitudeVariation((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 13: _t->setTargetItem((*reinterpret_cast< QQuickItem*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickTargetDirection::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTargetDirection::targetXChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickTargetDirection::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTargetDirection::targetYChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickTargetDirection::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTargetDirection::targetVariationChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickTargetDirection::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTargetDirection::magnitudeChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickTargetDirection::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTargetDirection::proprotionalMagnitudeChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickTargetDirection::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTargetDirection::magnitudeVariationChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickTargetDirection::*)(QQuickItem * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTargetDirection::targetItemChanged)) {
                *result = 6;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickTargetDirection *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< qreal*>(_v) = _t->targetX(); break;
        case 1: *reinterpret_cast< qreal*>(_v) = _t->targetY(); break;
        case 2: *reinterpret_cast< QQuickItem**>(_v) = _t->targetItem(); break;
        case 3: *reinterpret_cast< qreal*>(_v) = _t->targetVariation(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->proportionalMagnitude(); break;
        case 5: *reinterpret_cast< qreal*>(_v) = _t->magnitude(); break;
        case 6: *reinterpret_cast< qreal*>(_v) = _t->magnitudeVariation(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickTargetDirection *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTargetX(*reinterpret_cast< qreal*>(_v)); break;
        case 1: _t->setTargetY(*reinterpret_cast< qreal*>(_v)); break;
        case 2: _t->setTargetItem(*reinterpret_cast< QQuickItem**>(_v)); break;
        case 3: _t->setTargetVariation(*reinterpret_cast< qreal*>(_v)); break;
        case 4: _t->setProportionalMagnitude(*reinterpret_cast< bool*>(_v)); break;
        case 5: _t->setMagnitude(*reinterpret_cast< qreal*>(_v)); break;
        case 6: _t->setMagnitudeVariation(*reinterpret_cast< qreal*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickTargetDirection::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickDirection::staticMetaObject>(),
    qt_meta_stringdata_QQuickTargetDirection.data,
    qt_meta_data_QQuickTargetDirection,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickTargetDirection::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickTargetDirection::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickTargetDirection.stringdata0))
        return static_cast<void*>(this);
    return QQuickDirection::qt_metacast(_clname);
}

int QQuickTargetDirection::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickDirection::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 14;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 7;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickTargetDirection::targetXChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QQuickTargetDirection::targetYChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QQuickTargetDirection::targetVariationChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QQuickTargetDirection::magnitudeChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QQuickTargetDirection::proprotionalMagnitudeChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QQuickTargetDirection::magnitudeVariationChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QQuickTargetDirection::targetItemChanged(QQuickItem * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
