/****************************************************************************
** Meta object code from reading C++ file 'qquickparticleaffector_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/particles/qquickparticleaffector_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickparticleaffector_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickParticleAffector_t {
    QByteArrayData data[33];
    char stringdata0[444];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickParticleAffector_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickParticleAffector_t qt_meta_stringdata_QQuickParticleAffector = {
    {
QT_MOC_LITERAL(0, 0, 22), // "QQuickParticleAffector"
QT_MOC_LITERAL(1, 23, 11), // "QML.Element"
QT_MOC_LITERAL(2, 35, 16), // "ParticleAffector"
QT_MOC_LITERAL(3, 52, 13), // "QML.Creatable"
QT_MOC_LITERAL(4, 66, 5), // "false"
QT_MOC_LITERAL(5, 72, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(6, 94, 55), // "Abstract type. Use one of the..."
QT_MOC_LITERAL(7, 150, 13), // "systemChanged"
QT_MOC_LITERAL(8, 164, 0), // ""
QT_MOC_LITERAL(9, 165, 21), // "QQuickParticleSystem*"
QT_MOC_LITERAL(10, 187, 3), // "arg"
QT_MOC_LITERAL(11, 191, 13), // "groupsChanged"
QT_MOC_LITERAL(12, 205, 14), // "enabledChanged"
QT_MOC_LITERAL(13, 220, 11), // "onceChanged"
QT_MOC_LITERAL(14, 232, 12), // "shapeChanged"
QT_MOC_LITERAL(15, 245, 23), // "QQuickParticleExtruder*"
QT_MOC_LITERAL(16, 269, 8), // "affected"
QT_MOC_LITERAL(17, 278, 1), // "x"
QT_MOC_LITERAL(18, 280, 1), // "y"
QT_MOC_LITERAL(19, 282, 24), // "whenCollidingWithChanged"
QT_MOC_LITERAL(20, 307, 9), // "setSystem"
QT_MOC_LITERAL(21, 317, 9), // "setGroups"
QT_MOC_LITERAL(22, 327, 10), // "setEnabled"
QT_MOC_LITERAL(23, 338, 10), // "setOnceOff"
QT_MOC_LITERAL(24, 349, 8), // "setShape"
QT_MOC_LITERAL(25, 358, 20), // "setWhenCollidingWith"
QT_MOC_LITERAL(26, 379, 13), // "updateOffsets"
QT_MOC_LITERAL(27, 393, 6), // "system"
QT_MOC_LITERAL(28, 400, 6), // "groups"
QT_MOC_LITERAL(29, 407, 17), // "whenCollidingWith"
QT_MOC_LITERAL(30, 425, 7), // "enabled"
QT_MOC_LITERAL(31, 433, 4), // "once"
QT_MOC_LITERAL(32, 438, 5) // "shape"

    },
    "QQuickParticleAffector\0QML.Element\0"
    "ParticleAffector\0QML.Creatable\0false\0"
    "QML.UncreatableReason\0"
    "Abstract type. Use one of the inheriting types instead.\0"
    "systemChanged\0\0QQuickParticleSystem*\0"
    "arg\0groupsChanged\0enabledChanged\0"
    "onceChanged\0shapeChanged\0"
    "QQuickParticleExtruder*\0affected\0x\0y\0"
    "whenCollidingWithChanged\0setSystem\0"
    "setGroups\0setEnabled\0setOnceOff\0"
    "setShape\0setWhenCollidingWith\0"
    "updateOffsets\0system\0groups\0"
    "whenCollidingWith\0enabled\0once\0shape"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickParticleAffector[] = {

 // content:
       8,       // revision
       0,       // classname
       3,   14, // classinfo
      14,   20, // methods
       6,  132, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,

 // signals: name, argc, parameters, tag, flags
       7,    1,   90,    8, 0x06 /* Public */,
      11,    1,   93,    8, 0x06 /* Public */,
      12,    1,   96,    8, 0x06 /* Public */,
      13,    1,   99,    8, 0x06 /* Public */,
      14,    1,  102,    8, 0x06 /* Public */,
      16,    2,  105,    8, 0x06 /* Public */,
      19,    1,  110,    8, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      20,    1,  113,    8, 0x0a /* Public */,
      21,    1,  116,    8, 0x0a /* Public */,
      22,    1,  119,    8, 0x0a /* Public */,
      23,    1,  122,    8, 0x0a /* Public */,
      24,    1,  125,    8, 0x0a /* Public */,
      25,    1,  128,    8, 0x0a /* Public */,
      26,    0,  131,    8, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, QMetaType::QStringList,   10,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, 0x80000000 | 15,   10,
    QMetaType::Void, QMetaType::QReal, QMetaType::QReal,   17,   18,
    QMetaType::Void, QMetaType::QStringList,   10,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, QMetaType::QStringList,   10,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, 0x80000000 | 15,   10,
    QMetaType::Void, QMetaType::QStringList,   10,
    QMetaType::Void,

 // properties: name, type, flags
      27, 0x80000000 | 9, 0x0049510b,
      28, QMetaType::QStringList, 0x00495103,
      29, QMetaType::QStringList, 0x00495103,
      30, QMetaType::Bool, 0x00495103,
      31, QMetaType::Bool, 0x00495003,
      32, 0x80000000 | 15, 0x0049510b,

 // properties: notify_signal_id
       0,
       1,
       6,
       2,
       3,
       4,

       0        // eod
};

void QQuickParticleAffector::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickParticleAffector *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->systemChanged((*reinterpret_cast< QQuickParticleSystem*(*)>(_a[1]))); break;
        case 1: _t->groupsChanged((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 2: _t->enabledChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->onceChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->shapeChanged((*reinterpret_cast< QQuickParticleExtruder*(*)>(_a[1]))); break;
        case 5: _t->affected((*reinterpret_cast< qreal(*)>(_a[1])),(*reinterpret_cast< qreal(*)>(_a[2]))); break;
        case 6: _t->whenCollidingWithChanged((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 7: _t->setSystem((*reinterpret_cast< QQuickParticleSystem*(*)>(_a[1]))); break;
        case 8: _t->setGroups((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 9: _t->setEnabled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->setOnceOff((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->setShape((*reinterpret_cast< QQuickParticleExtruder*(*)>(_a[1]))); break;
        case 12: _t->setWhenCollidingWith((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 13: _t->updateOffsets(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickParticleSystem* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickParticleExtruder* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickParticleSystem* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickParticleExtruder* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickParticleAffector::*)(QQuickParticleSystem * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleAffector::systemChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickParticleAffector::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleAffector::groupsChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickParticleAffector::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleAffector::enabledChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickParticleAffector::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleAffector::onceChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickParticleAffector::*)(QQuickParticleExtruder * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleAffector::shapeChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickParticleAffector::*)(qreal , qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleAffector::affected)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickParticleAffector::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleAffector::whenCollidingWithChanged)) {
                *result = 6;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickParticleExtruder* >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickParticleSystem* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickParticleAffector *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQuickParticleSystem**>(_v) = _t->system(); break;
        case 1: *reinterpret_cast< QStringList*>(_v) = _t->groups(); break;
        case 2: *reinterpret_cast< QStringList*>(_v) = _t->whenCollidingWith(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->enabled(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->onceOff(); break;
        case 5: *reinterpret_cast< QQuickParticleExtruder**>(_v) = _t->shape(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickParticleAffector *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSystem(*reinterpret_cast< QQuickParticleSystem**>(_v)); break;
        case 1: _t->setGroups(*reinterpret_cast< QStringList*>(_v)); break;
        case 2: _t->setWhenCollidingWith(*reinterpret_cast< QStringList*>(_v)); break;
        case 3: _t->setEnabled(*reinterpret_cast< bool*>(_v)); break;
        case 4: _t->setOnceOff(*reinterpret_cast< bool*>(_v)); break;
        case 5: _t->setShape(*reinterpret_cast< QQuickParticleExtruder**>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickParticleAffector::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_meta_stringdata_QQuickParticleAffector.data,
    qt_meta_data_QQuickParticleAffector,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickParticleAffector::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickParticleAffector::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickParticleAffector.stringdata0))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int QQuickParticleAffector::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 6;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickParticleAffector::systemChanged(QQuickParticleSystem * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QQuickParticleAffector::groupsChanged(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QQuickParticleAffector::enabledChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QQuickParticleAffector::onceChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QQuickParticleAffector::shapeChanged(QQuickParticleExtruder * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QQuickParticleAffector::affected(qreal _t1, qreal _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QQuickParticleAffector::whenCollidingWithChanged(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
