/****************************************************************************
** Meta object code from reading C++ file 'plugin.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../kde-qtdeclarative/src/imports/window/plugin.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'plugin.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QWindowForeign_t {
    QByteArrayData data[7];
    char stringdata0[83];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QWindowForeign_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QWindowForeign_t qt_meta_stringdata_QWindowForeign = {
    {
QT_MOC_LITERAL(0, 0, 14), // "QWindowForeign"
QT_MOC_LITERAL(1, 15, 11), // "QML.Foreign"
QT_MOC_LITERAL(2, 27, 7), // "QWindow"
QT_MOC_LITERAL(3, 35, 11), // "QML.Element"
QT_MOC_LITERAL(4, 47, 9), // "anonymous"
QT_MOC_LITERAL(5, 57, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(6, 81, 1) // "1"

    },
    "QWindowForeign\0QML.Foreign\0QWindow\0"
    "QML.Element\0anonymous\0QML.AddedInMinorVersion\0"
    "1"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QWindowForeign[] = {

 // content:
       8,       // revision
       0,       // classname
       3,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject QWindowForeign::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_QWindowForeign.data,
    qt_meta_data_QWindowForeign,
    nullptr,
    nullptr,
    nullptr
} };

struct qt_meta_stringdata_QQuickWindowForeign_t {
    QByteArrayData data[9];
    char stringdata0[118];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickWindowForeign_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickWindowForeign_t qt_meta_stringdata_QQuickWindowForeign = {
    {
QT_MOC_LITERAL(0, 0, 19), // "QQuickWindowForeign"
QT_MOC_LITERAL(1, 20, 11), // "QML.Foreign"
QT_MOC_LITERAL(2, 32, 12), // "QQuickWindow"
QT_MOC_LITERAL(3, 45, 11), // "QML.Element"
QT_MOC_LITERAL(4, 57, 6), // "Window"
QT_MOC_LITERAL(5, 64, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(6, 88, 1), // "0"
QT_MOC_LITERAL(7, 90, 25), // "QML.RemovedInMinorVersion"
QT_MOC_LITERAL(8, 116, 1) // "1"

    },
    "QQuickWindowForeign\0QML.Foreign\0"
    "QQuickWindow\0QML.Element\0Window\0"
    "QML.AddedInMinorVersion\0""0\0"
    "QML.RemovedInMinorVersion\0""1"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickWindowForeign[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject QQuickWindowForeign::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_QQuickWindowForeign.data,
    qt_meta_data_QQuickWindowForeign,
    nullptr,
    nullptr,
    nullptr
} };

struct qt_meta_stringdata_QQuickWindowForeignAttached_t {
    QByteArrayData data[5];
    char stringdata0[83];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickWindowForeignAttached_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickWindowForeignAttached_t qt_meta_stringdata_QQuickWindowForeignAttached = {
    {
QT_MOC_LITERAL(0, 0, 27), // "QQuickWindowForeignAttached"
QT_MOC_LITERAL(1, 28, 11), // "QML.Foreign"
QT_MOC_LITERAL(2, 40, 20), // "QQuickWindowAttached"
QT_MOC_LITERAL(3, 61, 11), // "QML.Element"
QT_MOC_LITERAL(4, 73, 9) // "anonymous"

    },
    "QQuickWindowForeignAttached\0QML.Foreign\0"
    "QQuickWindowAttached\0QML.Element\0"
    "anonymous"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickWindowForeignAttached[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject QQuickWindowForeignAttached::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_QQuickWindowForeignAttached.data,
    qt_meta_data_QQuickWindowForeignAttached,
    nullptr,
    nullptr,
    nullptr
} };

struct qt_meta_stringdata_QQuickScreenInfoForeign_t {
    QByteArrayData data[11];
    char stringdata0[199];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickScreenInfoForeign_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickScreenInfoForeign_t qt_meta_stringdata_QQuickScreenInfoForeign = {
    {
QT_MOC_LITERAL(0, 0, 23), // "QQuickScreenInfoForeign"
QT_MOC_LITERAL(1, 24, 11), // "QML.Foreign"
QT_MOC_LITERAL(2, 36, 16), // "QQuickScreenInfo"
QT_MOC_LITERAL(3, 53, 11), // "QML.Element"
QT_MOC_LITERAL(4, 65, 10), // "ScreenInfo"
QT_MOC_LITERAL(5, 76, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(6, 100, 1), // "3"
QT_MOC_LITERAL(7, 102, 13), // "QML.Creatable"
QT_MOC_LITERAL(8, 116, 5), // "false"
QT_MOC_LITERAL(9, 122, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(10, 144, 54) // "ScreenInfo can only be used v..."

    },
    "QQuickScreenInfoForeign\0QML.Foreign\0"
    "QQuickScreenInfo\0QML.Element\0ScreenInfo\0"
    "QML.AddedInMinorVersion\0""3\0QML.Creatable\0"
    "false\0QML.UncreatableReason\0"
    "ScreenInfo can only be used via the attached property."
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickScreenInfoForeign[] = {

 // content:
       8,       // revision
       0,       // classname
       5,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,
       9,   10,

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject QQuickScreenInfoForeign::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_QQuickScreenInfoForeign.data,
    qt_meta_data_QQuickScreenInfoForeign,
    nullptr,
    nullptr,
    nullptr
} };

struct qt_meta_stringdata_QQuickScreenForeignAttached_t {
    QByteArrayData data[5];
    char stringdata0[83];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickScreenForeignAttached_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickScreenForeignAttached_t qt_meta_stringdata_QQuickScreenForeignAttached = {
    {
QT_MOC_LITERAL(0, 0, 27), // "QQuickScreenForeignAttached"
QT_MOC_LITERAL(1, 28, 11), // "QML.Foreign"
QT_MOC_LITERAL(2, 40, 20), // "QQuickScreenAttached"
QT_MOC_LITERAL(3, 61, 11), // "QML.Element"
QT_MOC_LITERAL(4, 73, 9) // "anonymous"

    },
    "QQuickScreenForeignAttached\0QML.Foreign\0"
    "QQuickScreenAttached\0QML.Element\0"
    "anonymous"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickScreenForeignAttached[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject QQuickScreenForeignAttached::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_QQuickScreenForeignAttached.data,
    qt_meta_data_QQuickScreenForeignAttached,
    nullptr,
    nullptr,
    nullptr
} };

struct qt_meta_stringdata_QQuickScreenForeign_t {
    QByteArrayData data[9];
    char stringdata0[157];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickScreenForeign_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickScreenForeign_t qt_meta_stringdata_QQuickScreenForeign = {
    {
QT_MOC_LITERAL(0, 0, 19), // "QQuickScreenForeign"
QT_MOC_LITERAL(1, 20, 11), // "QML.Foreign"
QT_MOC_LITERAL(2, 32, 12), // "QQuickScreen"
QT_MOC_LITERAL(3, 45, 11), // "QML.Element"
QT_MOC_LITERAL(4, 57, 6), // "Screen"
QT_MOC_LITERAL(5, 64, 13), // "QML.Creatable"
QT_MOC_LITERAL(6, 78, 5), // "false"
QT_MOC_LITERAL(7, 84, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(8, 106, 50) // "Screen can only be used via t..."

    },
    "QQuickScreenForeign\0QML.Foreign\0"
    "QQuickScreen\0QML.Element\0Screen\0"
    "QML.Creatable\0false\0QML.UncreatableReason\0"
    "Screen can only be used via the attached property."
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickScreenForeign[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject QQuickScreenForeign::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_QQuickScreenForeign.data,
    qt_meta_data_QQuickScreenForeign,
    nullptr,
    nullptr,
    nullptr
} };

struct qt_meta_stringdata_QQuickWindowQmlImplForeign_t {
    QByteArrayData data[7];
    char stringdata0[104];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickWindowQmlImplForeign_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickWindowQmlImplForeign_t qt_meta_stringdata_QQuickWindowQmlImplForeign = {
    {
QT_MOC_LITERAL(0, 0, 26), // "QQuickWindowQmlImplForeign"
QT_MOC_LITERAL(1, 27, 11), // "QML.Foreign"
QT_MOC_LITERAL(2, 39, 19), // "QQuickWindowQmlImpl"
QT_MOC_LITERAL(3, 59, 11), // "QML.Element"
QT_MOC_LITERAL(4, 71, 6), // "Window"
QT_MOC_LITERAL(5, 78, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(6, 102, 1) // "1"

    },
    "QQuickWindowQmlImplForeign\0QML.Foreign\0"
    "QQuickWindowQmlImpl\0QML.Element\0Window\0"
    "QML.AddedInMinorVersion\0""1"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickWindowQmlImplForeign[] = {

 // content:
       8,       // revision
       0,       // classname
       3,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject QQuickWindowQmlImplForeign::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_QQuickWindowQmlImplForeign.data,
    qt_meta_data_QQuickWindowQmlImplForeign,
    nullptr,
    nullptr,
    nullptr
} };

struct qt_meta_stringdata_QQuickRootItemForeign_t {
    QByteArrayData data[5];
    char stringdata0[71];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickRootItemForeign_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickRootItemForeign_t qt_meta_stringdata_QQuickRootItemForeign = {
    {
QT_MOC_LITERAL(0, 0, 21), // "QQuickRootItemForeign"
QT_MOC_LITERAL(1, 22, 11), // "QML.Foreign"
QT_MOC_LITERAL(2, 34, 14), // "QQuickRootItem"
QT_MOC_LITERAL(3, 49, 11), // "QML.Element"
QT_MOC_LITERAL(4, 61, 9) // "anonymous"

    },
    "QQuickRootItemForeign\0QML.Foreign\0"
    "QQuickRootItem\0QML.Element\0anonymous"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickRootItemForeign[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject QQuickRootItemForeign::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_QQuickRootItemForeign.data,
    qt_meta_data_QQuickRootItemForeign,
    nullptr,
    nullptr,
    nullptr
} };

QT_WARNING_POP
QT_END_MOC_NAMESPACE
