/********************************************************************************
** Form generated from reading UI file 'properties.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROPERTIES_H
#define UI_PROPERTIES_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Properties
{
public:
    QGridLayout *gridLayout;
    QSpacerItem *spacer;
    QLabel *label_3;
    QSpinBox *spinBox;
    QLabel *label;
    QPlainTextEdit *plainTextEdit;
    QLabel *label_2;
    QComboBox *comboBox;
    QSpacerItem *spacer_2;
    QGroupBox *groupBox;
    QPushButton *importButton;

    void setupUi(QWidget *Properties)
    {
        if (Properties->objectName().isEmpty())
            Properties->setObjectName(QString::fromUtf8("Properties"));
        Properties->resize(487, 627);
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(Properties->sizePolicy().hasHeightForWidth());
        Properties->setSizePolicy(sizePolicy);
        gridLayout = new QGridLayout(Properties);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        spacer = new QSpacerItem(20, 13, QSizePolicy::Minimum, QSizePolicy::Fixed);

        gridLayout->addItem(spacer, 0, 2, 1, 1);

        label_3 = new QLabel(Properties);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        gridLayout->addWidget(label_3, 1, 0, 1, 1);

        spinBox = new QSpinBox(Properties);
        spinBox->setObjectName(QString::fromUtf8("spinBox"));

        gridLayout->addWidget(spinBox, 1, 1, 1, 2);

        label = new QLabel(Properties);
        label->setObjectName(QString::fromUtf8("label"));

        gridLayout->addWidget(label, 2, 0, 1, 1);

        plainTextEdit = new QPlainTextEdit(Properties);
        plainTextEdit->setObjectName(QString::fromUtf8("plainTextEdit"));
        plainTextEdit->setMaximumSize(QSize(16777215, 128));
        plainTextEdit->setReadOnly(false);
        plainTextEdit->setPlainText(QString::fromUtf8(""));

        gridLayout->addWidget(plainTextEdit, 2, 1, 1, 2);

        label_2 = new QLabel(Properties);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout->addWidget(label_2, 3, 0, 1, 1);

        comboBox = new QComboBox(Properties);
        comboBox->setObjectName(QString::fromUtf8("comboBox"));
        QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(comboBox->sizePolicy().hasHeightForWidth());
        comboBox->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(comboBox, 3, 1, 1, 2);

        spacer_2 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        gridLayout->addItem(spacer_2, 6, 1, 1, 1);

        groupBox = new QGroupBox(Properties);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        QSizePolicy sizePolicy2(QSizePolicy::Preferred, QSizePolicy::Expanding);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(groupBox->sizePolicy().hasHeightForWidth());
        groupBox->setSizePolicy(sizePolicy2);
        groupBox->setMinimumSize(QSize(400, 0));
        groupBox->setFlat(true);

        gridLayout->addWidget(groupBox, 9, 0, 1, 3);

        importButton = new QPushButton(Properties);
        importButton->setObjectName(QString::fromUtf8("importButton"));

        gridLayout->addWidget(importButton, 4, 1, 1, 2);


        retranslateUi(Properties);

        QMetaObject::connectSlotsByName(Properties);
    } // setupUi

    void retranslateUi(QWidget *Properties)
    {
        Properties->setWindowTitle(QCoreApplication::translate("Properties", "Form", nullptr));
        label_3->setText(QCoreApplication::translate("Properties", "Duration", nullptr));
        label->setText(QCoreApplication::translate("Properties", "Code", nullptr));
        label_2->setText(QCoreApplication::translate("Properties", "Presets", nullptr));
        groupBox->setTitle(QCoreApplication::translate("Properties", "Control Points", nullptr));
        importButton->setText(QCoreApplication::translate("Properties", "Import AfterEffects Curve", nullptr));
    } // retranslateUi

};

namespace Ui {
    class Properties: public Ui_Properties {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PROPERTIES_H
