/****************************************************************************
** Meta object code from reading C++ file 'qquickmultipointtoucharea_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/quick/items/qquickmultipointtoucharea_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickmultipointtoucharea_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickTouchPoint_t {
    QByteArrayData data[37];
    char stringdata0[429];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickTouchPoint_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickTouchPoint_t qt_meta_stringdata_QQuickTouchPoint = {
    {
QT_MOC_LITERAL(0, 0, 16), // "QQuickTouchPoint"
QT_MOC_LITERAL(1, 17, 11), // "QML.Element"
QT_MOC_LITERAL(2, 29, 10), // "TouchPoint"
QT_MOC_LITERAL(3, 40, 14), // "pressedChanged"
QT_MOC_LITERAL(4, 55, 0), // ""
QT_MOC_LITERAL(5, 56, 14), // "pointIdChanged"
QT_MOC_LITERAL(6, 71, 15), // "uniqueIdChanged"
QT_MOC_LITERAL(7, 87, 8), // "xChanged"
QT_MOC_LITERAL(8, 96, 8), // "yChanged"
QT_MOC_LITERAL(9, 105, 23), // "ellipseDiametersChanged"
QT_MOC_LITERAL(10, 129, 15), // "pressureChanged"
QT_MOC_LITERAL(11, 145, 15), // "rotationChanged"
QT_MOC_LITERAL(12, 161, 15), // "velocityChanged"
QT_MOC_LITERAL(13, 177, 11), // "areaChanged"
QT_MOC_LITERAL(14, 189, 13), // "startXChanged"
QT_MOC_LITERAL(15, 203, 13), // "startYChanged"
QT_MOC_LITERAL(16, 217, 16), // "previousXChanged"
QT_MOC_LITERAL(17, 234, 16), // "previousYChanged"
QT_MOC_LITERAL(18, 251, 13), // "sceneXChanged"
QT_MOC_LITERAL(19, 265, 13), // "sceneYChanged"
QT_MOC_LITERAL(20, 279, 7), // "pointId"
QT_MOC_LITERAL(21, 287, 8), // "uniqueId"
QT_MOC_LITERAL(22, 296, 23), // "QPointingDeviceUniqueId"
QT_MOC_LITERAL(23, 320, 7), // "pressed"
QT_MOC_LITERAL(24, 328, 1), // "x"
QT_MOC_LITERAL(25, 330, 1), // "y"
QT_MOC_LITERAL(26, 332, 16), // "ellipseDiameters"
QT_MOC_LITERAL(27, 349, 8), // "pressure"
QT_MOC_LITERAL(28, 358, 8), // "rotation"
QT_MOC_LITERAL(29, 367, 8), // "velocity"
QT_MOC_LITERAL(30, 376, 4), // "area"
QT_MOC_LITERAL(31, 381, 6), // "startX"
QT_MOC_LITERAL(32, 388, 6), // "startY"
QT_MOC_LITERAL(33, 395, 9), // "previousX"
QT_MOC_LITERAL(34, 405, 9), // "previousY"
QT_MOC_LITERAL(35, 415, 6), // "sceneX"
QT_MOC_LITERAL(36, 422, 6) // "sceneY"

    },
    "QQuickTouchPoint\0QML.Element\0TouchPoint\0"
    "pressedChanged\0\0pointIdChanged\0"
    "uniqueIdChanged\0xChanged\0yChanged\0"
    "ellipseDiametersChanged\0pressureChanged\0"
    "rotationChanged\0velocityChanged\0"
    "areaChanged\0startXChanged\0startYChanged\0"
    "previousXChanged\0previousYChanged\0"
    "sceneXChanged\0sceneYChanged\0pointId\0"
    "uniqueId\0QPointingDeviceUniqueId\0"
    "pressed\0x\0y\0ellipseDiameters\0pressure\0"
    "rotation\0velocity\0area\0startX\0startY\0"
    "previousX\0previousY\0sceneX\0sceneY"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickTouchPoint[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      16,   16, // methods
      16,  128, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      16,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,  112,    4, 0x06 /* Public */,
       5,    0,  113,    4, 0x06 /* Public */,
       6,    0,  114,    4, 0x86 /* Public | MethodRevisioned */,
       7,    0,  115,    4, 0x06 /* Public */,
       8,    0,  116,    4, 0x06 /* Public */,
       9,    0,  117,    4, 0x86 /* Public | MethodRevisioned */,
      10,    0,  118,    4, 0x06 /* Public */,
      11,    0,  119,    4, 0x86 /* Public | MethodRevisioned */,
      12,    0,  120,    4, 0x06 /* Public */,
      13,    0,  121,    4, 0x06 /* Public */,
      14,    0,  122,    4, 0x06 /* Public */,
      15,    0,  123,    4, 0x06 /* Public */,
      16,    0,  124,    4, 0x06 /* Public */,
      17,    0,  125,    4, 0x06 /* Public */,
      18,    0,  126,    4, 0x06 /* Public */,
      19,    0,  127,    4, 0x06 /* Public */,

 // signals: revision
       0,
       0,
       9,
       0,
       0,
       9,
       0,
       9,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      20, QMetaType::Int, 0x00495001,
      21, 0x80000000 | 22, 0x00c95009,
      23, QMetaType::Bool, 0x00495001,
      24, QMetaType::QReal, 0x00495001,
      25, QMetaType::QReal, 0x00495001,
      26, QMetaType::QSizeF, 0x00c95001,
      27, QMetaType::QReal, 0x00495001,
      28, QMetaType::QReal, 0x00c95001,
      29, QMetaType::QVector2D, 0x00495001,
      30, QMetaType::QRectF, 0x00495001,
      31, QMetaType::QReal, 0x00495001,
      32, QMetaType::QReal, 0x00495001,
      33, QMetaType::QReal, 0x00495001,
      34, QMetaType::QReal, 0x00495001,
      35, QMetaType::QReal, 0x00495001,
      36, QMetaType::QReal, 0x00495001,

 // properties: notify_signal_id
       1,
       2,
       0,
       3,
       4,
       5,
       6,
       7,
       8,
       9,
      10,
      11,
      12,
      13,
      14,
      15,

 // properties: revision
       0,
       9,
       0,
       0,
       0,
       9,
       0,
       9,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,

       0        // eod
};

void QQuickTouchPoint::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickTouchPoint *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->pressedChanged(); break;
        case 1: _t->pointIdChanged(); break;
        case 2: _t->uniqueIdChanged(); break;
        case 3: _t->xChanged(); break;
        case 4: _t->yChanged(); break;
        case 5: _t->ellipseDiametersChanged(); break;
        case 6: _t->pressureChanged(); break;
        case 7: _t->rotationChanged(); break;
        case 8: _t->velocityChanged(); break;
        case 9: _t->areaChanged(); break;
        case 10: _t->startXChanged(); break;
        case 11: _t->startYChanged(); break;
        case 12: _t->previousXChanged(); break;
        case 13: _t->previousYChanged(); break;
        case 14: _t->sceneXChanged(); break;
        case 15: _t->sceneYChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickTouchPoint::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTouchPoint::pressedChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickTouchPoint::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTouchPoint::pointIdChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickTouchPoint::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTouchPoint::uniqueIdChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickTouchPoint::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTouchPoint::xChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickTouchPoint::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTouchPoint::yChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickTouchPoint::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTouchPoint::ellipseDiametersChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickTouchPoint::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTouchPoint::pressureChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickTouchPoint::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTouchPoint::rotationChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickTouchPoint::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTouchPoint::velocityChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickTouchPoint::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTouchPoint::areaChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QQuickTouchPoint::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTouchPoint::startXChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QQuickTouchPoint::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTouchPoint::startYChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QQuickTouchPoint::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTouchPoint::previousXChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QQuickTouchPoint::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTouchPoint::previousYChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QQuickTouchPoint::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTouchPoint::sceneXChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QQuickTouchPoint::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTouchPoint::sceneYChanged)) {
                *result = 15;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickTouchPoint *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->pointId(); break;
        case 1: *reinterpret_cast< QPointingDeviceUniqueId*>(_v) = _t->uniqueId(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->pressed(); break;
        case 3: *reinterpret_cast< qreal*>(_v) = _t->x(); break;
        case 4: *reinterpret_cast< qreal*>(_v) = _t->y(); break;
        case 5: *reinterpret_cast< QSizeF*>(_v) = _t->ellipseDiameters(); break;
        case 6: *reinterpret_cast< qreal*>(_v) = _t->pressure(); break;
        case 7: *reinterpret_cast< qreal*>(_v) = _t->rotation(); break;
        case 8: *reinterpret_cast< QVector2D*>(_v) = _t->velocity(); break;
        case 9: *reinterpret_cast< QRectF*>(_v) = _t->area(); break;
        case 10: *reinterpret_cast< qreal*>(_v) = _t->startX(); break;
        case 11: *reinterpret_cast< qreal*>(_v) = _t->startY(); break;
        case 12: *reinterpret_cast< qreal*>(_v) = _t->previousX(); break;
        case 13: *reinterpret_cast< qreal*>(_v) = _t->previousY(); break;
        case 14: *reinterpret_cast< qreal*>(_v) = _t->sceneX(); break;
        case 15: *reinterpret_cast< qreal*>(_v) = _t->sceneY(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickTouchPoint::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickTouchPoint.data,
    qt_meta_data_QQuickTouchPoint,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickTouchPoint::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickTouchPoint::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickTouchPoint.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickTouchPoint::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 16;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 16;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 16;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 16;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 16;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 16;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickTouchPoint::pressedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickTouchPoint::pointIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickTouchPoint::uniqueIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickTouchPoint::xChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickTouchPoint::yChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickTouchPoint::ellipseDiametersChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickTouchPoint::pressureChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQuickTouchPoint::rotationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QQuickTouchPoint::velocityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QQuickTouchPoint::areaChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QQuickTouchPoint::startXChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QQuickTouchPoint::startYChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void QQuickTouchPoint::previousXChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void QQuickTouchPoint::previousYChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void QQuickTouchPoint::sceneXChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void QQuickTouchPoint::sceneYChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}
struct qt_meta_stringdata_QQuickGrabGestureEvent_t {
    QByteArrayData data[12];
    char stringdata0[258];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickGrabGestureEvent_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickGrabGestureEvent_t qt_meta_stringdata_QQuickGrabGestureEvent = {
    {
QT_MOC_LITERAL(0, 0, 22), // "QQuickGrabGestureEvent"
QT_MOC_LITERAL(1, 23, 11), // "QML.Element"
QT_MOC_LITERAL(2, 35, 12), // "GestureEvent"
QT_MOC_LITERAL(3, 48, 13), // "QML.Creatable"
QT_MOC_LITERAL(4, 62, 5), // "false"
QT_MOC_LITERAL(5, 68, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(6, 90, 109), // "GestureEvent is only availabl..."
QT_MOC_LITERAL(7, 200, 4), // "grab"
QT_MOC_LITERAL(8, 205, 0), // ""
QT_MOC_LITERAL(9, 206, 11), // "touchPoints"
QT_MOC_LITERAL(10, 218, 25), // "QQmlListProperty<QObject>"
QT_MOC_LITERAL(11, 244, 13) // "dragThreshold"

    },
    "QQuickGrabGestureEvent\0QML.Element\0"
    "GestureEvent\0QML.Creatable\0false\0"
    "QML.UncreatableReason\0"
    "GestureEvent is only available in the context of handling the gestureS"
    "tarted signal from MultiPointTouchArea.\0"
    "grab\0\0touchPoints\0QQmlListProperty<QObject>\0"
    "dragThreshold"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickGrabGestureEvent[] = {

 // content:
       8,       // revision
       0,       // classname
       3,   14, // classinfo
       1,   20, // methods
       2,   26, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,

 // methods: name, argc, parameters, tag, flags
       7,    0,   25,    8, 0x02 /* Public */,

 // methods: parameters
    QMetaType::Void,

 // properties: name, type, flags
       9, 0x80000000 | 10, 0x00095409,
      11, QMetaType::QReal, 0x00095401,

       0        // eod
};

void QQuickGrabGestureEvent::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickGrabGestureEvent *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->grab(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlListProperty<QObject> >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickGrabGestureEvent *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQmlListProperty<QObject>*>(_v) = _t->touchPoints(); break;
        case 1: *reinterpret_cast< qreal*>(_v) = _t->dragThreshold(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickGrabGestureEvent::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickGrabGestureEvent.data,
    qt_meta_data_QQuickGrabGestureEvent,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickGrabGestureEvent::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickGrabGestureEvent::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickGrabGestureEvent.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickGrabGestureEvent::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
struct qt_meta_stringdata_QQuickMultiPointTouchArea_t {
    QByteArrayData data[21];
    char stringdata0[339];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickMultiPointTouchArea_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickMultiPointTouchArea_t qt_meta_stringdata_QQuickMultiPointTouchArea = {
    {
QT_MOC_LITERAL(0, 0, 25), // "QQuickMultiPointTouchArea"
QT_MOC_LITERAL(1, 26, 11), // "QML.Element"
QT_MOC_LITERAL(2, 38, 19), // "MultiPointTouchArea"
QT_MOC_LITERAL(3, 58, 7), // "pressed"
QT_MOC_LITERAL(4, 66, 0), // ""
QT_MOC_LITERAL(5, 67, 15), // "QList<QObject*>"
QT_MOC_LITERAL(6, 83, 11), // "touchPoints"
QT_MOC_LITERAL(7, 95, 7), // "updated"
QT_MOC_LITERAL(8, 103, 8), // "released"
QT_MOC_LITERAL(9, 112, 8), // "canceled"
QT_MOC_LITERAL(10, 121, 14), // "gestureStarted"
QT_MOC_LITERAL(11, 136, 23), // "QQuickGrabGestureEvent*"
QT_MOC_LITERAL(12, 160, 7), // "gesture"
QT_MOC_LITERAL(13, 168, 12), // "touchUpdated"
QT_MOC_LITERAL(14, 181, 25), // "minimumTouchPointsChanged"
QT_MOC_LITERAL(15, 207, 25), // "maximumTouchPointsChanged"
QT_MOC_LITERAL(16, 233, 19), // "mouseEnabledChanged"
QT_MOC_LITERAL(17, 253, 34), // "QQmlListProperty<QQuickTouchP..."
QT_MOC_LITERAL(18, 288, 18), // "minimumTouchPoints"
QT_MOC_LITERAL(19, 307, 18), // "maximumTouchPoints"
QT_MOC_LITERAL(20, 326, 12) // "mouseEnabled"

    },
    "QQuickMultiPointTouchArea\0QML.Element\0"
    "MultiPointTouchArea\0pressed\0\0"
    "QList<QObject*>\0touchPoints\0updated\0"
    "released\0canceled\0gestureStarted\0"
    "QQuickGrabGestureEvent*\0gesture\0"
    "touchUpdated\0minimumTouchPointsChanged\0"
    "maximumTouchPointsChanged\0mouseEnabledChanged\0"
    "QQmlListProperty<QQuickTouchPoint>\0"
    "minimumTouchPoints\0maximumTouchPoints\0"
    "mouseEnabled"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickMultiPointTouchArea[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       9,   16, // methods
       4,   82, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       9,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    1,   61,    4, 0x06 /* Public */,
       7,    1,   64,    4, 0x06 /* Public */,
       8,    1,   67,    4, 0x06 /* Public */,
       9,    1,   70,    4, 0x06 /* Public */,
      10,    1,   73,    4, 0x06 /* Public */,
      13,    1,   76,    4, 0x06 /* Public */,
      14,    0,   79,    4, 0x06 /* Public */,
      15,    0,   80,    4, 0x06 /* Public */,
      16,    0,   81,    4, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
       6, 0x80000000 | 17, 0x00095009,
      18, QMetaType::Int, 0x00495103,
      19, QMetaType::Int, 0x00495103,
      20, QMetaType::Bool, 0x00495103,

 // properties: notify_signal_id
       0,
       6,
       7,
       8,

       0        // eod
};

void QQuickMultiPointTouchArea::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickMultiPointTouchArea *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->pressed((*reinterpret_cast< const QList<QObject*>(*)>(_a[1]))); break;
        case 1: _t->updated((*reinterpret_cast< const QList<QObject*>(*)>(_a[1]))); break;
        case 2: _t->released((*reinterpret_cast< const QList<QObject*>(*)>(_a[1]))); break;
        case 3: _t->canceled((*reinterpret_cast< const QList<QObject*>(*)>(_a[1]))); break;
        case 4: _t->gestureStarted((*reinterpret_cast< QQuickGrabGestureEvent*(*)>(_a[1]))); break;
        case 5: _t->touchUpdated((*reinterpret_cast< const QList<QObject*>(*)>(_a[1]))); break;
        case 6: _t->minimumTouchPointsChanged(); break;
        case 7: _t->maximumTouchPointsChanged(); break;
        case 8: _t->mouseEnabledChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QObject*> >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QObject*> >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QObject*> >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QObject*> >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickGrabGestureEvent* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QObject*> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickMultiPointTouchArea::*)(const QList<QObject*> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMultiPointTouchArea::pressed)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickMultiPointTouchArea::*)(const QList<QObject*> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMultiPointTouchArea::updated)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickMultiPointTouchArea::*)(const QList<QObject*> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMultiPointTouchArea::released)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickMultiPointTouchArea::*)(const QList<QObject*> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMultiPointTouchArea::canceled)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickMultiPointTouchArea::*)(QQuickGrabGestureEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMultiPointTouchArea::gestureStarted)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickMultiPointTouchArea::*)(const QList<QObject*> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMultiPointTouchArea::touchUpdated)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickMultiPointTouchArea::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMultiPointTouchArea::minimumTouchPointsChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickMultiPointTouchArea::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMultiPointTouchArea::maximumTouchPointsChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickMultiPointTouchArea::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickMultiPointTouchArea::mouseEnabledChanged)) {
                *result = 8;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlListProperty<QQuickTouchPoint> >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickMultiPointTouchArea *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQmlListProperty<QQuickTouchPoint>*>(_v) = _t->touchPoints(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->minimumTouchPoints(); break;
        case 2: *reinterpret_cast< int*>(_v) = _t->maximumTouchPoints(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->mouseEnabled(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickMultiPointTouchArea *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setMinimumTouchPoints(*reinterpret_cast< int*>(_v)); break;
        case 2: _t->setMaximumTouchPoints(*reinterpret_cast< int*>(_v)); break;
        case 3: _t->setMouseEnabled(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickMultiPointTouchArea::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_meta_stringdata_QQuickMultiPointTouchArea.data,
    qt_meta_data_QQuickMultiPointTouchArea,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickMultiPointTouchArea::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickMultiPointTouchArea::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickMultiPointTouchArea.stringdata0))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int QQuickMultiPointTouchArea::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickMultiPointTouchArea::pressed(const QList<QObject*> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QQuickMultiPointTouchArea::updated(const QList<QObject*> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QQuickMultiPointTouchArea::released(const QList<QObject*> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QQuickMultiPointTouchArea::canceled(const QList<QObject*> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QQuickMultiPointTouchArea::gestureStarted(QQuickGrabGestureEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QQuickMultiPointTouchArea::touchUpdated(const QList<QObject*> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QQuickMultiPointTouchArea::minimumTouchPointsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQuickMultiPointTouchArea::maximumTouchPointsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QQuickMultiPointTouchArea::mouseEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
