/********************************************************************************
** Form generated from reading UI file 'pane.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PANE_H
#define UI_PANE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Pane
{
public:
    QGridLayout *gridLayout;
    QLabel *label;
    QWidget *widget;
    QHBoxLayout *horizontalLayout;
    QLabel *label_x;
    QDoubleSpinBox *p1_x;
    QLabel *label_y;
    QDoubleSpinBox *p1_y;
    QCheckBox *smooth;
    QSpacerItem *horizontalSpacer;

    void setupUi(QWidget *Pane)
    {
        if (Pane->objectName().isEmpty())
            Pane->setObjectName(QString::fromUtf8("Pane"));
        Pane->resize(416, 47);
        QSizePolicy sizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(Pane->sizePolicy().hasHeightForWidth());
        Pane->setSizePolicy(sizePolicy);
        gridLayout = new QGridLayout(Pane);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        label = new QLabel(Pane);
        label->setObjectName(QString::fromUtf8("label"));
        QFont font;
        font.setBold(true);
        font.setWeight(75);
        label->setFont(font);

        gridLayout->addWidget(label, 0, 0, 1, 1);

        widget = new QWidget(Pane);
        widget->setObjectName(QString::fromUtf8("widget"));
        horizontalLayout = new QHBoxLayout(widget);
        horizontalLayout->setSpacing(4);
        horizontalLayout->setContentsMargins(2, 2, 2, 2);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        label_x = new QLabel(widget);
        label_x->setObjectName(QString::fromUtf8("label_x"));

        horizontalLayout->addWidget(label_x);

        p1_x = new QDoubleSpinBox(widget);
        p1_x->setObjectName(QString::fromUtf8("p1_x"));
        p1_x->setDecimals(4);
        p1_x->setSingleStep(0.010000000000000);

        horizontalLayout->addWidget(p1_x);

        label_y = new QLabel(widget);
        label_y->setObjectName(QString::fromUtf8("label_y"));

        horizontalLayout->addWidget(label_y);

        p1_y = new QDoubleSpinBox(widget);
        p1_y->setObjectName(QString::fromUtf8("p1_y"));
        p1_y->setDecimals(4);
        p1_y->setMinimum(-10.000000000000000);
        p1_y->setMaximum(10.000000000000000);
        p1_y->setSingleStep(0.010000000000000);

        horizontalLayout->addWidget(p1_y);


        gridLayout->addWidget(widget, 0, 1, 1, 1);

        smooth = new QCheckBox(Pane);
        smooth->setObjectName(QString::fromUtf8("smooth"));

        gridLayout->addWidget(smooth, 0, 2, 1, 1);

        horizontalSpacer = new QSpacerItem(99, 10, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer, 0, 3, 1, 1, Qt::AlignVCenter);


        retranslateUi(Pane);

        QMetaObject::connectSlotsByName(Pane);
    } // setupUi

    void retranslateUi(QWidget *Pane)
    {
        Pane->setWindowTitle(QCoreApplication::translate("Pane", "Form", nullptr));
        label->setText(QCoreApplication::translate("Pane", "p1", nullptr));
        label_x->setText(QCoreApplication::translate("Pane", "x", nullptr));
        label_y->setText(QCoreApplication::translate("Pane", "y", nullptr));
        smooth->setText(QCoreApplication::translate("Pane", "smooth", nullptr));
    } // retranslateUi

};

namespace Ui {
    class Pane: public Ui_Pane {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PANE_H
