/****************************************************************************
** Meta object code from reading C++ file 'qquickparticleemitter_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/particles/qquickparticleemitter_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickparticleemitter_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickParticleEmitter_t {
    QByteArrayData data[65];
    char stringdata0[933];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickParticleEmitter_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickParticleEmitter_t qt_meta_stringdata_QQuickParticleEmitter = {
    {
QT_MOC_LITERAL(0, 0, 21), // "QQuickParticleEmitter"
QT_MOC_LITERAL(1, 22, 11), // "QML.Element"
QT_MOC_LITERAL(2, 34, 7), // "Emitter"
QT_MOC_LITERAL(3, 42, 13), // "emitParticles"
QT_MOC_LITERAL(4, 56, 0), // ""
QT_MOC_LITERAL(5, 57, 8), // "QJSValue"
QT_MOC_LITERAL(6, 66, 9), // "particles"
QT_MOC_LITERAL(7, 76, 25), // "particlesPerSecondChanged"
QT_MOC_LITERAL(8, 102, 23), // "particleDurationChanged"
QT_MOC_LITERAL(9, 126, 14), // "enabledChanged"
QT_MOC_LITERAL(10, 141, 13), // "systemChanged"
QT_MOC_LITERAL(11, 155, 21), // "QQuickParticleSystem*"
QT_MOC_LITERAL(12, 177, 3), // "arg"
QT_MOC_LITERAL(13, 181, 12), // "groupChanged"
QT_MOC_LITERAL(14, 194, 32), // "particleDurationVariationChanged"
QT_MOC_LITERAL(15, 227, 15), // "extruderChanged"
QT_MOC_LITERAL(16, 243, 23), // "QQuickParticleExtruder*"
QT_MOC_LITERAL(17, 267, 19), // "particleSizeChanged"
QT_MOC_LITERAL(18, 287, 22), // "particleEndSizeChanged"
QT_MOC_LITERAL(19, 310, 28), // "particleSizeVariationChanged"
QT_MOC_LITERAL(20, 339, 15), // "velocityChanged"
QT_MOC_LITERAL(21, 355, 16), // "QQuickDirection*"
QT_MOC_LITERAL(22, 372, 19), // "accelerationChanged"
QT_MOC_LITERAL(23, 392, 21), // "maximumEmittedChanged"
QT_MOC_LITERAL(24, 414, 20), // "particleCountChanged"
QT_MOC_LITERAL(25, 435, 27), // "velocityFromMovementChanged"
QT_MOC_LITERAL(26, 463, 16), // "startTimeChanged"
QT_MOC_LITERAL(27, 480, 5), // "pulse"
QT_MOC_LITERAL(28, 486, 12), // "milliseconds"
QT_MOC_LITERAL(29, 499, 5), // "burst"
QT_MOC_LITERAL(30, 505, 3), // "num"
QT_MOC_LITERAL(31, 509, 1), // "x"
QT_MOC_LITERAL(32, 511, 1), // "y"
QT_MOC_LITERAL(33, 513, 10), // "setEnabled"
QT_MOC_LITERAL(34, 524, 21), // "setParticlesPerSecond"
QT_MOC_LITERAL(35, 546, 19), // "setParticleDuration"
QT_MOC_LITERAL(36, 566, 9), // "setSystem"
QT_MOC_LITERAL(37, 576, 8), // "setGroup"
QT_MOC_LITERAL(38, 585, 28), // "setParticleDurationVariation"
QT_MOC_LITERAL(39, 614, 11), // "setExtruder"
QT_MOC_LITERAL(40, 626, 15), // "setParticleSize"
QT_MOC_LITERAL(41, 642, 18), // "setParticleEndSize"
QT_MOC_LITERAL(42, 661, 24), // "setParticleSizeVariation"
QT_MOC_LITERAL(43, 686, 11), // "setVelocity"
QT_MOC_LITERAL(44, 698, 15), // "setAcceleration"
QT_MOC_LITERAL(45, 714, 19), // "setMaxParticleCount"
QT_MOC_LITERAL(46, 734, 12), // "setStartTime"
QT_MOC_LITERAL(47, 747, 5), // "reset"
QT_MOC_LITERAL(48, 753, 6), // "system"
QT_MOC_LITERAL(49, 760, 5), // "group"
QT_MOC_LITERAL(50, 766, 5), // "shape"
QT_MOC_LITERAL(51, 772, 7), // "enabled"
QT_MOC_LITERAL(52, 780, 9), // "startTime"
QT_MOC_LITERAL(53, 790, 8), // "emitRate"
QT_MOC_LITERAL(54, 799, 8), // "lifeSpan"
QT_MOC_LITERAL(55, 808, 17), // "lifeSpanVariation"
QT_MOC_LITERAL(56, 826, 14), // "maximumEmitted"
QT_MOC_LITERAL(57, 841, 4), // "size"
QT_MOC_LITERAL(58, 846, 7), // "endSize"
QT_MOC_LITERAL(59, 854, 13), // "sizeVariation"
QT_MOC_LITERAL(60, 868, 8), // "velocity"
QT_MOC_LITERAL(61, 877, 12), // "acceleration"
QT_MOC_LITERAL(62, 890, 20), // "velocityFromMovement"
QT_MOC_LITERAL(63, 911, 8), // "Lifetime"
QT_MOC_LITERAL(64, 920, 12) // "InfiniteLife"

    },
    "QQuickParticleEmitter\0QML.Element\0"
    "Emitter\0emitParticles\0\0QJSValue\0"
    "particles\0particlesPerSecondChanged\0"
    "particleDurationChanged\0enabledChanged\0"
    "systemChanged\0QQuickParticleSystem*\0"
    "arg\0groupChanged\0particleDurationVariationChanged\0"
    "extruderChanged\0QQuickParticleExtruder*\0"
    "particleSizeChanged\0particleEndSizeChanged\0"
    "particleSizeVariationChanged\0"
    "velocityChanged\0QQuickDirection*\0"
    "accelerationChanged\0maximumEmittedChanged\0"
    "particleCountChanged\0velocityFromMovementChanged\0"
    "startTimeChanged\0pulse\0milliseconds\0"
    "burst\0num\0x\0y\0setEnabled\0setParticlesPerSecond\0"
    "setParticleDuration\0setSystem\0setGroup\0"
    "setParticleDurationVariation\0setExtruder\0"
    "setParticleSize\0setParticleEndSize\0"
    "setParticleSizeVariation\0setVelocity\0"
    "setAcceleration\0setMaxParticleCount\0"
    "setStartTime\0reset\0system\0group\0shape\0"
    "enabled\0startTime\0emitRate\0lifeSpan\0"
    "lifeSpanVariation\0maximumEmitted\0size\0"
    "endSize\0sizeVariation\0velocity\0"
    "acceleration\0velocityFromMovement\0"
    "Lifetime\0InfiniteLife"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickParticleEmitter[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      35,   16, // methods
      15,  294, // properties
       1,  354, // enums/sets
       0,    0, // constructors
       0,       // flags
      17,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    1,  191,    4, 0x06 /* Public */,
       7,    1,  194,    4, 0x06 /* Public */,
       8,    1,  197,    4, 0x06 /* Public */,
       9,    1,  200,    4, 0x06 /* Public */,
      10,    1,  203,    4, 0x06 /* Public */,
      13,    1,  206,    4, 0x06 /* Public */,
      14,    1,  209,    4, 0x06 /* Public */,
      15,    1,  212,    4, 0x06 /* Public */,
      17,    1,  215,    4, 0x06 /* Public */,
      18,    1,  218,    4, 0x06 /* Public */,
      19,    1,  221,    4, 0x06 /* Public */,
      20,    1,  224,    4, 0x06 /* Public */,
      22,    1,  227,    4, 0x06 /* Public */,
      23,    1,  230,    4, 0x06 /* Public */,
      24,    0,  233,    4, 0x06 /* Public */,
      25,    0,  234,    4, 0x06 /* Public */,
      26,    1,  235,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      27,    1,  238,    4, 0x0a /* Public */,
      29,    1,  241,    4, 0x0a /* Public */,
      29,    3,  244,    4, 0x0a /* Public */,
      33,    1,  251,    4, 0x0a /* Public */,
      34,    1,  254,    4, 0x0a /* Public */,
      35,    1,  257,    4, 0x0a /* Public */,
      36,    1,  260,    4, 0x0a /* Public */,
      37,    1,  263,    4, 0x0a /* Public */,
      38,    1,  266,    4, 0x0a /* Public */,
      39,    1,  269,    4, 0x0a /* Public */,
      40,    1,  272,    4, 0x0a /* Public */,
      41,    1,  275,    4, 0x0a /* Public */,
      42,    1,  278,    4, 0x0a /* Public */,
      43,    1,  281,    4, 0x0a /* Public */,
      44,    1,  284,    4, 0x0a /* Public */,
      45,    1,  287,    4, 0x0a /* Public */,
      46,    1,  290,    4, 0x0a /* Public */,
      47,    0,  293,    4, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, QMetaType::QReal,    4,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::Bool,    4,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void, QMetaType::QString,   12,
    QMetaType::Void, QMetaType::Int,   12,
    QMetaType::Void, 0x80000000 | 16,   12,
    QMetaType::Void, QMetaType::QReal,   12,
    QMetaType::Void, QMetaType::QReal,   12,
    QMetaType::Void, QMetaType::QReal,   12,
    QMetaType::Void, 0x80000000 | 21,   12,
    QMetaType::Void, 0x80000000 | 21,   12,
    QMetaType::Void, QMetaType::Int,   12,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   12,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,   28,
    QMetaType::Void, QMetaType::Int,   30,
    QMetaType::Void, QMetaType::Int, QMetaType::QReal, QMetaType::QReal,   30,   31,   32,
    QMetaType::Void, QMetaType::Bool,   12,
    QMetaType::Void, QMetaType::QReal,   12,
    QMetaType::Void, QMetaType::Int,   12,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void, QMetaType::QString,   12,
    QMetaType::Void, QMetaType::Int,   12,
    QMetaType::Void, 0x80000000 | 16,   12,
    QMetaType::Void, QMetaType::QReal,   12,
    QMetaType::Void, QMetaType::QReal,   12,
    QMetaType::Void, QMetaType::QReal,   12,
    QMetaType::Void, 0x80000000 | 21,   12,
    QMetaType::Void, 0x80000000 | 21,   12,
    QMetaType::Void, QMetaType::Int,   12,
    QMetaType::Void, QMetaType::Int,   12,
    QMetaType::Void,

 // properties: name, type, flags
      48, 0x80000000 | 11, 0x0049510b,
      49, QMetaType::QString, 0x00495103,
      50, 0x80000000 | 16, 0x0049500b,
      51, QMetaType::Bool, 0x00495103,
      52, QMetaType::Int, 0x00495103,
      53, QMetaType::QReal, 0x00495003,
      54, QMetaType::Int, 0x00495003,
      55, QMetaType::Int, 0x00495003,
      56, QMetaType::Int, 0x00495003,
      57, QMetaType::QReal, 0x00495003,
      58, QMetaType::QReal, 0x00495003,
      59, QMetaType::QReal, 0x00495003,
      60, 0x80000000 | 21, 0x0049510b,
      61, 0x80000000 | 21, 0x0049510b,
      62, QMetaType::QReal, 0x00495103,

 // properties: notify_signal_id
       4,
       5,
       7,
       3,
      16,
       1,
       2,
       6,
      13,
       8,
       9,
      10,
      11,
      12,
      15,

 // enums: name, alias, flags, count, data
      63,   63, 0x0,    1,  359,

 // enum data: key, value
      64, uint(QQuickParticleEmitter::InfiniteLife),

       0        // eod
};

void QQuickParticleEmitter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickParticleEmitter *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->emitParticles((*reinterpret_cast< const QJSValue(*)>(_a[1]))); break;
        case 1: _t->particlesPerSecondChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 2: _t->particleDurationChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->enabledChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->systemChanged((*reinterpret_cast< QQuickParticleSystem*(*)>(_a[1]))); break;
        case 5: _t->groupChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->particleDurationVariationChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->extruderChanged((*reinterpret_cast< QQuickParticleExtruder*(*)>(_a[1]))); break;
        case 8: _t->particleSizeChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 9: _t->particleEndSizeChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 10: _t->particleSizeVariationChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 11: _t->velocityChanged((*reinterpret_cast< QQuickDirection*(*)>(_a[1]))); break;
        case 12: _t->accelerationChanged((*reinterpret_cast< QQuickDirection*(*)>(_a[1]))); break;
        case 13: _t->maximumEmittedChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->particleCountChanged(); break;
        case 15: _t->velocityFromMovementChanged(); break;
        case 16: _t->startTimeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->pulse((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 18: _t->burst((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 19: _t->burst((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< qreal(*)>(_a[2])),(*reinterpret_cast< qreal(*)>(_a[3]))); break;
        case 20: _t->setEnabled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 21: _t->setParticlesPerSecond((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 22: _t->setParticleDuration((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 23: _t->setSystem((*reinterpret_cast< QQuickParticleSystem*(*)>(_a[1]))); break;
        case 24: _t->setGroup((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 25: _t->setParticleDurationVariation((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 26: _t->setExtruder((*reinterpret_cast< QQuickParticleExtruder*(*)>(_a[1]))); break;
        case 27: _t->setParticleSize((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 28: _t->setParticleEndSize((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 29: _t->setParticleSizeVariation((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 30: _t->setVelocity((*reinterpret_cast< QQuickDirection*(*)>(_a[1]))); break;
        case 31: _t->setAcceleration((*reinterpret_cast< QQuickDirection*(*)>(_a[1]))); break;
        case 32: _t->setMaxParticleCount((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 33: _t->setStartTime((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 34: _t->reset(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickParticleSystem* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickParticleExtruder* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickDirection* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickDirection* >(); break;
            }
            break;
        case 23:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickParticleSystem* >(); break;
            }
            break;
        case 26:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickParticleExtruder* >(); break;
            }
            break;
        case 30:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickDirection* >(); break;
            }
            break;
        case 31:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickDirection* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickParticleEmitter::*)(const QJSValue & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleEmitter::emitParticles)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickParticleEmitter::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleEmitter::particlesPerSecondChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickParticleEmitter::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleEmitter::particleDurationChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickParticleEmitter::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleEmitter::enabledChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickParticleEmitter::*)(QQuickParticleSystem * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleEmitter::systemChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickParticleEmitter::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleEmitter::groupChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickParticleEmitter::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleEmitter::particleDurationVariationChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickParticleEmitter::*)(QQuickParticleExtruder * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleEmitter::extruderChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickParticleEmitter::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleEmitter::particleSizeChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickParticleEmitter::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleEmitter::particleEndSizeChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QQuickParticleEmitter::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleEmitter::particleSizeVariationChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QQuickParticleEmitter::*)(QQuickDirection * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleEmitter::velocityChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QQuickParticleEmitter::*)(QQuickDirection * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleEmitter::accelerationChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QQuickParticleEmitter::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleEmitter::maximumEmittedChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QQuickParticleEmitter::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleEmitter::particleCountChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QQuickParticleEmitter::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleEmitter::velocityFromMovementChanged)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (QQuickParticleEmitter::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleEmitter::startTimeChanged)) {
                *result = 16;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 13:
        case 12:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickDirection* >(); break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickParticleExtruder* >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickParticleSystem* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickParticleEmitter *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQuickParticleSystem**>(_v) = _t->system(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->group(); break;
        case 2: *reinterpret_cast< QQuickParticleExtruder**>(_v) = _t->extruder(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->enabled(); break;
        case 4: *reinterpret_cast< int*>(_v) = _t->startTime(); break;
        case 5: *reinterpret_cast< qreal*>(_v) = _t->particlesPerSecond(); break;
        case 6: *reinterpret_cast< int*>(_v) = _t->particleDuration(); break;
        case 7: *reinterpret_cast< int*>(_v) = _t->particleDurationVariation(); break;
        case 8: *reinterpret_cast< int*>(_v) = _t->maxParticleCount(); break;
        case 9: *reinterpret_cast< qreal*>(_v) = _t->particleSize(); break;
        case 10: *reinterpret_cast< qreal*>(_v) = _t->particleEndSize(); break;
        case 11: *reinterpret_cast< qreal*>(_v) = _t->particleSizeVariation(); break;
        case 12: *reinterpret_cast< QQuickDirection**>(_v) = _t->velocity(); break;
        case 13: *reinterpret_cast< QQuickDirection**>(_v) = _t->acceleration(); break;
        case 14: *reinterpret_cast< qreal*>(_v) = _t->velocityFromMovement(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickParticleEmitter *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSystem(*reinterpret_cast< QQuickParticleSystem**>(_v)); break;
        case 1: _t->setGroup(*reinterpret_cast< QString*>(_v)); break;
        case 2: _t->setExtruder(*reinterpret_cast< QQuickParticleExtruder**>(_v)); break;
        case 3: _t->setEnabled(*reinterpret_cast< bool*>(_v)); break;
        case 4: _t->setStartTime(*reinterpret_cast< int*>(_v)); break;
        case 5: _t->setParticlesPerSecond(*reinterpret_cast< qreal*>(_v)); break;
        case 6: _t->setParticleDuration(*reinterpret_cast< int*>(_v)); break;
        case 7: _t->setParticleDurationVariation(*reinterpret_cast< int*>(_v)); break;
        case 8: _t->setMaxParticleCount(*reinterpret_cast< int*>(_v)); break;
        case 9: _t->setParticleSize(*reinterpret_cast< qreal*>(_v)); break;
        case 10: _t->setParticleEndSize(*reinterpret_cast< qreal*>(_v)); break;
        case 11: _t->setParticleSizeVariation(*reinterpret_cast< qreal*>(_v)); break;
        case 12: _t->setVelocity(*reinterpret_cast< QQuickDirection**>(_v)); break;
        case 13: _t->setAcceleration(*reinterpret_cast< QQuickDirection**>(_v)); break;
        case 14: _t->setVelocityFromMovement(*reinterpret_cast< qreal*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickParticleEmitter::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_meta_stringdata_QQuickParticleEmitter.data,
    qt_meta_data_QQuickParticleEmitter,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickParticleEmitter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickParticleEmitter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickParticleEmitter.stringdata0))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int QQuickParticleEmitter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 35)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 35;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 35)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 35;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 15;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickParticleEmitter::emitParticles(const QJSValue & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QQuickParticleEmitter::particlesPerSecondChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QQuickParticleEmitter::particleDurationChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QQuickParticleEmitter::enabledChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QQuickParticleEmitter::systemChanged(QQuickParticleSystem * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QQuickParticleEmitter::groupChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QQuickParticleEmitter::particleDurationVariationChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QQuickParticleEmitter::extruderChanged(QQuickParticleExtruder * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QQuickParticleEmitter::particleSizeChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QQuickParticleEmitter::particleEndSizeChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void QQuickParticleEmitter::particleSizeVariationChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void QQuickParticleEmitter::velocityChanged(QQuickDirection * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void QQuickParticleEmitter::accelerationChanged(QQuickDirection * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void QQuickParticleEmitter::maximumEmittedChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void QQuickParticleEmitter::particleCountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void QQuickParticleEmitter::velocityFromMovementChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void QQuickParticleEmitter::startTimeChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 16, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
