/****************************************************************************
** Meta object code from reading C++ file 'qquickpincharea_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/quick/items/qquickpincharea_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickpincharea_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickPinch_t {
    QByteArrayData data[33];
    char stringdata0[398];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickPinch_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickPinch_t qt_meta_stringdata_QQuickPinch = {
    {
QT_MOC_LITERAL(0, 0, 11), // "QQuickPinch"
QT_MOC_LITERAL(1, 12, 11), // "QML.Element"
QT_MOC_LITERAL(2, 24, 5), // "Pinch"
QT_MOC_LITERAL(3, 30, 13), // "targetChanged"
QT_MOC_LITERAL(4, 44, 0), // ""
QT_MOC_LITERAL(5, 45, 19), // "minimumScaleChanged"
QT_MOC_LITERAL(6, 65, 19), // "maximumScaleChanged"
QT_MOC_LITERAL(7, 85, 22), // "minimumRotationChanged"
QT_MOC_LITERAL(8, 108, 22), // "maximumRotationChanged"
QT_MOC_LITERAL(9, 131, 15), // "dragAxisChanged"
QT_MOC_LITERAL(10, 147, 15), // "minimumXChanged"
QT_MOC_LITERAL(11, 163, 15), // "maximumXChanged"
QT_MOC_LITERAL(12, 179, 15), // "minimumYChanged"
QT_MOC_LITERAL(13, 195, 15), // "maximumYChanged"
QT_MOC_LITERAL(14, 211, 13), // "activeChanged"
QT_MOC_LITERAL(15, 225, 6), // "target"
QT_MOC_LITERAL(16, 232, 11), // "QQuickItem*"
QT_MOC_LITERAL(17, 244, 12), // "minimumScale"
QT_MOC_LITERAL(18, 257, 12), // "maximumScale"
QT_MOC_LITERAL(19, 270, 15), // "minimumRotation"
QT_MOC_LITERAL(20, 286, 15), // "maximumRotation"
QT_MOC_LITERAL(21, 302, 8), // "dragAxis"
QT_MOC_LITERAL(22, 311, 4), // "Axis"
QT_MOC_LITERAL(23, 316, 8), // "minimumX"
QT_MOC_LITERAL(24, 325, 8), // "maximumX"
QT_MOC_LITERAL(25, 334, 8), // "minimumY"
QT_MOC_LITERAL(26, 343, 8), // "maximumY"
QT_MOC_LITERAL(27, 352, 6), // "active"
QT_MOC_LITERAL(28, 359, 6), // "NoDrag"
QT_MOC_LITERAL(29, 366, 5), // "XAxis"
QT_MOC_LITERAL(30, 372, 5), // "YAxis"
QT_MOC_LITERAL(31, 378, 9), // "XAndYAxis"
QT_MOC_LITERAL(32, 388, 9) // "XandYAxis"

    },
    "QQuickPinch\0QML.Element\0Pinch\0"
    "targetChanged\0\0minimumScaleChanged\0"
    "maximumScaleChanged\0minimumRotationChanged\0"
    "maximumRotationChanged\0dragAxisChanged\0"
    "minimumXChanged\0maximumXChanged\0"
    "minimumYChanged\0maximumYChanged\0"
    "activeChanged\0target\0QQuickItem*\0"
    "minimumScale\0maximumScale\0minimumRotation\0"
    "maximumRotation\0dragAxis\0Axis\0minimumX\0"
    "maximumX\0minimumY\0maximumY\0active\0"
    "NoDrag\0XAxis\0YAxis\0XAndYAxis\0XandYAxis"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickPinch[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      11,   16, // methods
      11,   82, // properties
       1,  126, // enums/sets
       0,    0, // constructors
       0,       // flags
      11,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   71,    4, 0x06 /* Public */,
       5,    0,   72,    4, 0x06 /* Public */,
       6,    0,   73,    4, 0x06 /* Public */,
       7,    0,   74,    4, 0x06 /* Public */,
       8,    0,   75,    4, 0x06 /* Public */,
       9,    0,   76,    4, 0x06 /* Public */,
      10,    0,   77,    4, 0x06 /* Public */,
      11,    0,   78,    4, 0x06 /* Public */,
      12,    0,   79,    4, 0x06 /* Public */,
      13,    0,   80,    4, 0x06 /* Public */,
      14,    0,   81,    4, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      15, 0x80000000 | 16, 0x0049510f,
      17, QMetaType::QReal, 0x00495103,
      18, QMetaType::QReal, 0x00495103,
      19, QMetaType::QReal, 0x00495103,
      20, QMetaType::QReal, 0x00495103,
      21, 0x80000000 | 22, 0x0049500b,
      23, QMetaType::QReal, 0x00495003,
      24, QMetaType::QReal, 0x00495003,
      25, QMetaType::QReal, 0x00495003,
      26, QMetaType::QReal, 0x00495003,
      27, QMetaType::Bool, 0x00495001,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,
       5,
       6,
       7,
       8,
       9,
      10,

 // enums: name, alias, flags, count, data
      22,   22, 0x0,    5,  131,

 // enum data: key, value
      28, uint(QQuickPinch::NoDrag),
      29, uint(QQuickPinch::XAxis),
      30, uint(QQuickPinch::YAxis),
      31, uint(QQuickPinch::XAndYAxis),
      32, uint(QQuickPinch::XandYAxis),

       0        // eod
};

void QQuickPinch::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickPinch *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->targetChanged(); break;
        case 1: _t->minimumScaleChanged(); break;
        case 2: _t->maximumScaleChanged(); break;
        case 3: _t->minimumRotationChanged(); break;
        case 4: _t->maximumRotationChanged(); break;
        case 5: _t->dragAxisChanged(); break;
        case 6: _t->minimumXChanged(); break;
        case 7: _t->maximumXChanged(); break;
        case 8: _t->minimumYChanged(); break;
        case 9: _t->maximumYChanged(); break;
        case 10: _t->activeChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickPinch::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPinch::targetChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickPinch::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPinch::minimumScaleChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickPinch::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPinch::maximumScaleChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickPinch::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPinch::minimumRotationChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickPinch::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPinch::maximumRotationChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickPinch::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPinch::dragAxisChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickPinch::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPinch::minimumXChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickPinch::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPinch::maximumXChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickPinch::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPinch::minimumYChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickPinch::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPinch::maximumYChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QQuickPinch::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPinch::activeChanged)) {
                *result = 10;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickPinch *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQuickItem**>(_v) = _t->target(); break;
        case 1: *reinterpret_cast< qreal*>(_v) = _t->minimumScale(); break;
        case 2: *reinterpret_cast< qreal*>(_v) = _t->maximumScale(); break;
        case 3: *reinterpret_cast< qreal*>(_v) = _t->minimumRotation(); break;
        case 4: *reinterpret_cast< qreal*>(_v) = _t->maximumRotation(); break;
        case 5: *reinterpret_cast< Axis*>(_v) = _t->axis(); break;
        case 6: *reinterpret_cast< qreal*>(_v) = _t->xmin(); break;
        case 7: *reinterpret_cast< qreal*>(_v) = _t->xmax(); break;
        case 8: *reinterpret_cast< qreal*>(_v) = _t->ymin(); break;
        case 9: *reinterpret_cast< qreal*>(_v) = _t->ymax(); break;
        case 10: *reinterpret_cast< bool*>(_v) = _t->active(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickPinch *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTarget(*reinterpret_cast< QQuickItem**>(_v)); break;
        case 1: _t->setMinimumScale(*reinterpret_cast< qreal*>(_v)); break;
        case 2: _t->setMaximumScale(*reinterpret_cast< qreal*>(_v)); break;
        case 3: _t->setMinimumRotation(*reinterpret_cast< qreal*>(_v)); break;
        case 4: _t->setMaximumRotation(*reinterpret_cast< qreal*>(_v)); break;
        case 5: _t->setAxis(*reinterpret_cast< Axis*>(_v)); break;
        case 6: _t->setXmin(*reinterpret_cast< qreal*>(_v)); break;
        case 7: _t->setXmax(*reinterpret_cast< qreal*>(_v)); break;
        case 8: _t->setYmin(*reinterpret_cast< qreal*>(_v)); break;
        case 9: _t->setYmax(*reinterpret_cast< qreal*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        QQuickPinch *_t = static_cast<QQuickPinch *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->resetTarget(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickPinch::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickPinch.data,
    qt_meta_data_QQuickPinch,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickPinch::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickPinch::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickPinch.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickPinch::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 11;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickPinch::targetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickPinch::minimumScaleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickPinch::maximumScaleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickPinch::minimumRotationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickPinch::maximumRotationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickPinch::dragAxisChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickPinch::minimumXChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQuickPinch::maximumXChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QQuickPinch::minimumYChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QQuickPinch::maximumYChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QQuickPinch::activeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}
struct qt_meta_stringdata_QQuickPinchEvent_t {
    QByteArrayData data[17];
    char stringdata0[180];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickPinchEvent_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickPinchEvent_t qt_meta_stringdata_QQuickPinchEvent = {
    {
QT_MOC_LITERAL(0, 0, 16), // "QQuickPinchEvent"
QT_MOC_LITERAL(1, 17, 11), // "QML.Element"
QT_MOC_LITERAL(2, 29, 9), // "anonymous"
QT_MOC_LITERAL(3, 39, 6), // "center"
QT_MOC_LITERAL(4, 46, 11), // "startCenter"
QT_MOC_LITERAL(5, 58, 14), // "previousCenter"
QT_MOC_LITERAL(6, 73, 5), // "scale"
QT_MOC_LITERAL(7, 79, 13), // "previousScale"
QT_MOC_LITERAL(8, 93, 5), // "angle"
QT_MOC_LITERAL(9, 99, 13), // "previousAngle"
QT_MOC_LITERAL(10, 113, 8), // "rotation"
QT_MOC_LITERAL(11, 122, 6), // "point1"
QT_MOC_LITERAL(12, 129, 11), // "startPoint1"
QT_MOC_LITERAL(13, 141, 6), // "point2"
QT_MOC_LITERAL(14, 148, 11), // "startPoint2"
QT_MOC_LITERAL(15, 160, 10), // "pointCount"
QT_MOC_LITERAL(16, 171, 8) // "accepted"

    },
    "QQuickPinchEvent\0QML.Element\0anonymous\0"
    "center\0startCenter\0previousCenter\0"
    "scale\0previousScale\0angle\0previousAngle\0"
    "rotation\0point1\0startPoint1\0point2\0"
    "startPoint2\0pointCount\0accepted"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickPinchEvent[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       0,    0, // methods
      14,   16, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,

 // properties: name, type, flags
       3, QMetaType::QPointF, 0x00095001,
       4, QMetaType::QPointF, 0x00095001,
       5, QMetaType::QPointF, 0x00095001,
       6, QMetaType::QReal, 0x00095001,
       7, QMetaType::QReal, 0x00095001,
       8, QMetaType::QReal, 0x00095001,
       9, QMetaType::QReal, 0x00095001,
      10, QMetaType::QReal, 0x00095001,
      11, QMetaType::QPointF, 0x00095001,
      12, QMetaType::QPointF, 0x00095001,
      13, QMetaType::QPointF, 0x00095001,
      14, QMetaType::QPointF, 0x00095001,
      15, QMetaType::Int, 0x00095001,
      16, QMetaType::Bool, 0x00095103,

       0        // eod
};

void QQuickPinchEvent::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{

#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickPinchEvent *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QPointF*>(_v) = _t->center(); break;
        case 1: *reinterpret_cast< QPointF*>(_v) = _t->startCenter(); break;
        case 2: *reinterpret_cast< QPointF*>(_v) = _t->previousCenter(); break;
        case 3: *reinterpret_cast< qreal*>(_v) = _t->scale(); break;
        case 4: *reinterpret_cast< qreal*>(_v) = _t->previousScale(); break;
        case 5: *reinterpret_cast< qreal*>(_v) = _t->angle(); break;
        case 6: *reinterpret_cast< qreal*>(_v) = _t->previousAngle(); break;
        case 7: *reinterpret_cast< qreal*>(_v) = _t->rotation(); break;
        case 8: *reinterpret_cast< QPointF*>(_v) = _t->point1(); break;
        case 9: *reinterpret_cast< QPointF*>(_v) = _t->startPoint1(); break;
        case 10: *reinterpret_cast< QPointF*>(_v) = _t->point2(); break;
        case 11: *reinterpret_cast< QPointF*>(_v) = _t->startPoint2(); break;
        case 12: *reinterpret_cast< int*>(_v) = _t->pointCount(); break;
        case 13: *reinterpret_cast< bool*>(_v) = _t->accepted(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickPinchEvent *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 13: _t->setAccepted(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickPinchEvent::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickPinchEvent.data,
    qt_meta_data_QQuickPinchEvent,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickPinchEvent::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickPinchEvent::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickPinchEvent.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickPinchEvent::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    
#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 14;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 14;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 14;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 14;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 14;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
struct qt_meta_stringdata_QQuickPinchArea_t {
    QByteArrayData data[13];
    char stringdata0[149];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickPinchArea_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickPinchArea_t qt_meta_stringdata_QQuickPinchArea = {
    {
QT_MOC_LITERAL(0, 0, 15), // "QQuickPinchArea"
QT_MOC_LITERAL(1, 16, 11), // "QML.Element"
QT_MOC_LITERAL(2, 28, 9), // "PinchArea"
QT_MOC_LITERAL(3, 38, 14), // "enabledChanged"
QT_MOC_LITERAL(4, 53, 0), // ""
QT_MOC_LITERAL(5, 54, 12), // "pinchStarted"
QT_MOC_LITERAL(6, 67, 17), // "QQuickPinchEvent*"
QT_MOC_LITERAL(7, 85, 5), // "pinch"
QT_MOC_LITERAL(8, 91, 12), // "pinchUpdated"
QT_MOC_LITERAL(9, 104, 13), // "pinchFinished"
QT_MOC_LITERAL(10, 118, 9), // "smartZoom"
QT_MOC_LITERAL(11, 128, 7), // "enabled"
QT_MOC_LITERAL(12, 136, 12) // "QQuickPinch*"

    },
    "QQuickPinchArea\0QML.Element\0PinchArea\0"
    "enabledChanged\0\0pinchStarted\0"
    "QQuickPinchEvent*\0pinch\0pinchUpdated\0"
    "pinchFinished\0smartZoom\0enabled\0"
    "QQuickPinch*"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickPinchArea[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       5,   16, // methods
       2,   59, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   46,    4, 0x06 /* Public */,
       5,    1,   47,    4, 0x06 /* Public */,
       8,    1,   50,    4, 0x06 /* Public */,
       9,    1,   53,    4, 0x06 /* Public */,
      10,    1,   56,    4, 0x86 /* Public | MethodRevisioned */,

 // signals: revision
       0,
       0,
       0,
       0,
       5,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, 0x80000000 | 6,    7,

 // properties: name, type, flags
      11, QMetaType::Bool, 0x00495103,
       7, 0x80000000 | 12, 0x00095409,

 // properties: notify_signal_id
       0,
       0,

       0        // eod
};

void QQuickPinchArea::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickPinchArea *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->enabledChanged(); break;
        case 1: _t->pinchStarted((*reinterpret_cast< QQuickPinchEvent*(*)>(_a[1]))); break;
        case 2: _t->pinchUpdated((*reinterpret_cast< QQuickPinchEvent*(*)>(_a[1]))); break;
        case 3: _t->pinchFinished((*reinterpret_cast< QQuickPinchEvent*(*)>(_a[1]))); break;
        case 4: _t->smartZoom((*reinterpret_cast< QQuickPinchEvent*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickPinchEvent* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickPinchEvent* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickPinchEvent* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickPinchEvent* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickPinchArea::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPinchArea::enabledChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickPinchArea::*)(QQuickPinchEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPinchArea::pinchStarted)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickPinchArea::*)(QQuickPinchEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPinchArea::pinchUpdated)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickPinchArea::*)(QQuickPinchEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPinchArea::pinchFinished)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickPinchArea::*)(QQuickPinchEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPinchArea::smartZoom)) {
                *result = 4;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickPinch* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickPinchArea *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->isEnabled(); break;
        case 1: *reinterpret_cast< QQuickPinch**>(_v) = _t->pinch(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickPinchArea *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setEnabled(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickPinchArea::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_meta_stringdata_QQuickPinchArea.data,
    qt_meta_data_QQuickPinchArea,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickPinchArea::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickPinchArea::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickPinchArea.stringdata0))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int QQuickPinchArea::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickPinchArea::enabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickPinchArea::pinchStarted(QQuickPinchEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QQuickPinchArea::pinchUpdated(QQuickPinchEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QQuickPinchArea::pinchFinished(QQuickPinchEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QQuickPinchArea::smartZoom(QQuickPinchEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
