/****************************************************************************
** Meta object code from reading C++ file 'qquicktrailemitter_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/particles/qquicktrailemitter_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquicktrailemitter_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickTrailEmitter_t {
    QByteArrayData data[28];
    char stringdata0[450];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickTrailEmitter_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickTrailEmitter_t qt_meta_stringdata_QQuickTrailEmitter = {
    {
QT_MOC_LITERAL(0, 0, 18), // "QQuickTrailEmitter"
QT_MOC_LITERAL(1, 19, 11), // "QML.Element"
QT_MOC_LITERAL(2, 31, 12), // "TrailEmitter"
QT_MOC_LITERAL(3, 44, 19), // "emitFollowParticles"
QT_MOC_LITERAL(4, 64, 0), // ""
QT_MOC_LITERAL(5, 65, 8), // "QJSValue"
QT_MOC_LITERAL(6, 74, 9), // "particles"
QT_MOC_LITERAL(7, 84, 8), // "followed"
QT_MOC_LITERAL(8, 93, 36), // "particlesPerParticlePerSecond..."
QT_MOC_LITERAL(9, 130, 3), // "arg"
QT_MOC_LITERAL(10, 134, 24), // "emitterXVariationChanged"
QT_MOC_LITERAL(11, 159, 24), // "emitterYVariationChanged"
QT_MOC_LITERAL(12, 184, 13), // "followChanged"
QT_MOC_LITERAL(13, 198, 20), // "emissionShapeChanged"
QT_MOC_LITERAL(14, 219, 23), // "QQuickParticleExtruder*"
QT_MOC_LITERAL(15, 243, 32), // "setParticlesPerParticlePerSecond"
QT_MOC_LITERAL(16, 276, 20), // "setEmitterXVariation"
QT_MOC_LITERAL(17, 297, 20), // "setEmitterYVariation"
QT_MOC_LITERAL(18, 318, 9), // "setFollow"
QT_MOC_LITERAL(19, 328, 16), // "setEmissionShape"
QT_MOC_LITERAL(20, 345, 24), // "recalcParticlesPerSecond"
QT_MOC_LITERAL(21, 370, 6), // "follow"
QT_MOC_LITERAL(22, 377, 19), // "emitRatePerParticle"
QT_MOC_LITERAL(23, 397, 9), // "emitShape"
QT_MOC_LITERAL(24, 407, 10), // "emitHeight"
QT_MOC_LITERAL(25, 418, 9), // "emitWidth"
QT_MOC_LITERAL(26, 428, 8), // "EmitSize"
QT_MOC_LITERAL(27, 437, 12) // "ParticleSize"

    },
    "QQuickTrailEmitter\0QML.Element\0"
    "TrailEmitter\0emitFollowParticles\0\0"
    "QJSValue\0particles\0followed\0"
    "particlesPerParticlePerSecondChanged\0"
    "arg\0emitterXVariationChanged\0"
    "emitterYVariationChanged\0followChanged\0"
    "emissionShapeChanged\0QQuickParticleExtruder*\0"
    "setParticlesPerParticlePerSecond\0"
    "setEmitterXVariation\0setEmitterYVariation\0"
    "setFollow\0setEmissionShape\0"
    "recalcParticlesPerSecond\0follow\0"
    "emitRatePerParticle\0emitShape\0emitHeight\0"
    "emitWidth\0EmitSize\0ParticleSize"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickTrailEmitter[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      12,   16, // methods
       5,  112, // properties
       1,  132, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    2,   76,    4, 0x06 /* Public */,
       8,    1,   81,    4, 0x06 /* Public */,
      10,    1,   84,    4, 0x06 /* Public */,
      11,    1,   87,    4, 0x06 /* Public */,
      12,    1,   90,    4, 0x06 /* Public */,
      13,    1,   93,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      15,    1,   96,    4, 0x0a /* Public */,
      16,    1,   99,    4, 0x0a /* Public */,
      17,    1,  102,    4, 0x0a /* Public */,
      18,    1,  105,    4, 0x0a /* Public */,
      19,    1,  108,    4, 0x0a /* Public */,
      20,    0,  111,    4, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 5, 0x80000000 | 5,    6,    7,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void, QMetaType::QReal,    9,
    QMetaType::Void, QMetaType::QReal,    9,
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::Void, 0x80000000 | 14,    9,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void, QMetaType::QReal,    9,
    QMetaType::Void, QMetaType::QReal,    9,
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::Void, 0x80000000 | 14,    9,
    QMetaType::Void,

 // properties: name, type, flags
      21, QMetaType::QString, 0x00495103,
      22, QMetaType::Int, 0x00495003,
      23, 0x80000000 | 14, 0x0049500b,
      24, QMetaType::QReal, 0x00495003,
      25, QMetaType::QReal, 0x00495003,

 // properties: notify_signal_id
       4,
       1,
       5,
       3,
       2,

 // enums: name, alias, flags, count, data
      26,   26, 0x0,    1,  137,

 // enum data: key, value
      27, uint(QQuickTrailEmitter::ParticleSize),

       0        // eod
};

void QQuickTrailEmitter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickTrailEmitter *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->emitFollowParticles((*reinterpret_cast< const QJSValue(*)>(_a[1])),(*reinterpret_cast< const QJSValue(*)>(_a[2]))); break;
        case 1: _t->particlesPerParticlePerSecondChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->emitterXVariationChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 3: _t->emitterYVariationChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 4: _t->followChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->emissionShapeChanged((*reinterpret_cast< QQuickParticleExtruder*(*)>(_a[1]))); break;
        case 6: _t->setParticlesPerParticlePerSecond((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->setEmitterXVariation((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 8: _t->setEmitterYVariation((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 9: _t->setFollow((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->setEmissionShape((*reinterpret_cast< QQuickParticleExtruder*(*)>(_a[1]))); break;
        case 11: _t->recalcParticlesPerSecond(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickParticleExtruder* >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickParticleExtruder* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickTrailEmitter::*)(const QJSValue & , const QJSValue & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTrailEmitter::emitFollowParticles)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickTrailEmitter::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTrailEmitter::particlesPerParticlePerSecondChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickTrailEmitter::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTrailEmitter::emitterXVariationChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickTrailEmitter::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTrailEmitter::emitterYVariationChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickTrailEmitter::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTrailEmitter::followChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickTrailEmitter::*)(QQuickParticleExtruder * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTrailEmitter::emissionShapeChanged)) {
                *result = 5;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickParticleExtruder* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickTrailEmitter *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->follow(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->particlesPerParticlePerSecond(); break;
        case 2: *reinterpret_cast< QQuickParticleExtruder**>(_v) = _t->emissonShape(); break;
        case 3: *reinterpret_cast< qreal*>(_v) = _t->emitterYVariation(); break;
        case 4: *reinterpret_cast< qreal*>(_v) = _t->emitterXVariation(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickTrailEmitter *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFollow(*reinterpret_cast< QString*>(_v)); break;
        case 1: _t->setParticlesPerParticlePerSecond(*reinterpret_cast< int*>(_v)); break;
        case 2: _t->setEmissionShape(*reinterpret_cast< QQuickParticleExtruder**>(_v)); break;
        case 3: _t->setEmitterYVariation(*reinterpret_cast< qreal*>(_v)); break;
        case 4: _t->setEmitterXVariation(*reinterpret_cast< qreal*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickTrailEmitter::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickParticleEmitter::staticMetaObject>(),
    qt_meta_stringdata_QQuickTrailEmitter.data,
    qt_meta_data_QQuickTrailEmitter,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickTrailEmitter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickTrailEmitter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickTrailEmitter.stringdata0))
        return static_cast<void*>(this);
    return QQuickParticleEmitter::qt_metacast(_clname);
}

int QQuickTrailEmitter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickParticleEmitter::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickTrailEmitter::emitFollowParticles(const QJSValue & _t1, const QJSValue & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QQuickTrailEmitter::particlesPerParticlePerSecondChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QQuickTrailEmitter::emitterXVariationChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QQuickTrailEmitter::emitterYVariationChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QQuickTrailEmitter::followChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QQuickTrailEmitter::emissionShapeChanged(QQuickParticleExtruder * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
