/****************************************************************************
** Meta object code from reading C++ file 'quicktestresult_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/qmltest/quicktestresult_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'quicktestresult_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QuickTestResult_t {
    QByteArrayData data[75];
    char stringdata0[853];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QuickTestResult_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QuickTestResult_t qt_meta_stringdata_QuickTestResult = {
    {
QT_MOC_LITERAL(0, 0, 15), // "QuickTestResult"
QT_MOC_LITERAL(1, 16, 18), // "programNameChanged"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 19), // "testCaseNameChanged"
QT_MOC_LITERAL(4, 56, 19), // "functionNameChanged"
QT_MOC_LITERAL(5, 76, 14), // "dataTagChanged"
QT_MOC_LITERAL(6, 91, 14), // "skippedChanged"
QT_MOC_LITERAL(7, 106, 5), // "reset"
QT_MOC_LITERAL(8, 112, 12), // "startLogging"
QT_MOC_LITERAL(9, 125, 11), // "stopLogging"
QT_MOC_LITERAL(10, 137, 13), // "initTestTable"
QT_MOC_LITERAL(11, 151, 14), // "clearTestTable"
QT_MOC_LITERAL(12, 166, 14), // "finishTestData"
QT_MOC_LITERAL(13, 181, 21), // "finishTestDataCleanup"
QT_MOC_LITERAL(14, 203, 18), // "finishTestFunction"
QT_MOC_LITERAL(15, 222, 9), // "stringify"
QT_MOC_LITERAL(16, 232, 15), // "QQmlV4Function*"
QT_MOC_LITERAL(17, 248, 4), // "args"
QT_MOC_LITERAL(18, 253, 4), // "fail"
QT_MOC_LITERAL(19, 258, 7), // "message"
QT_MOC_LITERAL(20, 266, 8), // "location"
QT_MOC_LITERAL(21, 275, 4), // "line"
QT_MOC_LITERAL(22, 280, 6), // "verify"
QT_MOC_LITERAL(23, 287, 7), // "success"
QT_MOC_LITERAL(24, 295, 7), // "compare"
QT_MOC_LITERAL(25, 303, 4), // "val1"
QT_MOC_LITERAL(26, 308, 4), // "val2"
QT_MOC_LITERAL(27, 313, 12), // "fuzzyCompare"
QT_MOC_LITERAL(28, 326, 6), // "actual"
QT_MOC_LITERAL(29, 333, 8), // "expected"
QT_MOC_LITERAL(30, 342, 5), // "delta"
QT_MOC_LITERAL(31, 348, 4), // "skip"
QT_MOC_LITERAL(32, 353, 10), // "expectFail"
QT_MOC_LITERAL(33, 364, 3), // "tag"
QT_MOC_LITERAL(34, 368, 7), // "comment"
QT_MOC_LITERAL(35, 376, 18), // "expectFailContinue"
QT_MOC_LITERAL(36, 395, 4), // "warn"
QT_MOC_LITERAL(37, 400, 13), // "ignoreWarning"
QT_MOC_LITERAL(38, 414, 8), // "QJSValue"
QT_MOC_LITERAL(39, 423, 4), // "wait"
QT_MOC_LITERAL(40, 428, 2), // "ms"
QT_MOC_LITERAL(41, 431, 5), // "sleep"
QT_MOC_LITERAL(42, 437, 16), // "waitForRendering"
QT_MOC_LITERAL(43, 454, 11), // "QQuickItem*"
QT_MOC_LITERAL(44, 466, 4), // "item"
QT_MOC_LITERAL(45, 471, 7), // "timeout"
QT_MOC_LITERAL(46, 479, 16), // "startMeasurement"
QT_MOC_LITERAL(47, 496, 12), // "beginDataRun"
QT_MOC_LITERAL(48, 509, 10), // "endDataRun"
QT_MOC_LITERAL(49, 520, 19), // "measurementAccepted"
QT_MOC_LITERAL(50, 540, 21), // "needsMoreMeasurements"
QT_MOC_LITERAL(51, 562, 14), // "startBenchmark"
QT_MOC_LITERAL(52, 577, 7), // "RunMode"
QT_MOC_LITERAL(53, 585, 7), // "runMode"
QT_MOC_LITERAL(54, 593, 15), // "isBenchmarkDone"
QT_MOC_LITERAL(55, 609, 13), // "nextBenchmark"
QT_MOC_LITERAL(56, 623, 13), // "stopBenchmark"
QT_MOC_LITERAL(57, 637, 9), // "grabImage"
QT_MOC_LITERAL(58, 647, 9), // "findChild"
QT_MOC_LITERAL(59, 657, 6), // "parent"
QT_MOC_LITERAL(60, 664, 10), // "objectName"
QT_MOC_LITERAL(61, 675, 17), // "isPolishScheduled"
QT_MOC_LITERAL(62, 693, 19), // "waitForItemPolished"
QT_MOC_LITERAL(63, 713, 12), // "testCaseName"
QT_MOC_LITERAL(64, 726, 12), // "functionName"
QT_MOC_LITERAL(65, 739, 7), // "dataTag"
QT_MOC_LITERAL(66, 747, 6), // "failed"
QT_MOC_LITERAL(67, 754, 7), // "skipped"
QT_MOC_LITERAL(68, 762, 9), // "passCount"
QT_MOC_LITERAL(69, 772, 9), // "failCount"
QT_MOC_LITERAL(70, 782, 9), // "skipCount"
QT_MOC_LITERAL(71, 792, 14), // "functionsToRun"
QT_MOC_LITERAL(72, 807, 9), // "tagsToRun"
QT_MOC_LITERAL(73, 817, 27), // "RepeatUntilValidMeasurement"
QT_MOC_LITERAL(74, 845, 7) // "RunOnce"

    },
    "QuickTestResult\0programNameChanged\0\0"
    "testCaseNameChanged\0functionNameChanged\0"
    "dataTagChanged\0skippedChanged\0reset\0"
    "startLogging\0stopLogging\0initTestTable\0"
    "clearTestTable\0finishTestData\0"
    "finishTestDataCleanup\0finishTestFunction\0"
    "stringify\0QQmlV4Function*\0args\0fail\0"
    "message\0location\0line\0verify\0success\0"
    "compare\0val1\0val2\0fuzzyCompare\0actual\0"
    "expected\0delta\0skip\0expectFail\0tag\0"
    "comment\0expectFailContinue\0warn\0"
    "ignoreWarning\0QJSValue\0wait\0ms\0sleep\0"
    "waitForRendering\0QQuickItem*\0item\0"
    "timeout\0startMeasurement\0beginDataRun\0"
    "endDataRun\0measurementAccepted\0"
    "needsMoreMeasurements\0startBenchmark\0"
    "RunMode\0runMode\0isBenchmarkDone\0"
    "nextBenchmark\0stopBenchmark\0grabImage\0"
    "findChild\0parent\0objectName\0"
    "isPolishScheduled\0waitForItemPolished\0"
    "testCaseName\0functionName\0dataTag\0"
    "failed\0skipped\0passCount\0failCount\0"
    "skipCount\0functionsToRun\0tagsToRun\0"
    "RepeatUntilValidMeasurement\0RunOnce"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QuickTestResult[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      40,   14, // methods
      10,  384, // properties
       1,  424, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  254,    2, 0x06 /* Public */,
       3,    0,  255,    2, 0x06 /* Public */,
       4,    0,  256,    2, 0x06 /* Public */,
       5,    0,  257,    2, 0x06 /* Public */,
       6,    0,  258,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    0,  259,    2, 0x0a /* Public */,
       8,    0,  260,    2, 0x0a /* Public */,
       9,    0,  261,    2, 0x0a /* Public */,
      10,    0,  262,    2, 0x0a /* Public */,
      11,    0,  263,    2, 0x0a /* Public */,
      12,    0,  264,    2, 0x0a /* Public */,
      13,    0,  265,    2, 0x0a /* Public */,
      14,    0,  266,    2, 0x0a /* Public */,
      15,    1,  267,    2, 0x0a /* Public */,
      18,    3,  270,    2, 0x0a /* Public */,
      22,    4,  277,    2, 0x0a /* Public */,
      24,    6,  286,    2, 0x0a /* Public */,
      27,    3,  299,    2, 0x0a /* Public */,
      31,    3,  306,    2, 0x0a /* Public */,
      32,    4,  313,    2, 0x0a /* Public */,
      35,    4,  322,    2, 0x0a /* Public */,
      36,    3,  331,    2, 0x0a /* Public */,
      37,    1,  338,    2, 0x0a /* Public */,
      39,    1,  341,    2, 0x0a /* Public */,
      41,    1,  344,    2, 0x0a /* Public */,
      42,    2,  347,    2, 0x0a /* Public */,
      42,    1,  352,    2, 0x2a /* Public | MethodCloned */,
      46,    0,  355,    2, 0x0a /* Public */,
      47,    0,  356,    2, 0x0a /* Public */,
      48,    0,  357,    2, 0x0a /* Public */,
      49,    0,  358,    2, 0x0a /* Public */,
      50,    0,  359,    2, 0x0a /* Public */,
      51,    2,  360,    2, 0x0a /* Public */,
      54,    0,  365,    2, 0x0a /* Public */,
      55,    0,  366,    2, 0x0a /* Public */,
      56,    0,  367,    2, 0x0a /* Public */,
      57,    1,  368,    2, 0x0a /* Public */,
      58,    2,  371,    2, 0x8a /* Public | MethodRevisioned */,
      61,    1,  376,    2, 0x8a /* Public | MethodRevisioned */,
      62,    2,  379,    2, 0x8a /* Public | MethodRevisioned */,

 // signals: revision
       0,
       0,
       0,
       0,
       0,

 // slots: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       1,
      13,
      13,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 16,   17,
    QMetaType::Void, QMetaType::QString, QMetaType::QUrl, QMetaType::Int,   19,   20,   21,
    QMetaType::Bool, QMetaType::Bool, QMetaType::QString, QMetaType::QUrl, QMetaType::Int,   23,   19,   20,   21,
    QMetaType::Bool, QMetaType::Bool, QMetaType::QString, QMetaType::QVariant, QMetaType::QVariant, QMetaType::QUrl, QMetaType::Int,   23,   19,   25,   26,   20,   21,
    QMetaType::Bool, QMetaType::QVariant, QMetaType::QVariant, QMetaType::QReal,   28,   29,   30,
    QMetaType::Void, QMetaType::QString, QMetaType::QUrl, QMetaType::Int,   19,   20,   21,
    QMetaType::Bool, QMetaType::QString, QMetaType::QString, QMetaType::QUrl, QMetaType::Int,   33,   34,   20,   21,
    QMetaType::Bool, QMetaType::QString, QMetaType::QString, QMetaType::QUrl, QMetaType::Int,   33,   34,   20,   21,
    QMetaType::Void, QMetaType::QString, QMetaType::QUrl, QMetaType::Int,   19,   20,   21,
    QMetaType::Void, 0x80000000 | 38,   19,
    QMetaType::Void, QMetaType::Int,   40,
    QMetaType::Void, QMetaType::Int,   40,
    QMetaType::Bool, 0x80000000 | 43, QMetaType::Int,   44,   45,
    QMetaType::Bool, 0x80000000 | 43,   44,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Void, 0x80000000 | 52, QMetaType::QString,   53,   33,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::QObjectStar, 0x80000000 | 43,   44,
    QMetaType::QObjectStar, QMetaType::QObjectStar, QMetaType::QString,   59,   60,
    QMetaType::Bool, 0x80000000 | 43,   44,
    QMetaType::Bool, 0x80000000 | 43, QMetaType::Int,   44,   45,

 // properties: name, type, flags
      63, QMetaType::QString, 0x00495103,
      64, QMetaType::QString, 0x00495103,
      65, QMetaType::QString, 0x00495103,
      66, QMetaType::Bool, 0x00095001,
      67, QMetaType::Bool, 0x00495103,
      68, QMetaType::Int, 0x00095001,
      69, QMetaType::Int, 0x00095001,
      70, QMetaType::Int, 0x00095001,
      71, QMetaType::QStringList, 0x00095001,
      72, QMetaType::QStringList, 0x00095001,

 // properties: notify_signal_id
       1,
       2,
       3,
       0,
       4,
       0,
       0,
       0,
       0,
       0,

 // enums: name, alias, flags, count, data
      52,   52, 0x0,    2,  429,

 // enum data: key, value
      73, uint(QuickTestResult::RepeatUntilValidMeasurement),
      74, uint(QuickTestResult::RunOnce),

       0        // eod
};

void QuickTestResult::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QuickTestResult *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->programNameChanged(); break;
        case 1: _t->testCaseNameChanged(); break;
        case 2: _t->functionNameChanged(); break;
        case 3: _t->dataTagChanged(); break;
        case 4: _t->skippedChanged(); break;
        case 5: _t->reset(); break;
        case 6: _t->startLogging(); break;
        case 7: _t->stopLogging(); break;
        case 8: _t->initTestTable(); break;
        case 9: _t->clearTestTable(); break;
        case 10: _t->finishTestData(); break;
        case 11: _t->finishTestDataCleanup(); break;
        case 12: _t->finishTestFunction(); break;
        case 13: _t->stringify((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        case 14: _t->fail((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 15: { bool _r = _t->verify((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 16: { bool _r = _t->compare((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QVariant(*)>(_a[3])),(*reinterpret_cast< const QVariant(*)>(_a[4])),(*reinterpret_cast< const QUrl(*)>(_a[5])),(*reinterpret_cast< int(*)>(_a[6])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 17: { bool _r = _t->fuzzyCompare((*reinterpret_cast< const QVariant(*)>(_a[1])),(*reinterpret_cast< const QVariant(*)>(_a[2])),(*reinterpret_cast< qreal(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 18: _t->skip((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 19: { bool _r = _t->expectFail((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 20: { bool _r = _t->expectFailContinue((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QUrl(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 21: _t->warn((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QUrl(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 22: _t->ignoreWarning((*reinterpret_cast< const QJSValue(*)>(_a[1]))); break;
        case 23: _t->wait((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 24: _t->sleep((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 25: { bool _r = _t->waitForRendering((*reinterpret_cast< QQuickItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 26: { bool _r = _t->waitForRendering((*reinterpret_cast< QQuickItem*(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 27: _t->startMeasurement(); break;
        case 28: _t->beginDataRun(); break;
        case 29: _t->endDataRun(); break;
        case 30: { bool _r = _t->measurementAccepted();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 31: { bool _r = _t->needsMoreMeasurements();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 32: _t->startBenchmark((*reinterpret_cast< RunMode(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 33: { bool _r = _t->isBenchmarkDone();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 34: _t->nextBenchmark(); break;
        case 35: _t->stopBenchmark(); break;
        case 36: { QObject* _r = _t->grabImage((*reinterpret_cast< QQuickItem*(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QObject**>(_a[0]) = std::move(_r); }  break;
        case 37: { QObject* _r = _t->findChild((*reinterpret_cast< QObject*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QObject**>(_a[0]) = std::move(_r); }  break;
        case 38: { bool _r = _t->isPolishScheduled((*reinterpret_cast< QQuickItem*(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 39: { bool _r = _t->waitForItemPolished((*reinterpret_cast< QQuickItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
            }
            break;
        case 25:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
            }
            break;
        case 26:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
            }
            break;
        case 36:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
            }
            break;
        case 38:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
            }
            break;
        case 39:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QuickTestResult::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QuickTestResult::programNameChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QuickTestResult::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QuickTestResult::testCaseNameChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QuickTestResult::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QuickTestResult::functionNameChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QuickTestResult::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QuickTestResult::dataTagChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QuickTestResult::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QuickTestResult::skippedChanged)) {
                *result = 4;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QuickTestResult *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->testCaseName(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->functionName(); break;
        case 2: *reinterpret_cast< QString*>(_v) = _t->dataTag(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->isFailed(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->isSkipped(); break;
        case 5: *reinterpret_cast< int*>(_v) = _t->passCount(); break;
        case 6: *reinterpret_cast< int*>(_v) = _t->failCount(); break;
        case 7: *reinterpret_cast< int*>(_v) = _t->skipCount(); break;
        case 8: *reinterpret_cast< QStringList*>(_v) = _t->functionsToRun(); break;
        case 9: *reinterpret_cast< QStringList*>(_v) = _t->tagsToRun(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QuickTestResult *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTestCaseName(*reinterpret_cast< QString*>(_v)); break;
        case 1: _t->setFunctionName(*reinterpret_cast< QString*>(_v)); break;
        case 2: _t->setDataTag(*reinterpret_cast< QString*>(_v)); break;
        case 4: _t->setSkipped(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QuickTestResult::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QuickTestResult.data,
    qt_meta_data_QuickTestResult,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QuickTestResult::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QuickTestResult::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QuickTestResult.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QuickTestResult::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 40)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 40;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 40)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 40;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 10;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QuickTestResult::programNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QuickTestResult::testCaseNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QuickTestResult::functionNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QuickTestResult::dataTagChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QuickTestResult::skippedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
