/****************************************************************************
** Meta object code from reading C++ file 'qquickpointattractor_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/particles/qquickpointattractor_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickpointattractor_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickAttractorAffector_t {
    QByteArrayData data[30];
    char stringdata0[413];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickAttractorAffector_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickAttractorAffector_t qt_meta_stringdata_QQuickAttractorAffector = {
    {
QT_MOC_LITERAL(0, 0, 23), // "QQuickAttractorAffector"
QT_MOC_LITERAL(1, 24, 11), // "QML.Element"
QT_MOC_LITERAL(2, 36, 9), // "Attractor"
QT_MOC_LITERAL(3, 46, 15), // "strengthChanged"
QT_MOC_LITERAL(4, 62, 0), // ""
QT_MOC_LITERAL(5, 63, 3), // "arg"
QT_MOC_LITERAL(6, 67, 13), // "pointXChanged"
QT_MOC_LITERAL(7, 81, 13), // "pointYChanged"
QT_MOC_LITERAL(8, 95, 24), // "affectedParameterChanged"
QT_MOC_LITERAL(9, 120, 20), // "AffectableParameters"
QT_MOC_LITERAL(10, 141, 29), // "proportionalToDistanceChanged"
QT_MOC_LITERAL(11, 171, 10), // "Proportion"
QT_MOC_LITERAL(12, 182, 11), // "setStrength"
QT_MOC_LITERAL(13, 194, 9), // "setPointX"
QT_MOC_LITERAL(14, 204, 9), // "setPointY"
QT_MOC_LITERAL(15, 214, 20), // "setAffectedParameter"
QT_MOC_LITERAL(16, 235, 25), // "setProportionalToDistance"
QT_MOC_LITERAL(17, 261, 8), // "strength"
QT_MOC_LITERAL(18, 270, 6), // "pointX"
QT_MOC_LITERAL(19, 277, 6), // "pointY"
QT_MOC_LITERAL(20, 284, 17), // "affectedParameter"
QT_MOC_LITERAL(21, 302, 22), // "proportionalToDistance"
QT_MOC_LITERAL(22, 325, 8), // "Constant"
QT_MOC_LITERAL(23, 334, 6), // "Linear"
QT_MOC_LITERAL(24, 341, 9), // "Quadratic"
QT_MOC_LITERAL(25, 351, 13), // "InverseLinear"
QT_MOC_LITERAL(26, 365, 16), // "InverseQuadratic"
QT_MOC_LITERAL(27, 382, 8), // "Position"
QT_MOC_LITERAL(28, 391, 8), // "Velocity"
QT_MOC_LITERAL(29, 400, 12) // "Acceleration"

    },
    "QQuickAttractorAffector\0QML.Element\0"
    "Attractor\0strengthChanged\0\0arg\0"
    "pointXChanged\0pointYChanged\0"
    "affectedParameterChanged\0AffectableParameters\0"
    "proportionalToDistanceChanged\0Proportion\0"
    "setStrength\0setPointX\0setPointY\0"
    "setAffectedParameter\0setProportionalToDistance\0"
    "strength\0pointX\0pointY\0affectedParameter\0"
    "proportionalToDistance\0Constant\0Linear\0"
    "Quadratic\0InverseLinear\0InverseQuadratic\0"
    "Position\0Velocity\0Acceleration"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickAttractorAffector[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      10,   16, // methods
       5,   96, // properties
       2,  116, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    1,   66,    4, 0x06 /* Public */,
       6,    1,   69,    4, 0x06 /* Public */,
       7,    1,   72,    4, 0x06 /* Public */,
       8,    1,   75,    4, 0x06 /* Public */,
      10,    1,   78,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      12,    1,   81,    4, 0x0a /* Public */,
      13,    1,   84,    4, 0x0a /* Public */,
      14,    1,   87,    4, 0x0a /* Public */,
      15,    1,   90,    4, 0x0a /* Public */,
      16,    1,   93,    4, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QReal,    5,
    QMetaType::Void, QMetaType::QReal,    5,
    QMetaType::Void, QMetaType::QReal,    5,
    QMetaType::Void, 0x80000000 | 9,    5,
    QMetaType::Void, 0x80000000 | 11,    5,

 // slots: parameters
    QMetaType::Void, QMetaType::QReal,    5,
    QMetaType::Void, QMetaType::QReal,    5,
    QMetaType::Void, QMetaType::QReal,    5,
    QMetaType::Void, 0x80000000 | 9,    5,
    QMetaType::Void, 0x80000000 | 11,    5,

 // properties: name, type, flags
      17, QMetaType::QReal, 0x00495103,
      18, QMetaType::QReal, 0x00495103,
      19, QMetaType::QReal, 0x00495103,
      20, 0x80000000 | 9, 0x0049510b,
      21, 0x80000000 | 11, 0x0049510b,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,

 // enums: name, alias, flags, count, data
      11,   11, 0x0,    5,  126,
       9,    9, 0x0,    3,  136,

 // enum data: key, value
      22, uint(QQuickAttractorAffector::Constant),
      23, uint(QQuickAttractorAffector::Linear),
      24, uint(QQuickAttractorAffector::Quadratic),
      25, uint(QQuickAttractorAffector::InverseLinear),
      26, uint(QQuickAttractorAffector::InverseQuadratic),
      27, uint(QQuickAttractorAffector::Position),
      28, uint(QQuickAttractorAffector::Velocity),
      29, uint(QQuickAttractorAffector::Acceleration),

       0        // eod
};

void QQuickAttractorAffector::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickAttractorAffector *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->strengthChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 1: _t->pointXChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 2: _t->pointYChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 3: _t->affectedParameterChanged((*reinterpret_cast< AffectableParameters(*)>(_a[1]))); break;
        case 4: _t->proportionalToDistanceChanged((*reinterpret_cast< Proportion(*)>(_a[1]))); break;
        case 5: _t->setStrength((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 6: _t->setPointX((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 7: _t->setPointY((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 8: _t->setAffectedParameter((*reinterpret_cast< AffectableParameters(*)>(_a[1]))); break;
        case 9: _t->setProportionalToDistance((*reinterpret_cast< Proportion(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickAttractorAffector::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAttractorAffector::strengthChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickAttractorAffector::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAttractorAffector::pointXChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickAttractorAffector::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAttractorAffector::pointYChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickAttractorAffector::*)(AffectableParameters );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAttractorAffector::affectedParameterChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickAttractorAffector::*)(Proportion );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAttractorAffector::proportionalToDistanceChanged)) {
                *result = 4;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickAttractorAffector *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< qreal*>(_v) = _t->strength(); break;
        case 1: *reinterpret_cast< qreal*>(_v) = _t->pointX(); break;
        case 2: *reinterpret_cast< qreal*>(_v) = _t->pointY(); break;
        case 3: *reinterpret_cast< AffectableParameters*>(_v) = _t->affectedParameter(); break;
        case 4: *reinterpret_cast< Proportion*>(_v) = _t->proportionalToDistance(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickAttractorAffector *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setStrength(*reinterpret_cast< qreal*>(_v)); break;
        case 1: _t->setPointX(*reinterpret_cast< qreal*>(_v)); break;
        case 2: _t->setPointY(*reinterpret_cast< qreal*>(_v)); break;
        case 3: _t->setAffectedParameter(*reinterpret_cast< AffectableParameters*>(_v)); break;
        case 4: _t->setProportionalToDistance(*reinterpret_cast< Proportion*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickAttractorAffector::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickParticleAffector::staticMetaObject>(),
    qt_meta_stringdata_QQuickAttractorAffector.data,
    qt_meta_data_QQuickAttractorAffector,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickAttractorAffector::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickAttractorAffector::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickAttractorAffector.stringdata0))
        return static_cast<void*>(this);
    return QQuickParticleAffector::qt_metacast(_clname);
}

int QQuickAttractorAffector::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickParticleAffector::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickAttractorAffector::strengthChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QQuickAttractorAffector::pointXChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QQuickAttractorAffector::pointYChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QQuickAttractorAffector::affectedParameterChanged(AffectableParameters _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QQuickAttractorAffector::proportionalToDistanceChanged(Proportion _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
