/****************************************************************************
** Meta object code from reading C++ file 'qqmldelegatecomponent_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../kde-qtdeclarative/src/imports/labsmodels/qqmldelegatecomponent_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qqmldelegatecomponent_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQmlDelegateChoice_t {
    QByteArrayData data[17];
    char stringdata0[193];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQmlDelegateChoice_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQmlDelegateChoice_t qt_meta_stringdata_QQmlDelegateChoice = {
    {
QT_MOC_LITERAL(0, 0, 18), // "QQmlDelegateChoice"
QT_MOC_LITERAL(1, 19, 15), // "DefaultProperty"
QT_MOC_LITERAL(2, 35, 8), // "delegate"
QT_MOC_LITERAL(3, 44, 11), // "QML.Element"
QT_MOC_LITERAL(4, 56, 14), // "DelegateChoice"
QT_MOC_LITERAL(5, 71, 16), // "roleValueChanged"
QT_MOC_LITERAL(6, 88, 0), // ""
QT_MOC_LITERAL(7, 89, 10), // "rowChanged"
QT_MOC_LITERAL(8, 100, 12), // "indexChanged"
QT_MOC_LITERAL(9, 113, 13), // "columnChanged"
QT_MOC_LITERAL(10, 127, 15), // "delegateChanged"
QT_MOC_LITERAL(11, 143, 7), // "changed"
QT_MOC_LITERAL(12, 151, 9), // "roleValue"
QT_MOC_LITERAL(13, 161, 3), // "row"
QT_MOC_LITERAL(14, 165, 5), // "index"
QT_MOC_LITERAL(15, 171, 6), // "column"
QT_MOC_LITERAL(16, 178, 14) // "QQmlComponent*"

    },
    "QQmlDelegateChoice\0DefaultProperty\0"
    "delegate\0QML.Element\0DelegateChoice\0"
    "roleValueChanged\0\0rowChanged\0indexChanged\0"
    "columnChanged\0delegateChanged\0changed\0"
    "roleValue\0row\0index\0column\0QQmlComponent*"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQmlDelegateChoice[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       6,   18, // methods
       5,   54, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

 // signals: name, argc, parameters, tag, flags
       5,    0,   48,    6, 0x06 /* Public */,
       7,    0,   49,    6, 0x06 /* Public */,
       8,    0,   50,    6, 0x06 /* Public */,
       9,    0,   51,    6, 0x06 /* Public */,
      10,    0,   52,    6, 0x06 /* Public */,
      11,    0,   53,    6, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      12, QMetaType::QVariant, 0x00495103,
      13, QMetaType::Int, 0x00495103,
      14, QMetaType::Int, 0x00495003,
      15, QMetaType::Int, 0x00495103,
       2, 0x80000000 | 16, 0x0049510b,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,

       0        // eod
};

void QQmlDelegateChoice::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQmlDelegateChoice *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->roleValueChanged(); break;
        case 1: _t->rowChanged(); break;
        case 2: _t->indexChanged(); break;
        case 3: _t->columnChanged(); break;
        case 4: _t->delegateChanged(); break;
        case 5: _t->changed(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQmlDelegateChoice::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlDelegateChoice::roleValueChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQmlDelegateChoice::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlDelegateChoice::rowChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQmlDelegateChoice::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlDelegateChoice::indexChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQmlDelegateChoice::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlDelegateChoice::columnChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQmlDelegateChoice::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlDelegateChoice::delegateChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQmlDelegateChoice::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlDelegateChoice::changed)) {
                *result = 5;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlComponent* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQmlDelegateChoice *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QVariant*>(_v) = _t->roleValue(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->row(); break;
        case 2: *reinterpret_cast< int*>(_v) = _t->row(); break;
        case 3: *reinterpret_cast< int*>(_v) = _t->column(); break;
        case 4: *reinterpret_cast< QQmlComponent**>(_v) = _t->delegate(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQmlDelegateChoice *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setRoleValue(*reinterpret_cast< QVariant*>(_v)); break;
        case 1: _t->setRow(*reinterpret_cast< int*>(_v)); break;
        case 2: _t->setRow(*reinterpret_cast< int*>(_v)); break;
        case 3: _t->setColumn(*reinterpret_cast< int*>(_v)); break;
        case 4: _t->setDelegate(*reinterpret_cast< QQmlComponent**>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQmlDelegateChoice::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQmlDelegateChoice.data,
    qt_meta_data_QQmlDelegateChoice,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQmlDelegateChoice::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlDelegateChoice::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQmlDelegateChoice.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQmlDelegateChoice::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQmlDelegateChoice::roleValueChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQmlDelegateChoice::rowChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQmlDelegateChoice::indexChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQmlDelegateChoice::columnChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQmlDelegateChoice::delegateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQmlDelegateChoice::changed()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
struct qt_meta_stringdata_QQmlDelegateChooser_t {
    QByteArrayData data[9];
    char stringdata0[127];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQmlDelegateChooser_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQmlDelegateChooser_t qt_meta_stringdata_QQmlDelegateChooser = {
    {
QT_MOC_LITERAL(0, 0, 19), // "QQmlDelegateChooser"
QT_MOC_LITERAL(1, 20, 15), // "DefaultProperty"
QT_MOC_LITERAL(2, 36, 7), // "choices"
QT_MOC_LITERAL(3, 44, 11), // "QML.Element"
QT_MOC_LITERAL(4, 56, 15), // "DelegateChooser"
QT_MOC_LITERAL(5, 72, 11), // "roleChanged"
QT_MOC_LITERAL(6, 84, 0), // ""
QT_MOC_LITERAL(7, 85, 4), // "role"
QT_MOC_LITERAL(8, 90, 36) // "QQmlListProperty<QQmlDelegate..."

    },
    "QQmlDelegateChooser\0DefaultProperty\0"
    "choices\0QML.Element\0DelegateChooser\0"
    "roleChanged\0\0role\0"
    "QQmlListProperty<QQmlDelegateChoice>"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQmlDelegateChooser[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       1,   18, // methods
       2,   24, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

 // signals: name, argc, parameters, tag, flags
       5,    0,   23,    6, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,

 // properties: name, type, flags
       7, QMetaType::QString, 0x00495103,
       2, 0x80000000 | 8, 0x00095409,

 // properties: notify_signal_id
       0,
       0,

       0        // eod
};

void QQmlDelegateChooser::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQmlDelegateChooser *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->roleChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQmlDelegateChooser::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlDelegateChooser::roleChanged)) {
                *result = 0;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlListProperty<QQmlDelegateChoice> >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQmlDelegateChooser *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->role(); break;
        case 1: *reinterpret_cast< QQmlListProperty<QQmlDelegateChoice>*>(_v) = _t->choices(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQmlDelegateChooser *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setRole(*reinterpret_cast< QString*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQmlDelegateChooser::staticMetaObject = { {
    QMetaObject::SuperData::link<QQmlAbstractDelegateComponent::staticMetaObject>(),
    qt_meta_stringdata_QQmlDelegateChooser.data,
    qt_meta_data_QQmlDelegateChooser,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQmlDelegateChooser::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlDelegateChooser::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQmlDelegateChooser.stringdata0))
        return static_cast<void*>(this);
    return QQmlAbstractDelegateComponent::qt_metacast(_clname);
}

int QQmlDelegateChooser::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQmlAbstractDelegateComponent::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQmlDelegateChooser::roleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
