/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#include <plugin.h>

void qml_register_types_QtQuick_Window()
{
    qmlRegisterTypesAndRevisions<QQuickRootItemForeign>("QtQuick.Window", 2);
    qmlRegisterTypesAndRevisions<QQuickScreenForeign>("QtQuick.Window", 2);
    qmlRegisterTypesAndRevisions<QQuickScreenForeignAttached>("QtQuick.Window", 2);
    qmlRegisterTypesAndRevisions<QQuickScreenInfoForeign>("QtQuick.Window", 2);
    qmlRegisterTypesAndRevisions<QQuickWindowForeign>("QtQuick.Window", 2);
    qmlRegisterTypesAndRevisions<QQuickWindowForeignAttached>("QtQuick.Window", 2);
    qmlRegisterTypesAndRevisions<QQuickWindowQmlImplForeign>("QtQuick.Window", 2);
    qmlRegisterTypesAndRevisions<QWindowForeign>("QtQuick.Window", 2);
    qmlRegisterModule("QtQuick.Window", 2, 15);
}

static const QQmlModuleRegistration registration("QtQuick.Window", 2, qml_register_types_QtQuick_Window);
