/****************************************************************************
** Meta object code from reading C++ file 'qquickimage_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/quick/items/qquickimage_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickimage_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickImage_t {
    QByteArrayData data[37];
    char stringdata0[505];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickImage_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickImage_t qt_meta_stringdata_QQuickImage = {
    {
QT_MOC_LITERAL(0, 0, 11), // "QQuickImage"
QT_MOC_LITERAL(1, 12, 11), // "QML.Element"
QT_MOC_LITERAL(2, 24, 5), // "Image"
QT_MOC_LITERAL(3, 30, 15), // "fillModeChanged"
QT_MOC_LITERAL(4, 46, 0), // ""
QT_MOC_LITERAL(5, 47, 22), // "paintedGeometryChanged"
QT_MOC_LITERAL(6, 70, 26), // "horizontalAlignmentChanged"
QT_MOC_LITERAL(7, 97, 10), // "HAlignment"
QT_MOC_LITERAL(8, 108, 9), // "alignment"
QT_MOC_LITERAL(9, 118, 24), // "verticalAlignmentChanged"
QT_MOC_LITERAL(10, 143, 10), // "VAlignment"
QT_MOC_LITERAL(11, 154, 13), // "mipmapChanged"
QT_MOC_LITERAL(12, 168, 20), // "autoTransformChanged"
QT_MOC_LITERAL(13, 189, 20), // "invalidateSceneGraph"
QT_MOC_LITERAL(14, 210, 21), // "sourceClipRectChanged"
QT_MOC_LITERAL(15, 232, 8), // "fillMode"
QT_MOC_LITERAL(16, 241, 8), // "FillMode"
QT_MOC_LITERAL(17, 250, 12), // "paintedWidth"
QT_MOC_LITERAL(18, 263, 13), // "paintedHeight"
QT_MOC_LITERAL(19, 277, 19), // "horizontalAlignment"
QT_MOC_LITERAL(20, 297, 17), // "verticalAlignment"
QT_MOC_LITERAL(21, 315, 6), // "mipmap"
QT_MOC_LITERAL(22, 322, 13), // "autoTransform"
QT_MOC_LITERAL(23, 336, 14), // "sourceClipRect"
QT_MOC_LITERAL(24, 351, 9), // "AlignLeft"
QT_MOC_LITERAL(25, 361, 10), // "AlignRight"
QT_MOC_LITERAL(26, 372, 12), // "AlignHCenter"
QT_MOC_LITERAL(27, 385, 8), // "AlignTop"
QT_MOC_LITERAL(28, 394, 11), // "AlignBottom"
QT_MOC_LITERAL(29, 406, 12), // "AlignVCenter"
QT_MOC_LITERAL(30, 419, 7), // "Stretch"
QT_MOC_LITERAL(31, 427, 17), // "PreserveAspectFit"
QT_MOC_LITERAL(32, 445, 18), // "PreserveAspectCrop"
QT_MOC_LITERAL(33, 464, 4), // "Tile"
QT_MOC_LITERAL(34, 469, 14), // "TileVertically"
QT_MOC_LITERAL(35, 484, 16), // "TileHorizontally"
QT_MOC_LITERAL(36, 501, 3) // "Pad"

    },
    "QQuickImage\0QML.Element\0Image\0"
    "fillModeChanged\0\0paintedGeometryChanged\0"
    "horizontalAlignmentChanged\0HAlignment\0"
    "alignment\0verticalAlignmentChanged\0"
    "VAlignment\0mipmapChanged\0autoTransformChanged\0"
    "invalidateSceneGraph\0sourceClipRectChanged\0"
    "fillMode\0FillMode\0paintedWidth\0"
    "paintedHeight\0horizontalAlignment\0"
    "verticalAlignment\0mipmap\0autoTransform\0"
    "sourceClipRect\0AlignLeft\0AlignRight\0"
    "AlignHCenter\0AlignTop\0AlignBottom\0"
    "AlignVCenter\0Stretch\0PreserveAspectFit\0"
    "PreserveAspectCrop\0Tile\0TileVertically\0"
    "TileHorizontally\0Pad"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickImage[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       7,   16, // methods
       8,   71, // properties
       3,  111, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   58,    4, 0x06 /* Public */,
       5,    0,   59,    4, 0x06 /* Public */,
       6,    1,   60,    4, 0x06 /* Public */,
       9,    1,   63,    4, 0x06 /* Public */,
      11,    1,   66,    4, 0x86 /* Public | MethodRevisioned */,
      12,    0,   69,    4, 0x86 /* Public | MethodRevisioned */,

 // slots: name, argc, parameters, tag, flags
      13,    0,   70,    4, 0x08 /* Private */,

 // signals: revision
       0,
       0,
       0,
       0,
       3,
       5,

 // slots: revision
       0,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, 0x80000000 | 10,    8,
    QMetaType::Void, QMetaType::Bool,    4,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,

 // properties: name, type, flags
      15, 0x80000000 | 16, 0x0049510b,
      17, QMetaType::QReal, 0x00495001,
      18, QMetaType::QReal, 0x00495001,
      19, 0x80000000 | 7, 0x0049510b,
      20, 0x80000000 | 10, 0x0049510b,
      21, QMetaType::Bool, 0x00c95103,
      22, QMetaType::Bool, 0x00c95103,
      23, QMetaType::QRectF, 0x00c95107,

 // properties: notify_signal_id
       0,
       1,
       1,
       2,
       3,
       4,
       5,
    1879048206,

 // properties: revision
       0,
       0,
       0,
       0,
       0,
       3,
       5,
      15,

 // enums: name, alias, flags, count, data
       7,    7, 0x0,    3,  126,
      10,   10, 0x0,    3,  132,
      16,   16, 0x0,    7,  138,

 // enum data: key, value
      24, uint(QQuickImage::AlignLeft),
      25, uint(QQuickImage::AlignRight),
      26, uint(QQuickImage::AlignHCenter),
      27, uint(QQuickImage::AlignTop),
      28, uint(QQuickImage::AlignBottom),
      29, uint(QQuickImage::AlignVCenter),
      30, uint(QQuickImage::Stretch),
      31, uint(QQuickImage::PreserveAspectFit),
      32, uint(QQuickImage::PreserveAspectCrop),
      33, uint(QQuickImage::Tile),
      34, uint(QQuickImage::TileVertically),
      35, uint(QQuickImage::TileHorizontally),
      36, uint(QQuickImage::Pad),

       0        // eod
};

void QQuickImage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickImage *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->fillModeChanged(); break;
        case 1: _t->paintedGeometryChanged(); break;
        case 2: _t->horizontalAlignmentChanged((*reinterpret_cast< HAlignment(*)>(_a[1]))); break;
        case 3: _t->verticalAlignmentChanged((*reinterpret_cast< VAlignment(*)>(_a[1]))); break;
        case 4: _t->mipmapChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->autoTransformChanged(); break;
        case 6: _t->invalidateSceneGraph(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickImage::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImage::fillModeChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickImage::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImage::paintedGeometryChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickImage::*)(HAlignment );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImage::horizontalAlignmentChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickImage::*)(VAlignment );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImage::verticalAlignmentChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickImage::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImage::mipmapChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickImage::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImage::autoTransformChanged)) {
                *result = 5;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickImage *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< FillMode*>(_v) = _t->fillMode(); break;
        case 1: *reinterpret_cast< qreal*>(_v) = _t->paintedWidth(); break;
        case 2: *reinterpret_cast< qreal*>(_v) = _t->paintedHeight(); break;
        case 3: *reinterpret_cast< HAlignment*>(_v) = _t->horizontalAlignment(); break;
        case 4: *reinterpret_cast< VAlignment*>(_v) = _t->verticalAlignment(); break;
        case 5: *reinterpret_cast< bool*>(_v) = _t->mipmap(); break;
        case 6: *reinterpret_cast< bool*>(_v) = _t->autoTransform(); break;
        case 7: *reinterpret_cast< QRectF*>(_v) = _t->sourceClipRect(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickImage *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFillMode(*reinterpret_cast< FillMode*>(_v)); break;
        case 3: _t->setHorizontalAlignment(*reinterpret_cast< HAlignment*>(_v)); break;
        case 4: _t->setVerticalAlignment(*reinterpret_cast< VAlignment*>(_v)); break;
        case 5: _t->setMipmap(*reinterpret_cast< bool*>(_v)); break;
        case 6: _t->setAutoTransform(*reinterpret_cast< bool*>(_v)); break;
        case 7: _t->setSourceClipRect(*reinterpret_cast< QRectF*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        QQuickImage *_t = static_cast<QQuickImage *>(_o);
        (void)_t;
        switch (_id) {
        case 7: _t->resetSourceClipRect(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickImage::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickImageBase::staticMetaObject>(),
    qt_meta_stringdata_QQuickImage.data,
    qt_meta_data_QQuickImage,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickImage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickImage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickImage.stringdata0))
        return static_cast<void*>(this);
    return QQuickImageBase::qt_metacast(_clname);
}

int QQuickImage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickImageBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 8;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickImage::fillModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickImage::paintedGeometryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickImage::horizontalAlignmentChanged(HAlignment _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QQuickImage::verticalAlignmentChanged(VAlignment _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QQuickImage::mipmapChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QQuickImage::autoTransformChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
// If you get a compile error in this function it can be because either
//     a) You are using a NOTIFY signal that does not exist. Fix it.
//     b) You are using a NOTIFY signal that does exist (in a parent class) but has a non-empty parameter list. This is a moc limitation.
Q_DECL_UNUSED static void checkNotifySignalValidity_QQuickImage(QQuickImage *t) {
    t->sourceClipRectChanged();
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
