/****************************************************************************
** Meta object code from reading C++ file 'qquickitemviewtransition_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/quick/items/qquickitemviewtransition_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickitemviewtransition_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickViewTransitionAttached_t {
    QByteArrayData data[22];
    char stringdata0[352];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickViewTransitionAttached_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickViewTransitionAttached_t qt_meta_stringdata_QQuickViewTransitionAttached = {
    {
QT_MOC_LITERAL(0, 0, 28), // "QQuickViewTransitionAttached"
QT_MOC_LITERAL(1, 29, 11), // "QML.Element"
QT_MOC_LITERAL(2, 41, 14), // "ViewTransition"
QT_MOC_LITERAL(3, 56, 13), // "QML.Creatable"
QT_MOC_LITERAL(4, 70, 5), // "false"
QT_MOC_LITERAL(5, 76, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(6, 98, 57), // "ViewTransition is only availa..."
QT_MOC_LITERAL(7, 156, 12), // "QML.Attached"
QT_MOC_LITERAL(8, 169, 12), // "indexChanged"
QT_MOC_LITERAL(9, 182, 0), // ""
QT_MOC_LITERAL(10, 183, 11), // "itemChanged"
QT_MOC_LITERAL(11, 195, 18), // "destinationChanged"
QT_MOC_LITERAL(12, 214, 20), // "targetIndexesChanged"
QT_MOC_LITERAL(13, 235, 18), // "targetItemsChanged"
QT_MOC_LITERAL(14, 254, 5), // "index"
QT_MOC_LITERAL(15, 260, 4), // "item"
QT_MOC_LITERAL(16, 265, 11), // "QQuickItem*"
QT_MOC_LITERAL(17, 277, 11), // "destination"
QT_MOC_LITERAL(18, 289, 13), // "targetIndexes"
QT_MOC_LITERAL(19, 303, 10), // "QList<int>"
QT_MOC_LITERAL(20, 314, 11), // "targetItems"
QT_MOC_LITERAL(21, 326, 25) // "QQmlListProperty<QObject>"

    },
    "QQuickViewTransitionAttached\0QML.Element\0"
    "ViewTransition\0QML.Creatable\0false\0"
    "QML.UncreatableReason\0"
    "ViewTransition is only available via attached properties.\0"
    "QML.Attached\0indexChanged\0\0itemChanged\0"
    "destinationChanged\0targetIndexesChanged\0"
    "targetItemsChanged\0index\0item\0QQuickItem*\0"
    "destination\0targetIndexes\0QList<int>\0"
    "targetItems\0QQmlListProperty<QObject>"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickViewTransitionAttached[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
       5,   22, // methods
       5,   52, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    0,

 // signals: name, argc, parameters, tag, flags
       8,    0,   47,    9, 0x06 /* Public */,
      10,    0,   48,    9, 0x06 /* Public */,
      11,    0,   49,    9, 0x06 /* Public */,
      12,    0,   50,    9, 0x06 /* Public */,
      13,    0,   51,    9, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      14, QMetaType::Int, 0x00495001,
      15, 0x80000000 | 16, 0x00495009,
      17, QMetaType::QPointF, 0x00495001,
      18, 0x80000000 | 19, 0x00495009,
      20, 0x80000000 | 21, 0x00495009,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,

       0        // eod
};

void QQuickViewTransitionAttached::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickViewTransitionAttached *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->indexChanged(); break;
        case 1: _t->itemChanged(); break;
        case 2: _t->destinationChanged(); break;
        case 3: _t->targetIndexesChanged(); break;
        case 4: _t->targetItemsChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickViewTransitionAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickViewTransitionAttached::indexChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickViewTransitionAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickViewTransitionAttached::itemChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickViewTransitionAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickViewTransitionAttached::destinationChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickViewTransitionAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickViewTransitionAttached::targetIndexesChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickViewTransitionAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickViewTransitionAttached::targetItemsChanged)) {
                *result = 4;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlListProperty<QObject> >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickViewTransitionAttached *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->index(); break;
        case 1: *reinterpret_cast< QQuickItem**>(_v) = _t->item(); break;
        case 2: *reinterpret_cast< QPointF*>(_v) = _t->destination(); break;
        case 3: *reinterpret_cast< QList<int>*>(_v) = _t->targetIndexes(); break;
        case 4: *reinterpret_cast< QQmlListProperty<QObject>*>(_v) = _t->targetItems(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickViewTransitionAttached::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickViewTransitionAttached.data,
    qt_meta_data_QQuickViewTransitionAttached,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickViewTransitionAttached::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickViewTransitionAttached::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickViewTransitionAttached.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickViewTransitionAttached::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickViewTransitionAttached::indexChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickViewTransitionAttached::itemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickViewTransitionAttached::destinationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickViewTransitionAttached::targetIndexesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickViewTransitionAttached::targetItemsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
