/****************************************************************************
** Meta object code from reading C++ file 'signaltransition.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../kde-qtdeclarative/src/imports/statemachine/signaltransition.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'signaltransition.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_SignalTransition_t {
    QByteArrayData data[12];
    char stringdata0[126];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SignalTransition_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SignalTransition_t qt_meta_stringdata_SignalTransition = {
    {
QT_MOC_LITERAL(0, 0, 16), // "SignalTransition"
QT_MOC_LITERAL(1, 17, 11), // "QML.Element"
QT_MOC_LITERAL(2, 29, 4), // "auto"
QT_MOC_LITERAL(3, 34, 12), // "guardChanged"
QT_MOC_LITERAL(4, 47, 0), // ""
QT_MOC_LITERAL(5, 48, 14), // "invokeYourself"
QT_MOC_LITERAL(6, 63, 16), // "qmlSignalChanged"
QT_MOC_LITERAL(7, 80, 6), // "invoke"
QT_MOC_LITERAL(8, 87, 6), // "signal"
QT_MOC_LITERAL(9, 94, 8), // "QJSValue"
QT_MOC_LITERAL(10, 103, 5), // "guard"
QT_MOC_LITERAL(11, 109, 16) // "QQmlScriptString"

    },
    "SignalTransition\0QML.Element\0auto\0"
    "guardChanged\0\0invokeYourself\0"
    "qmlSignalChanged\0invoke\0signal\0QJSValue\0"
    "guard\0QQmlScriptString"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SignalTransition[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       4,   16, // methods
       2,   40, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   36,    4, 0x06 /* Public */,
       5,    0,   37,    4, 0x06 /* Public */,
       6,    0,   38,    4, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
       7,    0,   39,    4, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void,

 // properties: name, type, flags
       8, 0x80000000 | 9, 0x0049510b,
      10, 0x80000000 | 11, 0x0049510b,

 // properties: notify_signal_id
       2,
       0,

       0        // eod
};

void SignalTransition::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SignalTransition *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->guardChanged(); break;
        case 1: _t->invokeYourself(); break;
        case 2: _t->qmlSignalChanged(); break;
        case 3: _t->invoke(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SignalTransition::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SignalTransition::guardChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (SignalTransition::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SignalTransition::invokeYourself)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (SignalTransition::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SignalTransition::qmlSignalChanged)) {
                *result = 2;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlScriptString >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<SignalTransition *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QJSValue*>(_v) = _t->signal(); break;
        case 1: *reinterpret_cast< QQmlScriptString*>(_v) = _t->guard(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<SignalTransition *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSignal(*reinterpret_cast< QJSValue*>(_v)); break;
        case 1: _t->setGuard(*reinterpret_cast< QQmlScriptString*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject SignalTransition::staticMetaObject = { {
    QMetaObject::SuperData::link<QSignalTransition::staticMetaObject>(),
    qt_meta_stringdata_SignalTransition.data,
    qt_meta_data_SignalTransition,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SignalTransition::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SignalTransition::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SignalTransition.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QSignalTransition::qt_metacast(_clname);
}

int SignalTransition::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSignalTransition::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void SignalTransition::guardChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void SignalTransition::invokeYourself()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void SignalTransition::qmlSignalChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
