/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#include <private/qquickaccessibleattached_p.h>
#include <private/qquickanchors_p.h>
#include <private/qquickanimatedimage_p.h>
#include <private/qquickanimatedsprite_p.h>
#include <private/qquickanimation_p.h>
#include <private/qquickanimationcontroller_p.h>
#include <private/qquickanimator_p.h>
#include <private/qquickapplication_p.h>
#include <private/qquickbehavior_p.h>
#include <private/qquickborderimage_p.h>
#include <private/qquickcanvasitem_p.h>
#include <private/qquickdrag_p.h>
#include <private/qquickdragaxis_p.h>
#include <private/qquickdraghandler_p.h>
#include <private/qquickdroparea_p.h>
#include <private/qquickevents_p_p.h>
#include <private/qquickflickable_p.h>
#include <private/qquickflickable_p_p.h>
#include <private/qquickflipable_p.h>
#include <private/qquickfocusscope_p.h>
#include <private/qquickfontloader_p.h>
#include <private/qquickfontmetrics_p.h>
#include <private/qquickforeignutils_p.h>
#include <private/qquickgraphicsinfo_p.h>
#include <private/qquickgridview_p.h>
#include <private/qquickhoverhandler_p.h>
#include <private/qquickimage_p.h>
#include <private/qquickimagebase_p.h>
#include <private/qquickitem_p.h>
#include <private/qquickitemanimation_p.h>
#include <private/qquickitemsmodule_p.h>
#include <private/qquickitemview_p.h>
#include <private/qquickitemviewtransition_p.h>
#include <private/qquicklistview_p.h>
#include <private/qquickloader_p.h>
#include <private/qquickmousearea_p.h>
#include <private/qquickmultipointtoucharea_p.h>
#include <private/qquickopenglinfo_p.h>
#include <private/qquickpath_p.h>
#include <private/qquickpathinterpolator_p.h>
#include <private/qquickpathview_p.h>
#include <private/qquickpincharea_p.h>
#include <private/qquickpinchhandler_p.h>
#include <private/qquickpointerhandler_p.h>
#include <private/qquickpointhandler_p.h>
#include <private/qquickpositioners_p.h>
#include <private/qquickpropertychanges_p.h>
#include <private/qquickrectangle_p.h>
#include <private/qquickrepeater_p.h>
#include <private/qquickscalegrid_p_p.h>
#include <private/qquickshadereffect_p.h>
#include <private/qquickshadereffectmesh_p.h>
#include <private/qquickshadereffectsource_p.h>
#include <private/qquickshortcut_p.h>
#include <private/qquicksmoothedanimation_p.h>
#include <private/qquickspringanimation_p.h>
#include <private/qquicksprite_p.h>
#include <private/qquickspritesequence_p.h>
#include <private/qquickstate_p.h>
#include <private/qquickstatechangescript_p.h>
#include <private/qquickstategroup_p.h>
#include <private/qquickstateoperations_p.h>
#include <private/qquicksystempalette_p.h>
#include <private/qquicktableview_p.h>
#include <private/qquicktaphandler_p.h>
#include <private/qquicktext_p.h>
#include <private/qquicktextedit_p.h>
#include <private/qquicktextinput_p.h>
#include <private/qquicktextmetrics_p.h>
#include <private/qquicktransition_p.h>
#include <private/qquicktranslate_p.h>
#include <private/qquickvalidator_p.h>
#include <private/qquickvaluetypes_p.h>
#include <private/qquickwheelhandler_p.h>
#include <qquickitem.h>
#include <qquickitemgrabresult.h>
#include <qquickpainteditem.h>
#include <qquicktextdocument.h>

void qml_register_types_QtQuick()
{
    qmlRegisterTypesAndRevisions<QInputMethodForeign>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QKeySequenceForeign>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QPointingDeviceUniqueIdForeign>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickAbstractAnimation>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickAccessibleAttached>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickAnchorAnimation>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickAnchorChanges>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickAnchorSet>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickAnchors>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickAnimatedImage>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickAnimatedSprite>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickAnimationController>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickAnimator>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickApplication>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickBasePositioner>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickBehavior>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickBorderImage>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickBorderImageMesh>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickCanvasItem>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickCloseEvent>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickColorAnimation>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickColorSpaceValueType>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickColumn>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickCurve>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickDoubleValidator>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickDrag>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickDragAttached>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickDragAxis>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickDragHandler>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickDropArea>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickDropAreaDrag>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickDropEvent>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickEnterKeyAttached>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickEventPoint>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickEventTabletPoint>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickEventTouchPoint>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickFlickable>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickFlickableVisibleArea>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickFlipable>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickFlow>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickFocusScope>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickFontLoader>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickFontMetrics>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickFontValueType>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickGrabGestureEvent>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickGradient>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickGradientStop>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickGraphicsInfo>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickGrid>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickGridMesh>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickGridView>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickHoverHandler>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickImage>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickImageBase>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickIntValidator>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickItem>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickItemGrabResult>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickItemLayer>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickItemView>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickKeyEvent>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickKeyNavigationAttached>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickKeysAttached>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickLayoutMirroringAttached>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickListView>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickLoader>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickMatrix4x4>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickMouseArea>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickMouseEvent>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickMultiPointTouchArea>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickNumberAnimation>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickOpacityAnimator>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickOpenGLInfo>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPaintedItem>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickParallelAnimation>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickParentAnimation>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickParentChange>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPath>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPathAngleArc>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPathAnimation>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPathArc>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPathAttribute>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPathCatmullRomCurve>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPathCubic>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPathElement>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPathInterpolator>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPathLine>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPathMove>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPathMultiline>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPathPercent>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPathPolyline>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPathQuad>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPathSvg>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPathText>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPathView>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPauseAnimation>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPen>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPinch>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPinchArea>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPinchEvent>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPinchHandler>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPointHandler>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPointerDevice>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPointerEvent>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPointerHandler>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPointerMouseEvent>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPointerScrollEvent>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPointerTouchEvent>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPropertyAction>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPropertyAnimation>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickPropertyChanges>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickRectangle>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickRepeater>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickRotation>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickRotationAnimation>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickRotationAnimator>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickRow>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickScale>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickScaleAnimator>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickScaleGrid>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickScriptAction>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickSequentialAnimation>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickShaderEffect>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickShaderEffectMesh>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickShaderEffectSource>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickShortcut>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickSmoothedAnimation>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickSpringAnimation>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickSprite>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickSpriteSequence>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickState>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickStateChangeScript>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickStateGroup>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickStateOperation>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickSystemPalette>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickTableView>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickTapHandler>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickText>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickTextDocument>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickTextEdit>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickTextInput>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickTextLine>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickTextMetrics>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickTouchPoint>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickTransform>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickTransition>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickTranslate>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickUniformAnimator>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickVector3dAnimation>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickViewSection>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickViewTransitionAttached>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickWheelEvent>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickWheelHandler>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickXAnimator>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QQuickYAnimator>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QRegExpValidatorForeign>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QRegularExpressionValidatorForeign>("QtQuick", 2);
    qmlRegisterTypesAndRevisions<QValidatorForeign>("QtQuick", 2);
    qmlRegisterModule("QtQuick", 2, 15);
}

static const QQmlModuleRegistration registration("QtQuick", 2, qml_register_types_QtQuick);
