/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#include <qqmldelegatecomponent_p.h>
#include <qqmltablemodel_p.h>
#include <qqmltablemodelcolumn_p.h>

void qml_register_types_Qt_labs_qmlmodels()
{
    qmlRegisterTypesAndRevisions<QQmlDelegateChoice>("Qt.labs.qmlmodels", 1);
    qmlRegisterTypesAndRevisions<QQmlDelegateChooser>("Qt.labs.qmlmodels", 1);
    qmlRegisterTypesAndRevisions<QQmlTableModel>("Qt.labs.qmlmodels", 1);
    qmlRegisterTypesAndRevisions<QQmlTableModelColumn>("Qt.labs.qmlmodels", 1);
    qmlRegisterModule("Qt.labs.qmlmodels", 1, 0);
}

static const QQmlModuleRegistration registration("Qt.labs.qmlmodels", 1, qml_register_types_Qt_labs_qmlmodels);
