/****************************************************************************
** Meta object code from reading C++ file 'qquickapplication_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/quick/util/qquickapplication_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickapplication_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickApplication_t {
    QByteArrayData data[24];
    char stringdata0[371];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickApplication_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickApplication_t qt_meta_stringdata_QQuickApplication = {
    {
QT_MOC_LITERAL(0, 0, 17), // "QQuickApplication"
QT_MOC_LITERAL(1, 18, 11), // "QML.Element"
QT_MOC_LITERAL(2, 30, 11), // "Application"
QT_MOC_LITERAL(3, 42, 13), // "QML.Creatable"
QT_MOC_LITERAL(4, 56, 5), // "false"
QT_MOC_LITERAL(5, 62, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(6, 84, 33), // "Application is an abstract cl..."
QT_MOC_LITERAL(7, 118, 13), // "activeChanged"
QT_MOC_LITERAL(8, 132, 0), // ""
QT_MOC_LITERAL(9, 133, 18), // "displayNameChanged"
QT_MOC_LITERAL(10, 152, 22), // "layoutDirectionChanged"
QT_MOC_LITERAL(11, 175, 12), // "stateChanged"
QT_MOC_LITERAL(12, 188, 20), // "Qt::ApplicationState"
QT_MOC_LITERAL(13, 209, 5), // "state"
QT_MOC_LITERAL(14, 215, 14), // "screensChanged"
QT_MOC_LITERAL(15, 230, 13), // "updateScreens"
QT_MOC_LITERAL(16, 244, 6), // "active"
QT_MOC_LITERAL(17, 251, 15), // "layoutDirection"
QT_MOC_LITERAL(18, 267, 19), // "Qt::LayoutDirection"
QT_MOC_LITERAL(19, 287, 23), // "supportsMultipleWindows"
QT_MOC_LITERAL(20, 311, 4), // "font"
QT_MOC_LITERAL(21, 316, 11), // "displayName"
QT_MOC_LITERAL(22, 328, 7), // "screens"
QT_MOC_LITERAL(23, 336, 34) // "QQmlListProperty<QQuickScreen..."

    },
    "QQuickApplication\0QML.Element\0Application\0"
    "QML.Creatable\0false\0QML.UncreatableReason\0"
    "Application is an abstract class.\0"
    "activeChanged\0\0displayNameChanged\0"
    "layoutDirectionChanged\0stateChanged\0"
    "Qt::ApplicationState\0state\0screensChanged\0"
    "updateScreens\0active\0layoutDirection\0"
    "Qt::LayoutDirection\0supportsMultipleWindows\0"
    "font\0displayName\0screens\0"
    "QQmlListProperty<QQuickScreenInfo>"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickApplication[] = {

 // content:
       8,       // revision
       0,       // classname
       3,   14, // classinfo
       6,   20, // methods
       7,   58, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,

 // signals: name, argc, parameters, tag, flags
       7,    0,   50,    8, 0x06 /* Public */,
       9,    0,   51,    8, 0x06 /* Public */,
      10,    0,   52,    8, 0x06 /* Public */,
      11,    1,   53,    8, 0x06 /* Public */,
      14,    0,   56,    8, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      15,    0,   57,    8, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,

 // properties: name, type, flags
      16, QMetaType::Bool, 0x00495001,
      17, 0x80000000 | 18, 0x00495009,
      19, QMetaType::Bool, 0x00095401,
      13, 0x80000000 | 12, 0x00495009,
      20, QMetaType::QFont, 0x00095401,
      21, QMetaType::QString, 0x00495103,
      22, 0x80000000 | 23, 0x00495009,

 // properties: notify_signal_id
       0,
       2,
       0,
       3,
       0,
       1,
       4,

       0        // eod
};

void QQuickApplication::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickApplication *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->activeChanged(); break;
        case 1: _t->displayNameChanged(); break;
        case 2: _t->layoutDirectionChanged(); break;
        case 3: _t->stateChanged((*reinterpret_cast< Qt::ApplicationState(*)>(_a[1]))); break;
        case 4: _t->screensChanged(); break;
        case 5: _t->updateScreens(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickApplication::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickApplication::activeChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickApplication::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickApplication::displayNameChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickApplication::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickApplication::layoutDirectionChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickApplication::*)(Qt::ApplicationState );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickApplication::stateChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickApplication::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickApplication::screensChanged)) {
                *result = 4;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 6:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlListProperty<QQuickScreenInfo> >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickApplication *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->active(); break;
        case 1: *reinterpret_cast< Qt::LayoutDirection*>(_v) = _t->layoutDirection(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->supportsMultipleWindows(); break;
        case 3: *reinterpret_cast< Qt::ApplicationState*>(_v) = _t->state(); break;
        case 4: *reinterpret_cast< QFont*>(_v) = _t->font(); break;
        case 5: *reinterpret_cast< QString*>(_v) = _t->displayName(); break;
        case 6: *reinterpret_cast< QQmlListProperty<QQuickScreenInfo>*>(_v) = _t->screens(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickApplication *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 5: _t->setDisplayName(*reinterpret_cast< QString*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickApplication::staticMetaObject = { {
    QMetaObject::SuperData::link<QQmlApplication::staticMetaObject>(),
    qt_meta_stringdata_QQuickApplication.data,
    qt_meta_data_QQuickApplication,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickApplication::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickApplication::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickApplication.stringdata0))
        return static_cast<void*>(this);
    return QQmlApplication::qt_metacast(_clname);
}

int QQuickApplication::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQmlApplication::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 7;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickApplication::activeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickApplication::displayNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickApplication::layoutDirectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickApplication::stateChanged(Qt::ApplicationState _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QQuickApplication::screensChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
