/****************************************************************************
** Meta object code from reading C++ file 'qquickboundaryrule_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../kde-qtdeclarative/src/imports/labsanimation/qquickboundaryrule_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickboundaryrule_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickBoundaryRule_t {
    QByteArrayData data[32];
    char stringdata0[479];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickBoundaryRule_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickBoundaryRule_t qt_meta_stringdata_QQuickBoundaryRule = {
    {
QT_MOC_LITERAL(0, 0, 18), // "QQuickBoundaryRule"
QT_MOC_LITERAL(1, 19, 11), // "QML.Element"
QT_MOC_LITERAL(2, 31, 12), // "BoundaryRule"
QT_MOC_LITERAL(3, 44, 14), // "enabledChanged"
QT_MOC_LITERAL(4, 59, 0), // ""
QT_MOC_LITERAL(5, 60, 14), // "minimumChanged"
QT_MOC_LITERAL(6, 75, 23), // "minimumOvershootChanged"
QT_MOC_LITERAL(7, 99, 14), // "maximumChanged"
QT_MOC_LITERAL(8, 114, 23), // "maximumOvershootChanged"
QT_MOC_LITERAL(9, 138, 21), // "overshootScaleChanged"
QT_MOC_LITERAL(10, 160, 23), // "currentOvershootChanged"
QT_MOC_LITERAL(11, 184, 20), // "peakOvershootChanged"
QT_MOC_LITERAL(12, 205, 22), // "overshootFilterChanged"
QT_MOC_LITERAL(13, 228, 13), // "easingChanged"
QT_MOC_LITERAL(14, 242, 21), // "returnDurationChanged"
QT_MOC_LITERAL(15, 264, 18), // "componentFinalized"
QT_MOC_LITERAL(16, 283, 14), // "returnToBounds"
QT_MOC_LITERAL(17, 298, 7), // "enabled"
QT_MOC_LITERAL(18, 306, 7), // "minimum"
QT_MOC_LITERAL(19, 314, 16), // "minimumOvershoot"
QT_MOC_LITERAL(20, 331, 7), // "maximum"
QT_MOC_LITERAL(21, 339, 16), // "maximumOvershoot"
QT_MOC_LITERAL(22, 356, 14), // "overshootScale"
QT_MOC_LITERAL(23, 371, 16), // "currentOvershoot"
QT_MOC_LITERAL(24, 388, 13), // "peakOvershoot"
QT_MOC_LITERAL(25, 402, 15), // "overshootFilter"
QT_MOC_LITERAL(26, 418, 15), // "OvershootFilter"
QT_MOC_LITERAL(27, 434, 6), // "easing"
QT_MOC_LITERAL(28, 441, 12), // "QEasingCurve"
QT_MOC_LITERAL(29, 454, 14), // "returnDuration"
QT_MOC_LITERAL(30, 469, 4), // "None"
QT_MOC_LITERAL(31, 474, 4) // "Peak"

    },
    "QQuickBoundaryRule\0QML.Element\0"
    "BoundaryRule\0enabledChanged\0\0"
    "minimumChanged\0minimumOvershootChanged\0"
    "maximumChanged\0maximumOvershootChanged\0"
    "overshootScaleChanged\0currentOvershootChanged\0"
    "peakOvershootChanged\0overshootFilterChanged\0"
    "easingChanged\0returnDurationChanged\0"
    "componentFinalized\0returnToBounds\0"
    "enabled\0minimum\0minimumOvershoot\0"
    "maximum\0maximumOvershoot\0overshootScale\0"
    "currentOvershoot\0peakOvershoot\0"
    "overshootFilter\0OvershootFilter\0easing\0"
    "QEasingCurve\0returnDuration\0None\0Peak"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickBoundaryRule[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      13,   16, // methods
      11,   94, // properties
       1,  138, // enums/sets
       0,    0, // constructors
       0,       // flags
      11,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   81,    4, 0x06 /* Public */,
       5,    0,   82,    4, 0x06 /* Public */,
       6,    0,   83,    4, 0x06 /* Public */,
       7,    0,   84,    4, 0x06 /* Public */,
       8,    0,   85,    4, 0x06 /* Public */,
       9,    0,   86,    4, 0x06 /* Public */,
      10,    0,   87,    4, 0x06 /* Public */,
      11,    0,   88,    4, 0x06 /* Public */,
      12,    0,   89,    4, 0x06 /* Public */,
      13,    0,   90,    4, 0x06 /* Public */,
      14,    0,   91,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      15,    0,   92,    4, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
      16,    0,   93,    4, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,

 // methods: parameters
    QMetaType::Bool,

 // properties: name, type, flags
      17, QMetaType::Bool, 0x00495103,
      18, QMetaType::QReal, 0x00495103,
      19, QMetaType::QReal, 0x00495103,
      20, QMetaType::QReal, 0x00495103,
      21, QMetaType::QReal, 0x00495103,
      22, QMetaType::QReal, 0x00495103,
      23, QMetaType::QReal, 0x00495001,
      24, QMetaType::QReal, 0x00495001,
      25, 0x80000000 | 26, 0x0049510b,
      27, 0x80000000 | 28, 0x0049510b,
      29, QMetaType::Int, 0x00495103,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,
       5,
       6,
       7,
       8,
       9,
      10,

 // enums: name, alias, flags, count, data
      26,   26, 0x0,    2,  143,

 // enum data: key, value
      30, uint(QQuickBoundaryRule::None),
      31, uint(QQuickBoundaryRule::Peak),

       0        // eod
};

void QQuickBoundaryRule::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickBoundaryRule *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->enabledChanged(); break;
        case 1: _t->minimumChanged(); break;
        case 2: _t->minimumOvershootChanged(); break;
        case 3: _t->maximumChanged(); break;
        case 4: _t->maximumOvershootChanged(); break;
        case 5: _t->overshootScaleChanged(); break;
        case 6: _t->currentOvershootChanged(); break;
        case 7: _t->peakOvershootChanged(); break;
        case 8: _t->overshootFilterChanged(); break;
        case 9: _t->easingChanged(); break;
        case 10: _t->returnDurationChanged(); break;
        case 11: _t->componentFinalized(); break;
        case 12: { bool _r = _t->returnToBounds();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickBoundaryRule::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickBoundaryRule::enabledChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickBoundaryRule::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickBoundaryRule::minimumChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickBoundaryRule::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickBoundaryRule::minimumOvershootChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickBoundaryRule::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickBoundaryRule::maximumChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickBoundaryRule::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickBoundaryRule::maximumOvershootChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickBoundaryRule::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickBoundaryRule::overshootScaleChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickBoundaryRule::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickBoundaryRule::currentOvershootChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickBoundaryRule::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickBoundaryRule::peakOvershootChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickBoundaryRule::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickBoundaryRule::overshootFilterChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickBoundaryRule::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickBoundaryRule::easingChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QQuickBoundaryRule::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickBoundaryRule::returnDurationChanged)) {
                *result = 10;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickBoundaryRule *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->enabled(); break;
        case 1: *reinterpret_cast< qreal*>(_v) = _t->minimum(); break;
        case 2: *reinterpret_cast< qreal*>(_v) = _t->minimumOvershoot(); break;
        case 3: *reinterpret_cast< qreal*>(_v) = _t->maximum(); break;
        case 4: *reinterpret_cast< qreal*>(_v) = _t->maximumOvershoot(); break;
        case 5: *reinterpret_cast< qreal*>(_v) = _t->overshootScale(); break;
        case 6: *reinterpret_cast< qreal*>(_v) = _t->currentOvershoot(); break;
        case 7: *reinterpret_cast< qreal*>(_v) = _t->peakOvershoot(); break;
        case 8: *reinterpret_cast< OvershootFilter*>(_v) = _t->overshootFilter(); break;
        case 9: *reinterpret_cast< QEasingCurve*>(_v) = _t->easing(); break;
        case 10: *reinterpret_cast< int*>(_v) = _t->returnDuration(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickBoundaryRule *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setEnabled(*reinterpret_cast< bool*>(_v)); break;
        case 1: _t->setMinimum(*reinterpret_cast< qreal*>(_v)); break;
        case 2: _t->setMinimumOvershoot(*reinterpret_cast< qreal*>(_v)); break;
        case 3: _t->setMaximum(*reinterpret_cast< qreal*>(_v)); break;
        case 4: _t->setMaximumOvershoot(*reinterpret_cast< qreal*>(_v)); break;
        case 5: _t->setOvershootScale(*reinterpret_cast< qreal*>(_v)); break;
        case 8: _t->setOvershootFilter(*reinterpret_cast< OvershootFilter*>(_v)); break;
        case 9: _t->setEasing(*reinterpret_cast< QEasingCurve*>(_v)); break;
        case 10: _t->setReturnDuration(*reinterpret_cast< int*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickBoundaryRule::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickBoundaryRule.data,
    qt_meta_data_QQuickBoundaryRule,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickBoundaryRule::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickBoundaryRule::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickBoundaryRule.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlPropertyValueInterceptor"))
        return static_cast< QQmlPropertyValueInterceptor*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlPropertyValueInterceptor"))
        return static_cast< QQmlPropertyValueInterceptor*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickBoundaryRule::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 11;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickBoundaryRule::enabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickBoundaryRule::minimumChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickBoundaryRule::minimumOvershootChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickBoundaryRule::maximumChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickBoundaryRule::maximumOvershootChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickBoundaryRule::overshootScaleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickBoundaryRule::currentOvershootChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQuickBoundaryRule::peakOvershootChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QQuickBoundaryRule::overshootFilterChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QQuickBoundaryRule::easingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QQuickBoundaryRule::returnDurationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
