/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>


void qml_register_types_Qt_labs_sharedimage()
{
    qmlRegisterModule("Qt.labs.sharedimage", 1, 0);
}

static const QQmlModuleRegistration registration("Qt.labs.sharedimage", 1, qml_register_types_Qt_labs_sharedimage);
