/****************************************************************************
** Meta object code from reading C++ file 'qquickpointerhandler_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/quick/handlers/qquickpointerhandler_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickpointerhandler_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickPointerHandler_t {
    QByteArrayData data[45];
    char stringdata0[792];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickPointerHandler_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickPointerHandler_t qt_meta_stringdata_QQuickPointerHandler = {
    {
QT_MOC_LITERAL(0, 0, 20), // "QQuickPointerHandler"
QT_MOC_LITERAL(1, 21, 11), // "QML.Element"
QT_MOC_LITERAL(2, 33, 14), // "PointerHandler"
QT_MOC_LITERAL(3, 48, 13), // "QML.Creatable"
QT_MOC_LITERAL(4, 62, 5), // "false"
QT_MOC_LITERAL(5, 68, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(6, 90, 41), // "PointerHandler is an abstract..."
QT_MOC_LITERAL(7, 132, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(8, 156, 2), // "12"
QT_MOC_LITERAL(9, 159, 14), // "enabledChanged"
QT_MOC_LITERAL(10, 174, 0), // ""
QT_MOC_LITERAL(11, 175, 13), // "activeChanged"
QT_MOC_LITERAL(12, 189, 13), // "targetChanged"
QT_MOC_LITERAL(13, 203, 13), // "marginChanged"
QT_MOC_LITERAL(14, 217, 20), // "dragThresholdChanged"
QT_MOC_LITERAL(15, 238, 11), // "grabChanged"
QT_MOC_LITERAL(16, 250, 32), // "QQuickEventPoint::GrabTransition"
QT_MOC_LITERAL(17, 283, 10), // "transition"
QT_MOC_LITERAL(18, 294, 17), // "QQuickEventPoint*"
QT_MOC_LITERAL(19, 312, 5), // "point"
QT_MOC_LITERAL(20, 318, 21), // "grabPermissionChanged"
QT_MOC_LITERAL(21, 340, 8), // "canceled"
QT_MOC_LITERAL(22, 349, 18), // "cursorShapeChanged"
QT_MOC_LITERAL(23, 368, 7), // "enabled"
QT_MOC_LITERAL(24, 376, 6), // "active"
QT_MOC_LITERAL(25, 383, 6), // "target"
QT_MOC_LITERAL(26, 390, 11), // "QQuickItem*"
QT_MOC_LITERAL(27, 402, 6), // "parent"
QT_MOC_LITERAL(28, 409, 15), // "grabPermissions"
QT_MOC_LITERAL(29, 425, 15), // "GrabPermissions"
QT_MOC_LITERAL(30, 441, 6), // "margin"
QT_MOC_LITERAL(31, 448, 13), // "dragThreshold"
QT_MOC_LITERAL(32, 462, 11), // "cursorShape"
QT_MOC_LITERAL(33, 474, 15), // "Qt::CursorShape"
QT_MOC_LITERAL(34, 490, 14), // "GrabPermission"
QT_MOC_LITERAL(35, 505, 17), // "TakeOverForbidden"
QT_MOC_LITERAL(36, 523, 33), // "CanTakeOverFromHandlersOfSame..."
QT_MOC_LITERAL(37, 557, 38), // "CanTakeOverFromHandlersOfDiff..."
QT_MOC_LITERAL(38, 596, 20), // "CanTakeOverFromItems"
QT_MOC_LITERAL(39, 617, 23), // "CanTakeOverFromAnything"
QT_MOC_LITERAL(40, 641, 36), // "ApprovesTakeOverByHandlersOfS..."
QT_MOC_LITERAL(41, 678, 41), // "ApprovesTakeOverByHandlersOfD..."
QT_MOC_LITERAL(42, 720, 23), // "ApprovesTakeOverByItems"
QT_MOC_LITERAL(43, 744, 20), // "ApprovesCancellation"
QT_MOC_LITERAL(44, 765, 26) // "ApprovesTakeOverByAnything"

    },
    "QQuickPointerHandler\0QML.Element\0"
    "PointerHandler\0QML.Creatable\0false\0"
    "QML.UncreatableReason\0"
    "PointerHandler is an abstract base class.\0"
    "QML.AddedInMinorVersion\0""12\0enabledChanged\0"
    "\0activeChanged\0targetChanged\0marginChanged\0"
    "dragThresholdChanged\0grabChanged\0"
    "QQuickEventPoint::GrabTransition\0"
    "transition\0QQuickEventPoint*\0point\0"
    "grabPermissionChanged\0canceled\0"
    "cursorShapeChanged\0enabled\0active\0"
    "target\0QQuickItem*\0parent\0grabPermissions\0"
    "GrabPermissions\0margin\0dragThreshold\0"
    "cursorShape\0Qt::CursorShape\0GrabPermission\0"
    "TakeOverForbidden\0CanTakeOverFromHandlersOfSameType\0"
    "CanTakeOverFromHandlersOfDifferentType\0"
    "CanTakeOverFromItems\0CanTakeOverFromAnything\0"
    "ApprovesTakeOverByHandlersOfSameType\0"
    "ApprovesTakeOverByHandlersOfDifferentType\0"
    "ApprovesTakeOverByItems\0ApprovesCancellation\0"
    "ApprovesTakeOverByAnything"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickPointerHandler[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
       9,   22, // methods
       8,   91, // properties
       1,  131, // enums/sets
       0,    0, // constructors
       0,       // flags
       9,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,

 // signals: name, argc, parameters, tag, flags
       9,    0,   76,   10, 0x06 /* Public */,
      11,    0,   77,   10, 0x06 /* Public */,
      12,    0,   78,   10, 0x06 /* Public */,
      13,    0,   79,   10, 0x06 /* Public */,
      14,    0,   80,   10, 0x86 /* Public | MethodRevisioned */,
      15,    2,   81,   10, 0x06 /* Public */,
      20,    0,   86,   10, 0x06 /* Public */,
      21,    1,   87,   10, 0x06 /* Public */,
      22,    0,   90,   10, 0x86 /* Public | MethodRevisioned */,

 // signals: revision
       0,
       0,
       0,
       0,
      15,
       0,
       0,
       0,
      15,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 16, 0x80000000 | 18,   17,   19,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 18,   19,
    QMetaType::Void,

 // properties: name, type, flags
      23, QMetaType::Bool, 0x00495103,
      24, QMetaType::Bool, 0x00495001,
      25, 0x80000000 | 26, 0x0049510b,
      27, 0x80000000 | 26, 0x00095409,
      28, 0x80000000 | 29, 0x0049510b,
      30, QMetaType::QReal, 0x00495103,
      31, QMetaType::Int, 0x00c95107,
      32, 0x80000000 | 33, 0x00c9510f,

 // properties: notify_signal_id
       0,
       1,
       2,
       0,
       6,
       3,
       4,
       8,

 // properties: revision
       0,
       0,
       0,
       0,
       0,
       0,
      15,
      15,

 // enums: name, alias, flags, count, data
      29,   34, 0x1,   10,  136,

 // enum data: key, value
      35, uint(QQuickPointerHandler::TakeOverForbidden),
      36, uint(QQuickPointerHandler::CanTakeOverFromHandlersOfSameType),
      37, uint(QQuickPointerHandler::CanTakeOverFromHandlersOfDifferentType),
      38, uint(QQuickPointerHandler::CanTakeOverFromItems),
      39, uint(QQuickPointerHandler::CanTakeOverFromAnything),
      40, uint(QQuickPointerHandler::ApprovesTakeOverByHandlersOfSameType),
      41, uint(QQuickPointerHandler::ApprovesTakeOverByHandlersOfDifferentType),
      42, uint(QQuickPointerHandler::ApprovesTakeOverByItems),
      43, uint(QQuickPointerHandler::ApprovesCancellation),
      44, uint(QQuickPointerHandler::ApprovesTakeOverByAnything),

       0        // eod
};

void QQuickPointerHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickPointerHandler *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->enabledChanged(); break;
        case 1: _t->activeChanged(); break;
        case 2: _t->targetChanged(); break;
        case 3: _t->marginChanged(); break;
        case 4: _t->dragThresholdChanged(); break;
        case 5: _t->grabChanged((*reinterpret_cast< QQuickEventPoint::GrabTransition(*)>(_a[1])),(*reinterpret_cast< QQuickEventPoint*(*)>(_a[2]))); break;
        case 6: _t->grabPermissionChanged(); break;
        case 7: _t->canceled((*reinterpret_cast< QQuickEventPoint*(*)>(_a[1]))); break;
        case 8: _t->cursorShapeChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickEventPoint* >(); break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickEventPoint::GrabTransition >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickEventPoint* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickPointerHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPointerHandler::enabledChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickPointerHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPointerHandler::activeChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickPointerHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPointerHandler::targetChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickPointerHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPointerHandler::marginChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickPointerHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPointerHandler::dragThresholdChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickPointerHandler::*)(QQuickEventPoint::GrabTransition , QQuickEventPoint * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPointerHandler::grabChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickPointerHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPointerHandler::grabPermissionChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickPointerHandler::*)(QQuickEventPoint * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPointerHandler::canceled)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickPointerHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPointerHandler::cursorShapeChanged)) {
                *result = 8;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickPointerHandler *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->enabled(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->active(); break;
        case 2: *reinterpret_cast< QQuickItem**>(_v) = _t->target(); break;
        case 3: *reinterpret_cast< QQuickItem**>(_v) = _t->parentItem(); break;
        case 4: *reinterpret_cast<int*>(_v) = QFlag(_t->grabPermissions()); break;
        case 5: *reinterpret_cast< qreal*>(_v) = _t->margin(); break;
        case 6: *reinterpret_cast< int*>(_v) = _t->dragThreshold(); break;
        case 7: *reinterpret_cast< Qt::CursorShape*>(_v) = _t->cursorShape(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickPointerHandler *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setEnabled(*reinterpret_cast< bool*>(_v)); break;
        case 2: _t->setTarget(*reinterpret_cast< QQuickItem**>(_v)); break;
        case 4: _t->setGrabPermissions(QFlag(*reinterpret_cast<int*>(_v))); break;
        case 5: _t->setMargin(*reinterpret_cast< qreal*>(_v)); break;
        case 6: _t->setDragThreshold(*reinterpret_cast< int*>(_v)); break;
        case 7: _t->setCursorShape(*reinterpret_cast< Qt::CursorShape*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        QQuickPointerHandler *_t = static_cast<QQuickPointerHandler *>(_o);
        (void)_t;
        switch (_id) {
        case 6: _t->resetDragThreshold(); break;
        case 7: _t->resetCursorShape(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickPointerHandler::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickPointerHandler.data,
    qt_meta_data_QQuickPointerHandler,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickPointerHandler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickPointerHandler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickPointerHandler.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickPointerHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 8;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickPointerHandler::enabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickPointerHandler::activeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickPointerHandler::targetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickPointerHandler::marginChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickPointerHandler::dragThresholdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickPointerHandler::grabChanged(QQuickEventPoint::GrabTransition _t1, QQuickEventPoint * _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QQuickPointerHandler::grabPermissionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQuickPointerHandler::canceled(QQuickEventPoint * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QQuickPointerHandler::cursorShapeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
