/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#include <private/qqmlabstractdelegatecomponent_p.h>
#include <private/qqmldelegatemodel_p.h>
#include <private/qqmlinstantiator_p.h>
#include <private/qqmllistmodel_p.h>
#include <private/qqmllistmodelworkeragent_p.h>
#include <private/qqmlmodelsmodule_p.h>
#include <private/qqmlobjectmodel_p.h>
#include <private/qquickpackage_p.h>

void qml_register_types_QtQml_Models()
{
    qmlRegisterTypesAndRevisions<QItemSelectionModelForeign>("QtQml.Models", 2);
    qmlRegisterTypesAndRevisions<QQmlAbstractDelegateComponent>("QtQml.Models", 2);
    qmlRegisterTypesAndRevisions<QQmlDelegateModel>("QtQml.Models", 2);
    qmlRegisterTypesAndRevisions<QQmlDelegateModelGroup>("QtQml.Models", 2);
    qmlRegisterTypesAndRevisions<QQmlInstanceModel>("QtQml.Models", 2);
    qmlRegisterTypesAndRevisions<QQmlInstantiator>("QtQml.Models", 2);
    qmlRegisterTypesAndRevisions<QQmlListElement>("QtQml.Models", 2);
    qmlRegisterTypesAndRevisions<QQmlListModel>("QtQml.Models", 2);
    qmlRegisterTypesAndRevisions<QQmlListModelWorkerAgent>("QtQml.Models", 2);
    qmlRegisterTypesAndRevisions<QQmlObjectModel>("QtQml.Models", 2);
    qmlRegisterTypesAndRevisions<QQuickPackage>("QtQml.Models", 2);
    qmlRegisterModule("QtQml.Models", 2, 15);
}

static const QQmlModuleRegistration registration("QtQml.Models", 2, qml_register_types_QtQml_Models);
