/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#include <qquicklocalstorage_p.h>

void qml_register_types_QtQuick_LocalStorage()
{
    qmlRegisterTypesAndRevisions<QQuickLocalStorage>("QtQuick.LocalStorage", 2);
    qmlRegisterModule("QtQuick.LocalStorage", 2, 15);
}

static const QQmlModuleRegistration registration("QtQuick.LocalStorage", 2, qml_register_types_QtQuick_LocalStorage);
