/****************************************************************************
** Meta object code from reading C++ file 'qquickdraghandler_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/quick/handlers/qquickdraghandler_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickdraghandler_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickDragHandler_t {
    QByteArrayData data[18];
    char stringdata0[217];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickDragHandler_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickDragHandler_t qt_meta_stringdata_QQuickDragHandler = {
    {
QT_MOC_LITERAL(0, 0, 17), // "QQuickDragHandler"
QT_MOC_LITERAL(1, 18, 11), // "QML.Element"
QT_MOC_LITERAL(2, 30, 11), // "DragHandler"
QT_MOC_LITERAL(3, 42, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(4, 66, 2), // "12"
QT_MOC_LITERAL(5, 69, 18), // "translationChanged"
QT_MOC_LITERAL(6, 88, 0), // ""
QT_MOC_LITERAL(7, 89, 15), // "snapModeChanged"
QT_MOC_LITERAL(8, 105, 5), // "xAxis"
QT_MOC_LITERAL(9, 111, 15), // "QQuickDragAxis*"
QT_MOC_LITERAL(10, 127, 5), // "yAxis"
QT_MOC_LITERAL(11, 133, 11), // "translation"
QT_MOC_LITERAL(12, 145, 8), // "snapMode"
QT_MOC_LITERAL(13, 154, 8), // "SnapMode"
QT_MOC_LITERAL(14, 163, 6), // "NoSnap"
QT_MOC_LITERAL(15, 170, 8), // "SnapAuto"
QT_MOC_LITERAL(16, 179, 26), // "SnapIfPressedOutsideTarget"
QT_MOC_LITERAL(17, 206, 10) // "SnapAlways"

    },
    "QQuickDragHandler\0QML.Element\0DragHandler\0"
    "QML.AddedInMinorVersion\0""12\0"
    "translationChanged\0\0snapModeChanged\0"
    "xAxis\0QQuickDragAxis*\0yAxis\0translation\0"
    "snapMode\0SnapMode\0NoSnap\0SnapAuto\0"
    "SnapIfPressedOutsideTarget\0SnapAlways"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickDragHandler[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       2,   18, // methods
       4,   32, // properties
       1,   52, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

 // signals: name, argc, parameters, tag, flags
       5,    0,   30,    6, 0x06 /* Public */,
       7,    0,   31,    6, 0x86 /* Public | MethodRevisioned */,

 // signals: revision
       0,
      14,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
       8, 0x80000000 | 9, 0x00095409,
      10, 0x80000000 | 9, 0x00095409,
      11, QMetaType::QVector2D, 0x00495001,
      12, 0x80000000 | 13, 0x00c9510b,

 // properties: notify_signal_id
       0,
       0,
       0,
       1,

 // properties: revision
       0,
       0,
       0,
      14,

 // enums: name, alias, flags, count, data
      13,   13, 0x0,    4,   57,

 // enum data: key, value
      14, uint(QQuickDragHandler::NoSnap),
      15, uint(QQuickDragHandler::SnapAuto),
      16, uint(QQuickDragHandler::SnapIfPressedOutsideTarget),
      17, uint(QQuickDragHandler::SnapAlways),

       0        // eod
};

void QQuickDragHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickDragHandler *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->translationChanged(); break;
        case 1: _t->snapModeChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickDragHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDragHandler::translationChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickDragHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDragHandler::snapModeChanged)) {
                *result = 1;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickDragAxis* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickDragHandler *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQuickDragAxis**>(_v) = _t->xAxis(); break;
        case 1: *reinterpret_cast< QQuickDragAxis**>(_v) = _t->yAxis(); break;
        case 2: *reinterpret_cast< QVector2D*>(_v) = _t->translation(); break;
        case 3: *reinterpret_cast< SnapMode*>(_v) = _t->snapMode(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickDragHandler *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 3: _t->setSnapMode(*reinterpret_cast< SnapMode*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickDragHandler::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickMultiPointHandler::staticMetaObject>(),
    qt_meta_stringdata_QQuickDragHandler.data,
    qt_meta_data_QQuickDragHandler,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickDragHandler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickDragHandler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickDragHandler.stringdata0))
        return static_cast<void*>(this);
    return QQuickMultiPointHandler::qt_metacast(_clname);
}

int QQuickDragHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickMultiPointHandler::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickDragHandler::translationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickDragHandler::snapModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
