/****************************************************************************
** Meta object code from reading C++ file 'qquickdragaxis_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/quick/handlers/qquickdragaxis_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickdragaxis_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickDragAxis_t {
    QByteArrayData data[16];
    char stringdata0[240];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickDragAxis_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickDragAxis_t qt_meta_stringdata_QQuickDragAxis = {
    {
QT_MOC_LITERAL(0, 0, 14), // "QQuickDragAxis"
QT_MOC_LITERAL(1, 15, 11), // "QML.Element"
QT_MOC_LITERAL(2, 27, 8), // "DragAxis"
QT_MOC_LITERAL(3, 36, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(4, 60, 2), // "12"
QT_MOC_LITERAL(5, 63, 13), // "QML.Creatable"
QT_MOC_LITERAL(6, 77, 5), // "false"
QT_MOC_LITERAL(7, 83, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(8, 105, 64), // "DragAxis is only available as..."
QT_MOC_LITERAL(9, 170, 14), // "minimumChanged"
QT_MOC_LITERAL(10, 185, 0), // ""
QT_MOC_LITERAL(11, 186, 14), // "maximumChanged"
QT_MOC_LITERAL(12, 201, 14), // "enabledChanged"
QT_MOC_LITERAL(13, 216, 7), // "minimum"
QT_MOC_LITERAL(14, 224, 7), // "maximum"
QT_MOC_LITERAL(15, 232, 7) // "enabled"

    },
    "QQuickDragAxis\0QML.Element\0DragAxis\0"
    "QML.AddedInMinorVersion\0""12\0QML.Creatable\0"
    "false\0QML.UncreatableReason\0"
    "DragAxis is only available as a grouped property of DragHandler.\0"
    "minimumChanged\0\0maximumChanged\0"
    "enabledChanged\0minimum\0maximum\0enabled"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickDragAxis[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
       3,   22, // methods
       3,   40, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,

 // signals: name, argc, parameters, tag, flags
       9,    0,   37,   10, 0x06 /* Public */,
      11,    0,   38,   10, 0x06 /* Public */,
      12,    0,   39,   10, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      13, QMetaType::QReal, 0x00495103,
      14, QMetaType::QReal, 0x00495103,
      15, QMetaType::Bool, 0x00495103,

 // properties: notify_signal_id
       0,
       1,
       2,

       0        // eod
};

void QQuickDragAxis::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickDragAxis *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->minimumChanged(); break;
        case 1: _t->maximumChanged(); break;
        case 2: _t->enabledChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickDragAxis::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDragAxis::minimumChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickDragAxis::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDragAxis::maximumChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickDragAxis::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickDragAxis::enabledChanged)) {
                *result = 2;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickDragAxis *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< qreal*>(_v) = _t->minimum(); break;
        case 1: *reinterpret_cast< qreal*>(_v) = _t->maximum(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->enabled(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickDragAxis *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setMinimum(*reinterpret_cast< qreal*>(_v)); break;
        case 1: _t->setMaximum(*reinterpret_cast< qreal*>(_v)); break;
        case 2: _t->setEnabled(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickDragAxis::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickDragAxis.data,
    qt_meta_data_QQuickDragAxis,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickDragAxis::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickDragAxis::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickDragAxis.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickDragAxis::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 3;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickDragAxis::minimumChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickDragAxis::maximumChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickDragAxis::enabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
