/****************************************************************************
** Meta object code from reading C++ file 'qquicktaphandler_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/quick/handlers/qquicktaphandler_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquicktaphandler_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickTapHandler_t {
    QByteArrayData data[26];
    char stringdata0[356];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickTapHandler_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickTapHandler_t qt_meta_stringdata_QQuickTapHandler = {
    {
QT_MOC_LITERAL(0, 0, 16), // "QQuickTapHandler"
QT_MOC_LITERAL(1, 17, 11), // "QML.Element"
QT_MOC_LITERAL(2, 29, 10), // "TapHandler"
QT_MOC_LITERAL(3, 40, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(4, 64, 2), // "12"
QT_MOC_LITERAL(5, 67, 14), // "pressedChanged"
QT_MOC_LITERAL(6, 82, 0), // ""
QT_MOC_LITERAL(7, 83, 15), // "tapCountChanged"
QT_MOC_LITERAL(8, 99, 15), // "timeHeldChanged"
QT_MOC_LITERAL(9, 115, 25), // "longPressThresholdChanged"
QT_MOC_LITERAL(10, 141, 20), // "gesturePolicyChanged"
QT_MOC_LITERAL(11, 162, 6), // "tapped"
QT_MOC_LITERAL(12, 169, 17), // "QQuickEventPoint*"
QT_MOC_LITERAL(13, 187, 10), // "eventPoint"
QT_MOC_LITERAL(14, 198, 12), // "singleTapped"
QT_MOC_LITERAL(15, 211, 12), // "doubleTapped"
QT_MOC_LITERAL(16, 224, 11), // "longPressed"
QT_MOC_LITERAL(17, 236, 7), // "pressed"
QT_MOC_LITERAL(18, 244, 8), // "tapCount"
QT_MOC_LITERAL(19, 253, 8), // "timeHeld"
QT_MOC_LITERAL(20, 262, 18), // "longPressThreshold"
QT_MOC_LITERAL(21, 281, 13), // "gesturePolicy"
QT_MOC_LITERAL(22, 295, 13), // "GesturePolicy"
QT_MOC_LITERAL(23, 309, 13), // "DragThreshold"
QT_MOC_LITERAL(24, 323, 12), // "WithinBounds"
QT_MOC_LITERAL(25, 336, 19) // "ReleaseWithinBounds"

    },
    "QQuickTapHandler\0QML.Element\0TapHandler\0"
    "QML.AddedInMinorVersion\0""12\0pressedChanged\0"
    "\0tapCountChanged\0timeHeldChanged\0"
    "longPressThresholdChanged\0"
    "gesturePolicyChanged\0tapped\0"
    "QQuickEventPoint*\0eventPoint\0singleTapped\0"
    "doubleTapped\0longPressed\0pressed\0"
    "tapCount\0timeHeld\0longPressThreshold\0"
    "gesturePolicy\0GesturePolicy\0DragThreshold\0"
    "WithinBounds\0ReleaseWithinBounds"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickTapHandler[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       9,   18, // methods
       5,   78, // properties
       1,   98, // enums/sets
       0,    0, // constructors
       0,       // flags
       9,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

 // signals: name, argc, parameters, tag, flags
       5,    0,   63,    6, 0x06 /* Public */,
       7,    0,   64,    6, 0x06 /* Public */,
       8,    0,   65,    6, 0x06 /* Public */,
       9,    0,   66,    6, 0x06 /* Public */,
      10,    0,   67,    6, 0x06 /* Public */,
      11,    1,   68,    6, 0x06 /* Public */,
      14,    1,   71,    6, 0x06 /* Public */,
      15,    1,   74,    6, 0x06 /* Public */,
      16,    0,   77,    6, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void,

 // properties: name, type, flags
      17, QMetaType::Bool, 0x00495001,
      18, QMetaType::Int, 0x00495001,
      19, QMetaType::QReal, 0x00495001,
      20, QMetaType::QReal, 0x00495103,
      21, 0x80000000 | 22, 0x0049510b,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,

 // enums: name, alias, flags, count, data
      22,   22, 0x0,    3,  103,

 // enum data: key, value
      23, uint(QQuickTapHandler::DragThreshold),
      24, uint(QQuickTapHandler::WithinBounds),
      25, uint(QQuickTapHandler::ReleaseWithinBounds),

       0        // eod
};

void QQuickTapHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickTapHandler *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->pressedChanged(); break;
        case 1: _t->tapCountChanged(); break;
        case 2: _t->timeHeldChanged(); break;
        case 3: _t->longPressThresholdChanged(); break;
        case 4: _t->gesturePolicyChanged(); break;
        case 5: _t->tapped((*reinterpret_cast< QQuickEventPoint*(*)>(_a[1]))); break;
        case 6: _t->singleTapped((*reinterpret_cast< QQuickEventPoint*(*)>(_a[1]))); break;
        case 7: _t->doubleTapped((*reinterpret_cast< QQuickEventPoint*(*)>(_a[1]))); break;
        case 8: _t->longPressed(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickEventPoint* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickEventPoint* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickEventPoint* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickTapHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTapHandler::pressedChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickTapHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTapHandler::tapCountChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickTapHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTapHandler::timeHeldChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickTapHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTapHandler::longPressThresholdChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickTapHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTapHandler::gesturePolicyChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickTapHandler::*)(QQuickEventPoint * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTapHandler::tapped)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickTapHandler::*)(QQuickEventPoint * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTapHandler::singleTapped)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickTapHandler::*)(QQuickEventPoint * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTapHandler::doubleTapped)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickTapHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTapHandler::longPressed)) {
                *result = 8;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickTapHandler *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->isPressed(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->tapCount(); break;
        case 2: *reinterpret_cast< qreal*>(_v) = _t->timeHeld(); break;
        case 3: *reinterpret_cast< qreal*>(_v) = _t->longPressThreshold(); break;
        case 4: *reinterpret_cast< GesturePolicy*>(_v) = _t->gesturePolicy(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickTapHandler *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 3: _t->setLongPressThreshold(*reinterpret_cast< qreal*>(_v)); break;
        case 4: _t->setGesturePolicy(*reinterpret_cast< GesturePolicy*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickTapHandler::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickSinglePointHandler::staticMetaObject>(),
    qt_meta_stringdata_QQuickTapHandler.data,
    qt_meta_data_QQuickTapHandler,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickTapHandler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickTapHandler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickTapHandler.stringdata0))
        return static_cast<void*>(this);
    return QQuickSinglePointHandler::qt_metacast(_clname);
}

int QQuickTapHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickSinglePointHandler::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickTapHandler::pressedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickTapHandler::tapCountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickTapHandler::timeHeldChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickTapHandler::longPressThresholdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickTapHandler::gesturePolicyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickTapHandler::tapped(QQuickEventPoint * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QQuickTapHandler::singleTapped(QQuickEventPoint * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QQuickTapHandler::doubleTapped(QQuickEventPoint * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QQuickTapHandler::longPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
