/****************************************************************************
** Meta object code from reading C++ file 'qquicktext_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/quick/items/qquicktext_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquicktext_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickText_t {
    QByteArrayData data[133];
    char stringdata0[1791];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickText_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickText_t qt_meta_stringdata_QQuickText = {
    {
QT_MOC_LITERAL(0, 0, 10), // "QQuickText"
QT_MOC_LITERAL(1, 11, 11), // "QML.Element"
QT_MOC_LITERAL(2, 23, 4), // "Text"
QT_MOC_LITERAL(3, 28, 11), // "textChanged"
QT_MOC_LITERAL(4, 40, 0), // ""
QT_MOC_LITERAL(5, 41, 4), // "text"
QT_MOC_LITERAL(6, 46, 13), // "linkActivated"
QT_MOC_LITERAL(7, 60, 4), // "link"
QT_MOC_LITERAL(8, 65, 11), // "linkHovered"
QT_MOC_LITERAL(9, 77, 11), // "fontChanged"
QT_MOC_LITERAL(10, 89, 4), // "font"
QT_MOC_LITERAL(11, 94, 12), // "colorChanged"
QT_MOC_LITERAL(12, 107, 16), // "linkColorChanged"
QT_MOC_LITERAL(13, 124, 12), // "styleChanged"
QT_MOC_LITERAL(14, 137, 21), // "QQuickText::TextStyle"
QT_MOC_LITERAL(15, 159, 5), // "style"
QT_MOC_LITERAL(16, 165, 17), // "styleColorChanged"
QT_MOC_LITERAL(17, 183, 26), // "horizontalAlignmentChanged"
QT_MOC_LITERAL(18, 210, 22), // "QQuickText::HAlignment"
QT_MOC_LITERAL(19, 233, 9), // "alignment"
QT_MOC_LITERAL(20, 243, 24), // "verticalAlignmentChanged"
QT_MOC_LITERAL(21, 268, 22), // "QQuickText::VAlignment"
QT_MOC_LITERAL(22, 291, 15), // "wrapModeChanged"
QT_MOC_LITERAL(23, 307, 16), // "lineCountChanged"
QT_MOC_LITERAL(24, 324, 16), // "truncatedChanged"
QT_MOC_LITERAL(25, 341, 23), // "maximumLineCountChanged"
QT_MOC_LITERAL(26, 365, 17), // "textFormatChanged"
QT_MOC_LITERAL(27, 383, 22), // "QQuickText::TextFormat"
QT_MOC_LITERAL(28, 406, 10), // "textFormat"
QT_MOC_LITERAL(29, 417, 16), // "elideModeChanged"
QT_MOC_LITERAL(30, 434, 25), // "QQuickText::TextElideMode"
QT_MOC_LITERAL(31, 460, 4), // "mode"
QT_MOC_LITERAL(32, 465, 18), // "contentSizeChanged"
QT_MOC_LITERAL(33, 484, 19), // "contentWidthChanged"
QT_MOC_LITERAL(34, 504, 12), // "contentWidth"
QT_MOC_LITERAL(35, 517, 20), // "contentHeightChanged"
QT_MOC_LITERAL(36, 538, 13), // "contentHeight"
QT_MOC_LITERAL(37, 552, 17), // "lineHeightChanged"
QT_MOC_LITERAL(38, 570, 10), // "lineHeight"
QT_MOC_LITERAL(39, 581, 21), // "lineHeightModeChanged"
QT_MOC_LITERAL(40, 603, 14), // "LineHeightMode"
QT_MOC_LITERAL(41, 618, 19), // "fontSizeModeChanged"
QT_MOC_LITERAL(42, 638, 23), // "minimumPixelSizeChanged"
QT_MOC_LITERAL(43, 662, 23), // "minimumPointSizeChanged"
QT_MOC_LITERAL(44, 686, 35), // "effectiveHorizontalAlignmentC..."
QT_MOC_LITERAL(45, 722, 11), // "lineLaidOut"
QT_MOC_LITERAL(46, 734, 15), // "QQuickTextLine*"
QT_MOC_LITERAL(47, 750, 4), // "line"
QT_MOC_LITERAL(48, 755, 14), // "baseUrlChanged"
QT_MOC_LITERAL(49, 770, 17), // "renderTypeChanged"
QT_MOC_LITERAL(50, 788, 14), // "paddingChanged"
QT_MOC_LITERAL(51, 803, 17), // "topPaddingChanged"
QT_MOC_LITERAL(52, 821, 18), // "leftPaddingChanged"
QT_MOC_LITERAL(53, 840, 19), // "rightPaddingChanged"
QT_MOC_LITERAL(54, 860, 20), // "bottomPaddingChanged"
QT_MOC_LITERAL(55, 881, 15), // "fontInfoChanged"
QT_MOC_LITERAL(56, 897, 14), // "q_updateLayout"
QT_MOC_LITERAL(57, 912, 17), // "triggerPreprocess"
QT_MOC_LITERAL(58, 930, 21), // "imageDownloadFinished"
QT_MOC_LITERAL(59, 952, 8), // "doLayout"
QT_MOC_LITERAL(60, 961, 11), // "forceLayout"
QT_MOC_LITERAL(61, 973, 6), // "linkAt"
QT_MOC_LITERAL(62, 980, 1), // "x"
QT_MOC_LITERAL(63, 982, 1), // "y"
QT_MOC_LITERAL(64, 984, 5), // "color"
QT_MOC_LITERAL(65, 990, 9), // "linkColor"
QT_MOC_LITERAL(66, 1000, 9), // "TextStyle"
QT_MOC_LITERAL(67, 1010, 10), // "styleColor"
QT_MOC_LITERAL(68, 1021, 19), // "horizontalAlignment"
QT_MOC_LITERAL(69, 1041, 10), // "HAlignment"
QT_MOC_LITERAL(70, 1052, 28), // "effectiveHorizontalAlignment"
QT_MOC_LITERAL(71, 1081, 17), // "verticalAlignment"
QT_MOC_LITERAL(72, 1099, 10), // "VAlignment"
QT_MOC_LITERAL(73, 1110, 8), // "wrapMode"
QT_MOC_LITERAL(74, 1119, 8), // "WrapMode"
QT_MOC_LITERAL(75, 1128, 9), // "lineCount"
QT_MOC_LITERAL(76, 1138, 9), // "truncated"
QT_MOC_LITERAL(77, 1148, 16), // "maximumLineCount"
QT_MOC_LITERAL(78, 1165, 10), // "TextFormat"
QT_MOC_LITERAL(79, 1176, 5), // "elide"
QT_MOC_LITERAL(80, 1182, 13), // "TextElideMode"
QT_MOC_LITERAL(81, 1196, 12), // "paintedWidth"
QT_MOC_LITERAL(82, 1209, 13), // "paintedHeight"
QT_MOC_LITERAL(83, 1223, 14), // "lineHeightMode"
QT_MOC_LITERAL(84, 1238, 7), // "baseUrl"
QT_MOC_LITERAL(85, 1246, 16), // "minimumPixelSize"
QT_MOC_LITERAL(86, 1263, 16), // "minimumPointSize"
QT_MOC_LITERAL(87, 1280, 12), // "fontSizeMode"
QT_MOC_LITERAL(88, 1293, 12), // "FontSizeMode"
QT_MOC_LITERAL(89, 1306, 10), // "renderType"
QT_MOC_LITERAL(90, 1317, 10), // "RenderType"
QT_MOC_LITERAL(91, 1328, 11), // "hoveredLink"
QT_MOC_LITERAL(92, 1340, 7), // "padding"
QT_MOC_LITERAL(93, 1348, 10), // "topPadding"
QT_MOC_LITERAL(94, 1359, 11), // "leftPadding"
QT_MOC_LITERAL(95, 1371, 12), // "rightPadding"
QT_MOC_LITERAL(96, 1384, 13), // "bottomPadding"
QT_MOC_LITERAL(97, 1398, 8), // "fontInfo"
QT_MOC_LITERAL(98, 1407, 8), // "QJSValue"
QT_MOC_LITERAL(99, 1416, 7), // "advance"
QT_MOC_LITERAL(100, 1424, 9), // "AlignLeft"
QT_MOC_LITERAL(101, 1434, 10), // "AlignRight"
QT_MOC_LITERAL(102, 1445, 12), // "AlignHCenter"
QT_MOC_LITERAL(103, 1458, 12), // "AlignJustify"
QT_MOC_LITERAL(104, 1471, 8), // "AlignTop"
QT_MOC_LITERAL(105, 1480, 11), // "AlignBottom"
QT_MOC_LITERAL(106, 1492, 12), // "AlignVCenter"
QT_MOC_LITERAL(107, 1505, 6), // "Normal"
QT_MOC_LITERAL(108, 1512, 7), // "Outline"
QT_MOC_LITERAL(109, 1520, 6), // "Raised"
QT_MOC_LITERAL(110, 1527, 6), // "Sunken"
QT_MOC_LITERAL(111, 1534, 9), // "PlainText"
QT_MOC_LITERAL(112, 1544, 8), // "RichText"
QT_MOC_LITERAL(113, 1553, 12), // "MarkdownText"
QT_MOC_LITERAL(114, 1566, 8), // "AutoText"
QT_MOC_LITERAL(115, 1575, 10), // "StyledText"
QT_MOC_LITERAL(116, 1586, 9), // "ElideLeft"
QT_MOC_LITERAL(117, 1596, 10), // "ElideRight"
QT_MOC_LITERAL(118, 1607, 11), // "ElideMiddle"
QT_MOC_LITERAL(119, 1619, 9), // "ElideNone"
QT_MOC_LITERAL(120, 1629, 6), // "NoWrap"
QT_MOC_LITERAL(121, 1636, 8), // "WordWrap"
QT_MOC_LITERAL(122, 1645, 12), // "WrapAnywhere"
QT_MOC_LITERAL(123, 1658, 28), // "WrapAtWordBoundaryOrAnywhere"
QT_MOC_LITERAL(124, 1687, 4), // "Wrap"
QT_MOC_LITERAL(125, 1692, 11), // "QtRendering"
QT_MOC_LITERAL(126, 1704, 15), // "NativeRendering"
QT_MOC_LITERAL(127, 1720, 18), // "ProportionalHeight"
QT_MOC_LITERAL(128, 1739, 11), // "FixedHeight"
QT_MOC_LITERAL(129, 1751, 9), // "FixedSize"
QT_MOC_LITERAL(130, 1761, 13), // "HorizontalFit"
QT_MOC_LITERAL(131, 1775, 11), // "VerticalFit"
QT_MOC_LITERAL(132, 1787, 3) // "Fit"

    },
    "QQuickText\0QML.Element\0Text\0textChanged\0"
    "\0text\0linkActivated\0link\0linkHovered\0"
    "fontChanged\0font\0colorChanged\0"
    "linkColorChanged\0styleChanged\0"
    "QQuickText::TextStyle\0style\0"
    "styleColorChanged\0horizontalAlignmentChanged\0"
    "QQuickText::HAlignment\0alignment\0"
    "verticalAlignmentChanged\0"
    "QQuickText::VAlignment\0wrapModeChanged\0"
    "lineCountChanged\0truncatedChanged\0"
    "maximumLineCountChanged\0textFormatChanged\0"
    "QQuickText::TextFormat\0textFormat\0"
    "elideModeChanged\0QQuickText::TextElideMode\0"
    "mode\0contentSizeChanged\0contentWidthChanged\0"
    "contentWidth\0contentHeightChanged\0"
    "contentHeight\0lineHeightChanged\0"
    "lineHeight\0lineHeightModeChanged\0"
    "LineHeightMode\0fontSizeModeChanged\0"
    "minimumPixelSizeChanged\0minimumPointSizeChanged\0"
    "effectiveHorizontalAlignmentChanged\0"
    "lineLaidOut\0QQuickTextLine*\0line\0"
    "baseUrlChanged\0renderTypeChanged\0"
    "paddingChanged\0topPaddingChanged\0"
    "leftPaddingChanged\0rightPaddingChanged\0"
    "bottomPaddingChanged\0fontInfoChanged\0"
    "q_updateLayout\0triggerPreprocess\0"
    "imageDownloadFinished\0doLayout\0"
    "forceLayout\0linkAt\0x\0y\0color\0linkColor\0"
    "TextStyle\0styleColor\0horizontalAlignment\0"
    "HAlignment\0effectiveHorizontalAlignment\0"
    "verticalAlignment\0VAlignment\0wrapMode\0"
    "WrapMode\0lineCount\0truncated\0"
    "maximumLineCount\0TextFormat\0elide\0"
    "TextElideMode\0paintedWidth\0paintedHeight\0"
    "lineHeightMode\0baseUrl\0minimumPixelSize\0"
    "minimumPointSize\0fontSizeMode\0"
    "FontSizeMode\0renderType\0RenderType\0"
    "hoveredLink\0padding\0topPadding\0"
    "leftPadding\0rightPadding\0bottomPadding\0"
    "fontInfo\0QJSValue\0advance\0AlignLeft\0"
    "AlignRight\0AlignHCenter\0AlignJustify\0"
    "AlignTop\0AlignBottom\0AlignVCenter\0"
    "Normal\0Outline\0Raised\0Sunken\0PlainText\0"
    "RichText\0MarkdownText\0AutoText\0"
    "StyledText\0ElideLeft\0ElideRight\0"
    "ElideMiddle\0ElideNone\0NoWrap\0WordWrap\0"
    "WrapAnywhere\0WrapAtWordBoundaryOrAnywhere\0"
    "Wrap\0QtRendering\0NativeRendering\0"
    "ProportionalHeight\0FixedHeight\0FixedSize\0"
    "HorizontalFit\0VerticalFit\0Fit"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickText[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      40,   16, // methods
      34,  328, // properties
       9,  498, // enums/sets
       0,    0, // constructors
       0,       // flags
      34,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    1,  256,    4, 0x06 /* Public */,
       6,    1,  259,    4, 0x06 /* Public */,
       8,    1,  262,    4, 0x86 /* Public | MethodRevisioned */,
       9,    1,  265,    4, 0x06 /* Public */,
      11,    0,  268,    4, 0x06 /* Public */,
      12,    0,  269,    4, 0x06 /* Public */,
      13,    1,  270,    4, 0x06 /* Public */,
      16,    0,  273,    4, 0x06 /* Public */,
      17,    1,  274,    4, 0x06 /* Public */,
      20,    1,  277,    4, 0x06 /* Public */,
      22,    0,  280,    4, 0x06 /* Public */,
      23,    0,  281,    4, 0x06 /* Public */,
      24,    0,  282,    4, 0x06 /* Public */,
      25,    0,  283,    4, 0x06 /* Public */,
      26,    1,  284,    4, 0x06 /* Public */,
      29,    1,  287,    4, 0x06 /* Public */,
      32,    0,  290,    4, 0x06 /* Public */,
      33,    1,  291,    4, 0x06 /* Public */,
      35,    1,  294,    4, 0x06 /* Public */,
      37,    1,  297,    4, 0x06 /* Public */,
      39,    1,  300,    4, 0x06 /* Public */,
      41,    0,  303,    4, 0x06 /* Public */,
      42,    0,  304,    4, 0x06 /* Public */,
      43,    0,  305,    4, 0x06 /* Public */,
      44,    0,  306,    4, 0x06 /* Public */,
      45,    1,  307,    4, 0x06 /* Public */,
      48,    0,  310,    4, 0x06 /* Public */,
      49,    0,  311,    4, 0x06 /* Public */,
      50,    0,  312,    4, 0x86 /* Public | MethodRevisioned */,
      51,    0,  313,    4, 0x86 /* Public | MethodRevisioned */,
      52,    0,  314,    4, 0x86 /* Public | MethodRevisioned */,
      53,    0,  315,    4, 0x86 /* Public | MethodRevisioned */,
      54,    0,  316,    4, 0x86 /* Public | MethodRevisioned */,
      55,    0,  317,    4, 0x86 /* Public | MethodRevisioned */,

 // slots: name, argc, parameters, tag, flags
      56,    0,  318,    4, 0x08 /* Private */,
      57,    0,  319,    4, 0x08 /* Private */,
      58,    0,  320,    4, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
      59,    0,  321,    4, 0x02 /* Public */,
      60,    0,  322,    4, 0x82 /* Public | MethodRevisioned */,
      61,    2,  323,    4, 0x82 /* Public | MethodRevisioned */,

 // signals: revision
       0,
       0,
       2,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       6,
       6,
       6,
       6,
       6,
       9,

 // slots: revision
       0,
       0,
       0,

 // methods: revision
       0,
       9,
       3,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void, QMetaType::QString,    7,
    QMetaType::Void, QMetaType::QString,    7,
    QMetaType::Void, QMetaType::QFont,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 14,   15,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 18,   19,
    QMetaType::Void, 0x80000000 | 21,   19,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 27,   28,
    QMetaType::Void, 0x80000000 | 30,   31,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QReal,   34,
    QMetaType::Void, QMetaType::QReal,   36,
    QMetaType::Void, QMetaType::QReal,   38,
    QMetaType::Void, 0x80000000 | 40,   31,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 46,   47,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::QString, QMetaType::QReal, QMetaType::QReal,   62,   63,

 // properties: name, type, flags
       5, QMetaType::QString, 0x00495103,
      10, QMetaType::QFont, 0x00495103,
      64, QMetaType::QColor, 0x00495103,
      65, QMetaType::QColor, 0x00495103,
      15, 0x80000000 | 66, 0x0049510b,
      67, QMetaType::QColor, 0x00495103,
      68, 0x80000000 | 69, 0x0049500f,
      70, 0x80000000 | 69, 0x00495009,
      71, 0x80000000 | 72, 0x0049500b,
      73, 0x80000000 | 74, 0x0049510b,
      75, QMetaType::Int, 0x00495001,
      76, QMetaType::Bool, 0x00495001,
      77, QMetaType::Int, 0x00495107,
      28, 0x80000000 | 78, 0x0049510b,
      79, 0x80000000 | 80, 0x0049500b,
      34, QMetaType::QReal, 0x00495001,
      36, QMetaType::QReal, 0x00495001,
      81, QMetaType::QReal, 0x00495001,
      82, QMetaType::QReal, 0x00495001,
      38, QMetaType::QReal, 0x00495103,
      83, 0x80000000 | 40, 0x0049510b,
      84, QMetaType::QUrl, 0x00495107,
      85, QMetaType::Int, 0x00495103,
      86, QMetaType::Int, 0x00495103,
      87, 0x80000000 | 88, 0x0049510b,
      89, 0x80000000 | 90, 0x0049510b,
      91, QMetaType::QString, 0x00c95001,
      92, QMetaType::QReal, 0x00c95107,
      93, QMetaType::QReal, 0x00c95107,
      94, QMetaType::QReal, 0x00c95107,
      95, QMetaType::QReal, 0x00c95107,
      96, QMetaType::QReal, 0x00c95107,
      97, 0x80000000 | 98, 0x00c95009,
      99, QMetaType::QSizeF, 0x00c95001,

 // properties: notify_signal_id
       0,
       3,
       4,
       5,
       6,
       7,
       8,
      24,
       9,
      10,
      11,
      12,
      13,
      14,
      15,
      17,
      18,
      17,
      18,
      19,
      20,
      26,
      22,
      23,
      21,
      27,
       2,
      28,
      29,
      30,
      31,
      32,
      33,
      16,

 // properties: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       2,
       6,
       6,
       6,
       6,
       6,
       9,
      10,

 // enums: name, alias, flags, count, data
      69,   69, 0x0,    4,  543,
      72,   72, 0x0,    3,  551,
      66,   66, 0x0,    4,  557,
      78,   78, 0x0,    5,  565,
      80,   80, 0x0,    4,  575,
      74,   74, 0x0,    5,  583,
      90,   90, 0x0,    2,  593,
      40,   40, 0x0,    2,  597,
      88,   88, 0x0,    4,  601,

 // enum data: key, value
     100, uint(QQuickText::AlignLeft),
     101, uint(QQuickText::AlignRight),
     102, uint(QQuickText::AlignHCenter),
     103, uint(QQuickText::AlignJustify),
     104, uint(QQuickText::AlignTop),
     105, uint(QQuickText::AlignBottom),
     106, uint(QQuickText::AlignVCenter),
     107, uint(QQuickText::Normal),
     108, uint(QQuickText::Outline),
     109, uint(QQuickText::Raised),
     110, uint(QQuickText::Sunken),
     111, uint(QQuickText::PlainText),
     112, uint(QQuickText::RichText),
     113, uint(QQuickText::MarkdownText),
     114, uint(QQuickText::AutoText),
     115, uint(QQuickText::StyledText),
     116, uint(QQuickText::ElideLeft),
     117, uint(QQuickText::ElideRight),
     118, uint(QQuickText::ElideMiddle),
     119, uint(QQuickText::ElideNone),
     120, uint(QQuickText::NoWrap),
     121, uint(QQuickText::WordWrap),
     122, uint(QQuickText::WrapAnywhere),
     123, uint(QQuickText::WrapAtWordBoundaryOrAnywhere),
     124, uint(QQuickText::Wrap),
     125, uint(QQuickText::QtRendering),
     126, uint(QQuickText::NativeRendering),
     127, uint(QQuickText::ProportionalHeight),
     128, uint(QQuickText::FixedHeight),
     129, uint(QQuickText::FixedSize),
     130, uint(QQuickText::HorizontalFit),
     131, uint(QQuickText::VerticalFit),
     132, uint(QQuickText::Fit),

       0        // eod
};

void QQuickText::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickText *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->textChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->linkActivated((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->linkHovered((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->fontChanged((*reinterpret_cast< const QFont(*)>(_a[1]))); break;
        case 4: _t->colorChanged(); break;
        case 5: _t->linkColorChanged(); break;
        case 6: _t->styleChanged((*reinterpret_cast< QQuickText::TextStyle(*)>(_a[1]))); break;
        case 7: _t->styleColorChanged(); break;
        case 8: _t->horizontalAlignmentChanged((*reinterpret_cast< QQuickText::HAlignment(*)>(_a[1]))); break;
        case 9: _t->verticalAlignmentChanged((*reinterpret_cast< QQuickText::VAlignment(*)>(_a[1]))); break;
        case 10: _t->wrapModeChanged(); break;
        case 11: _t->lineCountChanged(); break;
        case 12: _t->truncatedChanged(); break;
        case 13: _t->maximumLineCountChanged(); break;
        case 14: _t->textFormatChanged((*reinterpret_cast< QQuickText::TextFormat(*)>(_a[1]))); break;
        case 15: _t->elideModeChanged((*reinterpret_cast< QQuickText::TextElideMode(*)>(_a[1]))); break;
        case 16: _t->contentSizeChanged(); break;
        case 17: _t->contentWidthChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 18: _t->contentHeightChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 19: _t->lineHeightChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 20: _t->lineHeightModeChanged((*reinterpret_cast< LineHeightMode(*)>(_a[1]))); break;
        case 21: _t->fontSizeModeChanged(); break;
        case 22: _t->minimumPixelSizeChanged(); break;
        case 23: _t->minimumPointSizeChanged(); break;
        case 24: _t->effectiveHorizontalAlignmentChanged(); break;
        case 25: _t->lineLaidOut((*reinterpret_cast< QQuickTextLine*(*)>(_a[1]))); break;
        case 26: _t->baseUrlChanged(); break;
        case 27: _t->renderTypeChanged(); break;
        case 28: _t->paddingChanged(); break;
        case 29: _t->topPaddingChanged(); break;
        case 30: _t->leftPaddingChanged(); break;
        case 31: _t->rightPaddingChanged(); break;
        case 32: _t->bottomPaddingChanged(); break;
        case 33: _t->fontInfoChanged(); break;
        case 34: _t->q_updateLayout(); break;
        case 35: _t->triggerPreprocess(); break;
        case 36: _t->imageDownloadFinished(); break;
        case 37: _t->doLayout(); break;
        case 38: _t->forceLayout(); break;
        case 39: { QString _r = _t->linkAt((*reinterpret_cast< qreal(*)>(_a[1])),(*reinterpret_cast< qreal(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 25:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickTextLine* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickText::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::textChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::linkActivated)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::linkHovered)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)(const QFont & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::fontChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::colorChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::linkColorChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)(QQuickText::TextStyle );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::styleChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::styleColorChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)(QQuickText::HAlignment );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::horizontalAlignmentChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)(QQuickText::VAlignment );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::verticalAlignmentChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::wrapModeChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::lineCountChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::truncatedChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::maximumLineCountChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)(QQuickText::TextFormat );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::textFormatChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)(QQuickText::TextElideMode );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::elideModeChanged)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::contentSizeChanged)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::contentWidthChanged)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::contentHeightChanged)) {
                *result = 18;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::lineHeightChanged)) {
                *result = 19;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)(LineHeightMode );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::lineHeightModeChanged)) {
                *result = 20;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::fontSizeModeChanged)) {
                *result = 21;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::minimumPixelSizeChanged)) {
                *result = 22;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::minimumPointSizeChanged)) {
                *result = 23;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::effectiveHorizontalAlignmentChanged)) {
                *result = 24;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)(QQuickTextLine * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::lineLaidOut)) {
                *result = 25;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::baseUrlChanged)) {
                *result = 26;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::renderTypeChanged)) {
                *result = 27;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::paddingChanged)) {
                *result = 28;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::topPaddingChanged)) {
                *result = 29;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::leftPaddingChanged)) {
                *result = 30;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::rightPaddingChanged)) {
                *result = 31;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::bottomPaddingChanged)) {
                *result = 32;
                return;
            }
        }
        {
            using _t = void (QQuickText::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickText::fontInfoChanged)) {
                *result = 33;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 32:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickText *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->text(); break;
        case 1: *reinterpret_cast< QFont*>(_v) = _t->font(); break;
        case 2: *reinterpret_cast< QColor*>(_v) = _t->color(); break;
        case 3: *reinterpret_cast< QColor*>(_v) = _t->linkColor(); break;
        case 4: *reinterpret_cast< TextStyle*>(_v) = _t->style(); break;
        case 5: *reinterpret_cast< QColor*>(_v) = _t->styleColor(); break;
        case 6: *reinterpret_cast< HAlignment*>(_v) = _t->hAlign(); break;
        case 7: *reinterpret_cast< HAlignment*>(_v) = _t->effectiveHAlign(); break;
        case 8: *reinterpret_cast< VAlignment*>(_v) = _t->vAlign(); break;
        case 9: *reinterpret_cast< WrapMode*>(_v) = _t->wrapMode(); break;
        case 10: *reinterpret_cast< int*>(_v) = _t->lineCount(); break;
        case 11: *reinterpret_cast< bool*>(_v) = _t->truncated(); break;
        case 12: *reinterpret_cast< int*>(_v) = _t->maximumLineCount(); break;
        case 13: *reinterpret_cast< TextFormat*>(_v) = _t->textFormat(); break;
        case 14: *reinterpret_cast< TextElideMode*>(_v) = _t->elideMode(); break;
        case 15: *reinterpret_cast< qreal*>(_v) = _t->contentWidth(); break;
        case 16: *reinterpret_cast< qreal*>(_v) = _t->contentHeight(); break;
        case 17: *reinterpret_cast< qreal*>(_v) = _t->contentWidth(); break;
        case 18: *reinterpret_cast< qreal*>(_v) = _t->contentHeight(); break;
        case 19: *reinterpret_cast< qreal*>(_v) = _t->lineHeight(); break;
        case 20: *reinterpret_cast< LineHeightMode*>(_v) = _t->lineHeightMode(); break;
        case 21: *reinterpret_cast< QUrl*>(_v) = _t->baseUrl(); break;
        case 22: *reinterpret_cast< int*>(_v) = _t->minimumPixelSize(); break;
        case 23: *reinterpret_cast< int*>(_v) = _t->minimumPointSize(); break;
        case 24: *reinterpret_cast< FontSizeMode*>(_v) = _t->fontSizeMode(); break;
        case 25: *reinterpret_cast< RenderType*>(_v) = _t->renderType(); break;
        case 26: *reinterpret_cast< QString*>(_v) = _t->hoveredLink(); break;
        case 27: *reinterpret_cast< qreal*>(_v) = _t->padding(); break;
        case 28: *reinterpret_cast< qreal*>(_v) = _t->topPadding(); break;
        case 29: *reinterpret_cast< qreal*>(_v) = _t->leftPadding(); break;
        case 30: *reinterpret_cast< qreal*>(_v) = _t->rightPadding(); break;
        case 31: *reinterpret_cast< qreal*>(_v) = _t->bottomPadding(); break;
        case 32: *reinterpret_cast< QJSValue*>(_v) = _t->fontInfo(); break;
        case 33: *reinterpret_cast< QSizeF*>(_v) = _t->advance(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickText *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setText(*reinterpret_cast< QString*>(_v)); break;
        case 1: _t->setFont(*reinterpret_cast< QFont*>(_v)); break;
        case 2: _t->setColor(*reinterpret_cast< QColor*>(_v)); break;
        case 3: _t->setLinkColor(*reinterpret_cast< QColor*>(_v)); break;
        case 4: _t->setStyle(*reinterpret_cast< TextStyle*>(_v)); break;
        case 5: _t->setStyleColor(*reinterpret_cast< QColor*>(_v)); break;
        case 6: _t->setHAlign(*reinterpret_cast< HAlignment*>(_v)); break;
        case 8: _t->setVAlign(*reinterpret_cast< VAlignment*>(_v)); break;
        case 9: _t->setWrapMode(*reinterpret_cast< WrapMode*>(_v)); break;
        case 12: _t->setMaximumLineCount(*reinterpret_cast< int*>(_v)); break;
        case 13: _t->setTextFormat(*reinterpret_cast< TextFormat*>(_v)); break;
        case 14: _t->setElideMode(*reinterpret_cast< TextElideMode*>(_v)); break;
        case 19: _t->setLineHeight(*reinterpret_cast< qreal*>(_v)); break;
        case 20: _t->setLineHeightMode(*reinterpret_cast< LineHeightMode*>(_v)); break;
        case 21: _t->setBaseUrl(*reinterpret_cast< QUrl*>(_v)); break;
        case 22: _t->setMinimumPixelSize(*reinterpret_cast< int*>(_v)); break;
        case 23: _t->setMinimumPointSize(*reinterpret_cast< int*>(_v)); break;
        case 24: _t->setFontSizeMode(*reinterpret_cast< FontSizeMode*>(_v)); break;
        case 25: _t->setRenderType(*reinterpret_cast< RenderType*>(_v)); break;
        case 27: _t->setPadding(*reinterpret_cast< qreal*>(_v)); break;
        case 28: _t->setTopPadding(*reinterpret_cast< qreal*>(_v)); break;
        case 29: _t->setLeftPadding(*reinterpret_cast< qreal*>(_v)); break;
        case 30: _t->setRightPadding(*reinterpret_cast< qreal*>(_v)); break;
        case 31: _t->setBottomPadding(*reinterpret_cast< qreal*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        QQuickText *_t = static_cast<QQuickText *>(_o);
        (void)_t;
        switch (_id) {
        case 6: _t->resetHAlign(); break;
        case 12: _t->resetMaximumLineCount(); break;
        case 21: _t->resetBaseUrl(); break;
        case 27: _t->resetPadding(); break;
        case 28: _t->resetTopPadding(); break;
        case 29: _t->resetLeftPadding(); break;
        case 30: _t->resetRightPadding(); break;
        case 31: _t->resetBottomPadding(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickText::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickImplicitSizeItem::staticMetaObject>(),
    qt_meta_stringdata_QQuickText.data,
    qt_meta_data_QQuickText,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickText::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickText::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickText.stringdata0))
        return static_cast<void*>(this);
    return QQuickImplicitSizeItem::qt_metacast(_clname);
}

int QQuickText::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickImplicitSizeItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 40)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 40;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 40)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 40;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 34;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 34;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 34;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 34;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 34;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 34;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickText::textChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QQuickText::linkActivated(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QQuickText::linkHovered(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QQuickText::fontChanged(const QFont & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QQuickText::colorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickText::linkColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickText::styleChanged(QQuickText::TextStyle _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QQuickText::styleColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QQuickText::horizontalAlignmentChanged(QQuickText::HAlignment _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QQuickText::verticalAlignmentChanged(QQuickText::VAlignment _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void QQuickText::wrapModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QQuickText::lineCountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void QQuickText::truncatedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void QQuickText::maximumLineCountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void QQuickText::textFormatChanged(QQuickText::TextFormat _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void QQuickText::elideModeChanged(QQuickText::TextElideMode _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void QQuickText::contentSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void QQuickText::contentWidthChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 17, _a);
}

// SIGNAL 18
void QQuickText::contentHeightChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 18, _a);
}

// SIGNAL 19
void QQuickText::lineHeightChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 19, _a);
}

// SIGNAL 20
void QQuickText::lineHeightModeChanged(LineHeightMode _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 20, _a);
}

// SIGNAL 21
void QQuickText::fontSizeModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 21, nullptr);
}

// SIGNAL 22
void QQuickText::minimumPixelSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 22, nullptr);
}

// SIGNAL 23
void QQuickText::minimumPointSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 23, nullptr);
}

// SIGNAL 24
void QQuickText::effectiveHorizontalAlignmentChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 24, nullptr);
}

// SIGNAL 25
void QQuickText::lineLaidOut(QQuickTextLine * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 25, _a);
}

// SIGNAL 26
void QQuickText::baseUrlChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 26, nullptr);
}

// SIGNAL 27
void QQuickText::renderTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 27, nullptr);
}

// SIGNAL 28
void QQuickText::paddingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 28, nullptr);
}

// SIGNAL 29
void QQuickText::topPaddingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 29, nullptr);
}

// SIGNAL 30
void QQuickText::leftPaddingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 30, nullptr);
}

// SIGNAL 31
void QQuickText::rightPaddingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 31, nullptr);
}

// SIGNAL 32
void QQuickText::bottomPaddingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 32, nullptr);
}

// SIGNAL 33
void QQuickText::fontInfoChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 33, nullptr);
}
struct qt_meta_stringdata_QQuickTextLine_t {
    QByteArrayData data[10];
    char stringdata0[82];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickTextLine_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickTextLine_t qt_meta_stringdata_QQuickTextLine = {
    {
QT_MOC_LITERAL(0, 0, 14), // "QQuickTextLine"
QT_MOC_LITERAL(1, 15, 11), // "QML.Element"
QT_MOC_LITERAL(2, 27, 9), // "anonymous"
QT_MOC_LITERAL(3, 37, 6), // "number"
QT_MOC_LITERAL(4, 44, 5), // "width"
QT_MOC_LITERAL(5, 50, 6), // "height"
QT_MOC_LITERAL(6, 57, 1), // "x"
QT_MOC_LITERAL(7, 59, 1), // "y"
QT_MOC_LITERAL(8, 61, 13), // "implicitWidth"
QT_MOC_LITERAL(9, 75, 6) // "isLast"

    },
    "QQuickTextLine\0QML.Element\0anonymous\0"
    "number\0width\0height\0x\0y\0implicitWidth\0"
    "isLast"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickTextLine[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       0,    0, // methods
       7,   16, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,

 // properties: name, type, flags
       3, QMetaType::Int, 0x00095001,
       4, QMetaType::QReal, 0x00095103,
       5, QMetaType::QReal, 0x00095103,
       6, QMetaType::QReal, 0x00095103,
       7, QMetaType::QReal, 0x00095103,
       8, QMetaType::QReal, 0x00895001,
       9, QMetaType::Bool, 0x00895001,

 // properties: revision
       0,
       0,
       0,
       0,
       0,
      15,
      15,

       0        // eod
};

void QQuickTextLine::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{

#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickTextLine *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->number(); break;
        case 1: *reinterpret_cast< qreal*>(_v) = _t->width(); break;
        case 2: *reinterpret_cast< qreal*>(_v) = _t->height(); break;
        case 3: *reinterpret_cast< qreal*>(_v) = _t->x(); break;
        case 4: *reinterpret_cast< qreal*>(_v) = _t->y(); break;
        case 5: *reinterpret_cast< qreal*>(_v) = _t->implicitWidth(); break;
        case 6: *reinterpret_cast< bool*>(_v) = _t->isLast(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickTextLine *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setWidth(*reinterpret_cast< qreal*>(_v)); break;
        case 2: _t->setHeight(*reinterpret_cast< qreal*>(_v)); break;
        case 3: _t->setX(*reinterpret_cast< qreal*>(_v)); break;
        case 4: _t->setY(*reinterpret_cast< qreal*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickTextLine::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickTextLine.data,
    qt_meta_data_QQuickTextLine,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickTextLine::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickTextLine::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickTextLine.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickTextLine::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    
#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 7;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
