/****************************************************************************
** Meta object code from reading C++ file 'qquickimageparticle_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/particles/qquickimageparticle_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickimageparticle_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickImageParticle_t {
    QByteArrayData data[89];
    char stringdata0[1287];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickImageParticle_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickImageParticle_t qt_meta_stringdata_QQuickImageParticle = {
    {
QT_MOC_LITERAL(0, 0, 19), // "QQuickImageParticle"
QT_MOC_LITERAL(1, 20, 11), // "QML.Element"
QT_MOC_LITERAL(2, 32, 13), // "ImageParticle"
QT_MOC_LITERAL(3, 46, 12), // "imageChanged"
QT_MOC_LITERAL(4, 59, 0), // ""
QT_MOC_LITERAL(5, 60, 17), // "colortableChanged"
QT_MOC_LITERAL(6, 78, 16), // "sizetableChanged"
QT_MOC_LITERAL(7, 95, 19), // "opacitytableChanged"
QT_MOC_LITERAL(8, 115, 12), // "colorChanged"
QT_MOC_LITERAL(9, 128, 21), // "colorVariationChanged"
QT_MOC_LITERAL(10, 150, 21), // "alphaVariationChanged"
QT_MOC_LITERAL(11, 172, 3), // "arg"
QT_MOC_LITERAL(12, 176, 12), // "alphaChanged"
QT_MOC_LITERAL(13, 189, 19), // "redVariationChanged"
QT_MOC_LITERAL(14, 209, 21), // "greenVariationChanged"
QT_MOC_LITERAL(15, 231, 20), // "blueVariationChanged"
QT_MOC_LITERAL(16, 252, 15), // "rotationChanged"
QT_MOC_LITERAL(17, 268, 24), // "rotationVariationChanged"
QT_MOC_LITERAL(18, 293, 23), // "rotationVelocityChanged"
QT_MOC_LITERAL(19, 317, 32), // "rotationVelocityVariationChanged"
QT_MOC_LITERAL(20, 350, 19), // "autoRotationChanged"
QT_MOC_LITERAL(21, 370, 14), // "xVectorChanged"
QT_MOC_LITERAL(22, 385, 16), // "QQuickDirection*"
QT_MOC_LITERAL(23, 402, 14), // "yVectorChanged"
QT_MOC_LITERAL(24, 417, 25), // "spritesInterpolateChanged"
QT_MOC_LITERAL(25, 443, 26), // "bypassOptimizationsChanged"
QT_MOC_LITERAL(26, 470, 18), // "entryEffectChanged"
QT_MOC_LITERAL(27, 489, 11), // "EntryEffect"
QT_MOC_LITERAL(28, 501, 13), // "statusChanged"
QT_MOC_LITERAL(29, 515, 6), // "Status"
QT_MOC_LITERAL(30, 522, 11), // "reloadColor"
QT_MOC_LITERAL(31, 534, 8), // "Color4ub"
QT_MOC_LITERAL(32, 543, 1), // "c"
QT_MOC_LITERAL(33, 545, 19), // "QQuickParticleData*"
QT_MOC_LITERAL(34, 565, 1), // "d"
QT_MOC_LITERAL(35, 567, 17), // "setAlphaVariation"
QT_MOC_LITERAL(36, 585, 8), // "setAlpha"
QT_MOC_LITERAL(37, 594, 15), // "setRedVariation"
QT_MOC_LITERAL(38, 610, 17), // "setGreenVariation"
QT_MOC_LITERAL(39, 628, 16), // "setBlueVariation"
QT_MOC_LITERAL(40, 645, 11), // "setRotation"
QT_MOC_LITERAL(41, 657, 20), // "setRotationVariation"
QT_MOC_LITERAL(42, 678, 19), // "setRotationVelocity"
QT_MOC_LITERAL(43, 698, 28), // "setRotationVelocityVariation"
QT_MOC_LITERAL(44, 727, 15), // "setAutoRotation"
QT_MOC_LITERAL(45, 743, 10), // "setXVector"
QT_MOC_LITERAL(46, 754, 10), // "setYVector"
QT_MOC_LITERAL(47, 765, 21), // "setSpritesInterpolate"
QT_MOC_LITERAL(48, 787, 22), // "setBypassOptimizations"
QT_MOC_LITERAL(49, 810, 14), // "setEntryEffect"
QT_MOC_LITERAL(50, 825, 12), // "createEngine"
QT_MOC_LITERAL(51, 838, 13), // "spriteAdvance"
QT_MOC_LITERAL(52, 852, 11), // "spriteIndex"
QT_MOC_LITERAL(53, 864, 13), // "spritesUpdate"
QT_MOC_LITERAL(54, 878, 4), // "time"
QT_MOC_LITERAL(55, 883, 24), // "mainThreadFetchImageData"
QT_MOC_LITERAL(56, 908, 24), // "finishBuildParticleNodes"
QT_MOC_LITERAL(57, 933, 9), // "QSGNode**"
QT_MOC_LITERAL(58, 943, 1), // "n"
QT_MOC_LITERAL(59, 945, 6), // "source"
QT_MOC_LITERAL(60, 952, 7), // "sprites"
QT_MOC_LITERAL(61, 960, 30), // "QQmlListProperty<QQuickSprite>"
QT_MOC_LITERAL(62, 991, 6), // "status"
QT_MOC_LITERAL(63, 998, 10), // "colorTable"
QT_MOC_LITERAL(64, 1009, 9), // "sizeTable"
QT_MOC_LITERAL(65, 1019, 12), // "opacityTable"
QT_MOC_LITERAL(66, 1032, 5), // "color"
QT_MOC_LITERAL(67, 1038, 14), // "colorVariation"
QT_MOC_LITERAL(68, 1053, 12), // "redVariation"
QT_MOC_LITERAL(69, 1066, 14), // "greenVariation"
QT_MOC_LITERAL(70, 1081, 13), // "blueVariation"
QT_MOC_LITERAL(71, 1095, 5), // "alpha"
QT_MOC_LITERAL(72, 1101, 14), // "alphaVariation"
QT_MOC_LITERAL(73, 1116, 8), // "rotation"
QT_MOC_LITERAL(74, 1125, 17), // "rotationVariation"
QT_MOC_LITERAL(75, 1143, 16), // "rotationVelocity"
QT_MOC_LITERAL(76, 1160, 25), // "rotationVelocityVariation"
QT_MOC_LITERAL(77, 1186, 12), // "autoRotation"
QT_MOC_LITERAL(78, 1199, 7), // "xVector"
QT_MOC_LITERAL(79, 1207, 7), // "yVector"
QT_MOC_LITERAL(80, 1215, 18), // "spritesInterpolate"
QT_MOC_LITERAL(81, 1234, 11), // "entryEffect"
QT_MOC_LITERAL(82, 1246, 4), // "Null"
QT_MOC_LITERAL(83, 1251, 5), // "Ready"
QT_MOC_LITERAL(84, 1257, 7), // "Loading"
QT_MOC_LITERAL(85, 1265, 5), // "Error"
QT_MOC_LITERAL(86, 1271, 4), // "None"
QT_MOC_LITERAL(87, 1276, 4), // "Fade"
QT_MOC_LITERAL(88, 1281, 5) // "Scale"

    },
    "QQuickImageParticle\0QML.Element\0"
    "ImageParticle\0imageChanged\0\0"
    "colortableChanged\0sizetableChanged\0"
    "opacitytableChanged\0colorChanged\0"
    "colorVariationChanged\0alphaVariationChanged\0"
    "arg\0alphaChanged\0redVariationChanged\0"
    "greenVariationChanged\0blueVariationChanged\0"
    "rotationChanged\0rotationVariationChanged\0"
    "rotationVelocityChanged\0"
    "rotationVelocityVariationChanged\0"
    "autoRotationChanged\0xVectorChanged\0"
    "QQuickDirection*\0yVectorChanged\0"
    "spritesInterpolateChanged\0"
    "bypassOptimizationsChanged\0"
    "entryEffectChanged\0EntryEffect\0"
    "statusChanged\0Status\0reloadColor\0"
    "Color4ub\0c\0QQuickParticleData*\0d\0"
    "setAlphaVariation\0setAlpha\0setRedVariation\0"
    "setGreenVariation\0setBlueVariation\0"
    "setRotation\0setRotationVariation\0"
    "setRotationVelocity\0setRotationVelocityVariation\0"
    "setAutoRotation\0setXVector\0setYVector\0"
    "setSpritesInterpolate\0setBypassOptimizations\0"
    "setEntryEffect\0createEngine\0spriteAdvance\0"
    "spriteIndex\0spritesUpdate\0time\0"
    "mainThreadFetchImageData\0"
    "finishBuildParticleNodes\0QSGNode**\0n\0"
    "source\0sprites\0QQmlListProperty<QQuickSprite>\0"
    "status\0colorTable\0sizeTable\0opacityTable\0"
    "color\0colorVariation\0redVariation\0"
    "greenVariation\0blueVariation\0alpha\0"
    "alphaVariation\0rotation\0rotationVariation\0"
    "rotationVelocity\0rotationVelocityVariation\0"
    "autoRotation\0xVector\0yVector\0"
    "spritesInterpolate\0entryEffect\0Null\0"
    "Ready\0Loading\0Error\0None\0Fade\0Scale"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickImageParticle[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      44,   16, // methods
      22,  352, // properties
       2,  440, // enums/sets
       0,    0, // constructors
       0,       // flags
      22,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,  236,    4, 0x06 /* Public */,
       5,    0,  237,    4, 0x06 /* Public */,
       6,    0,  238,    4, 0x06 /* Public */,
       7,    0,  239,    4, 0x06 /* Public */,
       8,    0,  240,    4, 0x06 /* Public */,
       9,    0,  241,    4, 0x06 /* Public */,
      10,    1,  242,    4, 0x06 /* Public */,
      12,    1,  245,    4, 0x06 /* Public */,
      13,    1,  248,    4, 0x06 /* Public */,
      14,    1,  251,    4, 0x06 /* Public */,
      15,    1,  254,    4, 0x06 /* Public */,
      16,    1,  257,    4, 0x06 /* Public */,
      17,    1,  260,    4, 0x06 /* Public */,
      18,    1,  263,    4, 0x06 /* Public */,
      19,    1,  266,    4, 0x06 /* Public */,
      20,    1,  269,    4, 0x06 /* Public */,
      21,    1,  272,    4, 0x06 /* Public */,
      23,    1,  275,    4, 0x06 /* Public */,
      24,    1,  278,    4, 0x06 /* Public */,
      25,    1,  281,    4, 0x06 /* Public */,
      26,    1,  284,    4, 0x06 /* Public */,
      28,    1,  287,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      30,    2,  290,    4, 0x0a /* Public */,
      35,    1,  295,    4, 0x0a /* Public */,
      36,    1,  298,    4, 0x0a /* Public */,
      37,    1,  301,    4, 0x0a /* Public */,
      38,    1,  304,    4, 0x0a /* Public */,
      39,    1,  307,    4, 0x0a /* Public */,
      40,    1,  310,    4, 0x0a /* Public */,
      41,    1,  313,    4, 0x0a /* Public */,
      42,    1,  316,    4, 0x0a /* Public */,
      43,    1,  319,    4, 0x0a /* Public */,
      44,    1,  322,    4, 0x0a /* Public */,
      45,    1,  325,    4, 0x0a /* Public */,
      46,    1,  328,    4, 0x0a /* Public */,
      47,    1,  331,    4, 0x0a /* Public */,
      48,    1,  334,    4, 0x0a /* Public */,
      49,    1,  337,    4, 0x0a /* Public */,
      50,    0,  340,    4, 0x08 /* Private */,
      51,    1,  341,    4, 0x08 /* Private */,
      53,    1,  344,    4, 0x08 /* Private */,
      53,    0,  347,    4, 0x28 /* Private | MethodCloned */,
      55,    0,  348,    4, 0x08 /* Private */,
      56,    1,  349,    4, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QReal,   11,
    QMetaType::Void, QMetaType::QReal,   11,
    QMetaType::Void, QMetaType::QReal,   11,
    QMetaType::Void, QMetaType::QReal,   11,
    QMetaType::Void, QMetaType::QReal,   11,
    QMetaType::Void, QMetaType::QReal,   11,
    QMetaType::Void, QMetaType::QReal,   11,
    QMetaType::Void, QMetaType::QReal,   11,
    QMetaType::Void, QMetaType::QReal,   11,
    QMetaType::Void, QMetaType::Bool,   11,
    QMetaType::Void, 0x80000000 | 22,   11,
    QMetaType::Void, 0x80000000 | 22,   11,
    QMetaType::Void, QMetaType::Bool,   11,
    QMetaType::Void, QMetaType::Bool,   11,
    QMetaType::Void, 0x80000000 | 27,   11,
    QMetaType::Void, 0x80000000 | 29,   11,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 31, 0x80000000 | 33,   32,   34,
    QMetaType::Void, QMetaType::QReal,   11,
    QMetaType::Void, QMetaType::QReal,   11,
    QMetaType::Void, QMetaType::QReal,   11,
    QMetaType::Void, QMetaType::QReal,   11,
    QMetaType::Void, QMetaType::QReal,   11,
    QMetaType::Void, QMetaType::QReal,   11,
    QMetaType::Void, QMetaType::QReal,   11,
    QMetaType::Void, QMetaType::QReal,   11,
    QMetaType::Void, QMetaType::QReal,   11,
    QMetaType::Void, QMetaType::Bool,   11,
    QMetaType::Void, 0x80000000 | 22,   11,
    QMetaType::Void, 0x80000000 | 22,   11,
    QMetaType::Void, QMetaType::Bool,   11,
    QMetaType::Void, QMetaType::Bool,   11,
    QMetaType::Void, 0x80000000 | 27,   11,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   52,
    QMetaType::Void, QMetaType::QReal,   54,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 57,   58,

 // properties: name, type, flags
      59, QMetaType::QUrl, 0x00495003,
      60, 0x80000000 | 61, 0x00095009,
      62, 0x80000000 | 29, 0x00495009,
      63, QMetaType::QUrl, 0x00495003,
      64, QMetaType::QUrl, 0x00495003,
      65, QMetaType::QUrl, 0x00495003,
      66, QMetaType::QColor, 0x00495107,
      67, QMetaType::QReal, 0x00495107,
      68, QMetaType::QReal, 0x00495107,
      69, QMetaType::QReal, 0x00495107,
      70, QMetaType::QReal, 0x00495107,
      71, QMetaType::QReal, 0x00495107,
      72, QMetaType::QReal, 0x00495107,
      73, QMetaType::QReal, 0x00495107,
      74, QMetaType::QReal, 0x00495107,
      75, QMetaType::QReal, 0x00495107,
      76, QMetaType::QReal, 0x00495107,
      77, QMetaType::Bool, 0x00495107,
      78, 0x80000000 | 22, 0x0049510f,
      79, 0x80000000 | 22, 0x0049510f,
      80, QMetaType::Bool, 0x00495103,
      81, 0x80000000 | 27, 0x0049510b,

 // properties: notify_signal_id
       0,
       0,
      21,
       1,
       2,
       3,
       4,
       5,
       8,
       9,
      10,
       7,
       6,
      11,
      12,
      13,
      14,
      15,
      16,
      17,
      18,
      20,

 // enums: name, alias, flags, count, data
      29,   29, 0x0,    4,  450,
      27,   27, 0x0,    3,  458,

 // enum data: key, value
      82, uint(QQuickImageParticle::Null),
      83, uint(QQuickImageParticle::Ready),
      84, uint(QQuickImageParticle::Loading),
      85, uint(QQuickImageParticle::Error),
      86, uint(QQuickImageParticle::None),
      87, uint(QQuickImageParticle::Fade),
      88, uint(QQuickImageParticle::Scale),

       0        // eod
};

void QQuickImageParticle::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickImageParticle *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->imageChanged(); break;
        case 1: _t->colortableChanged(); break;
        case 2: _t->sizetableChanged(); break;
        case 3: _t->opacitytableChanged(); break;
        case 4: _t->colorChanged(); break;
        case 5: _t->colorVariationChanged(); break;
        case 6: _t->alphaVariationChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 7: _t->alphaChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 8: _t->redVariationChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 9: _t->greenVariationChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 10: _t->blueVariationChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 11: _t->rotationChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 12: _t->rotationVariationChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 13: _t->rotationVelocityChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 14: _t->rotationVelocityVariationChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 15: _t->autoRotationChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 16: _t->xVectorChanged((*reinterpret_cast< QQuickDirection*(*)>(_a[1]))); break;
        case 17: _t->yVectorChanged((*reinterpret_cast< QQuickDirection*(*)>(_a[1]))); break;
        case 18: _t->spritesInterpolateChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 19: _t->bypassOptimizationsChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 20: _t->entryEffectChanged((*reinterpret_cast< EntryEffect(*)>(_a[1]))); break;
        case 21: _t->statusChanged((*reinterpret_cast< Status(*)>(_a[1]))); break;
        case 22: _t->reloadColor((*reinterpret_cast< const Color4ub(*)>(_a[1])),(*reinterpret_cast< QQuickParticleData*(*)>(_a[2]))); break;
        case 23: _t->setAlphaVariation((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 24: _t->setAlpha((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 25: _t->setRedVariation((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 26: _t->setGreenVariation((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 27: _t->setBlueVariation((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 28: _t->setRotation((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 29: _t->setRotationVariation((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 30: _t->setRotationVelocity((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 31: _t->setRotationVelocityVariation((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 32: _t->setAutoRotation((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 33: _t->setXVector((*reinterpret_cast< QQuickDirection*(*)>(_a[1]))); break;
        case 34: _t->setYVector((*reinterpret_cast< QQuickDirection*(*)>(_a[1]))); break;
        case 35: _t->setSpritesInterpolate((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 36: _t->setBypassOptimizations((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 37: _t->setEntryEffect((*reinterpret_cast< EntryEffect(*)>(_a[1]))); break;
        case 38: _t->createEngine(); break;
        case 39: _t->spriteAdvance((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 40: _t->spritesUpdate((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 41: _t->spritesUpdate(); break;
        case 42: _t->mainThreadFetchImageData(); break;
        case 43: _t->finishBuildParticleNodes((*reinterpret_cast< QSGNode**(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickDirection* >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickDirection* >(); break;
            }
            break;
        case 33:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickDirection* >(); break;
            }
            break;
        case 34:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickDirection* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickImageParticle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageParticle::imageChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickImageParticle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageParticle::colortableChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickImageParticle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageParticle::sizetableChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickImageParticle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageParticle::opacitytableChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickImageParticle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageParticle::colorChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickImageParticle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageParticle::colorVariationChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickImageParticle::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageParticle::alphaVariationChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickImageParticle::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageParticle::alphaChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickImageParticle::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageParticle::redVariationChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickImageParticle::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageParticle::greenVariationChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QQuickImageParticle::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageParticle::blueVariationChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QQuickImageParticle::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageParticle::rotationChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QQuickImageParticle::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageParticle::rotationVariationChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QQuickImageParticle::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageParticle::rotationVelocityChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QQuickImageParticle::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageParticle::rotationVelocityVariationChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QQuickImageParticle::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageParticle::autoRotationChanged)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (QQuickImageParticle::*)(QQuickDirection * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageParticle::xVectorChanged)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (QQuickImageParticle::*)(QQuickDirection * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageParticle::yVectorChanged)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (QQuickImageParticle::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageParticle::spritesInterpolateChanged)) {
                *result = 18;
                return;
            }
        }
        {
            using _t = void (QQuickImageParticle::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageParticle::bypassOptimizationsChanged)) {
                *result = 19;
                return;
            }
        }
        {
            using _t = void (QQuickImageParticle::*)(EntryEffect );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageParticle::entryEffectChanged)) {
                *result = 20;
                return;
            }
        }
        {
            using _t = void (QQuickImageParticle::*)(Status );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickImageParticle::statusChanged)) {
                *result = 21;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 19:
        case 18:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickDirection* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickImageParticle *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QUrl*>(_v) = _t->image(); break;
        case 1: *reinterpret_cast< QQmlListProperty<QQuickSprite>*>(_v) = _t->sprites(); break;
        case 2: *reinterpret_cast< Status*>(_v) = _t->status(); break;
        case 3: *reinterpret_cast< QUrl*>(_v) = _t->colortable(); break;
        case 4: *reinterpret_cast< QUrl*>(_v) = _t->sizetable(); break;
        case 5: *reinterpret_cast< QUrl*>(_v) = _t->opacitytable(); break;
        case 6: *reinterpret_cast< QColor*>(_v) = _t->color(); break;
        case 7: *reinterpret_cast< qreal*>(_v) = _t->colorVariation(); break;
        case 8: *reinterpret_cast< qreal*>(_v) = _t->redVariation(); break;
        case 9: *reinterpret_cast< qreal*>(_v) = _t->greenVariation(); break;
        case 10: *reinterpret_cast< qreal*>(_v) = _t->blueVariation(); break;
        case 11: *reinterpret_cast< qreal*>(_v) = _t->alpha(); break;
        case 12: *reinterpret_cast< qreal*>(_v) = _t->alphaVariation(); break;
        case 13: *reinterpret_cast< qreal*>(_v) = _t->rotation(); break;
        case 14: *reinterpret_cast< qreal*>(_v) = _t->rotationVariation(); break;
        case 15: *reinterpret_cast< qreal*>(_v) = _t->rotationVelocity(); break;
        case 16: *reinterpret_cast< qreal*>(_v) = _t->rotationVelocityVariation(); break;
        case 17: *reinterpret_cast< bool*>(_v) = _t->autoRotation(); break;
        case 18: *reinterpret_cast< QQuickDirection**>(_v) = _t->xVector(); break;
        case 19: *reinterpret_cast< QQuickDirection**>(_v) = _t->yVector(); break;
        case 20: *reinterpret_cast< bool*>(_v) = _t->spritesInterpolate(); break;
        case 21: *reinterpret_cast< EntryEffect*>(_v) = _t->entryEffect(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickImageParticle *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setImage(*reinterpret_cast< QUrl*>(_v)); break;
        case 3: _t->setColortable(*reinterpret_cast< QUrl*>(_v)); break;
        case 4: _t->setSizetable(*reinterpret_cast< QUrl*>(_v)); break;
        case 5: _t->setOpacitytable(*reinterpret_cast< QUrl*>(_v)); break;
        case 6: _t->setColor(*reinterpret_cast< QColor*>(_v)); break;
        case 7: _t->setColorVariation(*reinterpret_cast< qreal*>(_v)); break;
        case 8: _t->setRedVariation(*reinterpret_cast< qreal*>(_v)); break;
        case 9: _t->setGreenVariation(*reinterpret_cast< qreal*>(_v)); break;
        case 10: _t->setBlueVariation(*reinterpret_cast< qreal*>(_v)); break;
        case 11: _t->setAlpha(*reinterpret_cast< qreal*>(_v)); break;
        case 12: _t->setAlphaVariation(*reinterpret_cast< qreal*>(_v)); break;
        case 13: _t->setRotation(*reinterpret_cast< qreal*>(_v)); break;
        case 14: _t->setRotationVariation(*reinterpret_cast< qreal*>(_v)); break;
        case 15: _t->setRotationVelocity(*reinterpret_cast< qreal*>(_v)); break;
        case 16: _t->setRotationVelocityVariation(*reinterpret_cast< qreal*>(_v)); break;
        case 17: _t->setAutoRotation(*reinterpret_cast< bool*>(_v)); break;
        case 18: _t->setXVector(*reinterpret_cast< QQuickDirection**>(_v)); break;
        case 19: _t->setYVector(*reinterpret_cast< QQuickDirection**>(_v)); break;
        case 20: _t->setSpritesInterpolate(*reinterpret_cast< bool*>(_v)); break;
        case 21: _t->setEntryEffect(*reinterpret_cast< EntryEffect*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        QQuickImageParticle *_t = static_cast<QQuickImageParticle *>(_o);
        (void)_t;
        switch (_id) {
        case 6: _t->resetColor(); break;
        case 7: _t->resetColor(); break;
        case 8: _t->resetColor(); break;
        case 9: _t->resetColor(); break;
        case 10: _t->resetColor(); break;
        case 11: _t->resetColor(); break;
        case 12: _t->resetColor(); break;
        case 13: _t->resetRotation(); break;
        case 14: _t->resetRotation(); break;
        case 15: _t->resetRotation(); break;
        case 16: _t->resetRotation(); break;
        case 17: _t->resetRotation(); break;
        case 18: _t->resetDeformation(); break;
        case 19: _t->resetDeformation(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickImageParticle::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickParticlePainter::staticMetaObject>(),
    qt_meta_stringdata_QQuickImageParticle.data,
    qt_meta_data_QQuickImageParticle,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickImageParticle::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickImageParticle::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickImageParticle.stringdata0))
        return static_cast<void*>(this);
    return QQuickParticlePainter::qt_metacast(_clname);
}

int QQuickImageParticle::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickParticlePainter::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 44)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 44;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 44)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 44;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 22;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 22;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 22;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 22;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 22;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickImageParticle::imageChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickImageParticle::colortableChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickImageParticle::sizetableChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickImageParticle::opacitytableChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickImageParticle::colorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickImageParticle::colorVariationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickImageParticle::alphaVariationChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QQuickImageParticle::alphaChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QQuickImageParticle::redVariationChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QQuickImageParticle::greenVariationChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void QQuickImageParticle::blueVariationChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void QQuickImageParticle::rotationChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void QQuickImageParticle::rotationVariationChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void QQuickImageParticle::rotationVelocityChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void QQuickImageParticle::rotationVelocityVariationChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void QQuickImageParticle::autoRotationChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void QQuickImageParticle::xVectorChanged(QQuickDirection * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 16, _a);
}

// SIGNAL 17
void QQuickImageParticle::yVectorChanged(QQuickDirection * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 17, _a);
}

// SIGNAL 18
void QQuickImageParticle::spritesInterpolateChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 18, _a);
}

// SIGNAL 19
void QQuickImageParticle::bypassOptimizationsChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 19, _a);
}

// SIGNAL 20
void QQuickImageParticle::entryEffectChanged(EntryEffect _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 20, _a);
}

// SIGNAL 21
void QQuickImageParticle::statusChanged(Status _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 21, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
