/****************************************************************************
** Meta object code from reading C++ file 'qqmlinstantiator_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/qmlmodels/qqmlinstantiator_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qqmlinstantiator_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQmlInstantiator_t {
    QByteArrayData data[27];
    char stringdata0[325];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQmlInstantiator_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQmlInstantiator_t qt_meta_stringdata_QQmlInstantiator = {
    {
QT_MOC_LITERAL(0, 0, 16), // "QQmlInstantiator"
QT_MOC_LITERAL(1, 17, 15), // "DefaultProperty"
QT_MOC_LITERAL(2, 33, 8), // "delegate"
QT_MOC_LITERAL(3, 42, 11), // "QML.Element"
QT_MOC_LITERAL(4, 54, 12), // "Instantiator"
QT_MOC_LITERAL(5, 67, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(6, 91, 2), // "14"
QT_MOC_LITERAL(7, 94, 12), // "modelChanged"
QT_MOC_LITERAL(8, 107, 0), // ""
QT_MOC_LITERAL(9, 108, 15), // "delegateChanged"
QT_MOC_LITERAL(10, 124, 12), // "countChanged"
QT_MOC_LITERAL(11, 137, 13), // "objectChanged"
QT_MOC_LITERAL(12, 151, 13), // "activeChanged"
QT_MOC_LITERAL(13, 165, 19), // "asynchronousChanged"
QT_MOC_LITERAL(14, 185, 11), // "objectAdded"
QT_MOC_LITERAL(15, 197, 5), // "index"
QT_MOC_LITERAL(16, 203, 6), // "object"
QT_MOC_LITERAL(17, 210, 13), // "objectRemoved"
QT_MOC_LITERAL(18, 224, 14), // "_q_createdItem"
QT_MOC_LITERAL(19, 239, 15), // "_q_modelUpdated"
QT_MOC_LITERAL(20, 255, 13), // "QQmlChangeSet"
QT_MOC_LITERAL(21, 269, 8), // "objectAt"
QT_MOC_LITERAL(22, 278, 6), // "active"
QT_MOC_LITERAL(23, 285, 12), // "asynchronous"
QT_MOC_LITERAL(24, 298, 5), // "model"
QT_MOC_LITERAL(25, 304, 5), // "count"
QT_MOC_LITERAL(26, 310, 14) // "QQmlComponent*"

    },
    "QQmlInstantiator\0DefaultProperty\0"
    "delegate\0QML.Element\0Instantiator\0"
    "QML.AddedInMinorVersion\0""14\0modelChanged\0"
    "\0delegateChanged\0countChanged\0"
    "objectChanged\0activeChanged\0"
    "asynchronousChanged\0objectAdded\0index\0"
    "object\0objectRemoved\0_q_createdItem\0"
    "_q_modelUpdated\0QQmlChangeSet\0objectAt\0"
    "active\0asynchronous\0model\0count\0"
    "QQmlComponent*"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQmlInstantiator[] = {

 // content:
       8,       // revision
       0,       // classname
       3,   14, // classinfo
      11,   20, // methods
       6,  104, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       8,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,

 // signals: name, argc, parameters, tag, flags
       7,    0,   75,    8, 0x06 /* Public */,
       9,    0,   76,    8, 0x06 /* Public */,
      10,    0,   77,    8, 0x06 /* Public */,
      11,    0,   78,    8, 0x06 /* Public */,
      12,    0,   79,    8, 0x06 /* Public */,
      13,    0,   80,    8, 0x06 /* Public */,
      14,    2,   81,    8, 0x06 /* Public */,
      17,    2,   86,    8, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      18,    2,   91,    8, 0x08 /* Private */,
      19,    2,   96,    8, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
      21,    1,  101,    8, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::QObjectStar,   15,   16,
    QMetaType::Void, QMetaType::Int, QMetaType::QObjectStar,   15,   16,

 // slots: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::QObjectStar,    8,    8,
    QMetaType::Void, 0x80000000 | 20, QMetaType::Bool,    8,    8,

 // methods: parameters
    QMetaType::QObjectStar, QMetaType::Int,   15,

 // properties: name, type, flags
      22, QMetaType::Bool, 0x00495103,
      23, QMetaType::Bool, 0x00495003,
      24, QMetaType::QVariant, 0x00495103,
      25, QMetaType::Int, 0x00495001,
       2, 0x80000000 | 26, 0x0049510b,
      16, QMetaType::QObjectStar, 0x00495001,

 // properties: notify_signal_id
       4,
       5,
       0,
       2,
       1,
       3,

       0        // eod
};

void QQmlInstantiator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQmlInstantiator *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->modelChanged(); break;
        case 1: _t->delegateChanged(); break;
        case 2: _t->countChanged(); break;
        case 3: _t->objectChanged(); break;
        case 4: _t->activeChanged(); break;
        case 5: _t->asynchronousChanged(); break;
        case 6: _t->objectAdded((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QObject*(*)>(_a[2]))); break;
        case 7: _t->objectRemoved((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QObject*(*)>(_a[2]))); break;
        case 8: _t->d_func()->_q_createdItem((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QObject*(*)>(_a[2]))); break;
        case 9: _t->d_func()->_q_modelUpdated((*reinterpret_cast< const QQmlChangeSet(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 10: { QObject* _r = _t->objectAt((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QObject**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQmlInstantiator::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlInstantiator::modelChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQmlInstantiator::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlInstantiator::delegateChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQmlInstantiator::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlInstantiator::countChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQmlInstantiator::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlInstantiator::objectChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQmlInstantiator::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlInstantiator::activeChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQmlInstantiator::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlInstantiator::asynchronousChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQmlInstantiator::*)(int , QObject * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlInstantiator::objectAdded)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQmlInstantiator::*)(int , QObject * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQmlInstantiator::objectRemoved)) {
                *result = 7;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlComponent* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQmlInstantiator *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->isActive(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->isAsync(); break;
        case 2: *reinterpret_cast< QVariant*>(_v) = _t->model(); break;
        case 3: *reinterpret_cast< int*>(_v) = _t->count(); break;
        case 4: *reinterpret_cast< QQmlComponent**>(_v) = _t->delegate(); break;
        case 5: *reinterpret_cast< QObject**>(_v) = _t->object(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQmlInstantiator *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setActive(*reinterpret_cast< bool*>(_v)); break;
        case 1: _t->setAsync(*reinterpret_cast< bool*>(_v)); break;
        case 2: _t->setModel(*reinterpret_cast< QVariant*>(_v)); break;
        case 4: _t->setDelegate(*reinterpret_cast< QQmlComponent**>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQmlInstantiator::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQmlInstantiator.data,
    qt_meta_data_QQmlInstantiator,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQmlInstantiator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQmlInstantiator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQmlInstantiator.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QObject::qt_metacast(_clname);
}

int QQmlInstantiator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 6;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQmlInstantiator::modelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQmlInstantiator::delegateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQmlInstantiator::countChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQmlInstantiator::objectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQmlInstantiator::activeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQmlInstantiator::asynchronousChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQmlInstantiator::objectAdded(int _t1, QObject * _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QQmlInstantiator::objectRemoved(int _t1, QObject * _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
