/****************************************************************************
** Meta object code from reading C++ file 'quicktestevent_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../kde-qtdeclarative/src/imports/testlib/quicktestevent_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'quicktestevent_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickTouchEventSequence_t {
    QByteArrayData data[13];
    char stringdata0[102];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickTouchEventSequence_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickTouchEventSequence_t qt_meta_stringdata_QQuickTouchEventSequence = {
    {
QT_MOC_LITERAL(0, 0, 24), // "QQuickTouchEventSequence"
QT_MOC_LITERAL(1, 25, 11), // "QML.Element"
QT_MOC_LITERAL(2, 37, 9), // "anonymous"
QT_MOC_LITERAL(3, 47, 5), // "press"
QT_MOC_LITERAL(4, 53, 0), // ""
QT_MOC_LITERAL(5, 54, 7), // "touchId"
QT_MOC_LITERAL(6, 62, 4), // "item"
QT_MOC_LITERAL(7, 67, 1), // "x"
QT_MOC_LITERAL(8, 69, 1), // "y"
QT_MOC_LITERAL(9, 71, 4), // "move"
QT_MOC_LITERAL(10, 76, 7), // "release"
QT_MOC_LITERAL(11, 84, 10), // "stationary"
QT_MOC_LITERAL(12, 95, 6) // "commit"

    },
    "QQuickTouchEventSequence\0QML.Element\0"
    "anonymous\0press\0\0touchId\0item\0x\0y\0"
    "move\0release\0stationary\0commit"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickTouchEventSequence[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       5,   16, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,

 // slots: name, argc, parameters, tag, flags
       3,    4,   41,    4, 0x0a /* Public */,
       9,    4,   50,    4, 0x0a /* Public */,
      10,    4,   59,    4, 0x0a /* Public */,
      11,    1,   68,    4, 0x0a /* Public */,
      12,    0,   71,    4, 0x0a /* Public */,

 // slots: parameters
    QMetaType::QObjectStar, QMetaType::Int, QMetaType::QObjectStar, QMetaType::QReal, QMetaType::QReal,    5,    6,    7,    8,
    QMetaType::QObjectStar, QMetaType::Int, QMetaType::QObjectStar, QMetaType::QReal, QMetaType::QReal,    5,    6,    7,    8,
    QMetaType::QObjectStar, QMetaType::Int, QMetaType::QObjectStar, QMetaType::QReal, QMetaType::QReal,    5,    6,    7,    8,
    QMetaType::QObjectStar, QMetaType::Int,    5,
    QMetaType::QObjectStar,

       0        // eod
};

void QQuickTouchEventSequence::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickTouchEventSequence *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { QObject* _r = _t->press((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QObject*(*)>(_a[2])),(*reinterpret_cast< qreal(*)>(_a[3])),(*reinterpret_cast< qreal(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< QObject**>(_a[0]) = std::move(_r); }  break;
        case 1: { QObject* _r = _t->move((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QObject*(*)>(_a[2])),(*reinterpret_cast< qreal(*)>(_a[3])),(*reinterpret_cast< qreal(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< QObject**>(_a[0]) = std::move(_r); }  break;
        case 2: { QObject* _r = _t->release((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QObject*(*)>(_a[2])),(*reinterpret_cast< qreal(*)>(_a[3])),(*reinterpret_cast< qreal(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< QObject**>(_a[0]) = std::move(_r); }  break;
        case 3: { QObject* _r = _t->stationary((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QObject**>(_a[0]) = std::move(_r); }  break;
        case 4: { QObject* _r = _t->commit();
            if (_a[0]) *reinterpret_cast< QObject**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QQuickTouchEventSequence::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickTouchEventSequence.data,
    qt_meta_data_QQuickTouchEventSequence,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickTouchEventSequence::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickTouchEventSequence::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickTouchEventSequence.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickTouchEventSequence::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}
struct qt_meta_stringdata_QuickTestEvent_t {
    QByteArrayData data[32];
    char stringdata0[341];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QuickTestEvent_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QuickTestEvent_t qt_meta_stringdata_QuickTestEvent = {
    {
QT_MOC_LITERAL(0, 0, 14), // "QuickTestEvent"
QT_MOC_LITERAL(1, 15, 11), // "QML.Element"
QT_MOC_LITERAL(2, 27, 9), // "TestEvent"
QT_MOC_LITERAL(3, 37, 8), // "keyPress"
QT_MOC_LITERAL(4, 46, 0), // ""
QT_MOC_LITERAL(5, 47, 3), // "key"
QT_MOC_LITERAL(6, 51, 9), // "modifiers"
QT_MOC_LITERAL(7, 61, 5), // "delay"
QT_MOC_LITERAL(8, 67, 10), // "keyRelease"
QT_MOC_LITERAL(9, 78, 8), // "keyClick"
QT_MOC_LITERAL(10, 87, 12), // "keyPressChar"
QT_MOC_LITERAL(11, 100, 9), // "character"
QT_MOC_LITERAL(12, 110, 14), // "keyReleaseChar"
QT_MOC_LITERAL(13, 125, 12), // "keyClickChar"
QT_MOC_LITERAL(14, 138, 11), // "keySequence"
QT_MOC_LITERAL(15, 150, 10), // "mousePress"
QT_MOC_LITERAL(16, 161, 4), // "item"
QT_MOC_LITERAL(17, 166, 1), // "x"
QT_MOC_LITERAL(18, 168, 1), // "y"
QT_MOC_LITERAL(19, 170, 6), // "button"
QT_MOC_LITERAL(20, 177, 12), // "mouseRelease"
QT_MOC_LITERAL(21, 190, 10), // "mouseClick"
QT_MOC_LITERAL(22, 201, 16), // "mouseDoubleClick"
QT_MOC_LITERAL(23, 218, 24), // "mouseDoubleClickSequence"
QT_MOC_LITERAL(24, 243, 9), // "mouseMove"
QT_MOC_LITERAL(25, 253, 7), // "buttons"
QT_MOC_LITERAL(26, 261, 10), // "mouseWheel"
QT_MOC_LITERAL(27, 272, 6), // "xDelta"
QT_MOC_LITERAL(28, 279, 6), // "yDelta"
QT_MOC_LITERAL(29, 286, 10), // "touchEvent"
QT_MOC_LITERAL(30, 297, 25), // "QQuickTouchEventSequence*"
QT_MOC_LITERAL(31, 323, 17) // "defaultMouseDelay"

    },
    "QuickTestEvent\0QML.Element\0TestEvent\0"
    "keyPress\0\0key\0modifiers\0delay\0keyRelease\0"
    "keyClick\0keyPressChar\0character\0"
    "keyReleaseChar\0keyClickChar\0keySequence\0"
    "mousePress\0item\0x\0y\0button\0mouseRelease\0"
    "mouseClick\0mouseDoubleClick\0"
    "mouseDoubleClickSequence\0mouseMove\0"
    "buttons\0mouseWheel\0xDelta\0yDelta\0"
    "touchEvent\0QQuickTouchEventSequence*\0"
    "defaultMouseDelay"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QuickTestEvent[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      16,   16, // methods
       1,  254, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,

 // slots: name, argc, parameters, tag, flags
       3,    3,  112,    4, 0x0a /* Public */,
       8,    3,  119,    4, 0x0a /* Public */,
       9,    3,  126,    4, 0x0a /* Public */,
      10,    3,  133,    4, 0x0a /* Public */,
      12,    3,  140,    4, 0x0a /* Public */,
      13,    3,  147,    4, 0x0a /* Public */,
      14,    1,  154,    4, 0x8a /* Public | MethodRevisioned */,
      15,    6,  157,    4, 0x0a /* Public */,
      20,    6,  170,    4, 0x0a /* Public */,
      21,    6,  183,    4, 0x0a /* Public */,
      22,    6,  196,    4, 0x0a /* Public */,
      23,    6,  209,    4, 0x0a /* Public */,
      24,    5,  222,    4, 0x0a /* Public */,
      26,    8,  233,    4, 0x0a /* Public */,
      29,    1,  250,    4, 0x0a /* Public */,
      29,    0,  253,    4, 0x2a /* Public | MethodCloned */,

 // slots: revision
       0,
       0,
       0,
       0,
       0,
       0,
       2,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,

 // slots: parameters
    QMetaType::Bool, QMetaType::Int, QMetaType::Int, QMetaType::Int,    5,    6,    7,
    QMetaType::Bool, QMetaType::Int, QMetaType::Int, QMetaType::Int,    5,    6,    7,
    QMetaType::Bool, QMetaType::Int, QMetaType::Int, QMetaType::Int,    5,    6,    7,
    QMetaType::Bool, QMetaType::QString, QMetaType::Int, QMetaType::Int,   11,    6,    7,
    QMetaType::Bool, QMetaType::QString, QMetaType::Int, QMetaType::Int,   11,    6,    7,
    QMetaType::Bool, QMetaType::QString, QMetaType::Int, QMetaType::Int,   11,    6,    7,
    QMetaType::Bool, QMetaType::QVariant,   14,
    QMetaType::Bool, QMetaType::QObjectStar, QMetaType::QReal, QMetaType::QReal, QMetaType::Int, QMetaType::Int, QMetaType::Int,   16,   17,   18,   19,    6,    7,
    QMetaType::Bool, QMetaType::QObjectStar, QMetaType::QReal, QMetaType::QReal, QMetaType::Int, QMetaType::Int, QMetaType::Int,   16,   17,   18,   19,    6,    7,
    QMetaType::Bool, QMetaType::QObjectStar, QMetaType::QReal, QMetaType::QReal, QMetaType::Int, QMetaType::Int, QMetaType::Int,   16,   17,   18,   19,    6,    7,
    QMetaType::Bool, QMetaType::QObjectStar, QMetaType::QReal, QMetaType::QReal, QMetaType::Int, QMetaType::Int, QMetaType::Int,   16,   17,   18,   19,    6,    7,
    QMetaType::Bool, QMetaType::QObjectStar, QMetaType::QReal, QMetaType::QReal, QMetaType::Int, QMetaType::Int, QMetaType::Int,   16,   17,   18,   19,    6,    7,
    QMetaType::Bool, QMetaType::QObjectStar, QMetaType::QReal, QMetaType::QReal, QMetaType::Int, QMetaType::Int,   16,   17,   18,    7,   25,
    QMetaType::Bool, QMetaType::QObjectStar, QMetaType::QReal, QMetaType::QReal, QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::Int,   16,   17,   18,   25,    6,   27,   28,    7,
    0x80000000 | 30, QMetaType::QObjectStar,   16,
    0x80000000 | 30,

 // properties: name, type, flags
      31, QMetaType::Int, 0x00095801,

       0        // eod
};

void QuickTestEvent::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QuickTestEvent *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { bool _r = _t->keyPress((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 1: { bool _r = _t->keyRelease((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 2: { bool _r = _t->keyClick((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->keyPressChar((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { bool _r = _t->keyReleaseChar((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->keyClickChar((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->keySequence((*reinterpret_cast< const QVariant(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { bool _r = _t->mousePress((*reinterpret_cast< QObject*(*)>(_a[1])),(*reinterpret_cast< qreal(*)>(_a[2])),(*reinterpret_cast< qreal(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4])),(*reinterpret_cast< int(*)>(_a[5])),(*reinterpret_cast< int(*)>(_a[6])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 8: { bool _r = _t->mouseRelease((*reinterpret_cast< QObject*(*)>(_a[1])),(*reinterpret_cast< qreal(*)>(_a[2])),(*reinterpret_cast< qreal(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4])),(*reinterpret_cast< int(*)>(_a[5])),(*reinterpret_cast< int(*)>(_a[6])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 9: { bool _r = _t->mouseClick((*reinterpret_cast< QObject*(*)>(_a[1])),(*reinterpret_cast< qreal(*)>(_a[2])),(*reinterpret_cast< qreal(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4])),(*reinterpret_cast< int(*)>(_a[5])),(*reinterpret_cast< int(*)>(_a[6])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 10: { bool _r = _t->mouseDoubleClick((*reinterpret_cast< QObject*(*)>(_a[1])),(*reinterpret_cast< qreal(*)>(_a[2])),(*reinterpret_cast< qreal(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4])),(*reinterpret_cast< int(*)>(_a[5])),(*reinterpret_cast< int(*)>(_a[6])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 11: { bool _r = _t->mouseDoubleClickSequence((*reinterpret_cast< QObject*(*)>(_a[1])),(*reinterpret_cast< qreal(*)>(_a[2])),(*reinterpret_cast< qreal(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4])),(*reinterpret_cast< int(*)>(_a[5])),(*reinterpret_cast< int(*)>(_a[6])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 12: { bool _r = _t->mouseMove((*reinterpret_cast< QObject*(*)>(_a[1])),(*reinterpret_cast< qreal(*)>(_a[2])),(*reinterpret_cast< qreal(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4])),(*reinterpret_cast< int(*)>(_a[5])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 13: { bool _r = _t->mouseWheel((*reinterpret_cast< QObject*(*)>(_a[1])),(*reinterpret_cast< qreal(*)>(_a[2])),(*reinterpret_cast< qreal(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4])),(*reinterpret_cast< int(*)>(_a[5])),(*reinterpret_cast< int(*)>(_a[6])),(*reinterpret_cast< int(*)>(_a[7])),(*reinterpret_cast< int(*)>(_a[8])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 14: { QQuickTouchEventSequence* _r = _t->touchEvent((*reinterpret_cast< QObject*(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QQuickTouchEventSequence**>(_a[0]) = std::move(_r); }  break;
        case 15: { QQuickTouchEventSequence* _r = _t->touchEvent();
            if (_a[0]) *reinterpret_cast< QQuickTouchEventSequence**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QuickTestEvent *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->defaultMouseDelay(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QuickTestEvent::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QuickTestEvent.data,
    qt_meta_data_QuickTestEvent,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QuickTestEvent::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QuickTestEvent::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QuickTestEvent.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QuickTestEvent::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 16;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
