/****************************************************************************
** Meta object code from reading C++ file 'qqmlengine_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/qml/qml/qqmlengine_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qqmlengine_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QObjectForeign_t {
    QByteArrayData data[7];
    char stringdata0[69];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QObjectForeign_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QObjectForeign_t qt_meta_stringdata_QObjectForeign = {
    {
QT_MOC_LITERAL(0, 0, 14), // "QObjectForeign"
QT_MOC_LITERAL(1, 15, 11), // "QML.Foreign"
QT_MOC_LITERAL(2, 27, 7), // "QObject"
QT_MOC_LITERAL(3, 35, 11), // "QML.Element"
QT_MOC_LITERAL(4, 47, 8), // "QtObject"
QT_MOC_LITERAL(5, 56, 8), // "QML.Root"
QT_MOC_LITERAL(6, 65, 3) // "QML"

    },
    "QObjectForeign\0QML.Foreign\0QObject\0"
    "QML.Element\0QtObject\0QML.Root\0QML"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QObjectForeign[] = {

 // content:
       8,       // revision
       0,       // classname
       3,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject QObjectForeign::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_QObjectForeign.data,
    qt_meta_data_QObjectForeign,
    nullptr,
    nullptr,
    nullptr
} };

QT_WARNING_POP
QT_END_MOC_NAMESPACE
