/****************************************************************************
** Meta object code from reading C++ file 'qquicktextinput_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/quick/items/qquicktextinput_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquicktextinput_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickTextInput_t {
    QByteArrayData data[173];
    char stringdata0[2554];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickTextInput_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickTextInput_t qt_meta_stringdata_QQuickTextInput = {
    {
QT_MOC_LITERAL(0, 0, 15), // "QQuickTextInput"
QT_MOC_LITERAL(1, 16, 11), // "QML.Element"
QT_MOC_LITERAL(2, 28, 9), // "TextInput"
QT_MOC_LITERAL(3, 38, 11), // "textChanged"
QT_MOC_LITERAL(4, 50, 0), // ""
QT_MOC_LITERAL(5, 51, 21), // "cursorPositionChanged"
QT_MOC_LITERAL(6, 73, 22), // "cursorRectangleChanged"
QT_MOC_LITERAL(7, 96, 21), // "selectionStartChanged"
QT_MOC_LITERAL(8, 118, 19), // "selectionEndChanged"
QT_MOC_LITERAL(9, 138, 19), // "selectedTextChanged"
QT_MOC_LITERAL(10, 158, 8), // "accepted"
QT_MOC_LITERAL(11, 167, 22), // "acceptableInputChanged"
QT_MOC_LITERAL(12, 190, 15), // "editingFinished"
QT_MOC_LITERAL(13, 206, 10), // "textEdited"
QT_MOC_LITERAL(14, 217, 12), // "colorChanged"
QT_MOC_LITERAL(15, 230, 21), // "selectionColorChanged"
QT_MOC_LITERAL(16, 252, 24), // "selectedTextColorChanged"
QT_MOC_LITERAL(17, 277, 11), // "fontChanged"
QT_MOC_LITERAL(18, 289, 4), // "font"
QT_MOC_LITERAL(19, 294, 26), // "horizontalAlignmentChanged"
QT_MOC_LITERAL(20, 321, 27), // "QQuickTextInput::HAlignment"
QT_MOC_LITERAL(21, 349, 9), // "alignment"
QT_MOC_LITERAL(22, 359, 24), // "verticalAlignmentChanged"
QT_MOC_LITERAL(23, 384, 27), // "QQuickTextInput::VAlignment"
QT_MOC_LITERAL(24, 412, 15), // "wrapModeChanged"
QT_MOC_LITERAL(25, 428, 15), // "readOnlyChanged"
QT_MOC_LITERAL(26, 444, 10), // "isReadOnly"
QT_MOC_LITERAL(27, 455, 20), // "cursorVisibleChanged"
QT_MOC_LITERAL(28, 476, 15), // "isCursorVisible"
QT_MOC_LITERAL(29, 492, 21), // "cursorDelegateChanged"
QT_MOC_LITERAL(30, 514, 20), // "overwriteModeChanged"
QT_MOC_LITERAL(31, 535, 13), // "overwriteMode"
QT_MOC_LITERAL(32, 549, 20), // "maximumLengthChanged"
QT_MOC_LITERAL(33, 570, 13), // "maximumLength"
QT_MOC_LITERAL(34, 584, 16), // "validatorChanged"
QT_MOC_LITERAL(35, 601, 16), // "inputMaskChanged"
QT_MOC_LITERAL(36, 618, 9), // "inputMask"
QT_MOC_LITERAL(37, 628, 15), // "echoModeChanged"
QT_MOC_LITERAL(38, 644, 25), // "QQuickTextInput::EchoMode"
QT_MOC_LITERAL(39, 670, 8), // "echoMode"
QT_MOC_LITERAL(40, 679, 24), // "passwordCharacterChanged"
QT_MOC_LITERAL(41, 704, 24), // "passwordMaskDelayChanged"
QT_MOC_LITERAL(42, 729, 5), // "delay"
QT_MOC_LITERAL(43, 735, 18), // "displayTextChanged"
QT_MOC_LITERAL(44, 754, 18), // "preeditTextChanged"
QT_MOC_LITERAL(45, 773, 25), // "activeFocusOnPressChanged"
QT_MOC_LITERAL(46, 799, 18), // "activeFocusOnPress"
QT_MOC_LITERAL(47, 818, 17), // "autoScrollChanged"
QT_MOC_LITERAL(48, 836, 10), // "autoScroll"
QT_MOC_LITERAL(49, 847, 20), // "selectByMouseChanged"
QT_MOC_LITERAL(50, 868, 13), // "selectByMouse"
QT_MOC_LITERAL(51, 882, 25), // "mouseSelectionModeChanged"
QT_MOC_LITERAL(52, 908, 30), // "QQuickTextInput::SelectionMode"
QT_MOC_LITERAL(53, 939, 4), // "mode"
QT_MOC_LITERAL(54, 944, 26), // "persistentSelectionChanged"
QT_MOC_LITERAL(55, 971, 15), // "canPasteChanged"
QT_MOC_LITERAL(56, 987, 14), // "canUndoChanged"
QT_MOC_LITERAL(57, 1002, 14), // "canRedoChanged"
QT_MOC_LITERAL(58, 1017, 27), // "inputMethodComposingChanged"
QT_MOC_LITERAL(59, 1045, 35), // "effectiveHorizontalAlignmentC..."
QT_MOC_LITERAL(60, 1081, 18), // "contentSizeChanged"
QT_MOC_LITERAL(61, 1100, 23), // "inputMethodHintsChanged"
QT_MOC_LITERAL(62, 1124, 17), // "renderTypeChanged"
QT_MOC_LITERAL(63, 1142, 14), // "paddingChanged"
QT_MOC_LITERAL(64, 1157, 17), // "topPaddingChanged"
QT_MOC_LITERAL(65, 1175, 18), // "leftPaddingChanged"
QT_MOC_LITERAL(66, 1194, 19), // "rightPaddingChanged"
QT_MOC_LITERAL(67, 1214, 20), // "bottomPaddingChanged"
QT_MOC_LITERAL(68, 1235, 9), // "selectAll"
QT_MOC_LITERAL(69, 1245, 10), // "selectWord"
QT_MOC_LITERAL(70, 1256, 6), // "select"
QT_MOC_LITERAL(71, 1263, 5), // "start"
QT_MOC_LITERAL(72, 1269, 3), // "end"
QT_MOC_LITERAL(73, 1273, 8), // "deselect"
QT_MOC_LITERAL(74, 1282, 13), // "isRightToLeft"
QT_MOC_LITERAL(75, 1296, 3), // "cut"
QT_MOC_LITERAL(76, 1300, 4), // "copy"
QT_MOC_LITERAL(77, 1305, 5), // "paste"
QT_MOC_LITERAL(78, 1311, 4), // "undo"
QT_MOC_LITERAL(79, 1316, 4), // "redo"
QT_MOC_LITERAL(80, 1321, 6), // "insert"
QT_MOC_LITERAL(81, 1328, 8), // "position"
QT_MOC_LITERAL(82, 1337, 4), // "text"
QT_MOC_LITERAL(83, 1342, 6), // "remove"
QT_MOC_LITERAL(84, 1349, 13), // "ensureVisible"
QT_MOC_LITERAL(85, 1363, 5), // "clear"
QT_MOC_LITERAL(86, 1369, 16), // "selectionChanged"
QT_MOC_LITERAL(87, 1386, 12), // "createCursor"
QT_MOC_LITERAL(88, 1399, 21), // "updateCursorRectangle"
QT_MOC_LITERAL(89, 1421, 6), // "scroll"
QT_MOC_LITERAL(90, 1428, 17), // "q_canPasteChanged"
QT_MOC_LITERAL(91, 1446, 17), // "q_updateAlignment"
QT_MOC_LITERAL(92, 1464, 17), // "triggerPreprocess"
QT_MOC_LITERAL(93, 1482, 18), // "q_validatorChanged"
QT_MOC_LITERAL(94, 1501, 10), // "positionAt"
QT_MOC_LITERAL(95, 1512, 15), // "QQmlV4Function*"
QT_MOC_LITERAL(96, 1528, 4), // "args"
QT_MOC_LITERAL(97, 1533, 19), // "positionToRectangle"
QT_MOC_LITERAL(98, 1553, 3), // "pos"
QT_MOC_LITERAL(99, 1557, 19), // "moveCursorSelection"
QT_MOC_LITERAL(100, 1577, 13), // "SelectionMode"
QT_MOC_LITERAL(101, 1591, 16), // "inputMethodQuery"
QT_MOC_LITERAL(102, 1608, 20), // "Qt::InputMethodQuery"
QT_MOC_LITERAL(103, 1629, 5), // "query"
QT_MOC_LITERAL(104, 1635, 8), // "argument"
QT_MOC_LITERAL(105, 1644, 7), // "getText"
QT_MOC_LITERAL(106, 1652, 6), // "length"
QT_MOC_LITERAL(107, 1659, 5), // "color"
QT_MOC_LITERAL(108, 1665, 14), // "selectionColor"
QT_MOC_LITERAL(109, 1680, 17), // "selectedTextColor"
QT_MOC_LITERAL(110, 1698, 19), // "horizontalAlignment"
QT_MOC_LITERAL(111, 1718, 10), // "HAlignment"
QT_MOC_LITERAL(112, 1729, 28), // "effectiveHorizontalAlignment"
QT_MOC_LITERAL(113, 1758, 17), // "verticalAlignment"
QT_MOC_LITERAL(114, 1776, 10), // "VAlignment"
QT_MOC_LITERAL(115, 1787, 8), // "wrapMode"
QT_MOC_LITERAL(116, 1796, 8), // "WrapMode"
QT_MOC_LITERAL(117, 1805, 8), // "readOnly"
QT_MOC_LITERAL(118, 1814, 13), // "cursorVisible"
QT_MOC_LITERAL(119, 1828, 14), // "cursorPosition"
QT_MOC_LITERAL(120, 1843, 15), // "cursorRectangle"
QT_MOC_LITERAL(121, 1859, 14), // "cursorDelegate"
QT_MOC_LITERAL(122, 1874, 14), // "QQmlComponent*"
QT_MOC_LITERAL(123, 1889, 14), // "selectionStart"
QT_MOC_LITERAL(124, 1904, 12), // "selectionEnd"
QT_MOC_LITERAL(125, 1917, 12), // "selectedText"
QT_MOC_LITERAL(126, 1930, 9), // "validator"
QT_MOC_LITERAL(127, 1940, 11), // "QValidator*"
QT_MOC_LITERAL(128, 1952, 16), // "inputMethodHints"
QT_MOC_LITERAL(129, 1969, 20), // "Qt::InputMethodHints"
QT_MOC_LITERAL(130, 1990, 15), // "acceptableInput"
QT_MOC_LITERAL(131, 2006, 8), // "EchoMode"
QT_MOC_LITERAL(132, 2015, 17), // "passwordCharacter"
QT_MOC_LITERAL(133, 2033, 17), // "passwordMaskDelay"
QT_MOC_LITERAL(134, 2051, 11), // "displayText"
QT_MOC_LITERAL(135, 2063, 11), // "preeditText"
QT_MOC_LITERAL(136, 2075, 18), // "mouseSelectionMode"
QT_MOC_LITERAL(137, 2094, 19), // "persistentSelection"
QT_MOC_LITERAL(138, 2114, 8), // "canPaste"
QT_MOC_LITERAL(139, 2123, 7), // "canUndo"
QT_MOC_LITERAL(140, 2131, 7), // "canRedo"
QT_MOC_LITERAL(141, 2139, 20), // "inputMethodComposing"
QT_MOC_LITERAL(142, 2160, 12), // "contentWidth"
QT_MOC_LITERAL(143, 2173, 13), // "contentHeight"
QT_MOC_LITERAL(144, 2187, 10), // "renderType"
QT_MOC_LITERAL(145, 2198, 10), // "RenderType"
QT_MOC_LITERAL(146, 2209, 7), // "padding"
QT_MOC_LITERAL(147, 2217, 10), // "topPadding"
QT_MOC_LITERAL(148, 2228, 11), // "leftPadding"
QT_MOC_LITERAL(149, 2240, 12), // "rightPadding"
QT_MOC_LITERAL(150, 2253, 13), // "bottomPadding"
QT_MOC_LITERAL(151, 2267, 6), // "Normal"
QT_MOC_LITERAL(152, 2274, 6), // "NoEcho"
QT_MOC_LITERAL(153, 2281, 8), // "Password"
QT_MOC_LITERAL(154, 2290, 18), // "PasswordEchoOnEdit"
QT_MOC_LITERAL(155, 2309, 9), // "AlignLeft"
QT_MOC_LITERAL(156, 2319, 10), // "AlignRight"
QT_MOC_LITERAL(157, 2330, 12), // "AlignHCenter"
QT_MOC_LITERAL(158, 2343, 8), // "AlignTop"
QT_MOC_LITERAL(159, 2352, 11), // "AlignBottom"
QT_MOC_LITERAL(160, 2364, 12), // "AlignVCenter"
QT_MOC_LITERAL(161, 2377, 6), // "NoWrap"
QT_MOC_LITERAL(162, 2384, 8), // "WordWrap"
QT_MOC_LITERAL(163, 2393, 12), // "WrapAnywhere"
QT_MOC_LITERAL(164, 2406, 28), // "WrapAtWordBoundaryOrAnywhere"
QT_MOC_LITERAL(165, 2435, 4), // "Wrap"
QT_MOC_LITERAL(166, 2440, 16), // "SelectCharacters"
QT_MOC_LITERAL(167, 2457, 11), // "SelectWords"
QT_MOC_LITERAL(168, 2469, 14), // "CursorPosition"
QT_MOC_LITERAL(169, 2484, 23), // "CursorBetweenCharacters"
QT_MOC_LITERAL(170, 2508, 17), // "CursorOnCharacter"
QT_MOC_LITERAL(171, 2526, 11), // "QtRendering"
QT_MOC_LITERAL(172, 2538, 15) // "NativeRendering"

    },
    "QQuickTextInput\0QML.Element\0TextInput\0"
    "textChanged\0\0cursorPositionChanged\0"
    "cursorRectangleChanged\0selectionStartChanged\0"
    "selectionEndChanged\0selectedTextChanged\0"
    "accepted\0acceptableInputChanged\0"
    "editingFinished\0textEdited\0colorChanged\0"
    "selectionColorChanged\0selectedTextColorChanged\0"
    "fontChanged\0font\0horizontalAlignmentChanged\0"
    "QQuickTextInput::HAlignment\0alignment\0"
    "verticalAlignmentChanged\0"
    "QQuickTextInput::VAlignment\0wrapModeChanged\0"
    "readOnlyChanged\0isReadOnly\0"
    "cursorVisibleChanged\0isCursorVisible\0"
    "cursorDelegateChanged\0overwriteModeChanged\0"
    "overwriteMode\0maximumLengthChanged\0"
    "maximumLength\0validatorChanged\0"
    "inputMaskChanged\0inputMask\0echoModeChanged\0"
    "QQuickTextInput::EchoMode\0echoMode\0"
    "passwordCharacterChanged\0"
    "passwordMaskDelayChanged\0delay\0"
    "displayTextChanged\0preeditTextChanged\0"
    "activeFocusOnPressChanged\0activeFocusOnPress\0"
    "autoScrollChanged\0autoScroll\0"
    "selectByMouseChanged\0selectByMouse\0"
    "mouseSelectionModeChanged\0"
    "QQuickTextInput::SelectionMode\0mode\0"
    "persistentSelectionChanged\0canPasteChanged\0"
    "canUndoChanged\0canRedoChanged\0"
    "inputMethodComposingChanged\0"
    "effectiveHorizontalAlignmentChanged\0"
    "contentSizeChanged\0inputMethodHintsChanged\0"
    "renderTypeChanged\0paddingChanged\0"
    "topPaddingChanged\0leftPaddingChanged\0"
    "rightPaddingChanged\0bottomPaddingChanged\0"
    "selectAll\0selectWord\0select\0start\0end\0"
    "deselect\0isRightToLeft\0cut\0copy\0paste\0"
    "undo\0redo\0insert\0position\0text\0remove\0"
    "ensureVisible\0clear\0selectionChanged\0"
    "createCursor\0updateCursorRectangle\0"
    "scroll\0q_canPasteChanged\0q_updateAlignment\0"
    "triggerPreprocess\0q_validatorChanged\0"
    "positionAt\0QQmlV4Function*\0args\0"
    "positionToRectangle\0pos\0moveCursorSelection\0"
    "SelectionMode\0inputMethodQuery\0"
    "Qt::InputMethodQuery\0query\0argument\0"
    "getText\0length\0color\0selectionColor\0"
    "selectedTextColor\0horizontalAlignment\0"
    "HAlignment\0effectiveHorizontalAlignment\0"
    "verticalAlignment\0VAlignment\0wrapMode\0"
    "WrapMode\0readOnly\0cursorVisible\0"
    "cursorPosition\0cursorRectangle\0"
    "cursorDelegate\0QQmlComponent*\0"
    "selectionStart\0selectionEnd\0selectedText\0"
    "validator\0QValidator*\0inputMethodHints\0"
    "Qt::InputMethodHints\0acceptableInput\0"
    "EchoMode\0passwordCharacter\0passwordMaskDelay\0"
    "displayText\0preeditText\0mouseSelectionMode\0"
    "persistentSelection\0canPaste\0canUndo\0"
    "canRedo\0inputMethodComposing\0contentWidth\0"
    "contentHeight\0renderType\0RenderType\0"
    "padding\0topPadding\0leftPadding\0"
    "rightPadding\0bottomPadding\0Normal\0"
    "NoEcho\0Password\0PasswordEchoOnEdit\0"
    "AlignLeft\0AlignRight\0AlignHCenter\0"
    "AlignTop\0AlignBottom\0AlignVCenter\0"
    "NoWrap\0WordWrap\0WrapAnywhere\0"
    "WrapAtWordBoundaryOrAnywhere\0Wrap\0"
    "SelectCharacters\0SelectWords\0"
    "CursorPosition\0CursorBetweenCharacters\0"
    "CursorOnCharacter\0QtRendering\0"
    "NativeRendering"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickTextInput[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      75,   16, // methods
      46,  607, // properties
       7,  837, // enums/sets
       0,    0, // constructors
       0,       // flags
      47,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,  466,    4, 0x06 /* Public */,
       5,    0,  467,    4, 0x06 /* Public */,
       6,    0,  468,    4, 0x06 /* Public */,
       7,    0,  469,    4, 0x06 /* Public */,
       8,    0,  470,    4, 0x06 /* Public */,
       9,    0,  471,    4, 0x06 /* Public */,
      10,    0,  472,    4, 0x06 /* Public */,
      11,    0,  473,    4, 0x06 /* Public */,
      12,    0,  474,    4, 0x86 /* Public | MethodRevisioned */,
      13,    0,  475,    4, 0x86 /* Public | MethodRevisioned */,
      14,    0,  476,    4, 0x06 /* Public */,
      15,    0,  477,    4, 0x06 /* Public */,
      16,    0,  478,    4, 0x06 /* Public */,
      17,    1,  479,    4, 0x06 /* Public */,
      19,    1,  482,    4, 0x06 /* Public */,
      22,    1,  485,    4, 0x06 /* Public */,
      24,    0,  488,    4, 0x06 /* Public */,
      25,    1,  489,    4, 0x06 /* Public */,
      27,    1,  492,    4, 0x06 /* Public */,
      29,    0,  495,    4, 0x06 /* Public */,
      30,    1,  496,    4, 0x06 /* Public */,
      32,    1,  499,    4, 0x06 /* Public */,
      34,    0,  502,    4, 0x06 /* Public */,
      35,    1,  503,    4, 0x06 /* Public */,
      37,    1,  506,    4, 0x06 /* Public */,
      40,    0,  509,    4, 0x06 /* Public */,
      41,    1,  510,    4, 0x86 /* Public | MethodRevisioned */,
      43,    0,  513,    4, 0x06 /* Public */,
      44,    0,  514,    4, 0x86 /* Public | MethodRevisioned */,
      45,    1,  515,    4, 0x06 /* Public */,
      47,    1,  518,    4, 0x06 /* Public */,
      49,    1,  521,    4, 0x06 /* Public */,
      51,    1,  524,    4, 0x06 /* Public */,
      54,    0,  527,    4, 0x06 /* Public */,
      55,    0,  528,    4, 0x06 /* Public */,
      56,    0,  529,    4, 0x06 /* Public */,
      57,    0,  530,    4, 0x06 /* Public */,
      58,    0,  531,    4, 0x06 /* Public */,
      59,    0,  532,    4, 0x06 /* Public */,
      60,    0,  533,    4, 0x06 /* Public */,
      61,    0,  534,    4, 0x06 /* Public */,
      62,    0,  535,    4, 0x06 /* Public */,
      63,    0,  536,    4, 0x86 /* Public | MethodRevisioned */,
      64,    0,  537,    4, 0x86 /* Public | MethodRevisioned */,
      65,    0,  538,    4, 0x86 /* Public | MethodRevisioned */,
      66,    0,  539,    4, 0x86 /* Public | MethodRevisioned */,
      67,    0,  540,    4, 0x86 /* Public | MethodRevisioned */,

 // slots: name, argc, parameters, tag, flags
      68,    0,  541,    4, 0x0a /* Public */,
      69,    0,  542,    4, 0x0a /* Public */,
      70,    2,  543,    4, 0x0a /* Public */,
      73,    0,  548,    4, 0x0a /* Public */,
      74,    2,  549,    4, 0x0a /* Public */,
      75,    0,  554,    4, 0x0a /* Public */,
      76,    0,  555,    4, 0x0a /* Public */,
      77,    0,  556,    4, 0x0a /* Public */,
      78,    0,  557,    4, 0x0a /* Public */,
      79,    0,  558,    4, 0x0a /* Public */,
      80,    2,  559,    4, 0x0a /* Public */,
      83,    2,  564,    4, 0x0a /* Public */,
      84,    1,  569,    4, 0x8a /* Public | MethodRevisioned */,
      85,    0,  572,    4, 0x8a /* Public | MethodRevisioned */,
      86,    0,  573,    4, 0x08 /* Private */,
      87,    0,  574,    4, 0x08 /* Private */,
      88,    1,  575,    4, 0x08 /* Private */,
      88,    0,  578,    4, 0x28 /* Private | MethodCloned */,
      90,    0,  579,    4, 0x08 /* Private */,
      91,    0,  580,    4, 0x08 /* Private */,
      92,    0,  581,    4, 0x08 /* Private */,
      93,    0,  582,    4, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
      94,    1,  583,    4, 0x02 /* Public */,
      97,    1,  586,    4, 0x02 /* Public */,
      99,    1,  589,    4, 0x02 /* Public */,
      99,    2,  592,    4, 0x02 /* Public */,
     101,    2,  597,    4, 0x82 /* Public | MethodRevisioned */,
     105,    2,  602,    4, 0x02 /* Public */,

 // signals: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       2,
       9,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       4,
       0,
       7,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       6,
       6,
       6,
       6,
       6,

 // slots: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       4,
       7,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,

 // methods: revision
       0,
       0,
       0,
       0,
       4,
       0,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QFont,   18,
    QMetaType::Void, 0x80000000 | 20,   21,
    QMetaType::Void, 0x80000000 | 23,   21,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   26,
    QMetaType::Void, QMetaType::Bool,   28,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   31,
    QMetaType::Void, QMetaType::Int,   33,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   36,
    QMetaType::Void, 0x80000000 | 38,   39,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   42,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   46,
    QMetaType::Void, QMetaType::Bool,   48,
    QMetaType::Void, QMetaType::Bool,   50,
    QMetaType::Void, 0x80000000 | 52,   53,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   71,   72,
    QMetaType::Void,
    QMetaType::Bool, QMetaType::Int, QMetaType::Int,   71,   72,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::QString,   81,   82,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   71,   72,
    QMetaType::Void, QMetaType::Int,   81,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   89,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void, 0x80000000 | 95,   96,
    QMetaType::QRectF, QMetaType::Int,   98,
    QMetaType::Void, QMetaType::Int,   98,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 100,   98,   53,
    QMetaType::QVariant, 0x80000000 | 102, QMetaType::QVariant,  103,  104,
    QMetaType::QString, QMetaType::Int, QMetaType::Int,   71,   72,

 // properties: name, type, flags
      82, QMetaType::QString, 0x00495103,
     106, QMetaType::Int, 0x00495001,
     107, QMetaType::QColor, 0x00495103,
     108, QMetaType::QColor, 0x00495103,
     109, QMetaType::QColor, 0x00495103,
      18, QMetaType::QFont, 0x00495103,
     110, 0x80000000 | 111, 0x0049500f,
     112, 0x80000000 | 111, 0x00495009,
     113, 0x80000000 | 114, 0x0049500b,
     115, 0x80000000 | 116, 0x0049510b,
     117, QMetaType::Bool, 0x00495103,
     118, QMetaType::Bool, 0x00495103,
     119, QMetaType::Int, 0x00495103,
     120, QMetaType::QRectF, 0x00495001,
     121, 0x80000000 | 122, 0x0049510b,
      31, QMetaType::Bool, 0x00495103,
     123, QMetaType::Int, 0x00495001,
     124, QMetaType::Int, 0x00495001,
     125, QMetaType::QString, 0x00495001,
      33, QMetaType::Int, 0x00495003,
     126, 0x80000000 | 127, 0x0049510b,
      36, QMetaType::QString, 0x00495103,
     128, 0x80000000 | 129, 0x0049510b,
     130, QMetaType::Bool, 0x00495001,
      39, 0x80000000 | 131, 0x0049510b,
      46, QMetaType::Bool, 0x00495003,
     132, QMetaType::QString, 0x00495103,
     133, QMetaType::Int, 0x00c95107,
     134, QMetaType::QString, 0x00495001,
     135, QMetaType::QString, 0x00c95001,
      48, QMetaType::Bool, 0x00495103,
      50, QMetaType::Bool, 0x00495103,
     136, 0x80000000 | 100, 0x0049510b,
     137, QMetaType::Bool, 0x00495103,
     138, QMetaType::Bool, 0x00495001,
     139, QMetaType::Bool, 0x00495001,
     140, QMetaType::Bool, 0x00495001,
     141, QMetaType::Bool, 0x00495001,
     142, QMetaType::QReal, 0x00495001,
     143, QMetaType::QReal, 0x00495001,
     144, 0x80000000 | 145, 0x0049510b,
     146, QMetaType::QReal, 0x00c95107,
     147, QMetaType::QReal, 0x00c95107,
     148, QMetaType::QReal, 0x00c95107,
     149, QMetaType::QReal, 0x00c95107,
     150, QMetaType::QReal, 0x00c95107,

 // properties: notify_signal_id
       0,
       0,
      10,
      11,
      12,
      13,
      14,
      38,
      15,
      16,
      17,
      18,
       1,
       2,
      19,
      20,
       3,
       4,
       5,
      21,
      22,
      23,
      40,
       7,
      24,
      29,
      25,
      26,
      27,
      28,
      30,
      31,
      32,
      33,
      34,
      35,
      36,
      37,
      39,
      39,
      41,
      42,
      43,
      44,
      45,
      46,

 // properties: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       4,
       0,
       7,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       6,
       6,
       6,
       6,
       6,

 // enums: name, alias, flags, count, data
     131,  131, 0x0,    4,  872,
     111,  111, 0x0,    3,  880,
     114,  114, 0x0,    3,  886,
     116,  116, 0x0,    5,  892,
     100,  100, 0x0,    2,  902,
     168,  168, 0x0,    2,  906,
     145,  145, 0x0,    2,  910,

 // enum data: key, value
     151, uint(QQuickTextInput::Normal),
     152, uint(QQuickTextInput::NoEcho),
     153, uint(QQuickTextInput::Password),
     154, uint(QQuickTextInput::PasswordEchoOnEdit),
     155, uint(QQuickTextInput::AlignLeft),
     156, uint(QQuickTextInput::AlignRight),
     157, uint(QQuickTextInput::AlignHCenter),
     158, uint(QQuickTextInput::AlignTop),
     159, uint(QQuickTextInput::AlignBottom),
     160, uint(QQuickTextInput::AlignVCenter),
     161, uint(QQuickTextInput::NoWrap),
     162, uint(QQuickTextInput::WordWrap),
     163, uint(QQuickTextInput::WrapAnywhere),
     164, uint(QQuickTextInput::WrapAtWordBoundaryOrAnywhere),
     165, uint(QQuickTextInput::Wrap),
     166, uint(QQuickTextInput::SelectCharacters),
     167, uint(QQuickTextInput::SelectWords),
     169, uint(QQuickTextInput::CursorBetweenCharacters),
     170, uint(QQuickTextInput::CursorOnCharacter),
     171, uint(QQuickTextInput::QtRendering),
     172, uint(QQuickTextInput::NativeRendering),

       0        // eod
};

void QQuickTextInput::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickTextInput *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->textChanged(); break;
        case 1: _t->cursorPositionChanged(); break;
        case 2: _t->cursorRectangleChanged(); break;
        case 3: _t->selectionStartChanged(); break;
        case 4: _t->selectionEndChanged(); break;
        case 5: _t->selectedTextChanged(); break;
        case 6: _t->accepted(); break;
        case 7: _t->acceptableInputChanged(); break;
        case 8: _t->editingFinished(); break;
        case 9: _t->textEdited(); break;
        case 10: _t->colorChanged(); break;
        case 11: _t->selectionColorChanged(); break;
        case 12: _t->selectedTextColorChanged(); break;
        case 13: _t->fontChanged((*reinterpret_cast< const QFont(*)>(_a[1]))); break;
        case 14: _t->horizontalAlignmentChanged((*reinterpret_cast< QQuickTextInput::HAlignment(*)>(_a[1]))); break;
        case 15: _t->verticalAlignmentChanged((*reinterpret_cast< QQuickTextInput::VAlignment(*)>(_a[1]))); break;
        case 16: _t->wrapModeChanged(); break;
        case 17: _t->readOnlyChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 18: _t->cursorVisibleChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 19: _t->cursorDelegateChanged(); break;
        case 20: _t->overwriteModeChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 21: _t->maximumLengthChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 22: _t->validatorChanged(); break;
        case 23: _t->inputMaskChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 24: _t->echoModeChanged((*reinterpret_cast< QQuickTextInput::EchoMode(*)>(_a[1]))); break;
        case 25: _t->passwordCharacterChanged(); break;
        case 26: _t->passwordMaskDelayChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 27: _t->displayTextChanged(); break;
        case 28: _t->preeditTextChanged(); break;
        case 29: _t->activeFocusOnPressChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 30: _t->autoScrollChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 31: _t->selectByMouseChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 32: _t->mouseSelectionModeChanged((*reinterpret_cast< QQuickTextInput::SelectionMode(*)>(_a[1]))); break;
        case 33: _t->persistentSelectionChanged(); break;
        case 34: _t->canPasteChanged(); break;
        case 35: _t->canUndoChanged(); break;
        case 36: _t->canRedoChanged(); break;
        case 37: _t->inputMethodComposingChanged(); break;
        case 38: _t->effectiveHorizontalAlignmentChanged(); break;
        case 39: _t->contentSizeChanged(); break;
        case 40: _t->inputMethodHintsChanged(); break;
        case 41: _t->renderTypeChanged(); break;
        case 42: _t->paddingChanged(); break;
        case 43: _t->topPaddingChanged(); break;
        case 44: _t->leftPaddingChanged(); break;
        case 45: _t->rightPaddingChanged(); break;
        case 46: _t->bottomPaddingChanged(); break;
        case 47: _t->selectAll(); break;
        case 48: _t->selectWord(); break;
        case 49: _t->select((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 50: _t->deselect(); break;
        case 51: { bool _r = _t->isRightToLeft((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 52: _t->cut(); break;
        case 53: _t->copy(); break;
        case 54: _t->paste(); break;
        case 55: _t->undo(); break;
        case 56: _t->redo(); break;
        case 57: _t->insert((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 58: _t->remove((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 59: _t->ensureVisible((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 60: _t->clear(); break;
        case 61: _t->selectionChanged(); break;
        case 62: _t->createCursor(); break;
        case 63: _t->updateCursorRectangle((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 64: _t->updateCursorRectangle(); break;
        case 65: _t->q_canPasteChanged(); break;
        case 66: _t->q_updateAlignment(); break;
        case 67: _t->triggerPreprocess(); break;
        case 68: _t->q_validatorChanged(); break;
        case 69: _t->positionAt((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        case 70: { QRectF _r = _t->positionToRectangle((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QRectF*>(_a[0]) = std::move(_r); }  break;
        case 71: _t->moveCursorSelection((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 72: _t->moveCursorSelection((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< SelectionMode(*)>(_a[2]))); break;
        case 73: { QVariant _r = _t->inputMethodQuery((*reinterpret_cast< Qt::InputMethodQuery(*)>(_a[1])),(*reinterpret_cast< const QVariant(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QVariant*>(_a[0]) = std::move(_r); }  break;
        case 74: { QString _r = _t->getText((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::textChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::cursorPositionChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::cursorRectangleChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::selectionStartChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::selectionEndChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::selectedTextChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::accepted)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::acceptableInputChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::editingFinished)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::textEdited)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::colorChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::selectionColorChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::selectedTextColorChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)(const QFont & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::fontChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)(QQuickTextInput::HAlignment );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::horizontalAlignmentChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)(QQuickTextInput::VAlignment );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::verticalAlignmentChanged)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::wrapModeChanged)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::readOnlyChanged)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::cursorVisibleChanged)) {
                *result = 18;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::cursorDelegateChanged)) {
                *result = 19;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::overwriteModeChanged)) {
                *result = 20;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::maximumLengthChanged)) {
                *result = 21;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::validatorChanged)) {
                *result = 22;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::inputMaskChanged)) {
                *result = 23;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)(QQuickTextInput::EchoMode );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::echoModeChanged)) {
                *result = 24;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::passwordCharacterChanged)) {
                *result = 25;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::passwordMaskDelayChanged)) {
                *result = 26;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::displayTextChanged)) {
                *result = 27;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::preeditTextChanged)) {
                *result = 28;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::activeFocusOnPressChanged)) {
                *result = 29;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::autoScrollChanged)) {
                *result = 30;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::selectByMouseChanged)) {
                *result = 31;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)(QQuickTextInput::SelectionMode );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::mouseSelectionModeChanged)) {
                *result = 32;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::persistentSelectionChanged)) {
                *result = 33;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::canPasteChanged)) {
                *result = 34;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::canUndoChanged)) {
                *result = 35;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::canRedoChanged)) {
                *result = 36;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::inputMethodComposingChanged)) {
                *result = 37;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::effectiveHorizontalAlignmentChanged)) {
                *result = 38;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::contentSizeChanged)) {
                *result = 39;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::inputMethodHintsChanged)) {
                *result = 40;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::renderTypeChanged)) {
                *result = 41;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::paddingChanged)) {
                *result = 42;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::topPaddingChanged)) {
                *result = 43;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::leftPaddingChanged)) {
                *result = 44;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::rightPaddingChanged)) {
                *result = 45;
                return;
            }
        }
        {
            using _t = void (QQuickTextInput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickTextInput::bottomPaddingChanged)) {
                *result = 46;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 14:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlComponent* >(); break;
        case 20:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QValidator* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickTextInput *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->text(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->length(); break;
        case 2: *reinterpret_cast< QColor*>(_v) = _t->color(); break;
        case 3: *reinterpret_cast< QColor*>(_v) = _t->selectionColor(); break;
        case 4: *reinterpret_cast< QColor*>(_v) = _t->selectedTextColor(); break;
        case 5: *reinterpret_cast< QFont*>(_v) = _t->font(); break;
        case 6: *reinterpret_cast< HAlignment*>(_v) = _t->hAlign(); break;
        case 7: *reinterpret_cast< HAlignment*>(_v) = _t->effectiveHAlign(); break;
        case 8: *reinterpret_cast< VAlignment*>(_v) = _t->vAlign(); break;
        case 9: *reinterpret_cast< WrapMode*>(_v) = _t->wrapMode(); break;
        case 10: *reinterpret_cast< bool*>(_v) = _t->isReadOnly(); break;
        case 11: *reinterpret_cast< bool*>(_v) = _t->isCursorVisible(); break;
        case 12: *reinterpret_cast< int*>(_v) = _t->cursorPosition(); break;
        case 13: *reinterpret_cast< QRectF*>(_v) = _t->cursorRectangle(); break;
        case 14: *reinterpret_cast< QQmlComponent**>(_v) = _t->cursorDelegate(); break;
        case 15: *reinterpret_cast< bool*>(_v) = _t->overwriteMode(); break;
        case 16: *reinterpret_cast< int*>(_v) = _t->selectionStart(); break;
        case 17: *reinterpret_cast< int*>(_v) = _t->selectionEnd(); break;
        case 18: *reinterpret_cast< QString*>(_v) = _t->selectedText(); break;
        case 19: *reinterpret_cast< int*>(_v) = _t->maxLength(); break;
        case 20: *reinterpret_cast< QValidator**>(_v) = _t->validator(); break;
        case 21: *reinterpret_cast< QString*>(_v) = _t->inputMask(); break;
        case 22: *reinterpret_cast< Qt::InputMethodHints*>(_v) = _t->inputMethodHints(); break;
        case 23: *reinterpret_cast< bool*>(_v) = _t->hasAcceptableInput(); break;
        case 24: *reinterpret_cast< EchoMode*>(_v) = _t->echoMode(); break;
        case 25: *reinterpret_cast< bool*>(_v) = _t->focusOnPress(); break;
        case 26: *reinterpret_cast< QString*>(_v) = _t->passwordCharacter(); break;
        case 27: *reinterpret_cast< int*>(_v) = _t->passwordMaskDelay(); break;
        case 28: *reinterpret_cast< QString*>(_v) = _t->displayText(); break;
        case 29: *reinterpret_cast< QString*>(_v) = _t->preeditText(); break;
        case 30: *reinterpret_cast< bool*>(_v) = _t->autoScroll(); break;
        case 31: *reinterpret_cast< bool*>(_v) = _t->selectByMouse(); break;
        case 32: *reinterpret_cast< SelectionMode*>(_v) = _t->mouseSelectionMode(); break;
        case 33: *reinterpret_cast< bool*>(_v) = _t->persistentSelection(); break;
        case 34: *reinterpret_cast< bool*>(_v) = _t->canPaste(); break;
        case 35: *reinterpret_cast< bool*>(_v) = _t->canUndo(); break;
        case 36: *reinterpret_cast< bool*>(_v) = _t->canRedo(); break;
        case 37: *reinterpret_cast< bool*>(_v) = _t->isInputMethodComposing(); break;
        case 38: *reinterpret_cast< qreal*>(_v) = _t->contentWidth(); break;
        case 39: *reinterpret_cast< qreal*>(_v) = _t->contentHeight(); break;
        case 40: *reinterpret_cast< RenderType*>(_v) = _t->renderType(); break;
        case 41: *reinterpret_cast< qreal*>(_v) = _t->padding(); break;
        case 42: *reinterpret_cast< qreal*>(_v) = _t->topPadding(); break;
        case 43: *reinterpret_cast< qreal*>(_v) = _t->leftPadding(); break;
        case 44: *reinterpret_cast< qreal*>(_v) = _t->rightPadding(); break;
        case 45: *reinterpret_cast< qreal*>(_v) = _t->bottomPadding(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickTextInput *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setText(*reinterpret_cast< QString*>(_v)); break;
        case 2: _t->setColor(*reinterpret_cast< QColor*>(_v)); break;
        case 3: _t->setSelectionColor(*reinterpret_cast< QColor*>(_v)); break;
        case 4: _t->setSelectedTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 5: _t->setFont(*reinterpret_cast< QFont*>(_v)); break;
        case 6: _t->setHAlign(*reinterpret_cast< HAlignment*>(_v)); break;
        case 8: _t->setVAlign(*reinterpret_cast< VAlignment*>(_v)); break;
        case 9: _t->setWrapMode(*reinterpret_cast< WrapMode*>(_v)); break;
        case 10: _t->setReadOnly(*reinterpret_cast< bool*>(_v)); break;
        case 11: _t->setCursorVisible(*reinterpret_cast< bool*>(_v)); break;
        case 12: _t->setCursorPosition(*reinterpret_cast< int*>(_v)); break;
        case 14: _t->setCursorDelegate(*reinterpret_cast< QQmlComponent**>(_v)); break;
        case 15: _t->setOverwriteMode(*reinterpret_cast< bool*>(_v)); break;
        case 19: _t->setMaxLength(*reinterpret_cast< int*>(_v)); break;
        case 20: _t->setValidator(*reinterpret_cast< QValidator**>(_v)); break;
        case 21: _t->setInputMask(*reinterpret_cast< QString*>(_v)); break;
        case 22: _t->setInputMethodHints(*reinterpret_cast< Qt::InputMethodHints*>(_v)); break;
        case 24: _t->setEchoMode(*reinterpret_cast< EchoMode*>(_v)); break;
        case 25: _t->setFocusOnPress(*reinterpret_cast< bool*>(_v)); break;
        case 26: _t->setPasswordCharacter(*reinterpret_cast< QString*>(_v)); break;
        case 27: _t->setPasswordMaskDelay(*reinterpret_cast< int*>(_v)); break;
        case 30: _t->setAutoScroll(*reinterpret_cast< bool*>(_v)); break;
        case 31: _t->setSelectByMouse(*reinterpret_cast< bool*>(_v)); break;
        case 32: _t->setMouseSelectionMode(*reinterpret_cast< SelectionMode*>(_v)); break;
        case 33: _t->setPersistentSelection(*reinterpret_cast< bool*>(_v)); break;
        case 40: _t->setRenderType(*reinterpret_cast< RenderType*>(_v)); break;
        case 41: _t->setPadding(*reinterpret_cast< qreal*>(_v)); break;
        case 42: _t->setTopPadding(*reinterpret_cast< qreal*>(_v)); break;
        case 43: _t->setLeftPadding(*reinterpret_cast< qreal*>(_v)); break;
        case 44: _t->setRightPadding(*reinterpret_cast< qreal*>(_v)); break;
        case 45: _t->setBottomPadding(*reinterpret_cast< qreal*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        QQuickTextInput *_t = static_cast<QQuickTextInput *>(_o);
        (void)_t;
        switch (_id) {
        case 6: _t->resetHAlign(); break;
        case 27: _t->resetPasswordMaskDelay(); break;
        case 41: _t->resetPadding(); break;
        case 42: _t->resetTopPadding(); break;
        case 43: _t->resetLeftPadding(); break;
        case 44: _t->resetRightPadding(); break;
        case 45: _t->resetBottomPadding(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickTextInput::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickImplicitSizeItem::staticMetaObject>(),
    qt_meta_stringdata_QQuickTextInput.data,
    qt_meta_data_QQuickTextInput,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickTextInput::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickTextInput::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickTextInput.stringdata0))
        return static_cast<void*>(this);
    return QQuickImplicitSizeItem::qt_metacast(_clname);
}

int QQuickTextInput::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickImplicitSizeItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 75)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 75;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 75)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 75;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 46;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 46;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 46;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 46;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 46;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 46;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickTextInput::textChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickTextInput::cursorPositionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickTextInput::cursorRectangleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickTextInput::selectionStartChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickTextInput::selectionEndChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickTextInput::selectedTextChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickTextInput::accepted()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQuickTextInput::acceptableInputChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QQuickTextInput::editingFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QQuickTextInput::textEdited()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QQuickTextInput::colorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QQuickTextInput::selectionColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void QQuickTextInput::selectedTextColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void QQuickTextInput::fontChanged(const QFont & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void QQuickTextInput::horizontalAlignmentChanged(QQuickTextInput::HAlignment _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void QQuickTextInput::verticalAlignmentChanged(QQuickTextInput::VAlignment _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void QQuickTextInput::wrapModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void QQuickTextInput::readOnlyChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 17, _a);
}

// SIGNAL 18
void QQuickTextInput::cursorVisibleChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 18, _a);
}

// SIGNAL 19
void QQuickTextInput::cursorDelegateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void QQuickTextInput::overwriteModeChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 20, _a);
}

// SIGNAL 21
void QQuickTextInput::maximumLengthChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 21, _a);
}

// SIGNAL 22
void QQuickTextInput::validatorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 22, nullptr);
}

// SIGNAL 23
void QQuickTextInput::inputMaskChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 23, _a);
}

// SIGNAL 24
void QQuickTextInput::echoModeChanged(QQuickTextInput::EchoMode _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 24, _a);
}

// SIGNAL 25
void QQuickTextInput::passwordCharacterChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 25, nullptr);
}

// SIGNAL 26
void QQuickTextInput::passwordMaskDelayChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 26, _a);
}

// SIGNAL 27
void QQuickTextInput::displayTextChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 27, nullptr);
}

// SIGNAL 28
void QQuickTextInput::preeditTextChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 28, nullptr);
}

// SIGNAL 29
void QQuickTextInput::activeFocusOnPressChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 29, _a);
}

// SIGNAL 30
void QQuickTextInput::autoScrollChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 30, _a);
}

// SIGNAL 31
void QQuickTextInput::selectByMouseChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 31, _a);
}

// SIGNAL 32
void QQuickTextInput::mouseSelectionModeChanged(QQuickTextInput::SelectionMode _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 32, _a);
}

// SIGNAL 33
void QQuickTextInput::persistentSelectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 33, nullptr);
}

// SIGNAL 34
void QQuickTextInput::canPasteChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 34, nullptr);
}

// SIGNAL 35
void QQuickTextInput::canUndoChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 35, nullptr);
}

// SIGNAL 36
void QQuickTextInput::canRedoChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 36, nullptr);
}

// SIGNAL 37
void QQuickTextInput::inputMethodComposingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 37, nullptr);
}

// SIGNAL 38
void QQuickTextInput::effectiveHorizontalAlignmentChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 38, nullptr);
}

// SIGNAL 39
void QQuickTextInput::contentSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 39, nullptr);
}

// SIGNAL 40
void QQuickTextInput::inputMethodHintsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 40, nullptr);
}

// SIGNAL 41
void QQuickTextInput::renderTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 41, nullptr);
}

// SIGNAL 42
void QQuickTextInput::paddingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 42, nullptr);
}

// SIGNAL 43
void QQuickTextInput::topPaddingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 43, nullptr);
}

// SIGNAL 44
void QQuickTextInput::leftPaddingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 44, nullptr);
}

// SIGNAL 45
void QQuickTextInput::rightPaddingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 45, nullptr);
}

// SIGNAL 46
void QQuickTextInput::bottomPaddingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 46, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
