/****************************************************************************
** Meta object code from reading C++ file 'qquickpinchhandler_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/quick/handlers/qquickpinchhandler_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickpinchhandler_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickPinchHandler_t {
    QByteArrayData data[30];
    char stringdata0[391];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickPinchHandler_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickPinchHandler_t qt_meta_stringdata_QQuickPinchHandler = {
    {
QT_MOC_LITERAL(0, 0, 18), // "QQuickPinchHandler"
QT_MOC_LITERAL(1, 19, 11), // "QML.Element"
QT_MOC_LITERAL(2, 31, 12), // "PinchHandler"
QT_MOC_LITERAL(3, 44, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(4, 68, 2), // "12"
QT_MOC_LITERAL(5, 71, 19), // "minimumScaleChanged"
QT_MOC_LITERAL(6, 91, 0), // ""
QT_MOC_LITERAL(7, 92, 19), // "maximumScaleChanged"
QT_MOC_LITERAL(8, 112, 22), // "minimumRotationChanged"
QT_MOC_LITERAL(9, 135, 22), // "maximumRotationChanged"
QT_MOC_LITERAL(10, 158, 15), // "minimumXChanged"
QT_MOC_LITERAL(11, 174, 15), // "maximumXChanged"
QT_MOC_LITERAL(12, 190, 15), // "minimumYChanged"
QT_MOC_LITERAL(13, 206, 15), // "maximumYChanged"
QT_MOC_LITERAL(14, 222, 7), // "updated"
QT_MOC_LITERAL(15, 230, 12), // "minimumScale"
QT_MOC_LITERAL(16, 243, 12), // "maximumScale"
QT_MOC_LITERAL(17, 256, 15), // "minimumRotation"
QT_MOC_LITERAL(18, 272, 15), // "maximumRotation"
QT_MOC_LITERAL(19, 288, 5), // "scale"
QT_MOC_LITERAL(20, 294, 11), // "activeScale"
QT_MOC_LITERAL(21, 306, 8), // "rotation"
QT_MOC_LITERAL(22, 315, 11), // "translation"
QT_MOC_LITERAL(23, 327, 8), // "minimumX"
QT_MOC_LITERAL(24, 336, 8), // "maximumX"
QT_MOC_LITERAL(25, 345, 8), // "minimumY"
QT_MOC_LITERAL(26, 354, 8), // "maximumY"
QT_MOC_LITERAL(27, 363, 5), // "xAxis"
QT_MOC_LITERAL(28, 369, 15), // "QQuickDragAxis*"
QT_MOC_LITERAL(29, 385, 5) // "yAxis"

    },
    "QQuickPinchHandler\0QML.Element\0"
    "PinchHandler\0QML.AddedInMinorVersion\0"
    "12\0minimumScaleChanged\0\0maximumScaleChanged\0"
    "minimumRotationChanged\0maximumRotationChanged\0"
    "minimumXChanged\0maximumXChanged\0"
    "minimumYChanged\0maximumYChanged\0updated\0"
    "minimumScale\0maximumScale\0minimumRotation\0"
    "maximumRotation\0scale\0activeScale\0"
    "rotation\0translation\0minimumX\0maximumX\0"
    "minimumY\0maximumY\0xAxis\0QQuickDragAxis*\0"
    "yAxis"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickPinchHandler[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       9,   18, // methods
      14,   72, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       9,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

 // signals: name, argc, parameters, tag, flags
       5,    0,   63,    6, 0x06 /* Public */,
       7,    0,   64,    6, 0x06 /* Public */,
       8,    0,   65,    6, 0x06 /* Public */,
       9,    0,   66,    6, 0x06 /* Public */,
      10,    0,   67,    6, 0x06 /* Public */,
      11,    0,   68,    6, 0x06 /* Public */,
      12,    0,   69,    6, 0x06 /* Public */,
      13,    0,   70,    6, 0x06 /* Public */,
      14,    0,   71,    6, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      15, QMetaType::QReal, 0x00495103,
      16, QMetaType::QReal, 0x00495103,
      17, QMetaType::QReal, 0x00495103,
      18, QMetaType::QReal, 0x00495103,
      19, QMetaType::QReal, 0x00495001,
      20, QMetaType::QReal, 0x00495001,
      21, QMetaType::QReal, 0x00495001,
      22, QMetaType::QVector2D, 0x00495001,
      23, QMetaType::QReal, 0x00495103,
      24, QMetaType::QReal, 0x00495103,
      25, QMetaType::QReal, 0x00495103,
      26, QMetaType::QReal, 0x00495103,
      27, 0x80000000 | 28, 0x00095409,
      29, 0x80000000 | 28, 0x00095409,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       8,
       8,
       8,
       8,
       4,
       5,
       6,
       7,
       0,
       0,

       0        // eod
};

void QQuickPinchHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickPinchHandler *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->minimumScaleChanged(); break;
        case 1: _t->maximumScaleChanged(); break;
        case 2: _t->minimumRotationChanged(); break;
        case 3: _t->maximumRotationChanged(); break;
        case 4: _t->minimumXChanged(); break;
        case 5: _t->maximumXChanged(); break;
        case 6: _t->minimumYChanged(); break;
        case 7: _t->maximumYChanged(); break;
        case 8: _t->updated(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickPinchHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPinchHandler::minimumScaleChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickPinchHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPinchHandler::maximumScaleChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickPinchHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPinchHandler::minimumRotationChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickPinchHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPinchHandler::maximumRotationChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickPinchHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPinchHandler::minimumXChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickPinchHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPinchHandler::maximumXChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickPinchHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPinchHandler::minimumYChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickPinchHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPinchHandler::maximumYChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickPinchHandler::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPinchHandler::updated)) {
                *result = 8;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 13:
        case 12:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickDragAxis* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickPinchHandler *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< qreal*>(_v) = _t->minimumScale(); break;
        case 1: *reinterpret_cast< qreal*>(_v) = _t->maximumScale(); break;
        case 2: *reinterpret_cast< qreal*>(_v) = _t->minimumRotation(); break;
        case 3: *reinterpret_cast< qreal*>(_v) = _t->maximumRotation(); break;
        case 4: *reinterpret_cast< qreal*>(_v) = _t->scale(); break;
        case 5: *reinterpret_cast< qreal*>(_v) = _t->activeScale(); break;
        case 6: *reinterpret_cast< qreal*>(_v) = _t->rotation(); break;
        case 7: *reinterpret_cast< QVector2D*>(_v) = _t->translation(); break;
        case 8: *reinterpret_cast< qreal*>(_v) = _t->minimumX(); break;
        case 9: *reinterpret_cast< qreal*>(_v) = _t->maximumX(); break;
        case 10: *reinterpret_cast< qreal*>(_v) = _t->minimumY(); break;
        case 11: *reinterpret_cast< qreal*>(_v) = _t->maximumY(); break;
        case 12: *reinterpret_cast< QQuickDragAxis**>(_v) = _t->xAxis(); break;
        case 13: *reinterpret_cast< QQuickDragAxis**>(_v) = _t->yAxis(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickPinchHandler *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setMinimumScale(*reinterpret_cast< qreal*>(_v)); break;
        case 1: _t->setMaximumScale(*reinterpret_cast< qreal*>(_v)); break;
        case 2: _t->setMinimumRotation(*reinterpret_cast< qreal*>(_v)); break;
        case 3: _t->setMaximumRotation(*reinterpret_cast< qreal*>(_v)); break;
        case 8: _t->setMinimumX(*reinterpret_cast< qreal*>(_v)); break;
        case 9: _t->setMaximumX(*reinterpret_cast< qreal*>(_v)); break;
        case 10: _t->setMinimumY(*reinterpret_cast< qreal*>(_v)); break;
        case 11: _t->setMaximumY(*reinterpret_cast< qreal*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickPinchHandler::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickMultiPointHandler::staticMetaObject>(),
    qt_meta_stringdata_QQuickPinchHandler.data,
    qt_meta_data_QQuickPinchHandler,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickPinchHandler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickPinchHandler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickPinchHandler.stringdata0))
        return static_cast<void*>(this);
    return QQuickMultiPointHandler::qt_metacast(_clname);
}

int QQuickPinchHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickMultiPointHandler::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 14;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 14;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 14;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 14;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 14;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickPinchHandler::minimumScaleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickPinchHandler::maximumScaleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickPinchHandler::minimumRotationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickPinchHandler::maximumRotationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickPinchHandler::minimumXChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickPinchHandler::maximumXChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickPinchHandler::minimumYChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQuickPinchHandler::maximumYChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QQuickPinchHandler::updated()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
