/****************************************************************************
** Meta object code from reading C++ file 'qquicklinearlayout_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../kde-qtdeclarative/src/imports/layouts/qquicklinearlayout_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquicklinearlayout_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickGridLayoutBase_t {
    QByteArrayData data[9];
    char stringdata0[129];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickGridLayoutBase_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickGridLayoutBase_t qt_meta_stringdata_QQuickGridLayoutBase = {
    {
QT_MOC_LITERAL(0, 0, 20), // "QQuickGridLayoutBase"
QT_MOC_LITERAL(1, 21, 11), // "QML.Element"
QT_MOC_LITERAL(2, 33, 9), // "anonymous"
QT_MOC_LITERAL(3, 43, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(4, 67, 1), // "1"
QT_MOC_LITERAL(5, 69, 22), // "layoutDirectionChanged"
QT_MOC_LITERAL(6, 92, 0), // ""
QT_MOC_LITERAL(7, 93, 15), // "layoutDirection"
QT_MOC_LITERAL(8, 109, 19) // "Qt::LayoutDirection"

    },
    "QQuickGridLayoutBase\0QML.Element\0"
    "anonymous\0QML.AddedInMinorVersion\0""1\0"
    "layoutDirectionChanged\0\0layoutDirection\0"
    "Qt::LayoutDirection"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickGridLayoutBase[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       1,   18, // methods
       1,   25, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

 // signals: name, argc, parameters, tag, flags
       5,    0,   24,    6, 0x86 /* Public | MethodRevisioned */,

 // signals: revision
       1,

 // signals: parameters
    QMetaType::Void,

 // properties: name, type, flags
       7, 0x80000000 | 8, 0x00c9510b,

 // properties: notify_signal_id
       0,

 // properties: revision
       1,

       0        // eod
};

void QQuickGridLayoutBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickGridLayoutBase *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->layoutDirectionChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickGridLayoutBase::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGridLayoutBase::layoutDirectionChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickGridLayoutBase *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< Qt::LayoutDirection*>(_v) = _t->layoutDirection(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickGridLayoutBase *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setLayoutDirection(*reinterpret_cast< Qt::LayoutDirection*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickGridLayoutBase::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickLayout::staticMetaObject>(),
    qt_meta_stringdata_QQuickGridLayoutBase.data,
    qt_meta_data_QQuickGridLayoutBase,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickGridLayoutBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickGridLayoutBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickGridLayoutBase.stringdata0))
        return static_cast<void*>(this);
    return QQuickLayout::qt_metacast(_clname);
}

int QQuickGridLayoutBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickLayout::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickGridLayoutBase::layoutDirectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_QQuickGridLayout_t {
    QByteArrayData data[17];
    char stringdata0[191];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickGridLayout_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickGridLayout_t qt_meta_stringdata_QQuickGridLayout = {
    {
QT_MOC_LITERAL(0, 0, 16), // "QQuickGridLayout"
QT_MOC_LITERAL(1, 17, 11), // "QML.Element"
QT_MOC_LITERAL(2, 29, 10), // "GridLayout"
QT_MOC_LITERAL(3, 40, 20), // "columnSpacingChanged"
QT_MOC_LITERAL(4, 61, 0), // ""
QT_MOC_LITERAL(5, 62, 17), // "rowSpacingChanged"
QT_MOC_LITERAL(6, 80, 14), // "columnsChanged"
QT_MOC_LITERAL(7, 95, 11), // "rowsChanged"
QT_MOC_LITERAL(8, 107, 11), // "flowChanged"
QT_MOC_LITERAL(9, 119, 13), // "columnSpacing"
QT_MOC_LITERAL(10, 133, 10), // "rowSpacing"
QT_MOC_LITERAL(11, 144, 7), // "columns"
QT_MOC_LITERAL(12, 152, 4), // "rows"
QT_MOC_LITERAL(13, 157, 4), // "flow"
QT_MOC_LITERAL(14, 162, 4), // "Flow"
QT_MOC_LITERAL(15, 167, 11), // "LeftToRight"
QT_MOC_LITERAL(16, 179, 11) // "TopToBottom"

    },
    "QQuickGridLayout\0QML.Element\0GridLayout\0"
    "columnSpacingChanged\0\0rowSpacingChanged\0"
    "columnsChanged\0rowsChanged\0flowChanged\0"
    "columnSpacing\0rowSpacing\0columns\0rows\0"
    "flow\0Flow\0LeftToRight\0TopToBottom"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickGridLayout[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       5,   16, // methods
       5,   46, // properties
       1,   66, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   41,    4, 0x06 /* Public */,
       5,    0,   42,    4, 0x06 /* Public */,
       6,    0,   43,    4, 0x06 /* Public */,
       7,    0,   44,    4, 0x06 /* Public */,
       8,    0,   45,    4, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
       9, QMetaType::QReal, 0x00495103,
      10, QMetaType::QReal, 0x00495103,
      11, QMetaType::Int, 0x00495103,
      12, QMetaType::Int, 0x00495103,
      13, 0x80000000 | 14, 0x0049510b,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,

 // enums: name, alias, flags, count, data
      14,   14, 0x0,    2,   71,

 // enum data: key, value
      15, uint(QQuickGridLayout::LeftToRight),
      16, uint(QQuickGridLayout::TopToBottom),

       0        // eod
};

void QQuickGridLayout::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickGridLayout *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->columnSpacingChanged(); break;
        case 1: _t->rowSpacingChanged(); break;
        case 2: _t->columnsChanged(); break;
        case 3: _t->rowsChanged(); break;
        case 4: _t->flowChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickGridLayout::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGridLayout::columnSpacingChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickGridLayout::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGridLayout::rowSpacingChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickGridLayout::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGridLayout::columnsChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickGridLayout::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGridLayout::rowsChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickGridLayout::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickGridLayout::flowChanged)) {
                *result = 4;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickGridLayout *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< qreal*>(_v) = _t->columnSpacing(); break;
        case 1: *reinterpret_cast< qreal*>(_v) = _t->rowSpacing(); break;
        case 2: *reinterpret_cast< int*>(_v) = _t->columns(); break;
        case 3: *reinterpret_cast< int*>(_v) = _t->rows(); break;
        case 4: *reinterpret_cast< Flow*>(_v) = _t->flow(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickGridLayout *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setColumnSpacing(*reinterpret_cast< qreal*>(_v)); break;
        case 1: _t->setRowSpacing(*reinterpret_cast< qreal*>(_v)); break;
        case 2: _t->setColumns(*reinterpret_cast< int*>(_v)); break;
        case 3: _t->setRows(*reinterpret_cast< int*>(_v)); break;
        case 4: _t->setFlow(*reinterpret_cast< Flow*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickGridLayout::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickGridLayoutBase::staticMetaObject>(),
    qt_meta_stringdata_QQuickGridLayout.data,
    qt_meta_data_QQuickGridLayout,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickGridLayout::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickGridLayout::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickGridLayout.stringdata0))
        return static_cast<void*>(this);
    return QQuickGridLayoutBase::qt_metacast(_clname);
}

int QQuickGridLayout::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickGridLayoutBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickGridLayout::columnSpacingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickGridLayout::rowSpacingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickGridLayout::columnsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickGridLayout::rowsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickGridLayout::flowChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
struct qt_meta_stringdata_QQuickLinearLayout_t {
    QByteArrayData data[4];
    char stringdata0[43];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickLinearLayout_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickLinearLayout_t qt_meta_stringdata_QQuickLinearLayout = {
    {
QT_MOC_LITERAL(0, 0, 18), // "QQuickLinearLayout"
QT_MOC_LITERAL(1, 19, 14), // "spacingChanged"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 7) // "spacing"

    },
    "QQuickLinearLayout\0spacingChanged\0\0"
    "spacing"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickLinearLayout[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       1,   20, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,

 // properties: name, type, flags
       3, QMetaType::QReal, 0x00495103,

 // properties: notify_signal_id
       0,

       0        // eod
};

void QQuickLinearLayout::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickLinearLayout *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->spacingChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickLinearLayout::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickLinearLayout::spacingChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickLinearLayout *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< qreal*>(_v) = _t->spacing(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickLinearLayout *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSpacing(*reinterpret_cast< qreal*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickLinearLayout::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickGridLayoutBase::staticMetaObject>(),
    qt_meta_stringdata_QQuickLinearLayout.data,
    qt_meta_data_QQuickLinearLayout,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickLinearLayout::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickLinearLayout::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickLinearLayout.stringdata0))
        return static_cast<void*>(this);
    return QQuickGridLayoutBase::qt_metacast(_clname);
}

int QQuickLinearLayout::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickGridLayoutBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickLinearLayout::spacingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_QQuickRowLayout_t {
    QByteArrayData data[3];
    char stringdata0[38];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickRowLayout_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickRowLayout_t qt_meta_stringdata_QQuickRowLayout = {
    {
QT_MOC_LITERAL(0, 0, 15), // "QQuickRowLayout"
QT_MOC_LITERAL(1, 16, 11), // "QML.Element"
QT_MOC_LITERAL(2, 28, 9) // "RowLayout"

    },
    "QQuickRowLayout\0QML.Element\0RowLayout"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickRowLayout[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,

       0        // eod
};

void QQuickRowLayout::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickRowLayout::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickLinearLayout::staticMetaObject>(),
    qt_meta_stringdata_QQuickRowLayout.data,
    qt_meta_data_QQuickRowLayout,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickRowLayout::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickRowLayout::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickRowLayout.stringdata0))
        return static_cast<void*>(this);
    return QQuickLinearLayout::qt_metacast(_clname);
}

int QQuickRowLayout::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickLinearLayout::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QQuickColumnLayout_t {
    QByteArrayData data[3];
    char stringdata0[44];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickColumnLayout_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickColumnLayout_t qt_meta_stringdata_QQuickColumnLayout = {
    {
QT_MOC_LITERAL(0, 0, 18), // "QQuickColumnLayout"
QT_MOC_LITERAL(1, 19, 11), // "QML.Element"
QT_MOC_LITERAL(2, 31, 12) // "ColumnLayout"

    },
    "QQuickColumnLayout\0QML.Element\0"
    "ColumnLayout"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickColumnLayout[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,

       0        // eod
};

void QQuickColumnLayout::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickColumnLayout::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickLinearLayout::staticMetaObject>(),
    qt_meta_stringdata_QQuickColumnLayout.data,
    qt_meta_data_QQuickColumnLayout,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickColumnLayout::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickColumnLayout::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickColumnLayout.stringdata0))
        return static_cast<void*>(this);
    return QQuickLinearLayout::qt_metacast(_clname);
}

int QQuickColumnLayout::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickLinearLayout::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
