/****************************************************************************
** Meta object code from reading C++ file 'qquickitemview_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/quick/items/qquickitemview_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickitemview_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickItemView_t {
    QByteArrayData data[125];
    char stringdata0[2004];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickItemView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickItemView_t qt_meta_stringdata_QQuickItemView = {
    {
QT_MOC_LITERAL(0, 0, 14), // "QQuickItemView"
QT_MOC_LITERAL(1, 15, 11), // "QML.Element"
QT_MOC_LITERAL(2, 27, 8), // "ItemView"
QT_MOC_LITERAL(3, 36, 13), // "QML.Creatable"
QT_MOC_LITERAL(4, 50, 5), // "false"
QT_MOC_LITERAL(5, 56, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(6, 78, 35), // "ItemView is an abstract base ..."
QT_MOC_LITERAL(7, 114, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(8, 138, 1), // "1"
QT_MOC_LITERAL(9, 140, 12), // "modelChanged"
QT_MOC_LITERAL(10, 153, 0), // ""
QT_MOC_LITERAL(11, 154, 15), // "delegateChanged"
QT_MOC_LITERAL(12, 170, 12), // "countChanged"
QT_MOC_LITERAL(13, 183, 19), // "currentIndexChanged"
QT_MOC_LITERAL(14, 203, 18), // "currentItemChanged"
QT_MOC_LITERAL(15, 222, 25), // "keyNavigationWrapsChanged"
QT_MOC_LITERAL(16, 248, 27), // "keyNavigationEnabledChanged"
QT_MOC_LITERAL(17, 276, 18), // "cacheBufferChanged"
QT_MOC_LITERAL(18, 295, 29), // "displayMarginBeginningChanged"
QT_MOC_LITERAL(19, 325, 23), // "displayMarginEndChanged"
QT_MOC_LITERAL(20, 349, 22), // "layoutDirectionChanged"
QT_MOC_LITERAL(21, 372, 31), // "effectiveLayoutDirectionChanged"
QT_MOC_LITERAL(22, 404, 30), // "verticalLayoutDirectionChanged"
QT_MOC_LITERAL(23, 435, 13), // "headerChanged"
QT_MOC_LITERAL(24, 449, 13), // "footerChanged"
QT_MOC_LITERAL(25, 463, 17), // "headerItemChanged"
QT_MOC_LITERAL(26, 481, 17), // "footerItemChanged"
QT_MOC_LITERAL(27, 499, 25), // "populateTransitionChanged"
QT_MOC_LITERAL(28, 525, 20), // "addTransitionChanged"
QT_MOC_LITERAL(29, 546, 29), // "addDisplacedTransitionChanged"
QT_MOC_LITERAL(30, 576, 21), // "moveTransitionChanged"
QT_MOC_LITERAL(31, 598, 30), // "moveDisplacedTransitionChanged"
QT_MOC_LITERAL(32, 629, 23), // "removeTransitionChanged"
QT_MOC_LITERAL(33, 653, 32), // "removeDisplacedTransitionChanged"
QT_MOC_LITERAL(34, 686, 26), // "displacedTransitionChanged"
QT_MOC_LITERAL(35, 713, 16), // "highlightChanged"
QT_MOC_LITERAL(36, 730, 20), // "highlightItemChanged"
QT_MOC_LITERAL(37, 751, 34), // "highlightFollowsCurrentItemCh..."
QT_MOC_LITERAL(38, 786, 25), // "highlightRangeModeChanged"
QT_MOC_LITERAL(39, 812, 30), // "preferredHighlightBeginChanged"
QT_MOC_LITERAL(40, 843, 28), // "preferredHighlightEndChanged"
QT_MOC_LITERAL(41, 872, 28), // "highlightMoveDurationChanged"
QT_MOC_LITERAL(42, 901, 17), // "reuseItemsChanged"
QT_MOC_LITERAL(43, 919, 14), // "destroyRemoved"
QT_MOC_LITERAL(44, 934, 11), // "createdItem"
QT_MOC_LITERAL(45, 946, 5), // "index"
QT_MOC_LITERAL(46, 952, 4), // "item"
QT_MOC_LITERAL(47, 957, 8), // "initItem"
QT_MOC_LITERAL(48, 966, 12), // "modelUpdated"
QT_MOC_LITERAL(49, 979, 13), // "QQmlChangeSet"
QT_MOC_LITERAL(50, 993, 9), // "changeSet"
QT_MOC_LITERAL(51, 1003, 5), // "reset"
QT_MOC_LITERAL(52, 1009, 14), // "destroyingItem"
QT_MOC_LITERAL(53, 1024, 12), // "onItemPooled"
QT_MOC_LITERAL(54, 1037, 10), // "modelIndex"
QT_MOC_LITERAL(55, 1048, 6), // "object"
QT_MOC_LITERAL(56, 1055, 12), // "onItemReused"
QT_MOC_LITERAL(57, 1068, 11), // "animStopped"
QT_MOC_LITERAL(58, 1080, 22), // "trackedPositionChanged"
QT_MOC_LITERAL(59, 1103, 19), // "positionViewAtIndex"
QT_MOC_LITERAL(60, 1123, 4), // "mode"
QT_MOC_LITERAL(61, 1128, 7), // "indexAt"
QT_MOC_LITERAL(62, 1136, 1), // "x"
QT_MOC_LITERAL(63, 1138, 1), // "y"
QT_MOC_LITERAL(64, 1140, 6), // "itemAt"
QT_MOC_LITERAL(65, 1147, 11), // "QQuickItem*"
QT_MOC_LITERAL(66, 1159, 11), // "itemAtIndex"
QT_MOC_LITERAL(67, 1171, 23), // "positionViewAtBeginning"
QT_MOC_LITERAL(68, 1195, 17), // "positionViewAtEnd"
QT_MOC_LITERAL(69, 1213, 11), // "forceLayout"
QT_MOC_LITERAL(70, 1225, 5), // "model"
QT_MOC_LITERAL(71, 1231, 8), // "delegate"
QT_MOC_LITERAL(72, 1240, 14), // "QQmlComponent*"
QT_MOC_LITERAL(73, 1255, 5), // "count"
QT_MOC_LITERAL(74, 1261, 12), // "currentIndex"
QT_MOC_LITERAL(75, 1274, 11), // "currentItem"
QT_MOC_LITERAL(76, 1286, 18), // "keyNavigationWraps"
QT_MOC_LITERAL(77, 1305, 20), // "keyNavigationEnabled"
QT_MOC_LITERAL(78, 1326, 11), // "cacheBuffer"
QT_MOC_LITERAL(79, 1338, 22), // "displayMarginBeginning"
QT_MOC_LITERAL(80, 1361, 16), // "displayMarginEnd"
QT_MOC_LITERAL(81, 1378, 15), // "layoutDirection"
QT_MOC_LITERAL(82, 1394, 19), // "Qt::LayoutDirection"
QT_MOC_LITERAL(83, 1414, 24), // "effectiveLayoutDirection"
QT_MOC_LITERAL(84, 1439, 23), // "verticalLayoutDirection"
QT_MOC_LITERAL(85, 1463, 23), // "VerticalLayoutDirection"
QT_MOC_LITERAL(86, 1487, 6), // "header"
QT_MOC_LITERAL(87, 1494, 10), // "headerItem"
QT_MOC_LITERAL(88, 1505, 6), // "footer"
QT_MOC_LITERAL(89, 1512, 10), // "footerItem"
QT_MOC_LITERAL(90, 1523, 8), // "populate"
QT_MOC_LITERAL(91, 1532, 17), // "QQuickTransition*"
QT_MOC_LITERAL(92, 1550, 3), // "add"
QT_MOC_LITERAL(93, 1554, 12), // "addDisplaced"
QT_MOC_LITERAL(94, 1567, 4), // "move"
QT_MOC_LITERAL(95, 1572, 13), // "moveDisplaced"
QT_MOC_LITERAL(96, 1586, 6), // "remove"
QT_MOC_LITERAL(97, 1593, 15), // "removeDisplaced"
QT_MOC_LITERAL(98, 1609, 9), // "displaced"
QT_MOC_LITERAL(99, 1619, 9), // "highlight"
QT_MOC_LITERAL(100, 1629, 13), // "highlightItem"
QT_MOC_LITERAL(101, 1643, 27), // "highlightFollowsCurrentItem"
QT_MOC_LITERAL(102, 1671, 18), // "highlightRangeMode"
QT_MOC_LITERAL(103, 1690, 18), // "HighlightRangeMode"
QT_MOC_LITERAL(104, 1709, 23), // "preferredHighlightBegin"
QT_MOC_LITERAL(105, 1733, 21), // "preferredHighlightEnd"
QT_MOC_LITERAL(106, 1755, 21), // "highlightMoveDuration"
QT_MOC_LITERAL(107, 1777, 10), // "reuseItems"
QT_MOC_LITERAL(108, 1788, 15), // "LayoutDirection"
QT_MOC_LITERAL(109, 1804, 11), // "LeftToRight"
QT_MOC_LITERAL(110, 1816, 11), // "RightToLeft"
QT_MOC_LITERAL(111, 1828, 19), // "VerticalTopToBottom"
QT_MOC_LITERAL(112, 1848, 19), // "VerticalBottomToTop"
QT_MOC_LITERAL(113, 1868, 11), // "TopToBottom"
QT_MOC_LITERAL(114, 1880, 11), // "BottomToTop"
QT_MOC_LITERAL(115, 1892, 16), // "NoHighlightRange"
QT_MOC_LITERAL(116, 1909, 10), // "ApplyRange"
QT_MOC_LITERAL(117, 1920, 20), // "StrictlyEnforceRange"
QT_MOC_LITERAL(118, 1941, 12), // "PositionMode"
QT_MOC_LITERAL(119, 1954, 9), // "Beginning"
QT_MOC_LITERAL(120, 1964, 6), // "Center"
QT_MOC_LITERAL(121, 1971, 3), // "End"
QT_MOC_LITERAL(122, 1975, 7), // "Visible"
QT_MOC_LITERAL(123, 1983, 7), // "Contain"
QT_MOC_LITERAL(124, 1991, 12) // "SnapPosition"

    },
    "QQuickItemView\0QML.Element\0ItemView\0"
    "QML.Creatable\0false\0QML.UncreatableReason\0"
    "ItemView is an abstract base class.\0"
    "QML.AddedInMinorVersion\0""1\0modelChanged\0"
    "\0delegateChanged\0countChanged\0"
    "currentIndexChanged\0currentItemChanged\0"
    "keyNavigationWrapsChanged\0"
    "keyNavigationEnabledChanged\0"
    "cacheBufferChanged\0displayMarginBeginningChanged\0"
    "displayMarginEndChanged\0layoutDirectionChanged\0"
    "effectiveLayoutDirectionChanged\0"
    "verticalLayoutDirectionChanged\0"
    "headerChanged\0footerChanged\0"
    "headerItemChanged\0footerItemChanged\0"
    "populateTransitionChanged\0"
    "addTransitionChanged\0addDisplacedTransitionChanged\0"
    "moveTransitionChanged\0"
    "moveDisplacedTransitionChanged\0"
    "removeTransitionChanged\0"
    "removeDisplacedTransitionChanged\0"
    "displacedTransitionChanged\0highlightChanged\0"
    "highlightItemChanged\0"
    "highlightFollowsCurrentItemChanged\0"
    "highlightRangeModeChanged\0"
    "preferredHighlightBeginChanged\0"
    "preferredHighlightEndChanged\0"
    "highlightMoveDurationChanged\0"
    "reuseItemsChanged\0destroyRemoved\0"
    "createdItem\0index\0item\0initItem\0"
    "modelUpdated\0QQmlChangeSet\0changeSet\0"
    "reset\0destroyingItem\0onItemPooled\0"
    "modelIndex\0object\0onItemReused\0"
    "animStopped\0trackedPositionChanged\0"
    "positionViewAtIndex\0mode\0indexAt\0x\0y\0"
    "itemAt\0QQuickItem*\0itemAtIndex\0"
    "positionViewAtBeginning\0positionViewAtEnd\0"
    "forceLayout\0model\0delegate\0QQmlComponent*\0"
    "count\0currentIndex\0currentItem\0"
    "keyNavigationWraps\0keyNavigationEnabled\0"
    "cacheBuffer\0displayMarginBeginning\0"
    "displayMarginEnd\0layoutDirection\0"
    "Qt::LayoutDirection\0effectiveLayoutDirection\0"
    "verticalLayoutDirection\0VerticalLayoutDirection\0"
    "header\0headerItem\0footer\0footerItem\0"
    "populate\0QQuickTransition*\0add\0"
    "addDisplaced\0move\0moveDisplaced\0remove\0"
    "removeDisplaced\0displaced\0highlight\0"
    "highlightItem\0highlightFollowsCurrentItem\0"
    "highlightRangeMode\0HighlightRangeMode\0"
    "preferredHighlightBegin\0preferredHighlightEnd\0"
    "highlightMoveDuration\0reuseItems\0"
    "LayoutDirection\0LeftToRight\0RightToLeft\0"
    "VerticalTopToBottom\0VerticalBottomToTop\0"
    "TopToBottom\0BottomToTop\0NoHighlightRange\0"
    "ApplyRange\0StrictlyEnforceRange\0"
    "PositionMode\0Beginning\0Center\0End\0"
    "Visible\0Contain\0SnapPosition"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickItemView[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
      49,   22, // methods
      33,  401, // properties
       4,  566, // enums/sets
       0,    0, // constructors
       0,       // flags
      33,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,

 // signals: name, argc, parameters, tag, flags
       9,    0,  316,   10, 0x06 /* Public */,
      11,    0,  317,   10, 0x06 /* Public */,
      12,    0,  318,   10, 0x06 /* Public */,
      13,    0,  319,   10, 0x06 /* Public */,
      14,    0,  320,   10, 0x06 /* Public */,
      15,    0,  321,   10, 0x06 /* Public */,
      16,    0,  322,   10, 0x86 /* Public | MethodRevisioned */,
      17,    0,  323,   10, 0x06 /* Public */,
      18,    0,  324,   10, 0x06 /* Public */,
      19,    0,  325,   10, 0x06 /* Public */,
      20,    0,  326,   10, 0x06 /* Public */,
      21,    0,  327,   10, 0x06 /* Public */,
      22,    0,  328,   10, 0x06 /* Public */,
      23,    0,  329,   10, 0x06 /* Public */,
      24,    0,  330,   10, 0x06 /* Public */,
      25,    0,  331,   10, 0x06 /* Public */,
      26,    0,  332,   10, 0x06 /* Public */,
      27,    0,  333,   10, 0x06 /* Public */,
      28,    0,  334,   10, 0x06 /* Public */,
      29,    0,  335,   10, 0x06 /* Public */,
      30,    0,  336,   10, 0x06 /* Public */,
      31,    0,  337,   10, 0x06 /* Public */,
      32,    0,  338,   10, 0x06 /* Public */,
      33,    0,  339,   10, 0x06 /* Public */,
      34,    0,  340,   10, 0x06 /* Public */,
      35,    0,  341,   10, 0x06 /* Public */,
      36,    0,  342,   10, 0x06 /* Public */,
      37,    0,  343,   10, 0x06 /* Public */,
      38,    0,  344,   10, 0x06 /* Public */,
      39,    0,  345,   10, 0x06 /* Public */,
      40,    0,  346,   10, 0x06 /* Public */,
      41,    0,  347,   10, 0x06 /* Public */,
      42,    0,  348,   10, 0x86 /* Public | MethodRevisioned */,

 // slots: name, argc, parameters, tag, flags
      43,    0,  349,   10, 0x09 /* Protected */,
      44,    2,  350,   10, 0x09 /* Protected */,
      47,    2,  355,   10, 0x09 /* Protected */,
      48,    2,  360,   10, 0x09 /* Protected */,
      52,    1,  365,   10, 0x09 /* Protected */,
      53,    2,  368,   10, 0x89 /* Protected | MethodRevisioned */,
      56,    2,  373,   10, 0x89 /* Protected | MethodRevisioned */,
      57,    0,  378,   10, 0x09 /* Protected */,
      58,    0,  379,   10, 0x09 /* Protected */,

 // methods: name, argc, parameters, tag, flags
      59,    2,  380,   10, 0x02 /* Public */,
      61,    2,  385,   10, 0x02 /* Public */,
      64,    2,  390,   10, 0x02 /* Public */,
      66,    1,  395,   10, 0x82 /* Public | MethodRevisioned */,
      67,    0,  398,   10, 0x02 /* Public */,
      68,    0,  399,   10, 0x02 /* Public */,
      69,    0,  400,   10, 0x82 /* Public | MethodRevisioned */,

 // signals: revision
       0,
       0,
       0,
       0,
       0,
       0,
       7,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
      15,

 // slots: revision
       0,
       0,
       0,
       0,
       0,
      15,
      15,
       0,
       0,

 // methods: revision
       0,
       0,
       0,
      13,
       0,
       0,
       1,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::QObjectStar,   45,   46,
    QMetaType::Void, QMetaType::Int, QMetaType::QObjectStar,   45,   46,
    QMetaType::Void, 0x80000000 | 49, QMetaType::Bool,   50,   51,
    QMetaType::Void, QMetaType::QObjectStar,   46,
    QMetaType::Void, QMetaType::Int, QMetaType::QObjectStar,   54,   55,
    QMetaType::Void, QMetaType::Int, QMetaType::QObjectStar,   54,   55,
    QMetaType::Void,
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   45,   60,
    QMetaType::Int, QMetaType::QReal, QMetaType::QReal,   62,   63,
    0x80000000 | 65, QMetaType::QReal, QMetaType::QReal,   62,   63,
    0x80000000 | 65, QMetaType::Int,   45,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      70, QMetaType::QVariant, 0x00495103,
      71, 0x80000000 | 72, 0x0049510b,
      73, QMetaType::Int, 0x00495001,
      74, QMetaType::Int, 0x00495103,
      75, 0x80000000 | 65, 0x00495009,
      76, QMetaType::Bool, 0x00495003,
      77, QMetaType::Bool, 0x00c95103,
      78, QMetaType::Int, 0x00495103,
      79, QMetaType::Int, 0x00c95103,
      80, QMetaType::Int, 0x00c95103,
      81, 0x80000000 | 82, 0x0049510b,
      83, 0x80000000 | 82, 0x00495009,
      84, 0x80000000 | 85, 0x0049510b,
      86, 0x80000000 | 72, 0x0049510b,
      87, 0x80000000 | 65, 0x00495009,
      88, 0x80000000 | 72, 0x0049510b,
      89, 0x80000000 | 65, 0x00495009,
      90, 0x80000000 | 91, 0x0049500b,
      92, 0x80000000 | 91, 0x0049500b,
      93, 0x80000000 | 91, 0x0049500b,
      94, 0x80000000 | 91, 0x0049500b,
      95, 0x80000000 | 91, 0x0049500b,
      96, 0x80000000 | 91, 0x0049500b,
      97, 0x80000000 | 91, 0x0049500b,
      98, 0x80000000 | 91, 0x0049500b,
      99, 0x80000000 | 72, 0x0049510b,
     100, 0x80000000 | 65, 0x00495009,
     101, QMetaType::Bool, 0x00495103,
     102, 0x80000000 | 103, 0x0049510b,
     104, QMetaType::QReal, 0x00495107,
     105, QMetaType::QReal, 0x00495107,
     106, QMetaType::Int, 0x00495103,
     107, QMetaType::Bool, 0x00c95103,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,
       5,
       6,
       7,
       8,
       9,
      10,
      11,
      12,
      13,
      15,
      14,
      16,
      17,
      18,
      19,
      20,
      21,
      22,
      23,
      24,
      25,
      26,
      27,
      28,
      29,
      30,
      31,
      32,

 // properties: revision
       0,
       0,
       0,
       0,
       0,
       0,
       7,
       0,
       3,
       3,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
      15,

 // enums: name, alias, flags, count, data
     108,  108, 0x0,    4,  586,
      85,   85, 0x0,    2,  594,
     103,  103, 0x0,    3,  598,
     118,  118, 0x0,    6,  604,

 // enum data: key, value
     109, uint(QQuickItemView::LeftToRight),
     110, uint(QQuickItemView::RightToLeft),
     111, uint(QQuickItemView::VerticalTopToBottom),
     112, uint(QQuickItemView::VerticalBottomToTop),
     113, uint(QQuickItemView::TopToBottom),
     114, uint(QQuickItemView::BottomToTop),
     115, uint(QQuickItemView::NoHighlightRange),
     116, uint(QQuickItemView::ApplyRange),
     117, uint(QQuickItemView::StrictlyEnforceRange),
     119, uint(QQuickItemView::Beginning),
     120, uint(QQuickItemView::Center),
     121, uint(QQuickItemView::End),
     122, uint(QQuickItemView::Visible),
     123, uint(QQuickItemView::Contain),
     124, uint(QQuickItemView::SnapPosition),

       0        // eod
};

void QQuickItemView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickItemView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->modelChanged(); break;
        case 1: _t->delegateChanged(); break;
        case 2: _t->countChanged(); break;
        case 3: _t->currentIndexChanged(); break;
        case 4: _t->currentItemChanged(); break;
        case 5: _t->keyNavigationWrapsChanged(); break;
        case 6: _t->keyNavigationEnabledChanged(); break;
        case 7: _t->cacheBufferChanged(); break;
        case 8: _t->displayMarginBeginningChanged(); break;
        case 9: _t->displayMarginEndChanged(); break;
        case 10: _t->layoutDirectionChanged(); break;
        case 11: _t->effectiveLayoutDirectionChanged(); break;
        case 12: _t->verticalLayoutDirectionChanged(); break;
        case 13: _t->headerChanged(); break;
        case 14: _t->footerChanged(); break;
        case 15: _t->headerItemChanged(); break;
        case 16: _t->footerItemChanged(); break;
        case 17: _t->populateTransitionChanged(); break;
        case 18: _t->addTransitionChanged(); break;
        case 19: _t->addDisplacedTransitionChanged(); break;
        case 20: _t->moveTransitionChanged(); break;
        case 21: _t->moveDisplacedTransitionChanged(); break;
        case 22: _t->removeTransitionChanged(); break;
        case 23: _t->removeDisplacedTransitionChanged(); break;
        case 24: _t->displacedTransitionChanged(); break;
        case 25: _t->highlightChanged(); break;
        case 26: _t->highlightItemChanged(); break;
        case 27: _t->highlightFollowsCurrentItemChanged(); break;
        case 28: _t->highlightRangeModeChanged(); break;
        case 29: _t->preferredHighlightBeginChanged(); break;
        case 30: _t->preferredHighlightEndChanged(); break;
        case 31: _t->highlightMoveDurationChanged(); break;
        case 32: _t->reuseItemsChanged(); break;
        case 33: _t->destroyRemoved(); break;
        case 34: _t->createdItem((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QObject*(*)>(_a[2]))); break;
        case 35: _t->initItem((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QObject*(*)>(_a[2]))); break;
        case 36: _t->modelUpdated((*reinterpret_cast< const QQmlChangeSet(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 37: _t->destroyingItem((*reinterpret_cast< QObject*(*)>(_a[1]))); break;
        case 38: _t->onItemPooled((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QObject*(*)>(_a[2]))); break;
        case 39: _t->onItemReused((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QObject*(*)>(_a[2]))); break;
        case 40: _t->animStopped(); break;
        case 41: _t->trackedPositionChanged(); break;
        case 42: _t->positionViewAtIndex((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 43: { int _r = _t->indexAt((*reinterpret_cast< qreal(*)>(_a[1])),(*reinterpret_cast< qreal(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 44: { QQuickItem* _r = _t->itemAt((*reinterpret_cast< qreal(*)>(_a[1])),(*reinterpret_cast< qreal(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QQuickItem**>(_a[0]) = std::move(_r); }  break;
        case 45: { QQuickItem* _r = _t->itemAtIndex((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QQuickItem**>(_a[0]) = std::move(_r); }  break;
        case 46: _t->positionViewAtBeginning(); break;
        case 47: _t->positionViewAtEnd(); break;
        case 48: _t->forceLayout(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::modelChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::delegateChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::countChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::currentIndexChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::currentItemChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::keyNavigationWrapsChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::keyNavigationEnabledChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::cacheBufferChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::displayMarginBeginningChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::displayMarginEndChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::layoutDirectionChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::effectiveLayoutDirectionChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::verticalLayoutDirectionChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::headerChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::footerChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::headerItemChanged)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::footerItemChanged)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::populateTransitionChanged)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::addTransitionChanged)) {
                *result = 18;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::addDisplacedTransitionChanged)) {
                *result = 19;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::moveTransitionChanged)) {
                *result = 20;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::moveDisplacedTransitionChanged)) {
                *result = 21;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::removeTransitionChanged)) {
                *result = 22;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::removeDisplacedTransitionChanged)) {
                *result = 23;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::displacedTransitionChanged)) {
                *result = 24;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::highlightChanged)) {
                *result = 25;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::highlightItemChanged)) {
                *result = 26;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::highlightFollowsCurrentItemChanged)) {
                *result = 27;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::highlightRangeModeChanged)) {
                *result = 28;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::preferredHighlightBeginChanged)) {
                *result = 29;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::preferredHighlightEndChanged)) {
                *result = 30;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::highlightMoveDurationChanged)) {
                *result = 31;
                return;
            }
        }
        {
            using _t = void (QQuickItemView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemView::reuseItemsChanged)) {
                *result = 32;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 25:
        case 15:
        case 13:
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlComponent* >(); break;
        case 26:
        case 16:
        case 14:
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickItemView *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QVariant*>(_v) = _t->model(); break;
        case 1: *reinterpret_cast< QQmlComponent**>(_v) = _t->delegate(); break;
        case 2: *reinterpret_cast< int*>(_v) = _t->count(); break;
        case 3: *reinterpret_cast< int*>(_v) = _t->currentIndex(); break;
        case 4: *reinterpret_cast< QQuickItem**>(_v) = _t->currentItem(); break;
        case 5: *reinterpret_cast< bool*>(_v) = _t->isWrapEnabled(); break;
        case 6: *reinterpret_cast< bool*>(_v) = _t->isKeyNavigationEnabled(); break;
        case 7: *reinterpret_cast< int*>(_v) = _t->cacheBuffer(); break;
        case 8: *reinterpret_cast< int*>(_v) = _t->displayMarginBeginning(); break;
        case 9: *reinterpret_cast< int*>(_v) = _t->displayMarginEnd(); break;
        case 10: *reinterpret_cast< Qt::LayoutDirection*>(_v) = _t->layoutDirection(); break;
        case 11: *reinterpret_cast< Qt::LayoutDirection*>(_v) = _t->effectiveLayoutDirection(); break;
        case 12: *reinterpret_cast< VerticalLayoutDirection*>(_v) = _t->verticalLayoutDirection(); break;
        case 13: *reinterpret_cast< QQmlComponent**>(_v) = _t->header(); break;
        case 14: *reinterpret_cast< QQuickItem**>(_v) = _t->headerItem(); break;
        case 15: *reinterpret_cast< QQmlComponent**>(_v) = _t->footer(); break;
        case 16: *reinterpret_cast< QQuickItem**>(_v) = _t->footerItem(); break;
        case 17: *reinterpret_cast< QQuickTransition**>(_v) = _t->populateTransition(); break;
        case 18: *reinterpret_cast< QQuickTransition**>(_v) = _t->addTransition(); break;
        case 19: *reinterpret_cast< QQuickTransition**>(_v) = _t->addDisplacedTransition(); break;
        case 20: *reinterpret_cast< QQuickTransition**>(_v) = _t->moveTransition(); break;
        case 21: *reinterpret_cast< QQuickTransition**>(_v) = _t->moveDisplacedTransition(); break;
        case 22: *reinterpret_cast< QQuickTransition**>(_v) = _t->removeTransition(); break;
        case 23: *reinterpret_cast< QQuickTransition**>(_v) = _t->removeDisplacedTransition(); break;
        case 24: *reinterpret_cast< QQuickTransition**>(_v) = _t->displacedTransition(); break;
        case 25: *reinterpret_cast< QQmlComponent**>(_v) = _t->highlight(); break;
        case 26: *reinterpret_cast< QQuickItem**>(_v) = _t->highlightItem(); break;
        case 27: *reinterpret_cast< bool*>(_v) = _t->highlightFollowsCurrentItem(); break;
        case 28: *reinterpret_cast< HighlightRangeMode*>(_v) = _t->highlightRangeMode(); break;
        case 29: *reinterpret_cast< qreal*>(_v) = _t->preferredHighlightBegin(); break;
        case 30: *reinterpret_cast< qreal*>(_v) = _t->preferredHighlightEnd(); break;
        case 31: *reinterpret_cast< int*>(_v) = _t->highlightMoveDuration(); break;
        case 32: *reinterpret_cast< bool*>(_v) = _t->reuseItems(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickItemView *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setModel(*reinterpret_cast< QVariant*>(_v)); break;
        case 1: _t->setDelegate(*reinterpret_cast< QQmlComponent**>(_v)); break;
        case 3: _t->setCurrentIndex(*reinterpret_cast< int*>(_v)); break;
        case 5: _t->setWrapEnabled(*reinterpret_cast< bool*>(_v)); break;
        case 6: _t->setKeyNavigationEnabled(*reinterpret_cast< bool*>(_v)); break;
        case 7: _t->setCacheBuffer(*reinterpret_cast< int*>(_v)); break;
        case 8: _t->setDisplayMarginBeginning(*reinterpret_cast< int*>(_v)); break;
        case 9: _t->setDisplayMarginEnd(*reinterpret_cast< int*>(_v)); break;
        case 10: _t->setLayoutDirection(*reinterpret_cast< Qt::LayoutDirection*>(_v)); break;
        case 12: _t->setVerticalLayoutDirection(*reinterpret_cast< VerticalLayoutDirection*>(_v)); break;
        case 13: _t->setHeader(*reinterpret_cast< QQmlComponent**>(_v)); break;
        case 15: _t->setFooter(*reinterpret_cast< QQmlComponent**>(_v)); break;
        case 17: _t->setPopulateTransition(*reinterpret_cast< QQuickTransition**>(_v)); break;
        case 18: _t->setAddTransition(*reinterpret_cast< QQuickTransition**>(_v)); break;
        case 19: _t->setAddDisplacedTransition(*reinterpret_cast< QQuickTransition**>(_v)); break;
        case 20: _t->setMoveTransition(*reinterpret_cast< QQuickTransition**>(_v)); break;
        case 21: _t->setMoveDisplacedTransition(*reinterpret_cast< QQuickTransition**>(_v)); break;
        case 22: _t->setRemoveTransition(*reinterpret_cast< QQuickTransition**>(_v)); break;
        case 23: _t->setRemoveDisplacedTransition(*reinterpret_cast< QQuickTransition**>(_v)); break;
        case 24: _t->setDisplacedTransition(*reinterpret_cast< QQuickTransition**>(_v)); break;
        case 25: _t->setHighlight(*reinterpret_cast< QQmlComponent**>(_v)); break;
        case 27: _t->setHighlightFollowsCurrentItem(*reinterpret_cast< bool*>(_v)); break;
        case 28: _t->setHighlightRangeMode(*reinterpret_cast< HighlightRangeMode*>(_v)); break;
        case 29: _t->setPreferredHighlightBegin(*reinterpret_cast< qreal*>(_v)); break;
        case 30: _t->setPreferredHighlightEnd(*reinterpret_cast< qreal*>(_v)); break;
        case 31: _t->setHighlightMoveDuration(*reinterpret_cast< int*>(_v)); break;
        case 32: _t->setReuseItems(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        QQuickItemView *_t = static_cast<QQuickItemView *>(_o);
        (void)_t;
        switch (_id) {
        case 29: _t->resetPreferredHighlightBegin(); break;
        case 30: _t->resetPreferredHighlightEnd(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickItemView::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickFlickable::staticMetaObject>(),
    qt_meta_stringdata_QQuickItemView.data,
    qt_meta_data_QQuickItemView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickItemView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickItemView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickItemView.stringdata0))
        return static_cast<void*>(this);
    return QQuickFlickable::qt_metacast(_clname);
}

int QQuickItemView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickFlickable::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 49)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 49;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 49)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 49;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 33;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 33;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 33;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 33;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 33;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 33;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickItemView::modelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickItemView::delegateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickItemView::countChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickItemView::currentIndexChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickItemView::currentItemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickItemView::keyNavigationWrapsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickItemView::keyNavigationEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQuickItemView::cacheBufferChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QQuickItemView::displayMarginBeginningChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QQuickItemView::displayMarginEndChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QQuickItemView::layoutDirectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QQuickItemView::effectiveLayoutDirectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void QQuickItemView::verticalLayoutDirectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void QQuickItemView::headerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void QQuickItemView::footerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void QQuickItemView::headerItemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void QQuickItemView::footerItemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void QQuickItemView::populateTransitionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void QQuickItemView::addTransitionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void QQuickItemView::addDisplacedTransitionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void QQuickItemView::moveTransitionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void QQuickItemView::moveDisplacedTransitionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 21, nullptr);
}

// SIGNAL 22
void QQuickItemView::removeTransitionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 22, nullptr);
}

// SIGNAL 23
void QQuickItemView::removeDisplacedTransitionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 23, nullptr);
}

// SIGNAL 24
void QQuickItemView::displacedTransitionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 24, nullptr);
}

// SIGNAL 25
void QQuickItemView::highlightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 25, nullptr);
}

// SIGNAL 26
void QQuickItemView::highlightItemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 26, nullptr);
}

// SIGNAL 27
void QQuickItemView::highlightFollowsCurrentItemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 27, nullptr);
}

// SIGNAL 28
void QQuickItemView::highlightRangeModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 28, nullptr);
}

// SIGNAL 29
void QQuickItemView::preferredHighlightBeginChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 29, nullptr);
}

// SIGNAL 30
void QQuickItemView::preferredHighlightEndChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 30, nullptr);
}

// SIGNAL 31
void QQuickItemView::highlightMoveDurationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 31, nullptr);
}

// SIGNAL 32
void QQuickItemView::reuseItemsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 32, nullptr);
}
struct qt_meta_stringdata_QQuickItemViewAttached_t {
    QByteArrayData data[19];
    char stringdata0[235];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickItemViewAttached_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickItemViewAttached_t qt_meta_stringdata_QQuickItemViewAttached = {
    {
QT_MOC_LITERAL(0, 0, 22), // "QQuickItemViewAttached"
QT_MOC_LITERAL(1, 23, 11), // "viewChanged"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 18), // "currentItemChanged"
QT_MOC_LITERAL(4, 55, 18), // "delayRemoveChanged"
QT_MOC_LITERAL(5, 74, 3), // "add"
QT_MOC_LITERAL(6, 78, 6), // "remove"
QT_MOC_LITERAL(7, 85, 14), // "sectionChanged"
QT_MOC_LITERAL(8, 100, 18), // "prevSectionChanged"
QT_MOC_LITERAL(9, 119, 18), // "nextSectionChanged"
QT_MOC_LITERAL(10, 138, 6), // "pooled"
QT_MOC_LITERAL(11, 145, 6), // "reused"
QT_MOC_LITERAL(12, 152, 4), // "view"
QT_MOC_LITERAL(13, 157, 15), // "QQuickItemView*"
QT_MOC_LITERAL(14, 173, 13), // "isCurrentItem"
QT_MOC_LITERAL(15, 187, 11), // "delayRemove"
QT_MOC_LITERAL(16, 199, 7), // "section"
QT_MOC_LITERAL(17, 207, 15), // "previousSection"
QT_MOC_LITERAL(18, 223, 11) // "nextSection"

    },
    "QQuickItemViewAttached\0viewChanged\0\0"
    "currentItemChanged\0delayRemoveChanged\0"
    "add\0remove\0sectionChanged\0prevSectionChanged\0"
    "nextSectionChanged\0pooled\0reused\0view\0"
    "QQuickItemView*\0isCurrentItem\0delayRemove\0"
    "section\0previousSection\0nextSection"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickItemViewAttached[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       6,   74, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      10,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   64,    2, 0x06 /* Public */,
       3,    0,   65,    2, 0x06 /* Public */,
       4,    0,   66,    2, 0x06 /* Public */,
       5,    0,   67,    2, 0x06 /* Public */,
       6,    0,   68,    2, 0x06 /* Public */,
       7,    0,   69,    2, 0x06 /* Public */,
       8,    0,   70,    2, 0x06 /* Public */,
       9,    0,   71,    2, 0x06 /* Public */,
      10,    0,   72,    2, 0x06 /* Public */,
      11,    0,   73,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      12, 0x80000000 | 13, 0x00495009,
      14, QMetaType::Bool, 0x00495001,
      15, QMetaType::Bool, 0x00495103,
      16, QMetaType::QString, 0x00495001,
      17, QMetaType::QString, 0x00495001,
      18, QMetaType::QString, 0x00495001,

 // properties: notify_signal_id
       0,
       1,
       2,
       5,
       6,
       7,

       0        // eod
};

void QQuickItemViewAttached::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickItemViewAttached *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->viewChanged(); break;
        case 1: _t->currentItemChanged(); break;
        case 2: _t->delayRemoveChanged(); break;
        case 3: _t->add(); break;
        case 4: _t->remove(); break;
        case 5: _t->sectionChanged(); break;
        case 6: _t->prevSectionChanged(); break;
        case 7: _t->nextSectionChanged(); break;
        case 8: _t->pooled(); break;
        case 9: _t->reused(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickItemViewAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemViewAttached::viewChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickItemViewAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemViewAttached::currentItemChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickItemViewAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemViewAttached::delayRemoveChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickItemViewAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemViewAttached::add)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickItemViewAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemViewAttached::remove)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickItemViewAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemViewAttached::sectionChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickItemViewAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemViewAttached::prevSectionChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickItemViewAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemViewAttached::nextSectionChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickItemViewAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemViewAttached::pooled)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickItemViewAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItemViewAttached::reused)) {
                *result = 9;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItemView* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickItemViewAttached *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQuickItemView**>(_v) = _t->view(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->isCurrentItem(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->delayRemove(); break;
        case 3: *reinterpret_cast< QString*>(_v) = _t->section(); break;
        case 4: *reinterpret_cast< QString*>(_v) = _t->prevSection(); break;
        case 5: *reinterpret_cast< QString*>(_v) = _t->nextSection(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickItemViewAttached *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 2: _t->setDelayRemove(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickItemViewAttached::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickItemViewAttached.data,
    qt_meta_data_QQuickItemViewAttached,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickItemViewAttached::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickItemViewAttached::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickItemViewAttached.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickItemViewAttached::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 6;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickItemViewAttached::viewChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickItemViewAttached::currentItemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickItemViewAttached::delayRemoveChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickItemViewAttached::add()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickItemViewAttached::remove()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickItemViewAttached::sectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickItemViewAttached::prevSectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQuickItemViewAttached::nextSectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QQuickItemViewAttached::pooled()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QQuickItemViewAttached::reused()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
