/****************************************************************************
** Meta object code from reading C++ file 'qsgadaptationlayer_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/quick/scenegraph/qsgadaptationlayer_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qsgadaptationlayer_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QSGLayer_t {
    QByteArrayData data[6];
    char stringdata0[80];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QSGLayer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QSGLayer_t qt_meta_stringdata_QSGLayer = {
    {
QT_MOC_LITERAL(0, 0, 8), // "QSGLayer"
QT_MOC_LITERAL(1, 9, 15), // "updateRequested"
QT_MOC_LITERAL(2, 25, 0), // ""
QT_MOC_LITERAL(3, 26, 24), // "scheduledUpdateCompleted"
QT_MOC_LITERAL(4, 51, 16), // "markDirtyTexture"
QT_MOC_LITERAL(5, 68, 11) // "invalidated"

    },
    "QSGLayer\0updateRequested\0\0"
    "scheduledUpdateCompleted\0markDirtyTexture\0"
    "invalidated"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QSGLayer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   34,    2, 0x06 /* Public */,
       3,    0,   35,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   36,    2, 0x0a /* Public */,
       5,    0,   37,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void QSGLayer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QSGLayer *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->updateRequested(); break;
        case 1: _t->scheduledUpdateCompleted(); break;
        case 2: _t->markDirtyTexture(); break;
        case 3: _t->invalidated(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QSGLayer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSGLayer::updateRequested)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QSGLayer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSGLayer::scheduledUpdateCompleted)) {
                *result = 1;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QSGLayer::staticMetaObject = { {
    QMetaObject::SuperData::link<QSGDynamicTexture::staticMetaObject>(),
    qt_meta_stringdata_QSGLayer.data,
    qt_meta_data_QSGLayer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QSGLayer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QSGLayer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QSGLayer.stringdata0))
        return static_cast<void*>(this);
    return QSGDynamicTexture::qt_metacast(_clname);
}

int QSGLayer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSGDynamicTexture::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void QSGLayer::updateRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QSGLayer::scheduledUpdateCompleted()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
struct qt_meta_stringdata_QSGGuiThreadShaderEffectManager_t {
    QByteArrayData data[11];
    char stringdata0[139];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QSGGuiThreadShaderEffectManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QSGGuiThreadShaderEffectManager_t qt_meta_stringdata_QSGGuiThreadShaderEffectManager = {
    {
QT_MOC_LITERAL(0, 0, 31), // "QSGGuiThreadShaderEffectManager"
QT_MOC_LITERAL(1, 32, 18), // "shaderCodePrepared"
QT_MOC_LITERAL(2, 51, 0), // ""
QT_MOC_LITERAL(3, 52, 2), // "ok"
QT_MOC_LITERAL(4, 55, 16), // "ShaderInfo::Type"
QT_MOC_LITERAL(5, 72, 8), // "typeHint"
QT_MOC_LITERAL(6, 81, 3), // "src"
QT_MOC_LITERAL(7, 85, 11), // "ShaderInfo*"
QT_MOC_LITERAL(8, 97, 6), // "result"
QT_MOC_LITERAL(9, 104, 14), // "textureChanged"
QT_MOC_LITERAL(10, 119, 19) // "logAndStatusChanged"

    },
    "QSGGuiThreadShaderEffectManager\0"
    "shaderCodePrepared\0\0ok\0ShaderInfo::Type\0"
    "typeHint\0src\0ShaderInfo*\0result\0"
    "textureChanged\0logAndStatusChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QSGGuiThreadShaderEffectManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    4,   29,    2, 0x06 /* Public */,
       9,    0,   38,    2, 0x06 /* Public */,
      10,    0,   39,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 4, QMetaType::QByteArray, 0x80000000 | 7,    3,    5,    6,    8,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void QSGGuiThreadShaderEffectManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QSGGuiThreadShaderEffectManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->shaderCodePrepared((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< ShaderInfo::Type(*)>(_a[2])),(*reinterpret_cast< const QByteArray(*)>(_a[3])),(*reinterpret_cast< ShaderInfo*(*)>(_a[4]))); break;
        case 1: _t->textureChanged(); break;
        case 2: _t->logAndStatusChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QSGGuiThreadShaderEffectManager::*)(bool , ShaderInfo::Type , const QByteArray & , ShaderInfo * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSGGuiThreadShaderEffectManager::shaderCodePrepared)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QSGGuiThreadShaderEffectManager::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSGGuiThreadShaderEffectManager::textureChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QSGGuiThreadShaderEffectManager::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QSGGuiThreadShaderEffectManager::logAndStatusChanged)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QSGGuiThreadShaderEffectManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QSGGuiThreadShaderEffectManager.data,
    qt_meta_data_QSGGuiThreadShaderEffectManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QSGGuiThreadShaderEffectManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QSGGuiThreadShaderEffectManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QSGGuiThreadShaderEffectManager.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QSGGuiThreadShaderEffectManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void QSGGuiThreadShaderEffectManager::shaderCodePrepared(bool _t1, ShaderInfo::Type _t2, const QByteArray & _t3, ShaderInfo * _t4)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QSGGuiThreadShaderEffectManager::textureChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QSGGuiThreadShaderEffectManager::logAndStatusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
