/****************************************************************************
** Meta object code from reading C++ file 'qquickitem.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/quick/items/qquickitem.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickitem.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickTransform_t {
    QByteArrayData data[5];
    char stringdata0[46];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickTransform_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickTransform_t qt_meta_stringdata_QQuickTransform = {
    {
QT_MOC_LITERAL(0, 0, 15), // "QQuickTransform"
QT_MOC_LITERAL(1, 16, 11), // "QML.Element"
QT_MOC_LITERAL(2, 28, 9), // "anonymous"
QT_MOC_LITERAL(3, 38, 6), // "update"
QT_MOC_LITERAL(4, 45, 0) // ""

    },
    "QQuickTransform\0QML.Element\0anonymous\0"
    "update\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickTransform[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       1,   16, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,

 // slots: name, argc, parameters, tag, flags
       3,    0,   21,    4, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void QQuickTransform::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickTransform *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->update(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickTransform::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickTransform.data,
    qt_meta_data_QQuickTransform,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickTransform::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickTransform::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickTransform.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickTransform::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_QQuickItem_t {
    QByteArrayData data[124];
    char stringdata0[1644];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickItem_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickItem_t qt_meta_stringdata_QQuickItem = {
    {
QT_MOC_LITERAL(0, 0, 10), // "QQuickItem"
QT_MOC_LITERAL(1, 11, 15), // "DefaultProperty"
QT_MOC_LITERAL(2, 27, 4), // "data"
QT_MOC_LITERAL(3, 32, 28), // "qt_QmlJSWrapperFactoryMethod"
QT_MOC_LITERAL(4, 61, 41), // "_q_createJSWrapper(QV4::Execu..."
QT_MOC_LITERAL(5, 103, 11), // "QML.Element"
QT_MOC_LITERAL(6, 115, 4), // "Item"
QT_MOC_LITERAL(7, 120, 19), // "childrenRectChanged"
QT_MOC_LITERAL(8, 140, 0), // ""
QT_MOC_LITERAL(9, 141, 21), // "baselineOffsetChanged"
QT_MOC_LITERAL(10, 163, 12), // "stateChanged"
QT_MOC_LITERAL(11, 176, 12), // "focusChanged"
QT_MOC_LITERAL(12, 189, 18), // "activeFocusChanged"
QT_MOC_LITERAL(13, 208, 23), // "activeFocusOnTabChanged"
QT_MOC_LITERAL(14, 232, 13), // "parentChanged"
QT_MOC_LITERAL(15, 246, 11), // "QQuickItem*"
QT_MOC_LITERAL(16, 258, 22), // "transformOriginChanged"
QT_MOC_LITERAL(17, 281, 15), // "TransformOrigin"
QT_MOC_LITERAL(18, 297, 13), // "smoothChanged"
QT_MOC_LITERAL(19, 311, 19), // "antialiasingChanged"
QT_MOC_LITERAL(20, 331, 11), // "clipChanged"
QT_MOC_LITERAL(21, 343, 13), // "windowChanged"
QT_MOC_LITERAL(22, 357, 13), // "QQuickWindow*"
QT_MOC_LITERAL(23, 371, 6), // "window"
QT_MOC_LITERAL(24, 378, 15), // "childrenChanged"
QT_MOC_LITERAL(25, 394, 14), // "opacityChanged"
QT_MOC_LITERAL(26, 409, 14), // "enabledChanged"
QT_MOC_LITERAL(27, 424, 14), // "visibleChanged"
QT_MOC_LITERAL(28, 439, 22), // "visibleChildrenChanged"
QT_MOC_LITERAL(29, 462, 15), // "rotationChanged"
QT_MOC_LITERAL(30, 478, 12), // "scaleChanged"
QT_MOC_LITERAL(31, 491, 8), // "xChanged"
QT_MOC_LITERAL(32, 500, 8), // "yChanged"
QT_MOC_LITERAL(33, 509, 12), // "widthChanged"
QT_MOC_LITERAL(34, 522, 13), // "heightChanged"
QT_MOC_LITERAL(35, 536, 8), // "zChanged"
QT_MOC_LITERAL(36, 545, 20), // "implicitWidthChanged"
QT_MOC_LITERAL(37, 566, 21), // "implicitHeightChanged"
QT_MOC_LITERAL(38, 588, 22), // "containmentMaskChanged"
QT_MOC_LITERAL(39, 611, 6), // "update"
QT_MOC_LITERAL(40, 618, 24), // "_q_resourceObjectDeleted"
QT_MOC_LITERAL(41, 643, 18), // "_q_createJSWrapper"
QT_MOC_LITERAL(42, 662, 21), // "QV4::ExecutionEngine*"
QT_MOC_LITERAL(43, 684, 11), // "grabToImage"
QT_MOC_LITERAL(44, 696, 8), // "QJSValue"
QT_MOC_LITERAL(45, 705, 8), // "callback"
QT_MOC_LITERAL(46, 714, 10), // "targetSize"
QT_MOC_LITERAL(47, 725, 8), // "contains"
QT_MOC_LITERAL(48, 734, 5), // "point"
QT_MOC_LITERAL(49, 740, 11), // "mapFromItem"
QT_MOC_LITERAL(50, 752, 15), // "QQmlV4Function*"
QT_MOC_LITERAL(51, 768, 9), // "mapToItem"
QT_MOC_LITERAL(52, 778, 13), // "mapFromGlobal"
QT_MOC_LITERAL(53, 792, 11), // "mapToGlobal"
QT_MOC_LITERAL(54, 804, 16), // "forceActiveFocus"
QT_MOC_LITERAL(55, 821, 15), // "Qt::FocusReason"
QT_MOC_LITERAL(56, 837, 6), // "reason"
QT_MOC_LITERAL(57, 844, 20), // "nextItemInFocusChain"
QT_MOC_LITERAL(58, 865, 7), // "forward"
QT_MOC_LITERAL(59, 873, 7), // "childAt"
QT_MOC_LITERAL(60, 881, 1), // "x"
QT_MOC_LITERAL(61, 883, 1), // "y"
QT_MOC_LITERAL(62, 885, 6), // "parent"
QT_MOC_LITERAL(63, 892, 25), // "QQmlListProperty<QObject>"
QT_MOC_LITERAL(64, 918, 9), // "resources"
QT_MOC_LITERAL(65, 928, 8), // "children"
QT_MOC_LITERAL(66, 937, 28), // "QQmlListProperty<QQuickItem>"
QT_MOC_LITERAL(67, 966, 1), // "z"
QT_MOC_LITERAL(68, 968, 5), // "width"
QT_MOC_LITERAL(69, 974, 6), // "height"
QT_MOC_LITERAL(70, 981, 7), // "opacity"
QT_MOC_LITERAL(71, 989, 7), // "enabled"
QT_MOC_LITERAL(72, 997, 7), // "visible"
QT_MOC_LITERAL(73, 1005, 15), // "visibleChildren"
QT_MOC_LITERAL(74, 1021, 6), // "states"
QT_MOC_LITERAL(75, 1028, 29), // "QQmlListProperty<QQuickState>"
QT_MOC_LITERAL(76, 1058, 11), // "transitions"
QT_MOC_LITERAL(77, 1070, 34), // "QQmlListProperty<QQuickTransi..."
QT_MOC_LITERAL(78, 1105, 5), // "state"
QT_MOC_LITERAL(79, 1111, 12), // "childrenRect"
QT_MOC_LITERAL(80, 1124, 7), // "anchors"
QT_MOC_LITERAL(81, 1132, 14), // "QQuickAnchors*"
QT_MOC_LITERAL(82, 1147, 4), // "left"
QT_MOC_LITERAL(83, 1152, 16), // "QQuickAnchorLine"
QT_MOC_LITERAL(84, 1169, 5), // "right"
QT_MOC_LITERAL(85, 1175, 16), // "horizontalCenter"
QT_MOC_LITERAL(86, 1192, 3), // "top"
QT_MOC_LITERAL(87, 1196, 6), // "bottom"
QT_MOC_LITERAL(88, 1203, 14), // "verticalCenter"
QT_MOC_LITERAL(89, 1218, 8), // "baseline"
QT_MOC_LITERAL(90, 1227, 14), // "baselineOffset"
QT_MOC_LITERAL(91, 1242, 4), // "clip"
QT_MOC_LITERAL(92, 1247, 5), // "focus"
QT_MOC_LITERAL(93, 1253, 11), // "activeFocus"
QT_MOC_LITERAL(94, 1265, 16), // "activeFocusOnTab"
QT_MOC_LITERAL(95, 1282, 8), // "rotation"
QT_MOC_LITERAL(96, 1291, 5), // "scale"
QT_MOC_LITERAL(97, 1297, 15), // "transformOrigin"
QT_MOC_LITERAL(98, 1313, 20), // "transformOriginPoint"
QT_MOC_LITERAL(99, 1334, 9), // "transform"
QT_MOC_LITERAL(100, 1344, 33), // "QQmlListProperty<QQuickTransf..."
QT_MOC_LITERAL(101, 1378, 6), // "smooth"
QT_MOC_LITERAL(102, 1385, 12), // "antialiasing"
QT_MOC_LITERAL(103, 1398, 13), // "implicitWidth"
QT_MOC_LITERAL(104, 1412, 14), // "implicitHeight"
QT_MOC_LITERAL(105, 1427, 15), // "containmentMask"
QT_MOC_LITERAL(106, 1443, 5), // "layer"
QT_MOC_LITERAL(107, 1449, 16), // "QQuickItemLayer*"
QT_MOC_LITERAL(108, 1466, 5), // "Flags"
QT_MOC_LITERAL(109, 1472, 4), // "Flag"
QT_MOC_LITERAL(110, 1477, 24), // "ItemClipsChildrenToShape"
QT_MOC_LITERAL(111, 1502, 22), // "ItemAcceptsInputMethod"
QT_MOC_LITERAL(112, 1525, 16), // "ItemIsFocusScope"
QT_MOC_LITERAL(113, 1542, 15), // "ItemHasContents"
QT_MOC_LITERAL(114, 1558, 16), // "ItemAcceptsDrops"
QT_MOC_LITERAL(115, 1575, 7), // "TopLeft"
QT_MOC_LITERAL(116, 1583, 3), // "Top"
QT_MOC_LITERAL(117, 1587, 8), // "TopRight"
QT_MOC_LITERAL(118, 1596, 4), // "Left"
QT_MOC_LITERAL(119, 1601, 6), // "Center"
QT_MOC_LITERAL(120, 1608, 5), // "Right"
QT_MOC_LITERAL(121, 1614, 10), // "BottomLeft"
QT_MOC_LITERAL(122, 1625, 6), // "Bottom"
QT_MOC_LITERAL(123, 1632, 11) // "BottomRight"

    },
    "QQuickItem\0DefaultProperty\0data\0"
    "qt_QmlJSWrapperFactoryMethod\0"
    "_q_createJSWrapper(QV4::ExecutionEngine*)\0"
    "QML.Element\0Item\0childrenRectChanged\0"
    "\0baselineOffsetChanged\0stateChanged\0"
    "focusChanged\0activeFocusChanged\0"
    "activeFocusOnTabChanged\0parentChanged\0"
    "QQuickItem*\0transformOriginChanged\0"
    "TransformOrigin\0smoothChanged\0"
    "antialiasingChanged\0clipChanged\0"
    "windowChanged\0QQuickWindow*\0window\0"
    "childrenChanged\0opacityChanged\0"
    "enabledChanged\0visibleChanged\0"
    "visibleChildrenChanged\0rotationChanged\0"
    "scaleChanged\0xChanged\0yChanged\0"
    "widthChanged\0heightChanged\0zChanged\0"
    "implicitWidthChanged\0implicitHeightChanged\0"
    "containmentMaskChanged\0update\0"
    "_q_resourceObjectDeleted\0_q_createJSWrapper\0"
    "QV4::ExecutionEngine*\0grabToImage\0"
    "QJSValue\0callback\0targetSize\0contains\0"
    "point\0mapFromItem\0QQmlV4Function*\0"
    "mapToItem\0mapFromGlobal\0mapToGlobal\0"
    "forceActiveFocus\0Qt::FocusReason\0"
    "reason\0nextItemInFocusChain\0forward\0"
    "childAt\0x\0y\0parent\0QQmlListProperty<QObject>\0"
    "resources\0children\0QQmlListProperty<QQuickItem>\0"
    "z\0width\0height\0opacity\0enabled\0visible\0"
    "visibleChildren\0states\0"
    "QQmlListProperty<QQuickState>\0transitions\0"
    "QQmlListProperty<QQuickTransition>\0"
    "state\0childrenRect\0anchors\0QQuickAnchors*\0"
    "left\0QQuickAnchorLine\0right\0"
    "horizontalCenter\0top\0bottom\0verticalCenter\0"
    "baseline\0baselineOffset\0clip\0focus\0"
    "activeFocus\0activeFocusOnTab\0rotation\0"
    "scale\0transformOrigin\0transformOriginPoint\0"
    "transform\0QQmlListProperty<QQuickTransform>\0"
    "smooth\0antialiasing\0implicitWidth\0"
    "implicitHeight\0containmentMask\0layer\0"
    "QQuickItemLayer*\0Flags\0Flag\0"
    "ItemClipsChildrenToShape\0"
    "ItemAcceptsInputMethod\0ItemIsFocusScope\0"
    "ItemHasContents\0ItemAcceptsDrops\0"
    "TopLeft\0Top\0TopRight\0Left\0Center\0Right\0"
    "BottomLeft\0Bottom\0BottomRight"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickItem[] = {

 // content:
       8,       // revision
       0,       // classname
       3,   14, // classinfo
      42,   20, // methods
      41,  366, // properties
       2,  571, // enums/sets
       0,    0, // constructors
       0,       // flags
      27,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,

 // signals: name, argc, parameters, tag, flags
       7,    1,  272,    8, 0x06 /* Public */,
       9,    1,  275,    8, 0x06 /* Public */,
      10,    1,  278,    8, 0x06 /* Public */,
      11,    1,  281,    8, 0x06 /* Public */,
      12,    1,  284,    8, 0x06 /* Public */,
      13,    1,  287,    8, 0x86 /* Public | MethodRevisioned */,
      14,    1,  290,    8, 0x06 /* Public */,
      16,    1,  293,    8, 0x06 /* Public */,
      18,    1,  296,    8, 0x06 /* Public */,
      19,    1,  299,    8, 0x06 /* Public */,
      20,    1,  302,    8, 0x06 /* Public */,
      21,    1,  305,    8, 0x86 /* Public | MethodRevisioned */,
      24,    0,  308,    8, 0x06 /* Public */,
      25,    0,  309,    8, 0x06 /* Public */,
      26,    0,  310,    8, 0x06 /* Public */,
      27,    0,  311,    8, 0x06 /* Public */,
      28,    0,  312,    8, 0x06 /* Public */,
      29,    0,  313,    8, 0x06 /* Public */,
      30,    0,  314,    8, 0x06 /* Public */,
      31,    0,  315,    8, 0x06 /* Public */,
      32,    0,  316,    8, 0x06 /* Public */,
      33,    0,  317,    8, 0x06 /* Public */,
      34,    0,  318,    8, 0x06 /* Public */,
      35,    0,  319,    8, 0x06 /* Public */,
      36,    0,  320,    8, 0x06 /* Public */,
      37,    0,  321,    8, 0x06 /* Public */,
      38,    0,  322,    8, 0x86 /* Public | MethodRevisioned */,

 // slots: name, argc, parameters, tag, flags
      39,    0,  323,    8, 0x0a /* Public */,
      40,    1,  324,    8, 0x08 /* Private */,
      41,    1,  327,    8, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
      43,    2,  330,    8, 0x82 /* Public | MethodRevisioned */,
      43,    1,  335,    8, 0xa2 /* Public | MethodCloned | MethodRevisioned */,
      47,    1,  338,    8, 0x02 /* Public */,
      49,    1,  341,    8, 0x02 /* Public */,
      51,    1,  344,    8, 0x02 /* Public */,
      52,    1,  347,    8, 0x82 /* Public | MethodRevisioned */,
      53,    1,  350,    8, 0x82 /* Public | MethodRevisioned */,
      54,    0,  353,    8, 0x02 /* Public */,
      54,    1,  354,    8, 0x02 /* Public */,
      57,    1,  357,    8, 0x82 /* Public | MethodRevisioned */,
      57,    0,  360,    8, 0xa2 /* Public | MethodCloned | MethodRevisioned */,
      59,    2,  361,    8, 0x02 /* Public */,

 // signals: revision
       0,
       0,
       0,
       0,
       0,
       1,
       0,
       0,
       0,
       0,
       0,
       1,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
      11,

 // slots: revision
       0,
       0,
       0,

 // methods: revision
       4,
       4,
       0,
       0,
       0,
       7,
       7,
       0,
       0,
       1,
       1,
       0,

 // signals: parameters
    QMetaType::Void, QMetaType::QRectF,    8,
    QMetaType::Void, QMetaType::QReal,    8,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::Bool,    8,
    QMetaType::Void, QMetaType::Bool,    8,
    QMetaType::Void, QMetaType::Bool,    8,
    QMetaType::Void, 0x80000000 | 15,    8,
    QMetaType::Void, 0x80000000 | 17,    8,
    QMetaType::Void, QMetaType::Bool,    8,
    QMetaType::Void, QMetaType::Bool,    8,
    QMetaType::Void, QMetaType::Bool,    8,
    QMetaType::Void, 0x80000000 | 22,   23,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QObjectStar,    8,
    QMetaType::ULongLong, 0x80000000 | 42,    8,

 // methods: parameters
    QMetaType::Bool, 0x80000000 | 44, QMetaType::QSize,   45,   46,
    QMetaType::Bool, 0x80000000 | 44,   45,
    QMetaType::Bool, QMetaType::QPointF,   48,
    QMetaType::Void, 0x80000000 | 50,    8,
    QMetaType::Void, 0x80000000 | 50,    8,
    QMetaType::Void, 0x80000000 | 50,    8,
    QMetaType::Void, 0x80000000 | 50,    8,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 55,   56,
    0x80000000 | 15, QMetaType::Bool,   58,
    0x80000000 | 15,
    0x80000000 | 15, QMetaType::QReal, QMetaType::QReal,   60,   61,

 // properties: name, type, flags
      62, 0x80000000 | 15, 0x0049480b,
       2, 0x80000000 | 63, 0x00094009,
      64, 0x80000000 | 63, 0x00094009,
      65, 0x80000000 | 66, 0x00494009,
      60, QMetaType::QReal, 0x00495903,
      61, QMetaType::QReal, 0x00495903,
      67, QMetaType::QReal, 0x00495903,
      68, QMetaType::QReal, 0x00495907,
      69, QMetaType::QReal, 0x00495907,
      70, QMetaType::QReal, 0x00495903,
      71, QMetaType::Bool, 0x00495103,
      72, QMetaType::Bool, 0x00495903,
      73, 0x80000000 | 66, 0x00494009,
      74, 0x80000000 | 75, 0x00094009,
      76, 0x80000000 | 77, 0x00094009,
      78, QMetaType::QString, 0x00495103,
      79, QMetaType::QRectF, 0x00494801,
      80, 0x80000000 | 81, 0x00094c09,
      82, 0x80000000 | 83, 0x00095c09,
      84, 0x80000000 | 83, 0x00095c09,
      85, 0x80000000 | 83, 0x00095c09,
      86, 0x80000000 | 83, 0x00095c09,
      87, 0x80000000 | 83, 0x00095c09,
      88, 0x80000000 | 83, 0x00095c09,
      89, 0x80000000 | 83, 0x00095c09,
      90, QMetaType::QReal, 0x00495103,
      91, QMetaType::Bool, 0x00495103,
      92, QMetaType::Bool, 0x00495903,
      93, QMetaType::Bool, 0x00495801,
      94, QMetaType::Bool, 0x00c95903,
      95, QMetaType::QReal, 0x00495103,
      96, QMetaType::QReal, 0x00495103,
      97, 0x80000000 | 17, 0x0049510b,
      98, QMetaType::QPointF, 0x00095001,
      99, 0x80000000 | 100, 0x00094809,
     101, QMetaType::Bool, 0x00495103,
     102, QMetaType::Bool, 0x00495107,
     103, QMetaType::QReal, 0x00495103,
     104, QMetaType::QReal, 0x00495103,
     105, QMetaType::QObjectStar, 0x00c95103,
     106, 0x80000000 | 107, 0x00094c09,

 // properties: notify_signal_id
       6,
       0,
       0,
      12,
      19,
      20,
      23,
      21,
      22,
      13,
      14,
      15,
      16,
       0,
       0,
       2,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       1,
      10,
       3,
       4,
       5,
      17,
      18,
       7,
       0,
       0,
       8,
       9,
      24,
      25,
      26,
       0,

 // properties: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       1,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
      11,
       0,

 // enums: name, alias, flags, count, data
     108,  109, 0x1,    5,  581,
      17,   17, 0x0,    9,  591,

 // enum data: key, value
     110, uint(QQuickItem::ItemClipsChildrenToShape),
     111, uint(QQuickItem::ItemAcceptsInputMethod),
     112, uint(QQuickItem::ItemIsFocusScope),
     113, uint(QQuickItem::ItemHasContents),
     114, uint(QQuickItem::ItemAcceptsDrops),
     115, uint(QQuickItem::TopLeft),
     116, uint(QQuickItem::Top),
     117, uint(QQuickItem::TopRight),
     118, uint(QQuickItem::Left),
     119, uint(QQuickItem::Center),
     120, uint(QQuickItem::Right),
     121, uint(QQuickItem::BottomLeft),
     122, uint(QQuickItem::Bottom),
     123, uint(QQuickItem::BottomRight),

       0        // eod
};

void QQuickItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickItem *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->childrenRectChanged((*reinterpret_cast< const QRectF(*)>(_a[1]))); break;
        case 1: _t->baselineOffsetChanged((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 2: _t->stateChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->focusChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->activeFocusChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->activeFocusOnTabChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->parentChanged((*reinterpret_cast< QQuickItem*(*)>(_a[1]))); break;
        case 7: _t->transformOriginChanged((*reinterpret_cast< TransformOrigin(*)>(_a[1]))); break;
        case 8: _t->smoothChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->antialiasingChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->clipChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->windowChanged((*reinterpret_cast< QQuickWindow*(*)>(_a[1]))); break;
        case 12: _t->childrenChanged(); break;
        case 13: _t->opacityChanged(); break;
        case 14: _t->enabledChanged(); break;
        case 15: _t->visibleChanged(); break;
        case 16: _t->visibleChildrenChanged(); break;
        case 17: _t->rotationChanged(); break;
        case 18: _t->scaleChanged(); break;
        case 19: _t->xChanged(); break;
        case 20: _t->yChanged(); break;
        case 21: _t->widthChanged(); break;
        case 22: _t->heightChanged(); break;
        case 23: _t->zChanged(); break;
        case 24: _t->implicitWidthChanged(); break;
        case 25: _t->implicitHeightChanged(); break;
        case 26: _t->containmentMaskChanged(); break;
        case 27: _t->update(); break;
        case 28: _t->d_func()->_q_resourceObjectDeleted((*reinterpret_cast< QObject*(*)>(_a[1]))); break;
        case 29: { quint64 _r = _t->d_func()->_q_createJSWrapper((*reinterpret_cast< QV4::ExecutionEngine*(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< quint64*>(_a[0]) = std::move(_r); }  break;
        case 30: { bool _r = _t->grabToImage((*reinterpret_cast< const QJSValue(*)>(_a[1])),(*reinterpret_cast< const QSize(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 31: { bool _r = _t->grabToImage((*reinterpret_cast< const QJSValue(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 32: { bool _r = _t->contains((*reinterpret_cast< const QPointF(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 33: _t->mapFromItem((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        case 34: _t->mapToItem((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        case 35: _t->mapFromGlobal((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        case 36: _t->mapToGlobal((*reinterpret_cast< QQmlV4Function*(*)>(_a[1]))); break;
        case 37: _t->forceActiveFocus(); break;
        case 38: _t->forceActiveFocus((*reinterpret_cast< Qt::FocusReason(*)>(_a[1]))); break;
        case 39: { QQuickItem* _r = _t->nextItemInFocusChain((*reinterpret_cast< bool(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QQuickItem**>(_a[0]) = std::move(_r); }  break;
        case 40: { QQuickItem* _r = _t->nextItemInFocusChain();
            if (_a[0]) *reinterpret_cast< QQuickItem**>(_a[0]) = std::move(_r); }  break;
        case 41: { QQuickItem* _r = _t->childAt((*reinterpret_cast< qreal(*)>(_a[1])),(*reinterpret_cast< qreal(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QQuickItem**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
            }
            break;
        case 30:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
            }
            break;
        case 31:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickItem::*)(const QRectF & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItem::childrenRectChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickItem::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItem::baselineOffsetChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickItem::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItem::stateChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickItem::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItem::focusChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickItem::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItem::activeFocusChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickItem::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItem::activeFocusOnTabChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickItem::*)(QQuickItem * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItem::parentChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickItem::*)(TransformOrigin );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItem::transformOriginChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickItem::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItem::smoothChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickItem::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItem::antialiasingChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QQuickItem::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItem::clipChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QQuickItem::*)(QQuickWindow * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItem::windowChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItem::childrenChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItem::opacityChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItem::enabledChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItem::visibleChanged)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (QQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItem::visibleChildrenChanged)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (QQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItem::rotationChanged)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (QQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItem::scaleChanged)) {
                *result = 18;
                return;
            }
        }
        {
            using _t = void (QQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItem::xChanged)) {
                *result = 19;
                return;
            }
        }
        {
            using _t = void (QQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItem::yChanged)) {
                *result = 20;
                return;
            }
        }
        {
            using _t = void (QQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItem::widthChanged)) {
                *result = 21;
                return;
            }
        }
        {
            using _t = void (QQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItem::heightChanged)) {
                *result = 22;
                return;
            }
        }
        {
            using _t = void (QQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItem::zChanged)) {
                *result = 23;
                return;
            }
        }
        {
            using _t = void (QQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItem::implicitWidthChanged)) {
                *result = 24;
                return;
            }
        }
        {
            using _t = void (QQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItem::implicitHeightChanged)) {
                *result = 25;
                return;
            }
        }
        {
            using _t = void (QQuickItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickItem::containmentMaskChanged)) {
                *result = 26;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlListProperty<QObject> >(); break;
        case 12:
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlListProperty<QQuickItem> >(); break;
        case 34:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlListProperty<QQuickTransform> >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickItem *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQuickItem**>(_v) = _t->parentItem(); break;
        case 1: *reinterpret_cast< QQmlListProperty<QObject>*>(_v) = _t->QQuickItem::d_func()->data(); break;
        case 2: *reinterpret_cast< QQmlListProperty<QObject>*>(_v) = _t->QQuickItem::d_func()->resources(); break;
        case 3: *reinterpret_cast< QQmlListProperty<QQuickItem>*>(_v) = _t->QQuickItem::d_func()->children(); break;
        case 4: *reinterpret_cast< qreal*>(_v) = _t->x(); break;
        case 5: *reinterpret_cast< qreal*>(_v) = _t->y(); break;
        case 6: *reinterpret_cast< qreal*>(_v) = _t->z(); break;
        case 7: *reinterpret_cast< qreal*>(_v) = _t->width(); break;
        case 8: *reinterpret_cast< qreal*>(_v) = _t->height(); break;
        case 9: *reinterpret_cast< qreal*>(_v) = _t->opacity(); break;
        case 10: *reinterpret_cast< bool*>(_v) = _t->isEnabled(); break;
        case 11: *reinterpret_cast< bool*>(_v) = _t->isVisible(); break;
        case 12: *reinterpret_cast< QQmlListProperty<QQuickItem>*>(_v) = _t->QQuickItem::d_func()->visibleChildren(); break;
        case 13: *reinterpret_cast< QQmlListProperty<QQuickState>*>(_v) = _t->QQuickItem::d_func()->states(); break;
        case 14: *reinterpret_cast< QQmlListProperty<QQuickTransition>*>(_v) = _t->QQuickItem::d_func()->transitions(); break;
        case 15: *reinterpret_cast< QString*>(_v) = _t->state(); break;
        case 16: *reinterpret_cast< QRectF*>(_v) = _t->childrenRect(); break;
        case 17: *reinterpret_cast< QQuickAnchors**>(_v) = _t->QQuickItem::d_func()->anchors(); break;
        case 18: *reinterpret_cast< QQuickAnchorLine*>(_v) = _t->QQuickItem::d_func()->left(); break;
        case 19: *reinterpret_cast< QQuickAnchorLine*>(_v) = _t->QQuickItem::d_func()->right(); break;
        case 20: *reinterpret_cast< QQuickAnchorLine*>(_v) = _t->QQuickItem::d_func()->horizontalCenter(); break;
        case 21: *reinterpret_cast< QQuickAnchorLine*>(_v) = _t->QQuickItem::d_func()->top(); break;
        case 22: *reinterpret_cast< QQuickAnchorLine*>(_v) = _t->QQuickItem::d_func()->bottom(); break;
        case 23: *reinterpret_cast< QQuickAnchorLine*>(_v) = _t->QQuickItem::d_func()->verticalCenter(); break;
        case 24: *reinterpret_cast< QQuickAnchorLine*>(_v) = _t->QQuickItem::d_func()->baseline(); break;
        case 25: *reinterpret_cast< qreal*>(_v) = _t->baselineOffset(); break;
        case 26: *reinterpret_cast< bool*>(_v) = _t->clip(); break;
        case 27: *reinterpret_cast< bool*>(_v) = _t->hasFocus(); break;
        case 28: *reinterpret_cast< bool*>(_v) = _t->hasActiveFocus(); break;
        case 29: *reinterpret_cast< bool*>(_v) = _t->activeFocusOnTab(); break;
        case 30: *reinterpret_cast< qreal*>(_v) = _t->rotation(); break;
        case 31: *reinterpret_cast< qreal*>(_v) = _t->scale(); break;
        case 32: *reinterpret_cast< TransformOrigin*>(_v) = _t->transformOrigin(); break;
        case 33: *reinterpret_cast< QPointF*>(_v) = _t->transformOriginPoint(); break;
        case 34: *reinterpret_cast< QQmlListProperty<QQuickTransform>*>(_v) = _t->transform(); break;
        case 35: *reinterpret_cast< bool*>(_v) = _t->smooth(); break;
        case 36: *reinterpret_cast< bool*>(_v) = _t->antialiasing(); break;
        case 37: *reinterpret_cast< qreal*>(_v) = _t->implicitWidth(); break;
        case 38: *reinterpret_cast< qreal*>(_v) = _t->implicitHeight(); break;
        case 39: *reinterpret_cast< QObject**>(_v) = _t->containmentMask(); break;
        case 40: *reinterpret_cast< QQuickItemLayer**>(_v) = _t->QQuickItem::d_func()->layer(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickItem *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setParentItem(*reinterpret_cast< QQuickItem**>(_v)); break;
        case 4: _t->setX(*reinterpret_cast< qreal*>(_v)); break;
        case 5: _t->setY(*reinterpret_cast< qreal*>(_v)); break;
        case 6: _t->setZ(*reinterpret_cast< qreal*>(_v)); break;
        case 7: _t->setWidth(*reinterpret_cast< qreal*>(_v)); break;
        case 8: _t->setHeight(*reinterpret_cast< qreal*>(_v)); break;
        case 9: _t->setOpacity(*reinterpret_cast< qreal*>(_v)); break;
        case 10: _t->setEnabled(*reinterpret_cast< bool*>(_v)); break;
        case 11: _t->setVisible(*reinterpret_cast< bool*>(_v)); break;
        case 15: _t->setState(*reinterpret_cast< QString*>(_v)); break;
        case 25: _t->setBaselineOffset(*reinterpret_cast< qreal*>(_v)); break;
        case 26: _t->setClip(*reinterpret_cast< bool*>(_v)); break;
        case 27: _t->setFocus(*reinterpret_cast< bool*>(_v)); break;
        case 29: _t->setActiveFocusOnTab(*reinterpret_cast< bool*>(_v)); break;
        case 30: _t->setRotation(*reinterpret_cast< qreal*>(_v)); break;
        case 31: _t->setScale(*reinterpret_cast< qreal*>(_v)); break;
        case 32: _t->setTransformOrigin(*reinterpret_cast< TransformOrigin*>(_v)); break;
        case 35: _t->setSmooth(*reinterpret_cast< bool*>(_v)); break;
        case 36: _t->setAntialiasing(*reinterpret_cast< bool*>(_v)); break;
        case 37: _t->setImplicitWidth(*reinterpret_cast< qreal*>(_v)); break;
        case 38: _t->setImplicitHeight(*reinterpret_cast< qreal*>(_v)); break;
        case 39: _t->setContainmentMask(*reinterpret_cast< QObject**>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        QQuickItem *_t = static_cast<QQuickItem *>(_o);
        (void)_t;
        switch (_id) {
        case 7: _t->resetWidth(); break;
        case 8: _t->resetHeight(); break;
        case 36: _t->resetAntialiasing(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickItem.data,
    qt_meta_data_QQuickItem,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickItem.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 42)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 42;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 42)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 42;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 41;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 41;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 41;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 41;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 41;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 41;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickItem::childrenRectChanged(const QRectF & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QQuickItem::baselineOffsetChanged(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QQuickItem::stateChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QQuickItem::focusChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QQuickItem::activeFocusChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QQuickItem::activeFocusOnTabChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QQuickItem::parentChanged(QQuickItem * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QQuickItem::transformOriginChanged(TransformOrigin _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QQuickItem::smoothChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QQuickItem::antialiasingChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void QQuickItem::clipChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void QQuickItem::windowChanged(QQuickWindow * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void QQuickItem::childrenChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void QQuickItem::opacityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void QQuickItem::enabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void QQuickItem::visibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void QQuickItem::visibleChildrenChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void QQuickItem::rotationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void QQuickItem::scaleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void QQuickItem::xChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void QQuickItem::yChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void QQuickItem::widthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 21, nullptr);
}

// SIGNAL 22
void QQuickItem::heightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 22, nullptr);
}

// SIGNAL 23
void QQuickItem::zChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 23, nullptr);
}

// SIGNAL 24
void QQuickItem::implicitWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 24, nullptr);
}

// SIGNAL 25
void QQuickItem::implicitHeightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 25, nullptr);
}

// SIGNAL 26
void QQuickItem::containmentMaskChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 26, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
