/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#include <qqmlsettings_p.h>

void qml_register_types_Qt_labs_settings()
{
    qmlRegisterTypesAndRevisions<QQmlSettings>("Qt.labs.settings", 1);
    qmlRegisterModule("Qt.labs.settings", 1, 1);
}

static const QQmlModuleRegistration registration("Qt.labs.settings", 1, qml_register_types_Qt_labs_settings);
