/****************************************************************************
** Meta object code from reading C++ file 'quicktestutil_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../kde-qtdeclarative/src/imports/testlib/quicktestutil_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'quicktestutil_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QuickTestUtil_t {
    QByteArrayData data[17];
    char stringdata0[195];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QuickTestUtil_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QuickTestUtil_t qt_meta_stringdata_QuickTestUtil = {
    {
QT_MOC_LITERAL(0, 0, 13), // "QuickTestUtil"
QT_MOC_LITERAL(1, 14, 11), // "QML.Element"
QT_MOC_LITERAL(2, 26, 8), // "TestUtil"
QT_MOC_LITERAL(3, 35, 30), // "printAvailableFunctionsChanged"
QT_MOC_LITERAL(4, 66, 0), // ""
QT_MOC_LITERAL(5, 67, 20), // "dragThresholdChanged"
QT_MOC_LITERAL(6, 88, 8), // "typeName"
QT_MOC_LITERAL(7, 97, 8), // "QJSValue"
QT_MOC_LITERAL(8, 106, 1), // "v"
QT_MOC_LITERAL(9, 108, 7), // "compare"
QT_MOC_LITERAL(10, 116, 3), // "act"
QT_MOC_LITERAL(11, 120, 3), // "exp"
QT_MOC_LITERAL(12, 124, 10), // "callerFile"
QT_MOC_LITERAL(13, 135, 10), // "frameIndex"
QT_MOC_LITERAL(14, 146, 10), // "callerLine"
QT_MOC_LITERAL(15, 157, 23), // "printAvailableFunctions"
QT_MOC_LITERAL(16, 181, 13) // "dragThreshold"

    },
    "QuickTestUtil\0QML.Element\0TestUtil\0"
    "printAvailableFunctionsChanged\0\0"
    "dragThresholdChanged\0typeName\0QJSValue\0"
    "v\0compare\0act\0exp\0callerFile\0frameIndex\0"
    "callerLine\0printAvailableFunctions\0"
    "dragThreshold"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QuickTestUtil[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       8,   16, // methods
       2,   74, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   56,    4, 0x06 /* Public */,
       5,    0,   57,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    1,   58,    4, 0x0a /* Public */,
       9,    2,   61,    4, 0x0a /* Public */,
      12,    1,   66,    4, 0x0a /* Public */,
      12,    0,   69,    4, 0x2a /* Public | MethodCloned */,
      14,    1,   70,    4, 0x0a /* Public */,
      14,    0,   73,    4, 0x2a /* Public | MethodCloned */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    0x80000000 | 7, QMetaType::QVariant,    8,
    QMetaType::Bool, QMetaType::QVariant, QMetaType::QVariant,   10,   11,
    0x80000000 | 7, QMetaType::Int,   13,
    0x80000000 | 7,
    QMetaType::Int, QMetaType::Int,   13,
    QMetaType::Int,

 // properties: name, type, flags
      15, QMetaType::Bool, 0x00495001,
      16, QMetaType::Int, 0x00495001,

 // properties: notify_signal_id
       0,
       1,

       0        // eod
};

void QuickTestUtil::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QuickTestUtil *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->printAvailableFunctionsChanged(); break;
        case 1: _t->dragThresholdChanged(); break;
        case 2: { QJSValue _r = _t->typeName((*reinterpret_cast< const QVariant(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->compare((*reinterpret_cast< const QVariant(*)>(_a[1])),(*reinterpret_cast< const QVariant(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { QJSValue _r = _t->callerFile((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 5: { QJSValue _r = _t->callerFile();
            if (_a[0]) *reinterpret_cast< QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 6: { int _r = _t->callerLine((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 7: { int _r = _t->callerLine();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QuickTestUtil::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QuickTestUtil::printAvailableFunctionsChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QuickTestUtil::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QuickTestUtil::dragThresholdChanged)) {
                *result = 1;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QuickTestUtil *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->printAvailableFunctions(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->dragThreshold(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QuickTestUtil::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QuickTestUtil.data,
    qt_meta_data_QuickTestUtil,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QuickTestUtil::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QuickTestUtil::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QuickTestUtil.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QuickTestUtil::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QuickTestUtil::printAvailableFunctionsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QuickTestUtil::dragThresholdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
