/****************************************************************************
** Meta object code from reading C++ file 'qquickaccessibleattached_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/quick/items/qquickaccessibleattached_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickaccessibleattached_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickAccessibleAttached_t {
    QByteArrayData data[62];
    char stringdata0[897];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickAccessibleAttached_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickAccessibleAttached_t qt_meta_stringdata_QQuickAccessibleAttached = {
    {
QT_MOC_LITERAL(0, 0, 24), // "QQuickAccessibleAttached"
QT_MOC_LITERAL(1, 25, 11), // "QML.Element"
QT_MOC_LITERAL(2, 37, 10), // "Accessible"
QT_MOC_LITERAL(3, 48, 13), // "QML.Creatable"
QT_MOC_LITERAL(4, 62, 5), // "false"
QT_MOC_LITERAL(5, 68, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(6, 90, 53), // "Accessible is only available ..."
QT_MOC_LITERAL(7, 144, 12), // "QML.Attached"
QT_MOC_LITERAL(8, 157, 16), // "checkableChanged"
QT_MOC_LITERAL(9, 174, 0), // ""
QT_MOC_LITERAL(10, 175, 3), // "arg"
QT_MOC_LITERAL(11, 179, 14), // "checkedChanged"
QT_MOC_LITERAL(12, 194, 15), // "editableChanged"
QT_MOC_LITERAL(13, 210, 16), // "focusableChanged"
QT_MOC_LITERAL(14, 227, 14), // "focusedChanged"
QT_MOC_LITERAL(15, 242, 16), // "multiLineChanged"
QT_MOC_LITERAL(16, 259, 15), // "readOnlyChanged"
QT_MOC_LITERAL(17, 275, 15), // "selectedChanged"
QT_MOC_LITERAL(18, 291, 17), // "selectableChanged"
QT_MOC_LITERAL(19, 309, 14), // "pressedChanged"
QT_MOC_LITERAL(20, 324, 22), // "checkStateMixedChanged"
QT_MOC_LITERAL(21, 347, 20), // "defaultButtonChanged"
QT_MOC_LITERAL(22, 368, 19), // "passwordEditChanged"
QT_MOC_LITERAL(23, 388, 21), // "selectableTextChanged"
QT_MOC_LITERAL(24, 410, 17), // "searchEditChanged"
QT_MOC_LITERAL(25, 428, 11), // "roleChanged"
QT_MOC_LITERAL(26, 440, 11), // "nameChanged"
QT_MOC_LITERAL(27, 452, 18), // "descriptionChanged"
QT_MOC_LITERAL(28, 471, 14), // "ignoredChanged"
QT_MOC_LITERAL(29, 486, 11), // "pressAction"
QT_MOC_LITERAL(30, 498, 12), // "toggleAction"
QT_MOC_LITERAL(31, 511, 14), // "increaseAction"
QT_MOC_LITERAL(32, 526, 14), // "decreaseAction"
QT_MOC_LITERAL(33, 541, 14), // "scrollUpAction"
QT_MOC_LITERAL(34, 556, 16), // "scrollDownAction"
QT_MOC_LITERAL(35, 573, 16), // "scrollLeftAction"
QT_MOC_LITERAL(36, 590, 17), // "scrollRightAction"
QT_MOC_LITERAL(37, 608, 18), // "previousPageAction"
QT_MOC_LITERAL(38, 627, 14), // "nextPageAction"
QT_MOC_LITERAL(39, 642, 12), // "valueChanged"
QT_MOC_LITERAL(40, 655, 21), // "cursorPositionChanged"
QT_MOC_LITERAL(41, 677, 10), // "setIgnored"
QT_MOC_LITERAL(42, 688, 7), // "ignored"
QT_MOC_LITERAL(43, 696, 4), // "role"
QT_MOC_LITERAL(44, 701, 17), // "QAccessible::Role"
QT_MOC_LITERAL(45, 719, 4), // "name"
QT_MOC_LITERAL(46, 724, 11), // "description"
QT_MOC_LITERAL(47, 736, 9), // "checkable"
QT_MOC_LITERAL(48, 746, 7), // "checked"
QT_MOC_LITERAL(49, 754, 8), // "editable"
QT_MOC_LITERAL(50, 763, 9), // "focusable"
QT_MOC_LITERAL(51, 773, 7), // "focused"
QT_MOC_LITERAL(52, 781, 9), // "multiLine"
QT_MOC_LITERAL(53, 791, 8), // "readOnly"
QT_MOC_LITERAL(54, 800, 8), // "selected"
QT_MOC_LITERAL(55, 809, 10), // "selectable"
QT_MOC_LITERAL(56, 820, 7), // "pressed"
QT_MOC_LITERAL(57, 828, 15), // "checkStateMixed"
QT_MOC_LITERAL(58, 844, 13), // "defaultButton"
QT_MOC_LITERAL(59, 858, 12), // "passwordEdit"
QT_MOC_LITERAL(60, 871, 14), // "selectableText"
QT_MOC_LITERAL(61, 886, 10) // "searchEdit"

    },
    "QQuickAccessibleAttached\0QML.Element\0"
    "Accessible\0QML.Creatable\0false\0"
    "QML.UncreatableReason\0"
    "Accessible is only available via attached properties.\0"
    "QML.Attached\0checkableChanged\0\0arg\0"
    "checkedChanged\0editableChanged\0"
    "focusableChanged\0focusedChanged\0"
    "multiLineChanged\0readOnlyChanged\0"
    "selectedChanged\0selectableChanged\0"
    "pressedChanged\0checkStateMixedChanged\0"
    "defaultButtonChanged\0passwordEditChanged\0"
    "selectableTextChanged\0searchEditChanged\0"
    "roleChanged\0nameChanged\0descriptionChanged\0"
    "ignoredChanged\0pressAction\0toggleAction\0"
    "increaseAction\0decreaseAction\0"
    "scrollUpAction\0scrollDownAction\0"
    "scrollLeftAction\0scrollRightAction\0"
    "previousPageAction\0nextPageAction\0"
    "valueChanged\0cursorPositionChanged\0"
    "setIgnored\0ignored\0role\0QAccessible::Role\0"
    "name\0description\0checkable\0checked\0"
    "editable\0focusable\0focused\0multiLine\0"
    "readOnly\0selected\0selectable\0pressed\0"
    "checkStateMixed\0defaultButton\0"
    "passwordEdit\0selectableText\0searchEdit"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickAccessibleAttached[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
      32,   22, // methods
      19,  246, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      29,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    0,

 // signals: name, argc, parameters, tag, flags
       8,    1,  182,    9, 0x06 /* Public */,
      11,    1,  185,    9, 0x06 /* Public */,
      12,    1,  188,    9, 0x06 /* Public */,
      13,    1,  191,    9, 0x06 /* Public */,
      14,    1,  194,    9, 0x06 /* Public */,
      15,    1,  197,    9, 0x06 /* Public */,
      16,    1,  200,    9, 0x06 /* Public */,
      17,    1,  203,    9, 0x06 /* Public */,
      18,    1,  206,    9, 0x06 /* Public */,
      19,    1,  209,    9, 0x06 /* Public */,
      20,    1,  212,    9, 0x06 /* Public */,
      21,    1,  215,    9, 0x06 /* Public */,
      22,    1,  218,    9, 0x06 /* Public */,
      23,    1,  221,    9, 0x06 /* Public */,
      24,    1,  224,    9, 0x06 /* Public */,
      25,    0,  227,    9, 0x06 /* Public */,
      26,    0,  228,    9, 0x06 /* Public */,
      27,    0,  229,    9, 0x06 /* Public */,
      28,    0,  230,    9, 0x06 /* Public */,
      29,    0,  231,    9, 0x06 /* Public */,
      30,    0,  232,    9, 0x06 /* Public */,
      31,    0,  233,    9, 0x06 /* Public */,
      32,    0,  234,    9, 0x06 /* Public */,
      33,    0,  235,    9, 0x06 /* Public */,
      34,    0,  236,    9, 0x06 /* Public */,
      35,    0,  237,    9, 0x06 /* Public */,
      36,    0,  238,    9, 0x06 /* Public */,
      37,    0,  239,    9, 0x06 /* Public */,
      38,    0,  240,    9, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      39,    0,  241,    9, 0x0a /* Public */,
      40,    0,  242,    9, 0x0a /* Public */,
      41,    1,  243,    9, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   42,

 // properties: name, type, flags
      43, 0x80000000 | 44, 0x0049590b,
      45, QMetaType::QString, 0x00495903,
      46, QMetaType::QString, 0x00495903,
      42, QMetaType::Bool, 0x00495903,
      47, QMetaType::Bool, 0x00495803,
      48, QMetaType::Bool, 0x00495803,
      49, QMetaType::Bool, 0x00495803,
      50, QMetaType::Bool, 0x00495803,
      51, QMetaType::Bool, 0x00495803,
      52, QMetaType::Bool, 0x00495803,
      53, QMetaType::Bool, 0x00495803,
      54, QMetaType::Bool, 0x00495803,
      55, QMetaType::Bool, 0x00495803,
      56, QMetaType::Bool, 0x00495803,
      57, QMetaType::Bool, 0x00495803,
      58, QMetaType::Bool, 0x00495803,
      59, QMetaType::Bool, 0x00495803,
      60, QMetaType::Bool, 0x00495803,
      61, QMetaType::Bool, 0x00495803,

 // properties: notify_signal_id
      15,
      16,
      17,
      18,
       0,
       1,
       2,
       3,
       4,
       5,
       6,
       7,
       8,
       9,
      10,
      11,
      12,
      13,
      14,

 // enums: name, alias, flags, count, data

 // enum data: key, value

       0        // eod
};

void QQuickAccessibleAttached::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickAccessibleAttached *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->checkableChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->checkedChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->editableChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->focusableChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->focusedChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->multiLineChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->readOnlyChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->selectedChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->selectableChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->pressedChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->checkStateMixedChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->defaultButtonChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 12: _t->passwordEditChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 13: _t->selectableTextChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 14: _t->searchEditChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 15: _t->roleChanged(); break;
        case 16: _t->nameChanged(); break;
        case 17: _t->descriptionChanged(); break;
        case 18: _t->ignoredChanged(); break;
        case 19: _t->pressAction(); break;
        case 20: _t->toggleAction(); break;
        case 21: _t->increaseAction(); break;
        case 22: _t->decreaseAction(); break;
        case 23: _t->scrollUpAction(); break;
        case 24: _t->scrollDownAction(); break;
        case 25: _t->scrollLeftAction(); break;
        case 26: _t->scrollRightAction(); break;
        case 27: _t->previousPageAction(); break;
        case 28: _t->nextPageAction(); break;
        case 29: _t->valueChanged(); break;
        case 30: _t->cursorPositionChanged(); break;
        case 31: _t->setIgnored((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickAccessibleAttached::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::checkableChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::checkedChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::editableChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::focusableChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::focusedChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::multiLineChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::readOnlyChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::selectedChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::selectableChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::pressedChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::checkStateMixedChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::defaultButtonChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::passwordEditChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::selectableTextChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::searchEditChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::roleChanged)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::nameChanged)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::descriptionChanged)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::ignoredChanged)) {
                *result = 18;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::pressAction)) {
                *result = 19;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::toggleAction)) {
                *result = 20;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::increaseAction)) {
                *result = 21;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::decreaseAction)) {
                *result = 22;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::scrollUpAction)) {
                *result = 23;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::scrollDownAction)) {
                *result = 24;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::scrollLeftAction)) {
                *result = 25;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::scrollRightAction)) {
                *result = 26;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::previousPageAction)) {
                *result = 27;
                return;
            }
        }
        {
            using _t = void (QQuickAccessibleAttached::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickAccessibleAttached::nextPageAction)) {
                *result = 28;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickAccessibleAttached *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QAccessible::Role*>(_v) = _t->role(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->name(); break;
        case 2: *reinterpret_cast< QString*>(_v) = _t->description(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->ignored(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->checkable(); break;
        case 5: *reinterpret_cast< bool*>(_v) = _t->checked(); break;
        case 6: *reinterpret_cast< bool*>(_v) = _t->editable(); break;
        case 7: *reinterpret_cast< bool*>(_v) = _t->focusable(); break;
        case 8: *reinterpret_cast< bool*>(_v) = _t->focused(); break;
        case 9: *reinterpret_cast< bool*>(_v) = _t->multiLine(); break;
        case 10: *reinterpret_cast< bool*>(_v) = _t->readOnly(); break;
        case 11: *reinterpret_cast< bool*>(_v) = _t->selected(); break;
        case 12: *reinterpret_cast< bool*>(_v) = _t->selectable(); break;
        case 13: *reinterpret_cast< bool*>(_v) = _t->pressed(); break;
        case 14: *reinterpret_cast< bool*>(_v) = _t->checkStateMixed(); break;
        case 15: *reinterpret_cast< bool*>(_v) = _t->defaultButton(); break;
        case 16: *reinterpret_cast< bool*>(_v) = _t->passwordEdit(); break;
        case 17: *reinterpret_cast< bool*>(_v) = _t->selectableText(); break;
        case 18: *reinterpret_cast< bool*>(_v) = _t->searchEdit(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickAccessibleAttached *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setRole(*reinterpret_cast< QAccessible::Role*>(_v)); break;
        case 1: _t->setName(*reinterpret_cast< QString*>(_v)); break;
        case 2: _t->setDescription(*reinterpret_cast< QString*>(_v)); break;
        case 3: _t->setIgnored(*reinterpret_cast< bool*>(_v)); break;
        case 4: _t->set_checkable(*reinterpret_cast< bool*>(_v)); break;
        case 5: _t->set_checked(*reinterpret_cast< bool*>(_v)); break;
        case 6: _t->set_editable(*reinterpret_cast< bool*>(_v)); break;
        case 7: _t->set_focusable(*reinterpret_cast< bool*>(_v)); break;
        case 8: _t->set_focused(*reinterpret_cast< bool*>(_v)); break;
        case 9: _t->set_multiLine(*reinterpret_cast< bool*>(_v)); break;
        case 10: _t->set_readOnly(*reinterpret_cast< bool*>(_v)); break;
        case 11: _t->set_selected(*reinterpret_cast< bool*>(_v)); break;
        case 12: _t->set_selectable(*reinterpret_cast< bool*>(_v)); break;
        case 13: _t->set_pressed(*reinterpret_cast< bool*>(_v)); break;
        case 14: _t->set_checkStateMixed(*reinterpret_cast< bool*>(_v)); break;
        case 15: _t->set_defaultButton(*reinterpret_cast< bool*>(_v)); break;
        case 16: _t->set_passwordEdit(*reinterpret_cast< bool*>(_v)); break;
        case 17: _t->set_selectableText(*reinterpret_cast< bool*>(_v)); break;
        case 18: _t->set_searchEdit(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

static const QMetaObject::SuperData qt_meta_extradata_QQuickAccessibleAttached[] = {
    QMetaObject::SuperData::link<QAccessible::staticMetaObject>(),
    nullptr
};

QT_INIT_METAOBJECT const QMetaObject QQuickAccessibleAttached::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickAccessibleAttached.data,
    qt_meta_data_QQuickAccessibleAttached,
    qt_static_metacall,
    qt_meta_extradata_QQuickAccessibleAttached,
    nullptr
} };


const QMetaObject *QQuickAccessibleAttached::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickAccessibleAttached::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickAccessibleAttached.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickAccessibleAttached::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 32)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 32;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 32)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 32;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 19;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 19;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 19;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 19;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 19;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickAccessibleAttached::checkableChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QQuickAccessibleAttached::checkedChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QQuickAccessibleAttached::editableChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QQuickAccessibleAttached::focusableChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QQuickAccessibleAttached::focusedChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QQuickAccessibleAttached::multiLineChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QQuickAccessibleAttached::readOnlyChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QQuickAccessibleAttached::selectedChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QQuickAccessibleAttached::selectableChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QQuickAccessibleAttached::pressedChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void QQuickAccessibleAttached::checkStateMixedChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void QQuickAccessibleAttached::defaultButtonChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void QQuickAccessibleAttached::passwordEditChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void QQuickAccessibleAttached::selectableTextChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void QQuickAccessibleAttached::searchEditChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void QQuickAccessibleAttached::roleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void QQuickAccessibleAttached::nameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void QQuickAccessibleAttached::descriptionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void QQuickAccessibleAttached::ignoredChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void QQuickAccessibleAttached::pressAction()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void QQuickAccessibleAttached::toggleAction()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void QQuickAccessibleAttached::increaseAction()
{
    QMetaObject::activate(this, &staticMetaObject, 21, nullptr);
}

// SIGNAL 22
void QQuickAccessibleAttached::decreaseAction()
{
    QMetaObject::activate(this, &staticMetaObject, 22, nullptr);
}

// SIGNAL 23
void QQuickAccessibleAttached::scrollUpAction()
{
    QMetaObject::activate(this, &staticMetaObject, 23, nullptr);
}

// SIGNAL 24
void QQuickAccessibleAttached::scrollDownAction()
{
    QMetaObject::activate(this, &staticMetaObject, 24, nullptr);
}

// SIGNAL 25
void QQuickAccessibleAttached::scrollLeftAction()
{
    QMetaObject::activate(this, &staticMetaObject, 25, nullptr);
}

// SIGNAL 26
void QQuickAccessibleAttached::scrollRightAction()
{
    QMetaObject::activate(this, &staticMetaObject, 26, nullptr);
}

// SIGNAL 27
void QQuickAccessibleAttached::previousPageAction()
{
    QMetaObject::activate(this, &staticMetaObject, 27, nullptr);
}

// SIGNAL 28
void QQuickAccessibleAttached::nextPageAction()
{
    QMetaObject::activate(this, &staticMetaObject, 28, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
