/****************************************************************************
** Meta object code from reading C++ file 'qquickpainteditem.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/quick/items/qquickpainteditem.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickpainteditem.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickPaintedItem_t {
    QByteArrayData data[26];
    char stringdata0[432];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickPaintedItem_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickPaintedItem_t qt_meta_stringdata_QQuickPaintedItem = {
    {
QT_MOC_LITERAL(0, 0, 17), // "QQuickPaintedItem"
QT_MOC_LITERAL(1, 18, 11), // "QML.Element"
QT_MOC_LITERAL(2, 30, 11), // "PaintedItem"
QT_MOC_LITERAL(3, 42, 13), // "QML.Creatable"
QT_MOC_LITERAL(4, 56, 5), // "false"
QT_MOC_LITERAL(5, 62, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(6, 84, 53), // "Cannot create instance of abs..."
QT_MOC_LITERAL(7, 138, 16), // "fillColorChanged"
QT_MOC_LITERAL(8, 155, 0), // ""
QT_MOC_LITERAL(9, 156, 19), // "contentsSizeChanged"
QT_MOC_LITERAL(10, 176, 20), // "contentsScaleChanged"
QT_MOC_LITERAL(11, 197, 19), // "renderTargetChanged"
QT_MOC_LITERAL(12, 217, 18), // "textureSizeChanged"
QT_MOC_LITERAL(13, 236, 20), // "invalidateSceneGraph"
QT_MOC_LITERAL(14, 257, 12), // "contentsSize"
QT_MOC_LITERAL(15, 270, 9), // "fillColor"
QT_MOC_LITERAL(16, 280, 13), // "contentsScale"
QT_MOC_LITERAL(17, 294, 12), // "renderTarget"
QT_MOC_LITERAL(18, 307, 12), // "RenderTarget"
QT_MOC_LITERAL(19, 320, 11), // "textureSize"
QT_MOC_LITERAL(20, 332, 5), // "Image"
QT_MOC_LITERAL(21, 338, 17), // "FramebufferObject"
QT_MOC_LITERAL(22, 356, 26), // "InvertedYFramebufferObject"
QT_MOC_LITERAL(23, 383, 16), // "PerformanceHints"
QT_MOC_LITERAL(24, 400, 15), // "PerformanceHint"
QT_MOC_LITERAL(25, 416, 15) // "FastFBOResizing"

    },
    "QQuickPaintedItem\0QML.Element\0PaintedItem\0"
    "QML.Creatable\0false\0QML.UncreatableReason\0"
    "Cannot create instance of abstract class PaintedItem.\0"
    "fillColorChanged\0\0contentsSizeChanged\0"
    "contentsScaleChanged\0renderTargetChanged\0"
    "textureSizeChanged\0invalidateSceneGraph\0"
    "contentsSize\0fillColor\0contentsScale\0"
    "renderTarget\0RenderTarget\0textureSize\0"
    "Image\0FramebufferObject\0"
    "InvertedYFramebufferObject\0PerformanceHints\0"
    "PerformanceHint\0FastFBOResizing"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickPaintedItem[] = {

 // content:
       8,       // revision
       0,       // classname
       3,   14, // classinfo
       6,   20, // methods
       5,   56, // properties
       2,   76, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,

 // signals: name, argc, parameters, tag, flags
       7,    0,   50,    8, 0x06 /* Public */,
       9,    0,   51,    8, 0x06 /* Public */,
      10,    0,   52,    8, 0x06 /* Public */,
      11,    0,   53,    8, 0x06 /* Public */,
      12,    0,   54,    8, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      13,    0,   55,    8, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,

 // properties: name, type, flags
      14, QMetaType::QSize, 0x00495103,
      15, QMetaType::QColor, 0x00495103,
      16, QMetaType::QReal, 0x00495103,
      17, 0x80000000 | 18, 0x0049510b,
      19, QMetaType::QSize, 0x00495103,

 // properties: notify_signal_id
       1,
       0,
       2,
       3,
       4,

 // enums: name, alias, flags, count, data
      18,   18, 0x0,    3,   86,
      23,   24, 0x1,    1,   92,

 // enum data: key, value
      20, uint(QQuickPaintedItem::Image),
      21, uint(QQuickPaintedItem::FramebufferObject),
      22, uint(QQuickPaintedItem::InvertedYFramebufferObject),
      25, uint(QQuickPaintedItem::FastFBOResizing),

       0        // eod
};

void QQuickPaintedItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickPaintedItem *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->fillColorChanged(); break;
        case 1: _t->contentsSizeChanged(); break;
        case 2: _t->contentsScaleChanged(); break;
        case 3: _t->renderTargetChanged(); break;
        case 4: _t->textureSizeChanged(); break;
        case 5: _t->invalidateSceneGraph(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickPaintedItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPaintedItem::fillColorChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickPaintedItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPaintedItem::contentsSizeChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickPaintedItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPaintedItem::contentsScaleChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickPaintedItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPaintedItem::renderTargetChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickPaintedItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickPaintedItem::textureSizeChanged)) {
                *result = 4;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickPaintedItem *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QSize*>(_v) = _t->contentsSize(); break;
        case 1: *reinterpret_cast< QColor*>(_v) = _t->fillColor(); break;
        case 2: *reinterpret_cast< qreal*>(_v) = _t->contentsScale(); break;
        case 3: *reinterpret_cast< RenderTarget*>(_v) = _t->renderTarget(); break;
        case 4: *reinterpret_cast< QSize*>(_v) = _t->textureSize(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickPaintedItem *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setContentsSize(*reinterpret_cast< QSize*>(_v)); break;
        case 1: _t->setFillColor(*reinterpret_cast< QColor*>(_v)); break;
        case 2: _t->setContentsScale(*reinterpret_cast< qreal*>(_v)); break;
        case 3: _t->setRenderTarget(*reinterpret_cast< RenderTarget*>(_v)); break;
        case 4: _t->setTextureSize(*reinterpret_cast< QSize*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickPaintedItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_meta_stringdata_QQuickPaintedItem.data,
    qt_meta_data_QQuickPaintedItem,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickPaintedItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickPaintedItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickPaintedItem.stringdata0))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int QQuickPaintedItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickPaintedItem::fillColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickPaintedItem::contentsSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickPaintedItem::contentsScaleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickPaintedItem::renderTargetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickPaintedItem::textureSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
