/****************************************************************************
** Meta object code from reading C++ file 'qquickshadereffectsource_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/quick/items/qquickshadereffectsource_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickshadereffectsource_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickShaderEffectSource_t {
    QByteArrayData data[45];
    char stringdata0[599];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickShaderEffectSource_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickShaderEffectSource_t qt_meta_stringdata_QQuickShaderEffectSource = {
    {
QT_MOC_LITERAL(0, 0, 24), // "QQuickShaderEffectSource"
QT_MOC_LITERAL(1, 25, 11), // "QML.Element"
QT_MOC_LITERAL(2, 37, 18), // "ShaderEffectSource"
QT_MOC_LITERAL(3, 56, 15), // "wrapModeChanged"
QT_MOC_LITERAL(4, 72, 0), // ""
QT_MOC_LITERAL(5, 73, 17), // "sourceItemChanged"
QT_MOC_LITERAL(6, 91, 17), // "sourceRectChanged"
QT_MOC_LITERAL(7, 109, 18), // "textureSizeChanged"
QT_MOC_LITERAL(8, 128, 13), // "formatChanged"
QT_MOC_LITERAL(9, 142, 11), // "liveChanged"
QT_MOC_LITERAL(10, 154, 17), // "hideSourceChanged"
QT_MOC_LITERAL(11, 172, 13), // "mipmapChanged"
QT_MOC_LITERAL(12, 186, 16), // "recursiveChanged"
QT_MOC_LITERAL(13, 203, 23), // "textureMirroringChanged"
QT_MOC_LITERAL(14, 227, 14), // "samplesChanged"
QT_MOC_LITERAL(15, 242, 24), // "scheduledUpdateCompleted"
QT_MOC_LITERAL(16, 267, 19), // "sourceItemDestroyed"
QT_MOC_LITERAL(17, 287, 4), // "item"
QT_MOC_LITERAL(18, 292, 20), // "invalidateSceneGraph"
QT_MOC_LITERAL(19, 313, 14), // "scheduleUpdate"
QT_MOC_LITERAL(20, 328, 8), // "wrapMode"
QT_MOC_LITERAL(21, 337, 8), // "WrapMode"
QT_MOC_LITERAL(22, 346, 10), // "sourceItem"
QT_MOC_LITERAL(23, 357, 11), // "QQuickItem*"
QT_MOC_LITERAL(24, 369, 10), // "sourceRect"
QT_MOC_LITERAL(25, 380, 11), // "textureSize"
QT_MOC_LITERAL(26, 392, 6), // "format"
QT_MOC_LITERAL(27, 399, 6), // "Format"
QT_MOC_LITERAL(28, 406, 4), // "live"
QT_MOC_LITERAL(29, 411, 10), // "hideSource"
QT_MOC_LITERAL(30, 422, 6), // "mipmap"
QT_MOC_LITERAL(31, 429, 9), // "recursive"
QT_MOC_LITERAL(32, 439, 16), // "textureMirroring"
QT_MOC_LITERAL(33, 456, 16), // "TextureMirroring"
QT_MOC_LITERAL(34, 473, 7), // "samples"
QT_MOC_LITERAL(35, 481, 11), // "ClampToEdge"
QT_MOC_LITERAL(36, 493, 18), // "RepeatHorizontally"
QT_MOC_LITERAL(37, 512, 16), // "RepeatVertically"
QT_MOC_LITERAL(38, 529, 6), // "Repeat"
QT_MOC_LITERAL(39, 536, 5), // "Alpha"
QT_MOC_LITERAL(40, 542, 3), // "RGB"
QT_MOC_LITERAL(41, 546, 4), // "RGBA"
QT_MOC_LITERAL(42, 551, 11), // "NoMirroring"
QT_MOC_LITERAL(43, 563, 18), // "MirrorHorizontally"
QT_MOC_LITERAL(44, 582, 16) // "MirrorVertically"

    },
    "QQuickShaderEffectSource\0QML.Element\0"
    "ShaderEffectSource\0wrapModeChanged\0\0"
    "sourceItemChanged\0sourceRectChanged\0"
    "textureSizeChanged\0formatChanged\0"
    "liveChanged\0hideSourceChanged\0"
    "mipmapChanged\0recursiveChanged\0"
    "textureMirroringChanged\0samplesChanged\0"
    "scheduledUpdateCompleted\0sourceItemDestroyed\0"
    "item\0invalidateSceneGraph\0scheduleUpdate\0"
    "wrapMode\0WrapMode\0sourceItem\0QQuickItem*\0"
    "sourceRect\0textureSize\0format\0Format\0"
    "live\0hideSource\0mipmap\0recursive\0"
    "textureMirroring\0TextureMirroring\0"
    "samples\0ClampToEdge\0RepeatHorizontally\0"
    "RepeatVertically\0Repeat\0Alpha\0RGB\0"
    "RGBA\0NoMirroring\0MirrorHorizontally\0"
    "MirrorVertically"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickShaderEffectSource[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      15,   16, // methods
      11,  108, // properties
       3,  163, // enums/sets
       0,    0, // constructors
       0,       // flags
      12,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   91,    4, 0x06 /* Public */,
       5,    0,   92,    4, 0x06 /* Public */,
       6,    0,   93,    4, 0x06 /* Public */,
       7,    0,   94,    4, 0x06 /* Public */,
       8,    0,   95,    4, 0x06 /* Public */,
       9,    0,   96,    4, 0x06 /* Public */,
      10,    0,   97,    4, 0x06 /* Public */,
      11,    0,   98,    4, 0x06 /* Public */,
      12,    0,   99,    4, 0x06 /* Public */,
      13,    0,  100,    4, 0x06 /* Public */,
      14,    0,  101,    4, 0x06 /* Public */,
      15,    0,  102,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      16,    1,  103,    4, 0x08 /* Private */,
      18,    0,  106,    4, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
      19,    0,  107,    4, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::QObjectStar,   17,
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void,

 // properties: name, type, flags
      20, 0x80000000 | 21, 0x0049510b,
      22, 0x80000000 | 23, 0x0049510b,
      24, QMetaType::QRectF, 0x00495103,
      25, QMetaType::QSize, 0x00495103,
      26, 0x80000000 | 27, 0x0049510b,
      28, QMetaType::Bool, 0x00495103,
      29, QMetaType::Bool, 0x00495103,
      30, QMetaType::Bool, 0x00495103,
      31, QMetaType::Bool, 0x00495103,
      32, 0x80000000 | 33, 0x00c9510b,
      34, QMetaType::Int, 0x00c95103,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,
       5,
       6,
       7,
       8,
       9,
      10,

 // properties: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       6,
       9,

 // enums: name, alias, flags, count, data
      21,   21, 0x0,    4,  178,
      27,   27, 0x0,    3,  186,
      33,   33, 0x0,    3,  192,

 // enum data: key, value
      35, uint(QQuickShaderEffectSource::ClampToEdge),
      36, uint(QQuickShaderEffectSource::RepeatHorizontally),
      37, uint(QQuickShaderEffectSource::RepeatVertically),
      38, uint(QQuickShaderEffectSource::Repeat),
      39, uint(QQuickShaderEffectSource::Alpha),
      40, uint(QQuickShaderEffectSource::RGB),
      41, uint(QQuickShaderEffectSource::RGBA),
      42, uint(QQuickShaderEffectSource::NoMirroring),
      43, uint(QQuickShaderEffectSource::MirrorHorizontally),
      44, uint(QQuickShaderEffectSource::MirrorVertically),

       0        // eod
};

void QQuickShaderEffectSource::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickShaderEffectSource *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->wrapModeChanged(); break;
        case 1: _t->sourceItemChanged(); break;
        case 2: _t->sourceRectChanged(); break;
        case 3: _t->textureSizeChanged(); break;
        case 4: _t->formatChanged(); break;
        case 5: _t->liveChanged(); break;
        case 6: _t->hideSourceChanged(); break;
        case 7: _t->mipmapChanged(); break;
        case 8: _t->recursiveChanged(); break;
        case 9: _t->textureMirroringChanged(); break;
        case 10: _t->samplesChanged(); break;
        case 11: _t->scheduledUpdateCompleted(); break;
        case 12: _t->sourceItemDestroyed((*reinterpret_cast< QObject*(*)>(_a[1]))); break;
        case 13: _t->invalidateSceneGraph(); break;
        case 14: _t->scheduleUpdate(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickShaderEffectSource::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickShaderEffectSource::wrapModeChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickShaderEffectSource::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickShaderEffectSource::sourceItemChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickShaderEffectSource::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickShaderEffectSource::sourceRectChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickShaderEffectSource::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickShaderEffectSource::textureSizeChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickShaderEffectSource::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickShaderEffectSource::formatChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickShaderEffectSource::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickShaderEffectSource::liveChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickShaderEffectSource::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickShaderEffectSource::hideSourceChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickShaderEffectSource::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickShaderEffectSource::mipmapChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickShaderEffectSource::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickShaderEffectSource::recursiveChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickShaderEffectSource::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickShaderEffectSource::textureMirroringChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QQuickShaderEffectSource::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickShaderEffectSource::samplesChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QQuickShaderEffectSource::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickShaderEffectSource::scheduledUpdateCompleted)) {
                *result = 11;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickShaderEffectSource *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< WrapMode*>(_v) = _t->wrapMode(); break;
        case 1: *reinterpret_cast< QQuickItem**>(_v) = _t->sourceItem(); break;
        case 2: *reinterpret_cast< QRectF*>(_v) = _t->sourceRect(); break;
        case 3: *reinterpret_cast< QSize*>(_v) = _t->textureSize(); break;
        case 4: *reinterpret_cast< Format*>(_v) = _t->format(); break;
        case 5: *reinterpret_cast< bool*>(_v) = _t->live(); break;
        case 6: *reinterpret_cast< bool*>(_v) = _t->hideSource(); break;
        case 7: *reinterpret_cast< bool*>(_v) = _t->mipmap(); break;
        case 8: *reinterpret_cast< bool*>(_v) = _t->recursive(); break;
        case 9: *reinterpret_cast< TextureMirroring*>(_v) = _t->textureMirroring(); break;
        case 10: *reinterpret_cast< int*>(_v) = _t->samples(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickShaderEffectSource *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setWrapMode(*reinterpret_cast< WrapMode*>(_v)); break;
        case 1: _t->setSourceItem(*reinterpret_cast< QQuickItem**>(_v)); break;
        case 2: _t->setSourceRect(*reinterpret_cast< QRectF*>(_v)); break;
        case 3: _t->setTextureSize(*reinterpret_cast< QSize*>(_v)); break;
        case 4: _t->setFormat(*reinterpret_cast< Format*>(_v)); break;
        case 5: _t->setLive(*reinterpret_cast< bool*>(_v)); break;
        case 6: _t->setHideSource(*reinterpret_cast< bool*>(_v)); break;
        case 7: _t->setMipmap(*reinterpret_cast< bool*>(_v)); break;
        case 8: _t->setRecursive(*reinterpret_cast< bool*>(_v)); break;
        case 9: _t->setTextureMirroring(*reinterpret_cast< TextureMirroring*>(_v)); break;
        case 10: _t->setSamples(*reinterpret_cast< int*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickShaderEffectSource::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_meta_stringdata_QQuickShaderEffectSource.data,
    qt_meta_data_QQuickShaderEffectSource,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickShaderEffectSource::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickShaderEffectSource::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickShaderEffectSource.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQuickItemChangeListener"))
        return static_cast< QQuickItemChangeListener*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int QQuickShaderEffectSource::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 15;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 11;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickShaderEffectSource::wrapModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickShaderEffectSource::sourceItemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickShaderEffectSource::sourceRectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickShaderEffectSource::textureSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickShaderEffectSource::formatChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickShaderEffectSource::liveChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickShaderEffectSource::hideSourceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQuickShaderEffectSource::mipmapChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QQuickShaderEffectSource::recursiveChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QQuickShaderEffectSource::textureMirroringChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QQuickShaderEffectSource::samplesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QQuickShaderEffectSource::scheduledUpdateCompleted()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
