/****************************************************************************
** Meta object code from reading C++ file 'qquickevents_p_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/quick/items/qquickevents_p_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickevents_p_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickKeyEvent_t {
    QByteArrayData data[13];
    char stringdata0[134];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickKeyEvent_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickKeyEvent_t qt_meta_stringdata_QQuickKeyEvent = {
    {
QT_MOC_LITERAL(0, 0, 14), // "QQuickKeyEvent"
QT_MOC_LITERAL(1, 15, 11), // "QML.Element"
QT_MOC_LITERAL(2, 27, 9), // "anonymous"
QT_MOC_LITERAL(3, 37, 7), // "matches"
QT_MOC_LITERAL(4, 45, 0), // ""
QT_MOC_LITERAL(5, 46, 25), // "QKeySequence::StandardKey"
QT_MOC_LITERAL(6, 72, 3), // "key"
QT_MOC_LITERAL(7, 76, 4), // "text"
QT_MOC_LITERAL(8, 81, 9), // "modifiers"
QT_MOC_LITERAL(9, 91, 12), // "isAutoRepeat"
QT_MOC_LITERAL(10, 104, 5), // "count"
QT_MOC_LITERAL(11, 110, 14), // "nativeScanCode"
QT_MOC_LITERAL(12, 125, 8) // "accepted"

    },
    "QQuickKeyEvent\0QML.Element\0anonymous\0"
    "matches\0\0QKeySequence::StandardKey\0"
    "key\0text\0modifiers\0isAutoRepeat\0count\0"
    "nativeScanCode\0accepted"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickKeyEvent[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       1,   16, // methods
       7,   25, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,

 // methods: name, argc, parameters, tag, flags
       3,    1,   22,    4, 0x82 /* Public | MethodRevisioned */,

 // methods: revision
       2,

 // methods: parameters
    QMetaType::Bool, 0x80000000 | 5,    6,

 // properties: name, type, flags
       6, QMetaType::Int, 0x00095401,
       7, QMetaType::QString, 0x00095401,
       8, QMetaType::Int, 0x00095401,
       9, QMetaType::Bool, 0x00095401,
      10, QMetaType::Int, 0x00095401,
      11, QMetaType::UInt, 0x00095401,
      12, QMetaType::Bool, 0x00095103,

       0        // eod
};

void QQuickKeyEvent::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickKeyEvent *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { bool _r = _t->matches((*reinterpret_cast< QKeySequence::StandardKey(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickKeyEvent *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->key(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->text(); break;
        case 2: *reinterpret_cast< int*>(_v) = _t->modifiers(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->isAutoRepeat(); break;
        case 4: *reinterpret_cast< int*>(_v) = _t->count(); break;
        case 5: *reinterpret_cast< quint32*>(_v) = _t->nativeScanCode(); break;
        case 6: *reinterpret_cast< bool*>(_v) = _t->isAccepted(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickKeyEvent *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 6: _t->setAccepted(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickKeyEvent::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickKeyEvent.data,
    qt_meta_data_QQuickKeyEvent,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickKeyEvent::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickKeyEvent::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickKeyEvent.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickKeyEvent::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 7;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
struct qt_meta_stringdata_QQuickMouseEvent_t {
    QByteArrayData data[13];
    char stringdata0[106];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickMouseEvent_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickMouseEvent_t qt_meta_stringdata_QQuickMouseEvent = {
    {
QT_MOC_LITERAL(0, 0, 16), // "QQuickMouseEvent"
QT_MOC_LITERAL(1, 17, 11), // "QML.Element"
QT_MOC_LITERAL(2, 29, 9), // "anonymous"
QT_MOC_LITERAL(3, 39, 1), // "x"
QT_MOC_LITERAL(4, 41, 1), // "y"
QT_MOC_LITERAL(5, 43, 6), // "button"
QT_MOC_LITERAL(6, 50, 7), // "buttons"
QT_MOC_LITERAL(7, 58, 9), // "modifiers"
QT_MOC_LITERAL(8, 68, 6), // "source"
QT_MOC_LITERAL(9, 75, 7), // "wasHeld"
QT_MOC_LITERAL(10, 83, 7), // "isClick"
QT_MOC_LITERAL(11, 91, 8), // "accepted"
QT_MOC_LITERAL(12, 100, 5) // "flags"

    },
    "QQuickMouseEvent\0QML.Element\0anonymous\0"
    "x\0y\0button\0buttons\0modifiers\0source\0"
    "wasHeld\0isClick\0accepted\0flags"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickMouseEvent[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       0,    0, // methods
      10,   16, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,

 // properties: name, type, flags
       3, QMetaType::QReal, 0x00095401,
       4, QMetaType::QReal, 0x00095401,
       5, QMetaType::Int, 0x00095401,
       6, QMetaType::Int, 0x00095401,
       7, QMetaType::Int, 0x00095401,
       8, QMetaType::Int, 0x00895401,
       9, QMetaType::Bool, 0x00095401,
      10, QMetaType::Bool, 0x00095401,
      11, QMetaType::Bool, 0x00095103,
      12, QMetaType::Int, 0x00895401,

 // properties: revision
       0,
       0,
       0,
       0,
       0,
       7,
       0,
       0,
       0,
      11,

       0        // eod
};

void QQuickMouseEvent::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{

#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickMouseEvent *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< qreal*>(_v) = _t->x(); break;
        case 1: *reinterpret_cast< qreal*>(_v) = _t->y(); break;
        case 2: *reinterpret_cast< int*>(_v) = _t->button(); break;
        case 3: *reinterpret_cast< int*>(_v) = _t->buttons(); break;
        case 4: *reinterpret_cast< int*>(_v) = _t->modifiers(); break;
        case 5: *reinterpret_cast< int*>(_v) = _t->source(); break;
        case 6: *reinterpret_cast< bool*>(_v) = _t->wasHeld(); break;
        case 7: *reinterpret_cast< bool*>(_v) = _t->isClick(); break;
        case 8: *reinterpret_cast< bool*>(_v) = _t->isAccepted(); break;
        case 9: *reinterpret_cast< int*>(_v) = _t->flags(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickMouseEvent *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 8: _t->setAccepted(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickMouseEvent::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickMouseEvent.data,
    qt_meta_data_QQuickMouseEvent,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickMouseEvent::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickMouseEvent::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickMouseEvent.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickMouseEvent::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    
#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 10;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
struct qt_meta_stringdata_QQuickWheelEvent_t {
    QByteArrayData data[11];
    char stringdata0[101];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickWheelEvent_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickWheelEvent_t qt_meta_stringdata_QQuickWheelEvent = {
    {
QT_MOC_LITERAL(0, 0, 16), // "QQuickWheelEvent"
QT_MOC_LITERAL(1, 17, 11), // "QML.Element"
QT_MOC_LITERAL(2, 29, 9), // "anonymous"
QT_MOC_LITERAL(3, 39, 1), // "x"
QT_MOC_LITERAL(4, 41, 1), // "y"
QT_MOC_LITERAL(5, 43, 10), // "angleDelta"
QT_MOC_LITERAL(6, 54, 10), // "pixelDelta"
QT_MOC_LITERAL(7, 65, 7), // "buttons"
QT_MOC_LITERAL(8, 73, 9), // "modifiers"
QT_MOC_LITERAL(9, 83, 8), // "inverted"
QT_MOC_LITERAL(10, 92, 8) // "accepted"

    },
    "QQuickWheelEvent\0QML.Element\0anonymous\0"
    "x\0y\0angleDelta\0pixelDelta\0buttons\0"
    "modifiers\0inverted\0accepted"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickWheelEvent[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       0,    0, // methods
       8,   16, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,

 // properties: name, type, flags
       3, QMetaType::QReal, 0x00095401,
       4, QMetaType::QReal, 0x00095401,
       5, QMetaType::QPoint, 0x00095401,
       6, QMetaType::QPoint, 0x00095401,
       7, QMetaType::Int, 0x00095401,
       8, QMetaType::Int, 0x00095401,
       9, QMetaType::Bool, 0x00095401,
      10, QMetaType::Bool, 0x00095103,

       0        // eod
};

void QQuickWheelEvent::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{

#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickWheelEvent *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< qreal*>(_v) = _t->x(); break;
        case 1: *reinterpret_cast< qreal*>(_v) = _t->y(); break;
        case 2: *reinterpret_cast< QPoint*>(_v) = _t->angleDelta(); break;
        case 3: *reinterpret_cast< QPoint*>(_v) = _t->pixelDelta(); break;
        case 4: *reinterpret_cast< int*>(_v) = _t->buttons(); break;
        case 5: *reinterpret_cast< int*>(_v) = _t->modifiers(); break;
        case 6: *reinterpret_cast< bool*>(_v) = _t->inverted(); break;
        case 7: *reinterpret_cast< bool*>(_v) = _t->isAccepted(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickWheelEvent *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 7: _t->setAccepted(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickWheelEvent::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickWheelEvent.data,
    qt_meta_data_QQuickWheelEvent,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickWheelEvent::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickWheelEvent::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickWheelEvent.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickWheelEvent::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    
#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 8;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
struct qt_meta_stringdata_QQuickCloseEvent_t {
    QByteArrayData data[4];
    char stringdata0[48];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickCloseEvent_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickCloseEvent_t qt_meta_stringdata_QQuickCloseEvent = {
    {
QT_MOC_LITERAL(0, 0, 16), // "QQuickCloseEvent"
QT_MOC_LITERAL(1, 17, 11), // "QML.Element"
QT_MOC_LITERAL(2, 29, 9), // "anonymous"
QT_MOC_LITERAL(3, 39, 8) // "accepted"

    },
    "QQuickCloseEvent\0QML.Element\0anonymous\0"
    "accepted"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickCloseEvent[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       0,    0, // methods
       1,   16, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,

 // properties: name, type, flags
       3, QMetaType::Bool, 0x00095103,

       0        // eod
};

void QQuickCloseEvent::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{

#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickCloseEvent *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->isAccepted(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickCloseEvent *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setAccepted(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickCloseEvent::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickCloseEvent.data,
    qt_meta_data_QQuickCloseEvent,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickCloseEvent::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickCloseEvent::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickCloseEvent.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickCloseEvent::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    
#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
struct qt_meta_stringdata_QQuickEventPoint_t {
    QByteArrayData data[35];
    char stringdata0[489];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickEventPoint_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickEventPoint_t qt_meta_stringdata_QQuickEventPoint = {
    {
QT_MOC_LITERAL(0, 0, 16), // "QQuickEventPoint"
QT_MOC_LITERAL(1, 17, 11), // "QML.Element"
QT_MOC_LITERAL(2, 29, 10), // "EventPoint"
QT_MOC_LITERAL(3, 40, 13), // "QML.Creatable"
QT_MOC_LITERAL(4, 54, 5), // "false"
QT_MOC_LITERAL(5, 60, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(6, 82, 57), // "EventPoint is only available ..."
QT_MOC_LITERAL(7, 140, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(8, 164, 2), // "12"
QT_MOC_LITERAL(9, 167, 5), // "event"
QT_MOC_LITERAL(10, 173, 19), // "QQuickPointerEvent*"
QT_MOC_LITERAL(11, 193, 8), // "position"
QT_MOC_LITERAL(12, 202, 13), // "scenePosition"
QT_MOC_LITERAL(13, 216, 18), // "scenePressPosition"
QT_MOC_LITERAL(14, 235, 17), // "sceneGrabPosition"
QT_MOC_LITERAL(15, 253, 5), // "state"
QT_MOC_LITERAL(16, 259, 5), // "State"
QT_MOC_LITERAL(17, 265, 7), // "pointId"
QT_MOC_LITERAL(18, 273, 8), // "timeHeld"
QT_MOC_LITERAL(19, 282, 8), // "velocity"
QT_MOC_LITERAL(20, 291, 8), // "accepted"
QT_MOC_LITERAL(21, 300, 16), // "exclusiveGrabber"
QT_MOC_LITERAL(22, 317, 6), // "States"
QT_MOC_LITERAL(23, 324, 7), // "Pressed"
QT_MOC_LITERAL(24, 332, 7), // "Updated"
QT_MOC_LITERAL(25, 340, 10), // "Stationary"
QT_MOC_LITERAL(26, 351, 8), // "Released"
QT_MOC_LITERAL(27, 360, 14), // "GrabTransition"
QT_MOC_LITERAL(28, 375, 11), // "GrabPassive"
QT_MOC_LITERAL(29, 387, 13), // "UngrabPassive"
QT_MOC_LITERAL(30, 401, 17), // "CancelGrabPassive"
QT_MOC_LITERAL(31, 419, 19), // "OverrideGrabPassive"
QT_MOC_LITERAL(32, 439, 13), // "GrabExclusive"
QT_MOC_LITERAL(33, 453, 15), // "UngrabExclusive"
QT_MOC_LITERAL(34, 469, 19) // "CancelGrabExclusive"

    },
    "QQuickEventPoint\0QML.Element\0EventPoint\0"
    "QML.Creatable\0false\0QML.UncreatableReason\0"
    "EventPoint is only available as a member of PointerEvent.\0"
    "QML.AddedInMinorVersion\0""12\0event\0"
    "QQuickPointerEvent*\0position\0scenePosition\0"
    "scenePressPosition\0sceneGrabPosition\0"
    "state\0State\0pointId\0timeHeld\0velocity\0"
    "accepted\0exclusiveGrabber\0States\0"
    "Pressed\0Updated\0Stationary\0Released\0"
    "GrabTransition\0GrabPassive\0UngrabPassive\0"
    "CancelGrabPassive\0OverrideGrabPassive\0"
    "GrabExclusive\0UngrabExclusive\0"
    "CancelGrabExclusive"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickEventPoint[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
       0,    0, // methods
      11,   22, // properties
       2,   55, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,

 // properties: name, type, flags
       9, 0x80000000 | 10, 0x00095409,
      11, QMetaType::QPointF, 0x00095401,
      12, QMetaType::QPointF, 0x00095401,
      13, QMetaType::QPointF, 0x00095401,
      14, QMetaType::QPointF, 0x00095401,
      15, 0x80000000 | 16, 0x00095409,
      17, QMetaType::Int, 0x00095401,
      18, QMetaType::QReal, 0x00095401,
      19, QMetaType::QVector2D, 0x00095401,
      20, QMetaType::Bool, 0x00095103,
      21, QMetaType::QObjectStar, 0x00095103,

 // enums: name, alias, flags, count, data
      22,   16, 0x1,    4,   65,
      27,   27, 0x0,    7,   73,

 // enum data: key, value
      23, uint(QQuickEventPoint::Pressed),
      24, uint(QQuickEventPoint::Updated),
      25, uint(QQuickEventPoint::Stationary),
      26, uint(QQuickEventPoint::Released),
      28, uint(QQuickEventPoint::GrabPassive),
      29, uint(QQuickEventPoint::UngrabPassive),
      30, uint(QQuickEventPoint::CancelGrabPassive),
      31, uint(QQuickEventPoint::OverrideGrabPassive),
      32, uint(QQuickEventPoint::GrabExclusive),
      33, uint(QQuickEventPoint::UngrabExclusive),
      34, uint(QQuickEventPoint::CancelGrabExclusive),

       0        // eod
};

void QQuickEventPoint::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickPointerEvent* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickEventPoint *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQuickPointerEvent**>(_v) = _t->pointerEvent(); break;
        case 1: *reinterpret_cast< QPointF*>(_v) = _t->position(); break;
        case 2: *reinterpret_cast< QPointF*>(_v) = _t->scenePosition(); break;
        case 3: *reinterpret_cast< QPointF*>(_v) = _t->scenePressPosition(); break;
        case 4: *reinterpret_cast< QPointF*>(_v) = _t->sceneGrabPosition(); break;
        case 5: *reinterpret_cast< State*>(_v) = _t->state(); break;
        case 6: *reinterpret_cast< int*>(_v) = _t->pointId(); break;
        case 7: *reinterpret_cast< qreal*>(_v) = _t->timeHeld(); break;
        case 8: *reinterpret_cast< QVector2D*>(_v) = _t->velocity(); break;
        case 9: *reinterpret_cast< bool*>(_v) = _t->isAccepted(); break;
        case 10: *reinterpret_cast< QObject**>(_v) = _t->exclusiveGrabber(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickEventPoint *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 9: _t->setAccepted(*reinterpret_cast< bool*>(_v)); break;
        case 10: _t->setExclusiveGrabber(*reinterpret_cast< QObject**>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
}

QT_INIT_METAOBJECT const QMetaObject QQuickEventPoint::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickEventPoint.data,
    qt_meta_data_QQuickEventPoint,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickEventPoint::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickEventPoint::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickEventPoint.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickEventPoint::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    
#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 11;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 11;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
struct qt_meta_stringdata_QQuickEventTouchPoint_t {
    QByteArrayData data[14];
    char stringdata0[250];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickEventTouchPoint_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickEventTouchPoint_t qt_meta_stringdata_QQuickEventTouchPoint = {
    {
QT_MOC_LITERAL(0, 0, 21), // "QQuickEventTouchPoint"
QT_MOC_LITERAL(1, 22, 11), // "QML.Element"
QT_MOC_LITERAL(2, 34, 15), // "EventTouchPoint"
QT_MOC_LITERAL(3, 50, 13), // "QML.Creatable"
QT_MOC_LITERAL(4, 64, 5), // "false"
QT_MOC_LITERAL(5, 70, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(6, 92, 62), // "EventTouchPoint is only avail..."
QT_MOC_LITERAL(7, 155, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(8, 179, 2), // "12"
QT_MOC_LITERAL(9, 182, 8), // "rotation"
QT_MOC_LITERAL(10, 191, 8), // "pressure"
QT_MOC_LITERAL(11, 200, 16), // "ellipseDiameters"
QT_MOC_LITERAL(12, 217, 8), // "uniqueId"
QT_MOC_LITERAL(13, 226, 23) // "QPointingDeviceUniqueId"

    },
    "QQuickEventTouchPoint\0QML.Element\0"
    "EventTouchPoint\0QML.Creatable\0false\0"
    "QML.UncreatableReason\0"
    "EventTouchPoint is only available as a member of PointerEvent.\0"
    "QML.AddedInMinorVersion\0""12\0rotation\0"
    "pressure\0ellipseDiameters\0uniqueId\0"
    "QPointingDeviceUniqueId"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickEventTouchPoint[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
       0,    0, // methods
       4,   22, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,

 // properties: name, type, flags
       9, QMetaType::QReal, 0x00095001,
      10, QMetaType::QReal, 0x00095001,
      11, QMetaType::QSizeF, 0x00095001,
      12, 0x80000000 | 13, 0x00095009,

       0        // eod
};

void QQuickEventTouchPoint::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{

#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickEventTouchPoint *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< qreal*>(_v) = _t->rotation(); break;
        case 1: *reinterpret_cast< qreal*>(_v) = _t->pressure(); break;
        case 2: *reinterpret_cast< QSizeF*>(_v) = _t->ellipseDiameters(); break;
        case 3: *reinterpret_cast< QPointingDeviceUniqueId*>(_v) = _t->uniqueId(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickEventTouchPoint::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickEventPoint::staticMetaObject>(),
    qt_meta_stringdata_QQuickEventTouchPoint.data,
    qt_meta_data_QQuickEventTouchPoint,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickEventTouchPoint::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickEventTouchPoint::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickEventTouchPoint.stringdata0))
        return static_cast<void*>(this);
    return QQuickEventPoint::qt_metacast(_clname);
}

int QQuickEventTouchPoint::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickEventPoint::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    
#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
struct qt_meta_stringdata_QQuickPointerEvent_t {
    QByteArrayData data[16];
    char stringdata0[288];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickPointerEvent_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickPointerEvent_t qt_meta_stringdata_QQuickPointerEvent = {
    {
QT_MOC_LITERAL(0, 0, 18), // "QQuickPointerEvent"
QT_MOC_LITERAL(1, 19, 11), // "QML.Element"
QT_MOC_LITERAL(2, 31, 12), // "PointerEvent"
QT_MOC_LITERAL(3, 44, 13), // "QML.Creatable"
QT_MOC_LITERAL(4, 58, 5), // "false"
QT_MOC_LITERAL(5, 64, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(6, 86, 82), // "PointerEvent is only availabl..."
QT_MOC_LITERAL(7, 169, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(8, 193, 2), // "12"
QT_MOC_LITERAL(9, 196, 6), // "device"
QT_MOC_LITERAL(10, 203, 20), // "QQuickPointerDevice*"
QT_MOC_LITERAL(11, 224, 9), // "modifiers"
QT_MOC_LITERAL(12, 234, 21), // "Qt::KeyboardModifiers"
QT_MOC_LITERAL(13, 256, 6), // "button"
QT_MOC_LITERAL(14, 263, 16), // "Qt::MouseButtons"
QT_MOC_LITERAL(15, 280, 7) // "buttons"

    },
    "QQuickPointerEvent\0QML.Element\0"
    "PointerEvent\0QML.Creatable\0false\0"
    "QML.UncreatableReason\0"
    "PointerEvent is only available as a parameter of several signals in Po"
    "interHandler\0"
    "QML.AddedInMinorVersion\0""12\0device\0"
    "QQuickPointerDevice*\0modifiers\0"
    "Qt::KeyboardModifiers\0button\0"
    "Qt::MouseButtons\0buttons"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickPointerEvent[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
       0,    0, // methods
       4,   22, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,

 // properties: name, type, flags
       9, 0x80000000 | 10, 0x00095409,
      11, 0x80000000 | 12, 0x00095409,
      13, 0x80000000 | 14, 0x00095409,
      15, 0x80000000 | 14, 0x00095409,

       0        // eod
};

void QQuickPointerEvent::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickPointerDevice* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickPointerEvent *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQuickPointerDevice**>(_v) = _t->device(); break;
        case 1: *reinterpret_cast< Qt::KeyboardModifiers*>(_v) = _t->modifiers(); break;
        case 2: *reinterpret_cast< Qt::MouseButtons*>(_v) = _t->button(); break;
        case 3: *reinterpret_cast< Qt::MouseButtons*>(_v) = _t->buttons(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
}

QT_INIT_METAOBJECT const QMetaObject QQuickPointerEvent::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickPointerEvent.data,
    qt_meta_data_QQuickPointerEvent,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickPointerEvent::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickPointerEvent::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickPointerEvent.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickPointerEvent::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    
#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
struct qt_meta_stringdata_QQuickSinglePointEvent_t {
    QByteArrayData data[1];
    char stringdata0[23];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickSinglePointEvent_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickSinglePointEvent_t qt_meta_stringdata_QQuickSinglePointEvent = {
    {
QT_MOC_LITERAL(0, 0, 22) // "QQuickSinglePointEvent"

    },
    "QQuickSinglePointEvent"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickSinglePointEvent[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QQuickSinglePointEvent::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickSinglePointEvent::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickPointerEvent::staticMetaObject>(),
    qt_meta_stringdata_QQuickSinglePointEvent.data,
    qt_meta_data_QQuickSinglePointEvent,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickSinglePointEvent::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickSinglePointEvent::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickSinglePointEvent.stringdata0))
        return static_cast<void*>(this);
    return QQuickPointerEvent::qt_metacast(_clname);
}

int QQuickSinglePointEvent::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickPointerEvent::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QQuickPointerMouseEvent_t {
    QByteArrayData data[9];
    char stringdata0[211];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickPointerMouseEvent_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickPointerMouseEvent_t qt_meta_stringdata_QQuickPointerMouseEvent = {
    {
QT_MOC_LITERAL(0, 0, 23), // "QQuickPointerMouseEvent"
QT_MOC_LITERAL(1, 24, 11), // "QML.Element"
QT_MOC_LITERAL(2, 36, 17), // "PointerMouseEvent"
QT_MOC_LITERAL(3, 54, 13), // "QML.Creatable"
QT_MOC_LITERAL(4, 68, 5), // "false"
QT_MOC_LITERAL(5, 74, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(6, 96, 87), // "PointerMouseEvent is only ava..."
QT_MOC_LITERAL(7, 184, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(8, 208, 2) // "12"

    },
    "QQuickPointerMouseEvent\0QML.Element\0"
    "PointerMouseEvent\0QML.Creatable\0false\0"
    "QML.UncreatableReason\0"
    "PointerMouseEvent is only available as a parameter of several signals "
    "in PointerHandler\0"
    "QML.AddedInMinorVersion\0""12"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickPointerMouseEvent[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,

       0        // eod
};

void QQuickPointerMouseEvent::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickPointerMouseEvent::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickSinglePointEvent::staticMetaObject>(),
    qt_meta_stringdata_QQuickPointerMouseEvent.data,
    qt_meta_data_QQuickPointerMouseEvent,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickPointerMouseEvent::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickPointerMouseEvent::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickPointerMouseEvent.stringdata0))
        return static_cast<void*>(this);
    return QQuickSinglePointEvent::qt_metacast(_clname);
}

int QQuickPointerMouseEvent::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickSinglePointEvent::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QQuickPointerTouchEvent_t {
    QByteArrayData data[9];
    char stringdata0[211];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickPointerTouchEvent_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickPointerTouchEvent_t qt_meta_stringdata_QQuickPointerTouchEvent = {
    {
QT_MOC_LITERAL(0, 0, 23), // "QQuickPointerTouchEvent"
QT_MOC_LITERAL(1, 24, 11), // "QML.Element"
QT_MOC_LITERAL(2, 36, 17), // "PointerTouchEvent"
QT_MOC_LITERAL(3, 54, 13), // "QML.Creatable"
QT_MOC_LITERAL(4, 68, 5), // "false"
QT_MOC_LITERAL(5, 74, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(6, 96, 87), // "PointerTouchEvent is only ava..."
QT_MOC_LITERAL(7, 184, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(8, 208, 2) // "12"

    },
    "QQuickPointerTouchEvent\0QML.Element\0"
    "PointerTouchEvent\0QML.Creatable\0false\0"
    "QML.UncreatableReason\0"
    "PointerTouchEvent is only available as a parameter of several signals "
    "in PointerHandler\0"
    "QML.AddedInMinorVersion\0""12"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickPointerTouchEvent[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,

       0        // eod
};

void QQuickPointerTouchEvent::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickPointerTouchEvent::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickPointerEvent::staticMetaObject>(),
    qt_meta_stringdata_QQuickPointerTouchEvent.data,
    qt_meta_data_QQuickPointerTouchEvent,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickPointerTouchEvent::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickPointerTouchEvent::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickPointerTouchEvent.stringdata0))
        return static_cast<void*>(this);
    return QQuickPointerEvent::qt_metacast(_clname);
}

int QQuickPointerTouchEvent::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickPointerEvent::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QQuickEventTabletPoint_t {
    QByteArrayData data[13];
    char stringdata0[226];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickEventTabletPoint_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickEventTabletPoint_t qt_meta_stringdata_QQuickEventTabletPoint = {
    {
QT_MOC_LITERAL(0, 0, 22), // "QQuickEventTabletPoint"
QT_MOC_LITERAL(1, 23, 11), // "QML.Element"
QT_MOC_LITERAL(2, 35, 16), // "EventTabletPoint"
QT_MOC_LITERAL(3, 52, 13), // "QML.Creatable"
QT_MOC_LITERAL(4, 66, 5), // "false"
QT_MOC_LITERAL(5, 72, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(6, 94, 62), // "EventTouchPoint is only avail..."
QT_MOC_LITERAL(7, 157, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(8, 181, 2), // "15"
QT_MOC_LITERAL(9, 184, 8), // "rotation"
QT_MOC_LITERAL(10, 193, 8), // "pressure"
QT_MOC_LITERAL(11, 202, 18), // "tangentialPressure"
QT_MOC_LITERAL(12, 221, 4) // "tilt"

    },
    "QQuickEventTabletPoint\0QML.Element\0"
    "EventTabletPoint\0QML.Creatable\0false\0"
    "QML.UncreatableReason\0"
    "EventTouchPoint is only available as a member of PointerEvent.\0"
    "QML.AddedInMinorVersion\0""15\0rotation\0"
    "pressure\0tangentialPressure\0tilt"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickEventTabletPoint[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
       0,    0, // methods
       4,   22, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,

 // properties: name, type, flags
       9, QMetaType::QReal, 0x00095001,
      10, QMetaType::QReal, 0x00095001,
      11, QMetaType::QReal, 0x00095001,
      12, QMetaType::QVector2D, 0x00095001,

       0        // eod
};

void QQuickEventTabletPoint::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{

#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickEventTabletPoint *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< qreal*>(_v) = _t->rotation(); break;
        case 1: *reinterpret_cast< qreal*>(_v) = _t->pressure(); break;
        case 2: *reinterpret_cast< qreal*>(_v) = _t->tangentialPressure(); break;
        case 3: *reinterpret_cast< QVector2D*>(_v) = _t->tilt(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickEventTabletPoint::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickEventPoint::staticMetaObject>(),
    qt_meta_stringdata_QQuickEventTabletPoint.data,
    qt_meta_data_QQuickEventTabletPoint,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickEventTabletPoint::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickEventTabletPoint::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickEventTabletPoint.stringdata0))
        return static_cast<void*>(this);
    return QQuickEventPoint::qt_metacast(_clname);
}

int QQuickEventTabletPoint::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickEventPoint::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    
#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
struct qt_meta_stringdata_QQuickPointerTabletEvent_t {
    QByteArrayData data[1];
    char stringdata0[25];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickPointerTabletEvent_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickPointerTabletEvent_t qt_meta_stringdata_QQuickPointerTabletEvent = {
    {
QT_MOC_LITERAL(0, 0, 24) // "QQuickPointerTabletEvent"

    },
    "QQuickPointerTabletEvent"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickPointerTabletEvent[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QQuickPointerTabletEvent::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickPointerTabletEvent::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickSinglePointEvent::staticMetaObject>(),
    qt_meta_stringdata_QQuickPointerTabletEvent.data,
    qt_meta_data_QQuickPointerTabletEvent,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickPointerTabletEvent::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickPointerTabletEvent::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickPointerTabletEvent.stringdata0))
        return static_cast<void*>(this);
    return QQuickSinglePointEvent::qt_metacast(_clname);
}

int QQuickPointerTabletEvent::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickSinglePointEvent::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_QQuickPointerNativeGestureEvent_t {
    QByteArrayData data[4];
    char stringdata0[65];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickPointerNativeGestureEvent_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickPointerNativeGestureEvent_t qt_meta_stringdata_QQuickPointerNativeGestureEvent = {
    {
QT_MOC_LITERAL(0, 0, 31), // "QQuickPointerNativeGestureEvent"
QT_MOC_LITERAL(1, 32, 4), // "type"
QT_MOC_LITERAL(2, 37, 21), // "Qt::NativeGestureType"
QT_MOC_LITERAL(3, 59, 5) // "value"

    },
    "QQuickPointerNativeGestureEvent\0type\0"
    "Qt::NativeGestureType\0value"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickPointerNativeGestureEvent[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       2,   14, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // properties: name, type, flags
       1, 0x80000000 | 2, 0x00095409,
       3, QMetaType::QReal, 0x00095401,

       0        // eod
};

void QQuickPointerNativeGestureEvent::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{

#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickPointerNativeGestureEvent *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< Qt::NativeGestureType*>(_v) = _t->type(); break;
        case 1: *reinterpret_cast< qreal*>(_v) = _t->value(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickPointerNativeGestureEvent::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickSinglePointEvent::staticMetaObject>(),
    qt_meta_stringdata_QQuickPointerNativeGestureEvent.data,
    qt_meta_data_QQuickPointerNativeGestureEvent,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickPointerNativeGestureEvent::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickPointerNativeGestureEvent::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickPointerNativeGestureEvent.stringdata0))
        return static_cast<void*>(this);
    return QQuickSinglePointEvent::qt_metacast(_clname);
}

int QQuickPointerNativeGestureEvent::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickSinglePointEvent::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    
#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
struct qt_meta_stringdata_QQuickPointerScrollEvent_t {
    QByteArrayData data[14];
    char stringdata0[257];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickPointerScrollEvent_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickPointerScrollEvent_t qt_meta_stringdata_QQuickPointerScrollEvent = {
    {
QT_MOC_LITERAL(0, 0, 24), // "QQuickPointerScrollEvent"
QT_MOC_LITERAL(1, 25, 11), // "QML.Element"
QT_MOC_LITERAL(2, 37, 18), // "PointerScrollEvent"
QT_MOC_LITERAL(3, 56, 13), // "QML.Creatable"
QT_MOC_LITERAL(4, 70, 5), // "false"
QT_MOC_LITERAL(5, 76, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(6, 98, 72), // "PointerScrollEvent is only av..."
QT_MOC_LITERAL(7, 171, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(8, 195, 2), // "14"
QT_MOC_LITERAL(9, 198, 10), // "angleDelta"
QT_MOC_LITERAL(10, 209, 10), // "pixelDelta"
QT_MOC_LITERAL(11, 220, 13), // "hasAngleDelta"
QT_MOC_LITERAL(12, 234, 13), // "hasPixelDelta"
QT_MOC_LITERAL(13, 248, 8) // "inverted"

    },
    "QQuickPointerScrollEvent\0QML.Element\0"
    "PointerScrollEvent\0QML.Creatable\0false\0"
    "QML.UncreatableReason\0"
    "PointerScrollEvent is only available via the WheelHandler::wheel signa"
    "l.\0"
    "QML.AddedInMinorVersion\0""14\0angleDelta\0"
    "pixelDelta\0hasAngleDelta\0hasPixelDelta\0"
    "inverted"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickPointerScrollEvent[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
       0,    0, // methods
       5,   22, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,

 // properties: name, type, flags
       9, QMetaType::QVector2D, 0x00095401,
      10, QMetaType::QVector2D, 0x00095401,
      11, QMetaType::Bool, 0x00095401,
      12, QMetaType::Bool, 0x00095401,
      13, QMetaType::Bool, 0x00095401,

       0        // eod
};

void QQuickPointerScrollEvent::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{

#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickPointerScrollEvent *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QVector2D*>(_v) = _t->angleDelta(); break;
        case 1: *reinterpret_cast< QVector2D*>(_v) = _t->pixelDelta(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->hasAngleDelta(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->hasPixelDelta(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->isInverted(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickPointerScrollEvent::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickSinglePointEvent::staticMetaObject>(),
    qt_meta_stringdata_QQuickPointerScrollEvent.data,
    qt_meta_data_QQuickPointerScrollEvent,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickPointerScrollEvent::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickPointerScrollEvent::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickPointerScrollEvent.stringdata0))
        return static_cast<void*>(this);
    return QQuickSinglePointEvent::qt_metacast(_clname);
}

int QQuickPointerScrollEvent::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickSinglePointEvent::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    
#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
struct qt_meta_stringdata_QQuickPointerDevice_t {
    QByteArrayData data[47];
    char stringdata0[563];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickPointerDevice_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickPointerDevice_t qt_meta_stringdata_QQuickPointerDevice = {
    {
QT_MOC_LITERAL(0, 0, 19), // "QQuickPointerDevice"
QT_MOC_LITERAL(1, 20, 11), // "QML.Element"
QT_MOC_LITERAL(2, 32, 13), // "PointerDevice"
QT_MOC_LITERAL(3, 46, 13), // "QML.Creatable"
QT_MOC_LITERAL(4, 60, 5), // "false"
QT_MOC_LITERAL(5, 66, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(6, 88, 62), // "PointerDevice is only availab..."
QT_MOC_LITERAL(7, 151, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(8, 175, 2), // "12"
QT_MOC_LITERAL(9, 178, 4), // "type"
QT_MOC_LITERAL(10, 183, 10), // "DeviceType"
QT_MOC_LITERAL(11, 194, 11), // "pointerType"
QT_MOC_LITERAL(12, 206, 11), // "PointerType"
QT_MOC_LITERAL(13, 218, 12), // "capabilities"
QT_MOC_LITERAL(14, 231, 12), // "Capabilities"
QT_MOC_LITERAL(15, 244, 18), // "maximumTouchPoints"
QT_MOC_LITERAL(16, 263, 11), // "buttonCount"
QT_MOC_LITERAL(17, 275, 4), // "name"
QT_MOC_LITERAL(18, 280, 8), // "uniqueId"
QT_MOC_LITERAL(19, 289, 23), // "QPointingDeviceUniqueId"
QT_MOC_LITERAL(20, 313, 11), // "DeviceTypes"
QT_MOC_LITERAL(21, 325, 13), // "UnknownDevice"
QT_MOC_LITERAL(22, 339, 5), // "Mouse"
QT_MOC_LITERAL(23, 345, 11), // "TouchScreen"
QT_MOC_LITERAL(24, 357, 8), // "TouchPad"
QT_MOC_LITERAL(25, 366, 4), // "Puck"
QT_MOC_LITERAL(26, 371, 6), // "Stylus"
QT_MOC_LITERAL(27, 378, 8), // "Airbrush"
QT_MOC_LITERAL(28, 387, 10), // "AllDevices"
QT_MOC_LITERAL(29, 398, 12), // "PointerTypes"
QT_MOC_LITERAL(30, 411, 14), // "GenericPointer"
QT_MOC_LITERAL(31, 426, 6), // "Finger"
QT_MOC_LITERAL(32, 433, 3), // "Pen"
QT_MOC_LITERAL(33, 437, 6), // "Eraser"
QT_MOC_LITERAL(34, 444, 6), // "Cursor"
QT_MOC_LITERAL(35, 451, 15), // "AllPointerTypes"
QT_MOC_LITERAL(36, 467, 14), // "CapabilityFlag"
QT_MOC_LITERAL(37, 482, 8), // "Position"
QT_MOC_LITERAL(38, 491, 4), // "Area"
QT_MOC_LITERAL(39, 496, 8), // "Pressure"
QT_MOC_LITERAL(40, 505, 8), // "Velocity"
QT_MOC_LITERAL(41, 514, 14), // "MouseEmulation"
QT_MOC_LITERAL(42, 529, 6), // "Scroll"
QT_MOC_LITERAL(43, 536, 5), // "Hover"
QT_MOC_LITERAL(44, 542, 8), // "Rotation"
QT_MOC_LITERAL(45, 551, 5), // "XTilt"
QT_MOC_LITERAL(46, 557, 5) // "YTilt"

    },
    "QQuickPointerDevice\0QML.Element\0"
    "PointerDevice\0QML.Creatable\0false\0"
    "QML.UncreatableReason\0"
    "PointerDevice is only available as a property of PointerEvent.\0"
    "QML.AddedInMinorVersion\0""12\0type\0"
    "DeviceType\0pointerType\0PointerType\0"
    "capabilities\0Capabilities\0maximumTouchPoints\0"
    "buttonCount\0name\0uniqueId\0"
    "QPointingDeviceUniqueId\0DeviceTypes\0"
    "UnknownDevice\0Mouse\0TouchScreen\0"
    "TouchPad\0Puck\0Stylus\0Airbrush\0AllDevices\0"
    "PointerTypes\0GenericPointer\0Finger\0"
    "Pen\0Eraser\0Cursor\0AllPointerTypes\0"
    "CapabilityFlag\0Position\0Area\0Pressure\0"
    "Velocity\0MouseEmulation\0Scroll\0Hover\0"
    "Rotation\0XTilt\0YTilt"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickPointerDevice[] = {

 // content:
       8,       // revision
       0,       // classname
       4,   14, // classinfo
       0,    0, // methods
       7,   22, // properties
       3,   43, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,

 // properties: name, type, flags
       9, 0x80000000 | 10, 0x00095409,
      11, 0x80000000 | 12, 0x00095409,
      13, 0x80000000 | 14, 0x00095409,
      15, QMetaType::Int, 0x00095401,
      16, QMetaType::Int, 0x00095401,
      17, QMetaType::QString, 0x00095401,
      18, 0x80000000 | 19, 0x00095409,

 // enums: name, alias, flags, count, data
      20,   10, 0x1,    8,   58,
      29,   12, 0x1,    6,   74,
      14,   36, 0x1,   10,   86,

 // enum data: key, value
      21, uint(QQuickPointerDevice::UnknownDevice),
      22, uint(QQuickPointerDevice::Mouse),
      23, uint(QQuickPointerDevice::TouchScreen),
      24, uint(QQuickPointerDevice::TouchPad),
      25, uint(QQuickPointerDevice::Puck),
      26, uint(QQuickPointerDevice::Stylus),
      27, uint(QQuickPointerDevice::Airbrush),
      28, uint(QQuickPointerDevice::AllDevices),
      30, uint(QQuickPointerDevice::GenericPointer),
      31, uint(QQuickPointerDevice::Finger),
      32, uint(QQuickPointerDevice::Pen),
      33, uint(QQuickPointerDevice::Eraser),
      34, uint(QQuickPointerDevice::Cursor),
      35, uint(QQuickPointerDevice::AllPointerTypes),
      37, uint(QQuickPointerDevice::Position),
      38, uint(QQuickPointerDevice::Area),
      39, uint(QQuickPointerDevice::Pressure),
      40, uint(QQuickPointerDevice::Velocity),
      41, uint(QQuickPointerDevice::MouseEmulation),
      42, uint(QQuickPointerDevice::Scroll),
      43, uint(QQuickPointerDevice::Hover),
      44, uint(QQuickPointerDevice::Rotation),
      45, uint(QQuickPointerDevice::XTilt),
      46, uint(QQuickPointerDevice::YTilt),

       0        // eod
};

void QQuickPointerDevice::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{

#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickPointerDevice *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< DeviceType*>(_v) = _t->type(); break;
        case 1: *reinterpret_cast< PointerType*>(_v) = _t->pointerType(); break;
        case 2: *reinterpret_cast<int*>(_v) = QFlag(_t->capabilities()); break;
        case 3: *reinterpret_cast< int*>(_v) = _t->maximumTouchPoints(); break;
        case 4: *reinterpret_cast< int*>(_v) = _t->buttonCount(); break;
        case 5: *reinterpret_cast< QString*>(_v) = _t->name(); break;
        case 6: *reinterpret_cast< QPointingDeviceUniqueId*>(_v) = _t->uniqueId(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickPointerDevice::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickPointerDevice.data,
    qt_meta_data_QQuickPointerDevice,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickPointerDevice::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickPointerDevice::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickPointerDevice.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickPointerDevice::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    
#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 7;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
