/****************************************************************************
** Meta object code from reading C++ file 'qquickwindow.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/quick/items/qquickwindow.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickWindow_t {
    QByteArrayData data[61];
    char stringdata0[987];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickWindow_t qt_meta_stringdata_QQuickWindow = {
    {
QT_MOC_LITERAL(0, 0, 12), // "QQuickWindow"
QT_MOC_LITERAL(1, 13, 15), // "DefaultProperty"
QT_MOC_LITERAL(2, 29, 4), // "data"
QT_MOC_LITERAL(3, 34, 12), // "frameSwapped"
QT_MOC_LITERAL(4, 47, 0), // ""
QT_MOC_LITERAL(5, 48, 20), // "openglContextCreated"
QT_MOC_LITERAL(6, 69, 15), // "QOpenGLContext*"
QT_MOC_LITERAL(7, 85, 7), // "context"
QT_MOC_LITERAL(8, 93, 21), // "sceneGraphInitialized"
QT_MOC_LITERAL(9, 115, 21), // "sceneGraphInvalidated"
QT_MOC_LITERAL(10, 137, 19), // "beforeSynchronizing"
QT_MOC_LITERAL(11, 157, 18), // "afterSynchronizing"
QT_MOC_LITERAL(12, 176, 15), // "beforeRendering"
QT_MOC_LITERAL(13, 192, 14), // "afterRendering"
QT_MOC_LITERAL(14, 207, 14), // "afterAnimating"
QT_MOC_LITERAL(15, 222, 21), // "sceneGraphAboutToStop"
QT_MOC_LITERAL(16, 244, 7), // "closing"
QT_MOC_LITERAL(17, 252, 17), // "QQuickCloseEvent*"
QT_MOC_LITERAL(18, 270, 5), // "close"
QT_MOC_LITERAL(19, 276, 12), // "colorChanged"
QT_MOC_LITERAL(20, 289, 22), // "activeFocusItemChanged"
QT_MOC_LITERAL(21, 312, 15), // "sceneGraphError"
QT_MOC_LITERAL(22, 328, 29), // "QQuickWindow::SceneGraphError"
QT_MOC_LITERAL(23, 358, 5), // "error"
QT_MOC_LITERAL(24, 364, 7), // "message"
QT_MOC_LITERAL(25, 372, 25), // "beforeRenderPassRecording"
QT_MOC_LITERAL(26, 398, 24), // "afterRenderPassRecording"
QT_MOC_LITERAL(27, 423, 6), // "update"
QT_MOC_LITERAL(28, 430, 16), // "releaseResources"
QT_MOC_LITERAL(29, 447, 11), // "maybeUpdate"
QT_MOC_LITERAL(30, 459, 17), // "cleanupSceneGraph"
QT_MOC_LITERAL(31, 477, 18), // "physicalDpiChanged"
QT_MOC_LITERAL(32, 496, 19), // "handleScreenChanged"
QT_MOC_LITERAL(33, 516, 8), // "QScreen*"
QT_MOC_LITERAL(34, 525, 6), // "screen"
QT_MOC_LITERAL(35, 532, 25), // "setTransientParent_helper"
QT_MOC_LITERAL(36, 558, 13), // "QQuickWindow*"
QT_MOC_LITERAL(37, 572, 6), // "window"
QT_MOC_LITERAL(38, 579, 16), // "runJobsAfterSwap"
QT_MOC_LITERAL(39, 596, 29), // "handleApplicationStateChanged"
QT_MOC_LITERAL(40, 626, 20), // "Qt::ApplicationState"
QT_MOC_LITERAL(41, 647, 5), // "state"
QT_MOC_LITERAL(42, 653, 25), // "QQmlListProperty<QObject>"
QT_MOC_LITERAL(43, 679, 5), // "color"
QT_MOC_LITERAL(44, 685, 11), // "contentItem"
QT_MOC_LITERAL(45, 697, 11), // "QQuickItem*"
QT_MOC_LITERAL(46, 709, 15), // "activeFocusItem"
QT_MOC_LITERAL(47, 725, 20), // "CreateTextureOptions"
QT_MOC_LITERAL(48, 746, 19), // "CreateTextureOption"
QT_MOC_LITERAL(49, 766, 22), // "TextureHasAlphaChannel"
QT_MOC_LITERAL(50, 789, 17), // "TextureHasMipmaps"
QT_MOC_LITERAL(51, 807, 20), // "TextureOwnsGLTexture"
QT_MOC_LITERAL(52, 828, 18), // "TextureCanUseAtlas"
QT_MOC_LITERAL(53, 847, 15), // "TextureIsOpaque"
QT_MOC_LITERAL(54, 863, 15), // "SceneGraphError"
QT_MOC_LITERAL(55, 879, 19), // "ContextNotAvailable"
QT_MOC_LITERAL(56, 899, 14), // "TextRenderType"
QT_MOC_LITERAL(57, 914, 15), // "QtTextRendering"
QT_MOC_LITERAL(58, 930, 19), // "NativeTextRendering"
QT_MOC_LITERAL(59, 950, 16), // "NativeObjectType"
QT_MOC_LITERAL(60, 967, 19) // "NativeObjectTexture"

    },
    "QQuickWindow\0DefaultProperty\0data\0"
    "frameSwapped\0\0openglContextCreated\0"
    "QOpenGLContext*\0context\0sceneGraphInitialized\0"
    "sceneGraphInvalidated\0beforeSynchronizing\0"
    "afterSynchronizing\0beforeRendering\0"
    "afterRendering\0afterAnimating\0"
    "sceneGraphAboutToStop\0closing\0"
    "QQuickCloseEvent*\0close\0colorChanged\0"
    "activeFocusItemChanged\0sceneGraphError\0"
    "QQuickWindow::SceneGraphError\0error\0"
    "message\0beforeRenderPassRecording\0"
    "afterRenderPassRecording\0update\0"
    "releaseResources\0maybeUpdate\0"
    "cleanupSceneGraph\0physicalDpiChanged\0"
    "handleScreenChanged\0QScreen*\0screen\0"
    "setTransientParent_helper\0QQuickWindow*\0"
    "window\0runJobsAfterSwap\0"
    "handleApplicationStateChanged\0"
    "Qt::ApplicationState\0state\0"
    "QQmlListProperty<QObject>\0color\0"
    "contentItem\0QQuickItem*\0activeFocusItem\0"
    "CreateTextureOptions\0CreateTextureOption\0"
    "TextureHasAlphaChannel\0TextureHasMipmaps\0"
    "TextureOwnsGLTexture\0TextureCanUseAtlas\0"
    "TextureIsOpaque\0SceneGraphError\0"
    "ContextNotAvailable\0TextRenderType\0"
    "QtTextRendering\0NativeTextRendering\0"
    "NativeObjectType\0NativeObjectTexture"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickWindow[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      25,   16, // methods
       4,  207, // properties
       4,  227, // enums/sets
       0,    0, // constructors
       0,       // flags
      16,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,  166,    4, 0x06 /* Public */,
       5,    1,  167,    4, 0x86 /* Public | MethodRevisioned */,
       8,    0,  170,    4, 0x06 /* Public */,
       9,    0,  171,    4, 0x06 /* Public */,
      10,    0,  172,    4, 0x06 /* Public */,
      11,    0,  173,    4, 0x86 /* Public | MethodRevisioned */,
      12,    0,  174,    4, 0x06 /* Public */,
      13,    0,  175,    4, 0x06 /* Public */,
      14,    0,  176,    4, 0x86 /* Public | MethodRevisioned */,
      15,    0,  177,    4, 0x86 /* Public | MethodRevisioned */,
      16,    1,  178,    4, 0x86 /* Public | MethodRevisioned */,
      19,    1,  181,    4, 0x06 /* Public */,
      20,    0,  184,    4, 0x86 /* Public | MethodRevisioned */,
      21,    2,  185,    4, 0x86 /* Public | MethodRevisioned */,
      25,    0,  190,    4, 0x86 /* Public | MethodRevisioned */,
      26,    0,  191,    4, 0x86 /* Public | MethodRevisioned */,

 // slots: name, argc, parameters, tag, flags
      27,    0,  192,    4, 0x0a /* Public */,
      28,    0,  193,    4, 0x0a /* Public */,
      29,    0,  194,    4, 0x08 /* Private */,
      30,    0,  195,    4, 0x08 /* Private */,
      31,    0,  196,    4, 0x08 /* Private */,
      32,    1,  197,    4, 0x08 /* Private */,
      35,    1,  200,    4, 0x08 /* Private */,
      38,    0,  203,    4, 0x08 /* Private */,
      39,    1,  204,    4, 0x08 /* Private */,

 // signals: revision
       0,
       2,
       0,
       0,
       0,
       2,
       0,
       0,
       2,
       2,
       1,
       0,
       1,
       2,
      14,
      14,

 // slots: revision
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 17,   18,
    QMetaType::Void, QMetaType::QColor,    4,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 22, QMetaType::QString,   23,   24,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 33,   34,
    QMetaType::Void, 0x80000000 | 36,   37,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 40,   41,

 // properties: name, type, flags
       2, 0x80000000 | 42, 0x00094009,
      43, QMetaType::QColor, 0x00495103,
      44, 0x80000000 | 45, 0x00095409,
      46, 0x80000000 | 45, 0x00c95009,

 // properties: notify_signal_id
       0,
      11,
       0,
      12,

 // properties: revision
       0,
       0,
       0,
       1,

 // enums: name, alias, flags, count, data
      47,   48, 0x1,    5,  247,
      54,   54, 0x0,    1,  257,
      56,   56, 0x0,    2,  259,
      59,   59, 0x0,    1,  263,

 // enum data: key, value
      49, uint(QQuickWindow::TextureHasAlphaChannel),
      50, uint(QQuickWindow::TextureHasMipmaps),
      51, uint(QQuickWindow::TextureOwnsGLTexture),
      52, uint(QQuickWindow::TextureCanUseAtlas),
      53, uint(QQuickWindow::TextureIsOpaque),
      55, uint(QQuickWindow::ContextNotAvailable),
      57, uint(QQuickWindow::QtTextRendering),
      58, uint(QQuickWindow::NativeTextRendering),
      60, uint(QQuickWindow::NativeObjectTexture),

       0        // eod
};

void QQuickWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickWindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->frameSwapped(); break;
        case 1: _t->openglContextCreated((*reinterpret_cast< QOpenGLContext*(*)>(_a[1]))); break;
        case 2: _t->sceneGraphInitialized(); break;
        case 3: _t->sceneGraphInvalidated(); break;
        case 4: _t->beforeSynchronizing(); break;
        case 5: _t->afterSynchronizing(); break;
        case 6: _t->beforeRendering(); break;
        case 7: _t->afterRendering(); break;
        case 8: _t->afterAnimating(); break;
        case 9: _t->sceneGraphAboutToStop(); break;
        case 10: _t->closing((*reinterpret_cast< QQuickCloseEvent*(*)>(_a[1]))); break;
        case 11: _t->colorChanged((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 12: _t->activeFocusItemChanged(); break;
        case 13: _t->sceneGraphError((*reinterpret_cast< QQuickWindow::SceneGraphError(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 14: _t->beforeRenderPassRecording(); break;
        case 15: _t->afterRenderPassRecording(); break;
        case 16: _t->update(); break;
        case 17: _t->releaseResources(); break;
        case 18: _t->maybeUpdate(); break;
        case 19: _t->cleanupSceneGraph(); break;
        case 20: _t->physicalDpiChanged(); break;
        case 21: _t->handleScreenChanged((*reinterpret_cast< QScreen*(*)>(_a[1]))); break;
        case 22: _t->setTransientParent_helper((*reinterpret_cast< QQuickWindow*(*)>(_a[1]))); break;
        case 23: _t->runJobsAfterSwap(); break;
        case 24: _t->handleApplicationStateChanged((*reinterpret_cast< Qt::ApplicationState(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickWindow* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWindow::frameSwapped)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickWindow::*)(QOpenGLContext * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWindow::openglContextCreated)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWindow::sceneGraphInitialized)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWindow::sceneGraphInvalidated)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QQuickWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWindow::beforeSynchronizing)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QQuickWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWindow::afterSynchronizing)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QQuickWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWindow::beforeRendering)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QQuickWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWindow::afterRendering)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QQuickWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWindow::afterAnimating)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QQuickWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWindow::sceneGraphAboutToStop)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QQuickWindow::*)(QQuickCloseEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWindow::closing)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QQuickWindow::*)(const QColor & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWindow::colorChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QQuickWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWindow::activeFocusItemChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QQuickWindow::*)(QQuickWindow::SceneGraphError , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWindow::sceneGraphError)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QQuickWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWindow::beforeRenderPassRecording)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QQuickWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickWindow::afterRenderPassRecording)) {
                *result = 15;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlListProperty<QObject> >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickWindow *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQmlListProperty<QObject>*>(_v) = _t->QQuickWindow::d_func()->data(); break;
        case 1: *reinterpret_cast< QColor*>(_v) = _t->color(); break;
        case 2: *reinterpret_cast< QQuickItem**>(_v) = _t->contentItem(); break;
        case 3: *reinterpret_cast< QQuickItem**>(_v) = _t->activeFocusItem(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickWindow *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setColor(*reinterpret_cast< QColor*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QWindow::staticMetaObject>(),
    qt_meta_stringdata_QQuickWindow.data,
    qt_meta_data_QQuickWindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickWindow.stringdata0))
        return static_cast<void*>(this);
    return QWindow::qt_metacast(_clname);
}

int QQuickWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickWindow::frameSwapped()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickWindow::openglContextCreated(QOpenGLContext * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QQuickWindow::sceneGraphInitialized()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickWindow::sceneGraphInvalidated()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QQuickWindow::beforeSynchronizing()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QQuickWindow::afterSynchronizing()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QQuickWindow::beforeRendering()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QQuickWindow::afterRendering()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QQuickWindow::afterAnimating()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QQuickWindow::sceneGraphAboutToStop()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QQuickWindow::closing(QQuickCloseEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void QQuickWindow::colorChanged(const QColor & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void QQuickWindow::activeFocusItemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void QQuickWindow::sceneGraphError(QQuickWindow::SceneGraphError _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void QQuickWindow::beforeRenderPassRecording()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void QQuickWindow::afterRenderPassRecording()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
