/****************************************************************************
** Meta object code from reading C++ file 'qquickopenglinfo_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/quick/items/qquickopenglinfo_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickopenglinfo_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickOpenGLInfo_t {
    QByteArrayData data[31];
    char stringdata0[447];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickOpenGLInfo_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickOpenGLInfo_t qt_meta_stringdata_QQuickOpenGLInfo = {
    {
QT_MOC_LITERAL(0, 0, 16), // "QQuickOpenGLInfo"
QT_MOC_LITERAL(1, 17, 11), // "QML.Element"
QT_MOC_LITERAL(2, 29, 10), // "OpenGLInfo"
QT_MOC_LITERAL(3, 40, 13), // "QML.Creatable"
QT_MOC_LITERAL(4, 54, 5), // "false"
QT_MOC_LITERAL(5, 60, 21), // "QML.UncreatableReason"
QT_MOC_LITERAL(6, 82, 53), // "OpenGLInfo is only available ..."
QT_MOC_LITERAL(7, 136, 23), // "QML.AddedInMinorVersion"
QT_MOC_LITERAL(8, 160, 1), // "4"
QT_MOC_LITERAL(9, 162, 12), // "QML.Attached"
QT_MOC_LITERAL(10, 175, 19), // "majorVersionChanged"
QT_MOC_LITERAL(11, 195, 0), // ""
QT_MOC_LITERAL(12, 196, 19), // "minorVersionChanged"
QT_MOC_LITERAL(13, 216, 14), // "profileChanged"
QT_MOC_LITERAL(14, 231, 21), // "renderableTypeChanged"
QT_MOC_LITERAL(15, 253, 12), // "updateFormat"
QT_MOC_LITERAL(16, 266, 9), // "setWindow"
QT_MOC_LITERAL(17, 276, 13), // "QQuickWindow*"
QT_MOC_LITERAL(18, 290, 6), // "window"
QT_MOC_LITERAL(19, 297, 12), // "majorVersion"
QT_MOC_LITERAL(20, 310, 12), // "minorVersion"
QT_MOC_LITERAL(21, 323, 7), // "profile"
QT_MOC_LITERAL(22, 331, 14), // "ContextProfile"
QT_MOC_LITERAL(23, 346, 14), // "renderableType"
QT_MOC_LITERAL(24, 361, 14), // "RenderableType"
QT_MOC_LITERAL(25, 376, 9), // "NoProfile"
QT_MOC_LITERAL(26, 386, 11), // "CoreProfile"
QT_MOC_LITERAL(27, 398, 20), // "CompatibilityProfile"
QT_MOC_LITERAL(28, 419, 11), // "Unspecified"
QT_MOC_LITERAL(29, 431, 6), // "OpenGL"
QT_MOC_LITERAL(30, 438, 8) // "OpenGLES"

    },
    "QQuickOpenGLInfo\0QML.Element\0OpenGLInfo\0"
    "QML.Creatable\0false\0QML.UncreatableReason\0"
    "OpenGLInfo is only available via attached properties.\0"
    "QML.AddedInMinorVersion\0""4\0QML.Attached\0"
    "majorVersionChanged\0\0minorVersionChanged\0"
    "profileChanged\0renderableTypeChanged\0"
    "updateFormat\0setWindow\0QQuickWindow*\0"
    "window\0majorVersion\0minorVersion\0"
    "profile\0ContextProfile\0renderableType\0"
    "RenderableType\0NoProfile\0CoreProfile\0"
    "CompatibilityProfile\0Unspecified\0"
    "OpenGL\0OpenGLES"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickOpenGLInfo[] = {

 // content:
       8,       // revision
       0,       // classname
       5,   14, // classinfo
       6,   24, // methods
       4,   62, // properties
       2,   78, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,
       5,    6,
       7,    8,
       9,    0,

 // signals: name, argc, parameters, tag, flags
      10,    0,   54,   11, 0x06 /* Public */,
      12,    0,   55,   11, 0x06 /* Public */,
      13,    0,   56,   11, 0x06 /* Public */,
      14,    0,   57,   11, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      15,    0,   58,   11, 0x08 /* Private */,
      16,    1,   59,   11, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 17,   18,

 // properties: name, type, flags
      19, QMetaType::Int, 0x00495801,
      20, QMetaType::Int, 0x00495801,
      21, 0x80000000 | 22, 0x00495809,
      23, 0x80000000 | 24, 0x00495809,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,

 // enums: name, alias, flags, count, data
      22,   22, 0x0,    3,   88,
      24,   24, 0x0,    3,   94,

 // enum data: key, value
      25, uint(QQuickOpenGLInfo::NoProfile),
      26, uint(QQuickOpenGLInfo::CoreProfile),
      27, uint(QQuickOpenGLInfo::CompatibilityProfile),
      28, uint(QQuickOpenGLInfo::Unspecified),
      29, uint(QQuickOpenGLInfo::OpenGL),
      30, uint(QQuickOpenGLInfo::OpenGLES),

       0        // eod
};

void QQuickOpenGLInfo::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickOpenGLInfo *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->majorVersionChanged(); break;
        case 1: _t->minorVersionChanged(); break;
        case 2: _t->profileChanged(); break;
        case 3: _t->renderableTypeChanged(); break;
        case 4: _t->updateFormat(); break;
        case 5: _t->setWindow((*reinterpret_cast< QQuickWindow*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickOpenGLInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickOpenGLInfo::majorVersionChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickOpenGLInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickOpenGLInfo::minorVersionChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickOpenGLInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickOpenGLInfo::profileChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickOpenGLInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickOpenGLInfo::renderableTypeChanged)) {
                *result = 3;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickOpenGLInfo *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->majorVersion(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->minorVersion(); break;
        case 2: *reinterpret_cast< ContextProfile*>(_v) = _t->profile(); break;
        case 3: *reinterpret_cast< RenderableType*>(_v) = _t->renderableType(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickOpenGLInfo::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QQuickOpenGLInfo.data,
    qt_meta_data_QQuickOpenGLInfo,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickOpenGLInfo::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickOpenGLInfo::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickOpenGLInfo.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QQuickOpenGLInfo::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickOpenGLInfo::majorVersionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickOpenGLInfo::minorVersionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QQuickOpenGLInfo::profileChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QQuickOpenGLInfo::renderableTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
