/****************************************************************************
** Meta object code from reading C++ file 'qquickparticlesystem_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kde-qtdeclarative/src/particles/qquickparticlesystem_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qquickparticlesystem_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QQuickParticleSystem_t {
    QByteArrayData data[28];
    char stringdata0[292];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickParticleSystem_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickParticleSystem_t qt_meta_stringdata_QQuickParticleSystem = {
    {
QT_MOC_LITERAL(0, 0, 20), // "QQuickParticleSystem"
QT_MOC_LITERAL(1, 21, 11), // "QML.Element"
QT_MOC_LITERAL(2, 33, 14), // "ParticleSystem"
QT_MOC_LITERAL(3, 48, 17), // "systemInitialized"
QT_MOC_LITERAL(4, 66, 0), // ""
QT_MOC_LITERAL(5, 67, 14), // "runningChanged"
QT_MOC_LITERAL(6, 82, 3), // "arg"
QT_MOC_LITERAL(7, 86, 13), // "pausedChanged"
QT_MOC_LITERAL(8, 100, 12), // "emptyChanged"
QT_MOC_LITERAL(9, 113, 5), // "start"
QT_MOC_LITERAL(10, 119, 4), // "stop"
QT_MOC_LITERAL(11, 124, 7), // "restart"
QT_MOC_LITERAL(12, 132, 5), // "pause"
QT_MOC_LITERAL(13, 138, 6), // "resume"
QT_MOC_LITERAL(14, 145, 5), // "reset"
QT_MOC_LITERAL(15, 151, 10), // "setRunning"
QT_MOC_LITERAL(16, 162, 9), // "setPaused"
QT_MOC_LITERAL(17, 172, 8), // "duration"
QT_MOC_LITERAL(18, 181, 15), // "emittersChanged"
QT_MOC_LITERAL(19, 197, 11), // "loadPainter"
QT_MOC_LITERAL(20, 209, 22), // "QQuickParticlePainter*"
QT_MOC_LITERAL(21, 232, 1), // "p"
QT_MOC_LITERAL(22, 234, 12), // "createEngine"
QT_MOC_LITERAL(23, 247, 19), // "particleStateChange"
QT_MOC_LITERAL(24, 267, 3), // "idx"
QT_MOC_LITERAL(25, 271, 7), // "running"
QT_MOC_LITERAL(26, 279, 6), // "paused"
QT_MOC_LITERAL(27, 286, 5) // "empty"

    },
    "QQuickParticleSystem\0QML.Element\0"
    "ParticleSystem\0systemInitialized\0\0"
    "runningChanged\0arg\0pausedChanged\0"
    "emptyChanged\0start\0stop\0restart\0pause\0"
    "resume\0reset\0setRunning\0setPaused\0"
    "duration\0emittersChanged\0loadPainter\0"
    "QQuickParticlePainter*\0p\0createEngine\0"
    "particleStateChange\0idx\0running\0paused\0"
    "empty"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickParticleSystem[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      17,   16, // methods
       3,  132, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,  101,    4, 0x06 /* Public */,
       5,    1,  102,    4, 0x06 /* Public */,
       7,    1,  105,    4, 0x06 /* Public */,
       8,    1,  108,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    0,  111,    4, 0x0a /* Public */,
      10,    0,  112,    4, 0x0a /* Public */,
      11,    0,  113,    4, 0x0a /* Public */,
      12,    0,  114,    4, 0x0a /* Public */,
      13,    0,  115,    4, 0x0a /* Public */,
      14,    0,  116,    4, 0x0a /* Public */,
      15,    1,  117,    4, 0x0a /* Public */,
      16,    1,  120,    4, 0x0a /* Public */,
      17,    0,  123,    4, 0x0a /* Public */,
      18,    0,  124,    4, 0x08 /* Private */,
      19,    1,  125,    4, 0x08 /* Private */,
      22,    0,  128,    4, 0x08 /* Private */,
      23,    1,  129,    4, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    6,
    QMetaType::Void, QMetaType::Bool,    6,
    QMetaType::Void, QMetaType::Bool,    6,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    6,
    QMetaType::Void, QMetaType::Bool,    6,
    QMetaType::Int,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 20,   21,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   24,

 // properties: name, type, flags
      25, QMetaType::Bool, 0x00495103,
      26, QMetaType::Bool, 0x00495103,
      27, QMetaType::Bool, 0x00495001,

 // properties: notify_signal_id
       1,
       2,
       3,

       0        // eod
};

void QQuickParticleSystem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QQuickParticleSystem *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->systemInitialized(); break;
        case 1: _t->runningChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->pausedChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->emptyChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->start(); break;
        case 5: _t->stop(); break;
        case 6: _t->restart(); break;
        case 7: _t->pause(); break;
        case 8: _t->resume(); break;
        case 9: _t->reset(); break;
        case 10: _t->setRunning((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->setPaused((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 12: { int _r = _t->duration();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->emittersChanged(); break;
        case 14: _t->loadPainter((*reinterpret_cast< QQuickParticlePainter*(*)>(_a[1]))); break;
        case 15: _t->createEngine(); break;
        case 16: _t->particleStateChange((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QQuickParticleSystem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleSystem::systemInitialized)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QQuickParticleSystem::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleSystem::runningChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QQuickParticleSystem::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleSystem::pausedChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QQuickParticleSystem::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QQuickParticleSystem::emptyChanged)) {
                *result = 3;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QQuickParticleSystem *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->isRunning(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->isPaused(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->isEmpty(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QQuickParticleSystem *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setRunning(*reinterpret_cast< bool*>(_v)); break;
        case 1: _t->setPaused(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QQuickParticleSystem::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_meta_stringdata_QQuickParticleSystem.data,
    qt_meta_data_QQuickParticleSystem,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickParticleSystem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickParticleSystem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickParticleSystem.stringdata0))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int QQuickParticleSystem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 17;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 3;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QQuickParticleSystem::systemInitialized()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QQuickParticleSystem::runningChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QQuickParticleSystem::pausedChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QQuickParticleSystem::emptyChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
struct qt_meta_stringdata_QQuickParticleSystemAnimation_t {
    QByteArrayData data[1];
    char stringdata0[30];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QQuickParticleSystemAnimation_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QQuickParticleSystemAnimation_t qt_meta_stringdata_QQuickParticleSystemAnimation = {
    {
QT_MOC_LITERAL(0, 0, 29) // "QQuickParticleSystemAnimation"

    },
    "QQuickParticleSystemAnimation"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QQuickParticleSystemAnimation[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void QQuickParticleSystemAnimation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QQuickParticleSystemAnimation::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractAnimation::staticMetaObject>(),
    qt_meta_stringdata_QQuickParticleSystemAnimation.data,
    qt_meta_data_QQuickParticleSystemAnimation,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QQuickParticleSystemAnimation::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QQuickParticleSystemAnimation::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QQuickParticleSystemAnimation.stringdata0))
        return static_cast<void*>(this);
    return QAbstractAnimation::qt_metacast(_clname);
}

int QQuickParticleSystemAnimation::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractAnimation::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
